@@ -1,25 +1,25 @@
 /*
-    This file is part of KAddressBook.                                  
-    Copyright (c) 2002 Mike Pilone <mpilone@slac.com>                   
-                                                                        
+    This file is part of KAddressBook.
+    Copyright (c) 2002 Mike Pilone <mpilone@slac.com>
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or   
-    (at your option) any later version.                                 
-                                                                        
-    This program is distributed in the hope that it will be useful,     
-    but WITHOUT ANY WARRANTY; without even the implied warranty of      
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the        
-    GNU General Public License for more details.                        
-                                                                        
-    You should have received a copy of the GNU General Public License   
-    along with this program; if not, write to the Free Software         
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           
-                                                                        
-    As a special exception, permission is given to link this program    
-    with any edition of Qt, and distribute the resulting executable,    
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
+    As a special exception, permission is given to link this program
+    with any edition of Qt, and distribute the resulting executable,
     without including the source code for Qt in the source distribution.
-*/                                                                      
+*/
 
 #include <qcheckbox.h>
 #include <qhbox.h>
@@ -33,6 +33,7 @@
 #include <qtooltip.h>
 
 #include <kabc/resource.h>
+#include <kabc/stdaddressbook.h>
 #include <kaccelmanager.h>
 #include <kapplication.h>
 #include <kconfig.h>
@@ -55,8 +56,8 @@
 
 #include "addresseditwidget.h"
 #include "advancedcustomfields.h"
-#include "core.h"
 #include "emaileditwidget.h"
+#include "imeditwidget.h"
 #include "kabprefs.h"
 #include "keywidget.h"
 #include "nameeditdialog.h"
@@ -65,15 +66,14 @@
 
 #include "addresseeeditorwidget.h"
 
-AddresseeEditorWidget::AddresseeEditorWidget( KAB::Core *core, bool isExtension, 
-                                              QWidget *parent, const char *name )
-  : AddresseeEditorBase( core, isExtension, parent, name ),
+AddresseeEditorWidget::AddresseeEditorWidget( QWidget *parent, const char *name )
+  : AddresseeEditorBase( parent, name ),
     mBlockSignals( false ), mReadOnly( false )
 {
   kdDebug(5720) << "AddresseeEditorWidget()" << endl;
 
   initGUI();
-  mCategoryDialog = 0;
+  mCategorySelectDialog = 0;
   mCategoryEditDialog = 0;
 
   // Load the empty addressee as defaults
@@ -85,11 +85,11 @@ AddresseeEditorWidget::AddresseeEditorWidget( KAB::Core *core, bool isExtension,
 AddresseeEditorWidget::~AddresseeEditorWidget()
 {
   kdDebug(5720) << "~AddresseeEditorWidget()" << endl;
-}  
+}
 
 void AddresseeEditorWidget::setAddressee( const KABC::Addressee &addr )
 {
-  if ( mAddressee.uid() == addr.uid() ) 
+  if ( mAddressee.uid() == addr.uid() )
 	  return;
 
   mAddressee = addr;
@@ -134,30 +134,30 @@ void AddresseeEditorWidget::setupTab1()
   QGridLayout *layout = new QGridLayout( tab1, 11, 7 );
   layout->setMargin( KDialogBase::marginHint() );
   layout->setSpacing( KDialogBase::spacingHint() );
-  
+
   QLabel *label;
   KSeparator* bar;
   QPushButton *button;
-  
+
   //////////////////////////////////
   // Upper left group (person info)
-  
+
   // Person icon
   label = new QLabel( tab1 );
   label->setPixmap( KGlobal::iconLoader()->loadIcon( "personal", KIcon::Desktop,
                                                       KIcon::SizeMedium ) );
   layout->addMultiCellWidget( label, 0, 1, 0, 0 );
-  
+
   // First name
   button = new QPushButton( i18n( "Edit Name..." ), tab1 );
   QToolTip::add( button, i18n( "Edit the contact's name" ) );
   mNameEdit = new KLineEdit( tab1, "mNameEdit" );
-  connect( mNameEdit, SIGNAL( textChanged( const QString& ) ), 
+  connect( mNameEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( nameTextChanged( const QString& ) ) );
   connect( button, SIGNAL( clicked() ), SLOT( nameButtonClicked() ) );
   mNameLabel = new KSqueezedTextLabel( tab1 );
 
-  if ( KABPrefs::instance()->mAutomaticNameParsing ) {
+  if ( KABPrefs::instance()->automaticNameParsing() ) {
     mNameLabel->hide();
     mNameEdit->show();
   } else {
@@ -168,58 +168,58 @@ void AddresseeEditorWidget::setupTab1()
   layout->addWidget( button, 0, 1 );
   layout->addWidget( mNameEdit, 0, 2 );
   layout->addWidget( mNameLabel, 0, 2 );
-  label = new QLabel( i18n( "%1:" ).arg( KABC::Addressee::roleLabel() ), tab1 );
+  label = new QLabel( i18n( "<roleLabel>:", "%1:" ).arg( KABC::Addressee::roleLabel() ), tab1 );
   mRoleEdit = new KLineEdit( tab1 );
   connect( mRoleEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mRoleEdit );
   layout->addWidget( label, 1, 1 );
   layout->addWidget( mRoleEdit, 1, 2 );
-  
+
   // Organization
-  label = new QLabel( i18n( "%1:" ).arg( KABC::Addressee::organizationLabel() ), tab1 );
+  label = new QLabel( i18n( "<organizationLabel>:", "%1:" ).arg( KABC::Addressee::organizationLabel() ), tab1 );
   mOrgEdit = new KLineEdit( tab1 );
   label->setBuddy( mOrgEdit );
-  connect( mOrgEdit, SIGNAL( textChanged( const QString& ) ), 
+  connect( mOrgEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( organizationTextChanged( const QString& ) ) );
   layout->addWidget( label, 2, 1 );
   layout->addWidget( mOrgEdit, 2, 2 );
-  
+
   // File as (formatted name)
   label = new QLabel( i18n( "Formatted name:" ), tab1 );
   mFormattedNameLabel = new KSqueezedTextLabel( tab1 );
   layout->addWidget( label, 3, 1 );
   layout->addWidget( mFormattedNameLabel, 3, 2 );
-  
+
   // Left hand separator. This separator doesn't go all the way
   // across so the dialog still flows from top to bottom
   bar = new KSeparator( KSeparator::HLine, tab1 );
   layout->addMultiCellWidget( bar, 4, 4, 0, 2 );
-  
+
   //////////////////////////////////////
   // Phone numbers (upper right)
   label = new QLabel( tab1 );
   label->setPixmap( KGlobal::iconLoader()->loadIcon( "kaddressbook",
                     KIcon::Desktop, KIcon::SizeMedium ) );
   layout->addMultiCellWidget( label, 0, 1, 3, 3 );
-  
+
   mPhoneEditWidget = new PhoneEditWidget( tab1 );
   connect( mPhoneEditWidget, SIGNAL( modified() ), SLOT( emitModified() ) );
-  layout->addMultiCellWidget( mPhoneEditWidget, 0, 3, 4, 6 ); 
+  layout->addMultiCellWidget( mPhoneEditWidget, 0, 3, 4, 6 );
 
   bar = new KSeparator( KSeparator::HLine, tab1 );
   layout->addMultiCellWidget( bar, 4, 4, 3, 6 );
-  
+
   //////////////////////////////////////
   // Addresses (lower left)
   label = new QLabel( tab1 );
-  label->setPixmap( KGlobal::iconLoader()->loadIcon( "gohome", KIcon::Desktop,
+  label->setPixmap( KGlobal::iconLoader()->loadIcon( "kfm_home", KIcon::Desktop,
                                                      KIcon::SizeMedium ) );
   layout->addMultiCellWidget( label, 5, 6, 0, 0 );
-  
+
   mAddressEditWidget = new AddressEditWidget( tab1 );
   connect( mAddressEditWidget, SIGNAL( modified() ), SLOT( emitModified() ) );
-  layout->addMultiCellWidget( mAddressEditWidget, 5, 9, 1, 2 );
+  layout->addMultiCellWidget( mAddressEditWidget, 5, 10, 1, 2 );
 
   //////////////////////////////////////
   // Email / Web (lower right)
@@ -236,50 +236,59 @@ void AddresseeEditorWidget::setupTab1()
   bar = new KSeparator( KSeparator::HLine, tab1 );
   layout->addMultiCellWidget( bar, 7, 7, 3, 6 );
 
+  QHBoxLayout *homePageLayout = new QHBoxLayout( 0, 11, 7 );
+
   label = new QLabel( tab1 );
   label->setPixmap( KGlobal::iconLoader()->loadIcon( "homepage", KIcon::Desktop,
                                                      KIcon::SizeMedium ) );
-  layout->addMultiCellWidget( label, 8, 9, 3, 3 );
+  homePageLayout->addWidget( label );
 
-  label = new QLabel( i18n( "%1:" ).arg( KABC::Addressee::urlLabel() ), tab1 );
+  label = new QLabel( i18n( "<urlLabel>:", "%1:" ).arg( KABC::Addressee::urlLabel() ), tab1 );
   mURLEdit = new KLineEdit( tab1 );
-  connect( mURLEdit, SIGNAL( textChanged( const QString& ) ), 
+  connect( mURLEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mURLEdit );
-  layout->addWidget( label, 8, 4 );
-  layout->addMultiCellWidget( mURLEdit, 8, 8, 5, 6 );
+  homePageLayout->addWidget( label );
+  homePageLayout->addWidget( mURLEdit );
+  layout->addMultiCellLayout( homePageLayout, 8, 8, 3, 6 );
+
+  QHBoxLayout *blogLayout = new QHBoxLayout( 0, 11, 7 );
+  label = new QLabel( i18n("Blog feed:"), tab1 );
+  blogLayout->addWidget( label );
+  mBlogEdit = new KLineEdit( tab1 );
+  blogLayout->addWidget( mBlogEdit );
+  connect( mBlogEdit, SIGNAL( textChanged( const QString & ) ),
+           SLOT( textChanged( const QString & ) ) );
+  label->setBuddy( mBlogEdit );
+  layout->addMultiCellLayout( blogLayout, 9, 9, 4, 6 );
+
+  mIMWidget = new IMEditWidget( tab1, mAddressee );
+  connect( mIMWidget, SIGNAL( modified() ), SLOT( emitModified() ) );
+  layout->addMultiCellWidget( mIMWidget, 10, 10, 4, 6 );
 
-  label = new QLabel( i18n( "&IM address:" ), tab1 );
-  mIMAddressEdit = new KLineEdit( tab1 );
-  connect( mIMAddressEdit, SIGNAL( textChanged( const QString& ) ), 
-           SLOT( textChanged( const QString& ) ) );
-  label->setBuddy( mIMAddressEdit );
-  layout->addWidget( label, 9, 4 );
-  layout->addMultiCellWidget( mIMAddressEdit, 9, 9, 5, 6 );
-  
   layout->addColSpacing( 6, 50 );
-  
+
   bar = new KSeparator( KSeparator::HLine, tab1 );
-  layout->addMultiCellWidget( bar, 10, 10, 0, 6 );
-  
+  layout->addMultiCellWidget( bar, 11, 11, 0, 6 );
+
   ///////////////////////////////////////
   QHBox *categoryBox = new QHBox( tab1 );
   categoryBox->setSpacing( KDialogBase::spacingHint() );
-  
+
   // Categories
   mCategoryButton = new QPushButton( i18n( "Select Categories..." ), categoryBox );
-  connect( mCategoryButton, SIGNAL( clicked() ), SLOT( categoryButtonClicked() ) );
+  connect( mCategoryButton, SIGNAL( clicked() ), SLOT( selectCategories() ) );
 
   mCategoryEdit = new KLineEdit( categoryBox );
   mCategoryEdit->setReadOnly( true );
-  connect( mCategoryEdit, SIGNAL( textChanged( const QString& ) ), 
+  connect( mCategoryEdit, SIGNAL( textChanged( const QString& ) ),
            SLOT( textChanged( const QString& ) ) );
 
   mSecrecyWidget = new SecrecyWidget( categoryBox );
   connect( mSecrecyWidget, SIGNAL( changed() ), SLOT( emitModified() ) );
 
-  layout->addMultiCellWidget( categoryBox, 11, 11, 0, 6 );
-  
+  layout->addMultiCellWidget( categoryBox, 12, 12, 0, 6 );
+
   // Build the layout and add to the tab widget
   layout->activate(); // required
 
@@ -294,19 +303,19 @@ void AddresseeEditorWidget::setupTab2()
   QGridLayout *layout = new QGridLayout( tab2, 6, 6 );
   layout->setMargin( KDialogBase::marginHint() );
   layout->setSpacing( KDialogBase::spacingHint() );
-  
+
   QLabel *label;
   KSeparator* bar;
-  
+
   ///////////////////////
   // Office info
-  
+
   // Department
   label = new QLabel( tab2 );
   label->setPixmap( KGlobal::iconLoader()->loadIcon( "folder", KIcon::Desktop,
                                                      KIcon::SizeMedium ) );
   layout->addMultiCellWidget( label, 0, 1, 0, 0 );
-  
+
   label = new QLabel( i18n( "Department:" ), tab2 );
   layout->addWidget( label, 0, 1 );
   mDepartmentEdit = new KLineEdit( tab2 );
@@ -314,7 +323,7 @@ void AddresseeEditorWidget::setupTab2()
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mDepartmentEdit );
   layout->addWidget( mDepartmentEdit, 0, 2 );
-  
+
   label = new QLabel( i18n( "Office:" ), tab2 );
   layout->addWidget( label, 1, 1 );
   mOfficeEdit = new KLineEdit( tab2 );
@@ -322,7 +331,7 @@ void AddresseeEditorWidget::setupTab2()
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mOfficeEdit );
   layout->addWidget( mOfficeEdit, 1, 2 );
-  
+
   label = new QLabel( i18n( "Profession:" ), tab2 );
   layout->addWidget( label, 2, 1 );
   mProfessionEdit = new KLineEdit( tab2 );
@@ -330,7 +339,7 @@ void AddresseeEditorWidget::setupTab2()
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mProfessionEdit );
   layout->addWidget( mProfessionEdit, 2, 2 );
-  
+
   label = new QLabel( i18n( "Manager\'s name:" ), tab2 );
   layout->addWidget( label, 0, 3 );
   mManagerEdit = new KPIM::AddresseeLineEdit( tab2 );
@@ -338,7 +347,7 @@ void AddresseeEditorWidget::setupTab2()
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mManagerEdit );
   layout->addMultiCellWidget( mManagerEdit, 0, 0, 4, 5 );
-  
+
   label = new QLabel( i18n( "Assistant's name:" ), tab2 );
   layout->addWidget( label, 1, 3 );
   mAssistantEdit = new KPIM::AddresseeLineEdit( tab2 );
@@ -346,18 +355,26 @@ void AddresseeEditorWidget::setupTab2()
            SLOT( textChanged( const QString& ) ) );
   label->setBuddy( mAssistantEdit );
   layout->addMultiCellWidget( mAssistantEdit, 1, 1, 4, 5 );
-  
+
+  label = new QLabel( i18n( "<titleLabel>:", "%1:" ).arg( KABC::Addressee::titleLabel() ), tab2 );
+  layout->addWidget( label, 2, 3 );
+  mTitleEdit = new KLineEdit( tab2 );
+  connect( mTitleEdit, SIGNAL( textChanged( const QString& ) ),
+           SLOT( textChanged( const QString& ) ) );
+  label->setBuddy( mTitleEdit );
+  layout->addMultiCellWidget( mTitleEdit, 2, 2, 4, 5 );
+
   bar = new KSeparator( KSeparator::HLine, tab2 );
   layout->addMultiCellWidget( bar, 3, 3, 0, 5 );
-  
+
   /////////////////////////////////////////////////
   // Personal info
-  
+
   label = new QLabel( tab2 );
   label->setPixmap( KGlobal::iconLoader()->loadIcon( "personal", KIcon::Desktop,
                                                      KIcon::SizeMedium ) );
   layout->addMultiCellWidget( label, 4, 5, 0, 0 );
-  
+
   label = new QLabel( i18n( "Nickname:" ), tab2 );
   layout->addWidget( label, 4, 1 );
   mNicknameEdit = new KLineEdit( tab2 );
@@ -366,7 +383,7 @@ void AddresseeEditorWidget::setupTab2()
   label->setBuddy( mNicknameEdit );
   layout->addWidget( mNicknameEdit, 4, 2 );
 
-  label = new QLabel( i18n( "Spouse's name:" ), tab2 );
+  label = new QLabel( i18n( "Partner's name:" ), tab2 );
   layout->addWidget( label, 5, 1 );
   mSpouseEdit = new KPIM::AddresseeLineEdit( tab2 );
   connect( mSpouseEdit, SIGNAL( textChanged( const QString& ) ),
@@ -377,32 +394,26 @@ void AddresseeEditorWidget::setupTab2()
   label = new QLabel( i18n( "Birthdate:" ), tab2 );
   layout->addWidget( label, 4, 3 );
   mBirthdayPicker = new KDateEdit( tab2 );
-  mBirthdayPicker->setHandleInvalid( true );
-  connect( mBirthdayPicker, SIGNAL( dateChanged( QDate ) ),
-           SLOT( dateChanged( QDate ) ) );
-  connect( mBirthdayPicker, SIGNAL( invalidDateEntered() ),
-           SLOT( invalidDate() ) );
+  connect( mBirthdayPicker, SIGNAL( dateChanged( const QDate& ) ),
+           SLOT( dateChanged( const QDate& ) ) );
   connect( mBirthdayPicker, SIGNAL( textChanged( const QString& ) ),
            SLOT( emitModified() ) );
   label->setBuddy( mBirthdayPicker );
   layout->addWidget( mBirthdayPicker, 4, 4 );
-  
+
   label = new QLabel( i18n( "Anniversary:" ), tab2 );
   layout->addWidget( label, 5, 3 );
   mAnniversaryPicker = new KDateEdit( tab2 );
-  mAnniversaryPicker->setHandleInvalid( true );
-  connect( mAnniversaryPicker, SIGNAL( dateChanged( QDate ) ),
-           SLOT( dateChanged( QDate ) ) );
-  connect( mAnniversaryPicker, SIGNAL( invalidDateEntered() ),
-           SLOT( invalidDate() ) );
+  connect( mAnniversaryPicker, SIGNAL( dateChanged( const QDate& ) ),
+           SLOT( dateChanged( const QDate& ) ) );
   connect( mAnniversaryPicker, SIGNAL( textChanged( const QString& ) ),
            SLOT( emitModified() ) );
   label->setBuddy( mAnniversaryPicker );
   layout->addWidget( mAnniversaryPicker, 5, 4 );
-  
+
   bar = new KSeparator( KSeparator::HLine, tab2 );
   layout->addMultiCellWidget( bar, 6, 6, 0, 5 );
-  
+
    //////////////////////////////////////
   // Notes
   label = new QLabel( i18n( "Note:" ), tab2 );
@@ -444,7 +455,7 @@ void AddresseeEditorWidget::setupAdditionalTabs()
     }
 
     KAB::ContactEditorWidget *widget
-              = manager->factory( i )->createWidget( core()->addressBook(),
+              = manager->factory( i )->createWidget( KABC::StdAddressBook::self( true ),
                                                      page );
     if ( widget )
       page->addWidget( widget );
@@ -458,15 +469,15 @@ void AddresseeEditorWidget::setupAdditionalTabs()
 
 void AddresseeEditorWidget::setupCustomFieldsTabs()
 {
-  QStringList activePages = KABPrefs::instance()->mAdvancedCustomFields;
+  QStringList activePages = KABPrefs::instance()->advancedCustomFields();
 
-  QStringList list = KGlobal::dirs()->findAllResources( "data", "kaddressbook/contacteditorpages/*.ui", true, true );
-  for ( QStringList::iterator it = list.begin(); it != list.end(); ++it ) {
+  const QStringList list = KGlobal::dirs()->findAllResources( "data", "kaddressbook/contacteditorpages/*.ui", true, true );
+  for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it ) {
     if ( activePages.find( (*it).mid( (*it).findRev('/') + 1 ) ) == activePages.end() )
       continue;
 
     ContactEditorTabPage *page = new ContactEditorTabPage( mTabWidget );
-    AdvancedCustomFields *wdg = new AdvancedCustomFields( *it, core()->addressBook(), page );
+    AdvancedCustomFields *wdg = new AdvancedCustomFields( *it, KABC::StdAddressBook::self( true ), page );
     if ( wdg ) {
       mTabPages.insert( wdg->pageIdentifier(), page );
       mTabWidget->addTab( page, wdg->pageTitle() );
@@ -487,7 +498,7 @@ void AddresseeEditorWidget::load()
   // Block signals in case anything tries to emit modified
   // CS: This doesn't seem to work.
   bool block = signalsBlocked();
-  blockSignals( true ); 
+  blockSignals( true );
   mBlockSignals = true; // used for internal signal blocking
 
   mNameEdit->blockSignals( true );
@@ -520,26 +531,30 @@ void AddresseeEditorWidget::load()
   mOrgEdit->setText( mAddressee.organization() );
   mURLEdit->setURL( mAddressee.url() );
   mURLEdit->home( false );
+  mBlogEdit->setURL( mAddressee.custom( "KADDRESSBOOK", "BlogFeed" ) );
   mNoteEdit->setText( mAddressee.note() );
   mEmailWidget->setEmails( mAddressee.emails() );
   mPhoneEditWidget->setPhoneNumbers( mAddressee.phoneNumbers() );
   mAddressEditWidget->setAddresses( mAddressee, mAddressee.addresses() );
   mBirthdayPicker->setDate( mAddressee.birthday().date() );
-  mAnniversaryPicker->setDate( QDate::fromString( mAddressee.custom(
-                               "KADDRESSBOOK", "X-Anniversary" ), Qt::ISODate) );
+
+  QString anniversaryStr = mAddressee.custom( "KADDRESSBOOK", "X-Anniversary" );
+  QDate anniversary = (anniversaryStr.isEmpty() ? QDate() : QDate::fromString( anniversaryStr, Qt::ISODate ));
+  mAnniversaryPicker->setDate( anniversary );
   mNicknameEdit->setText( mAddressee.nickName() );
   mCategoryEdit->setText( mAddressee.categories().join( "," ) );
 
   mSecrecyWidget->setSecrecy( mAddressee.secrecy() );
 
   // Load customs
-  mIMAddressEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-IMAddress" ) );
+  mIMWidget->setPreferredIM( mAddressee.custom( "KADDRESSBOOK", "X-IMAddress" ) );
   mSpouseEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-SpousesName" ) );
   mManagerEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-ManagersName" ) );
   mAssistantEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-AssistantsName" ) );
   mDepartmentEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-Department" ) );
   mOfficeEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-Office" ) );
   mProfessionEdit->setText( mAddressee.custom( "KADDRESSBOOK", "X-Profession" ) );
+  mTitleEdit->setText( mAddressee.title() );
 
   QDictIterator<ContactEditorTabPage> it( mTabPages );
   for ( ; it.current(); ++it )
@@ -558,37 +573,67 @@ void AddresseeEditorWidget::save()
   mAddressee.setRole( mRoleEdit->text() );
   mAddressee.setOrganization( mOrgEdit->text() );
   mAddressee.setUrl( KURL( mURLEdit->text().stripWhiteSpace() ) );
+  if ( !mBlogEdit->text().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "BlogFeed", mBlogEdit->text() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "BlogFeed" );
+
   mAddressee.setNote( mNoteEdit->text() );
-  if ( mBirthdayPicker->inputIsValid() )
+  if ( mBirthdayPicker->date().isValid() )
     mAddressee.setBirthday( QDateTime( mBirthdayPicker->date() ) );
   else
     mAddressee.setBirthday( QDateTime() );
-  
+
   mAddressee.setNickName( mNicknameEdit->text() );
   mAddressee.setCategories( QStringList::split( ",", mCategoryEdit->text() ) );
 
   mAddressee.setSecrecy( mSecrecyWidget->secrecy() );
 
   // save custom fields
-  mAddressee.insertCustom( "KADDRESSBOOK", "X-IMAddress", mIMAddressEdit->text() );
-  mAddressee.insertCustom( "KADDRESSBOOK", "X-SpousesName", mSpouseEdit->text() );
-  mAddressee.insertCustom( "KADDRESSBOOK", "X-ManagersName", mManagerEdit->text() );
-  mAddressee.insertCustom( "KADDRESSBOOK", "X-AssistantsName", mAssistantEdit->text() );
-  mAddressee.insertCustom( "KADDRESSBOOK", "X-Department", mDepartmentEdit->text() );
-  mAddressee.insertCustom( "KADDRESSBOOK", "X-Office", mOfficeEdit->text() );
-  mAddressee.insertCustom( "KADDRESSBOOK", "X-Profession", mProfessionEdit->text() );
-  if ( mAnniversaryPicker->inputIsValid() )
+  if ( !mIMWidget->preferredIM().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "X-IMAddress", mIMWidget->preferredIM() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "X-IMAddress" );
+  if ( !mSpouseEdit->text().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "X-SpousesName", mSpouseEdit->text() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "X-SpousesName" );
+  if ( !mManagerEdit->text().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "X-ManagersName", mManagerEdit->text() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "X-ManagersName" );
+  if ( !mAssistantEdit->text().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "X-AssistantsName", mAssistantEdit->text() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "X-AssistantsName" );
+
+  if ( !mDepartmentEdit->text().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "X-Department", mDepartmentEdit->text() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "X-Department" );
+  if ( !mOfficeEdit->text().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "X-Office", mOfficeEdit->text() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "X-Office" );
+  if ( !mProfessionEdit->text().isEmpty() )
+    mAddressee.insertCustom( "KADDRESSBOOK", "X-Profession", mProfessionEdit->text() );
+  else
+    mAddressee.removeCustom( "KADDRESSBOOK", "X-Profession" );
+
+  if ( mAnniversaryPicker->date().isValid() )
     mAddressee.insertCustom( "KADDRESSBOOK", "X-Anniversary",
                              mAnniversaryPicker->date().toString( Qt::ISODate ) );
   else
     mAddressee.removeCustom( "KADDRESSBOOK", "X-Anniversary" );
 
+  mAddressee.setTitle( mTitleEdit->text() );
+
   // Save the email addresses
   mAddressee.setEmails( mEmailWidget->emails() );
 
   // Save the phone numbers
   KABC::PhoneNumber::List phoneNumbers;
-  KABC::PhoneNumber::List::Iterator phoneIter;
+  KABC::PhoneNumber::List::ConstIterator phoneIter;
   phoneNumbers = mAddressee.phoneNumbers();
   for ( phoneIter = phoneNumbers.begin(); phoneIter != phoneNumbers.end();
         ++phoneIter )
@@ -601,12 +646,12 @@ void AddresseeEditorWidget::save()
 
   // Save the addresses
   KABC::Address::List addresses;
-  KABC::Address::List::Iterator addressIter;
+  KABC::Address::List::ConstIterator addressIter;
   addresses = mAddressee.addresses();
   for ( addressIter = addresses.begin(); addressIter != addresses.end();
         ++addressIter )
     mAddressee.removeAddress( *addressIter );
-    
+
   addresses = mAddressEditWidget->addresses();
   for ( addressIter = addresses.begin(); addressIter != addresses.end();
         ++addressIter )
@@ -659,6 +704,8 @@ void AddresseeEditorWidget::organizationTextChanged( const QString &text )
 
   nameBoxChanged();
 
+  mAddressEditWidget->updateAddressee( mAddressee );
+
   emitModified();
 }
 
@@ -681,13 +728,15 @@ void AddresseeEditorWidget::nameBoxChanged()
     mFormattedNameLabel->setText( NameEditDialog::formattedName( mAddressee, mFormattedNameType ) );
     mAddressee.setFormattedName( NameEditDialog::formattedName( mAddressee, mFormattedNameType ) );
   }
+
+  mAddressEditWidget->updateAddressee( mAddressee );
 }
 
 void AddresseeEditorWidget::nameButtonClicked()
 {
   // show the name dialog.
   NameEditDialog dialog( mAddressee, mFormattedNameType, mReadOnly, this );
-  
+
   if ( dialog.exec() ) {
     if ( dialog.changed() ) {
       mAddressee.setFamilyName( dialog.familyName() );
@@ -708,28 +757,26 @@ void AddresseeEditorWidget::nameButtonClicked()
 
       // Update the combo box.
       nameBoxChanged();
-    
+
       emitModified();
     }
   }
 }
 
-void AddresseeEditorWidget::categoryButtonClicked()
+void AddresseeEditorWidget::selectCategories()
 {
   // Show the category dialog
-  if ( mCategoryDialog == 0 ) {
-    mCategoryDialog = new KPIM::CategorySelectDialog( KABPrefs::instance(), this );
-    connect( mCategoryDialog, SIGNAL( categoriesSelected( const QStringList& ) ),
-             SLOT(categoriesSelected( const QStringList& ) ) );
-    connect( mCategoryDialog, SIGNAL( editCategories() ), SLOT( editCategories() ) );
+  if ( mCategorySelectDialog == 0 ) {
+    mCategorySelectDialog = new KPIM::CategorySelectDialog( KABPrefs::instance(), this );
+    connect( mCategorySelectDialog, SIGNAL( categoriesSelected( const QStringList& ) ),
+             this, SLOT( categoriesSelected( const QStringList& ) ) );
+    connect( mCategorySelectDialog, SIGNAL( editCategories() ),
+             this, SLOT( editCategories() ) );
   }
 
-  QStringList customCategories = QStringList::split( ",", mCategoryEdit->text() );
-
-  mCategoryDialog->setCategories( customCategories );
-  mCategoryDialog->setSelected( customCategories );
-  mCategoryDialog->show();
-  mCategoryDialog->raise();
+  mCategorySelectDialog->setSelected( QStringList::split( ",", mCategoryEdit->text() ) );
+  mCategorySelectDialog->show();
+  mCategorySelectDialog->raise();
 }
 
 void AddresseeEditorWidget::categoriesSelected( const QStringList &list )
@@ -742,32 +789,24 @@ void AddresseeEditorWidget::editCategories()
   if ( mCategoryEditDialog == 0 ) {
     mCategoryEditDialog = new KPIM::CategoryEditDialog( KABPrefs::instance(), this );
     connect( mCategoryEditDialog, SIGNAL( categoryConfigChanged() ),
-             SLOT( categoryButtonClicked() ) );
+             mCategorySelectDialog, SLOT( updateCategoryConfig() ) );
   }
-  
+
   mCategoryEditDialog->show();
   mCategoryEditDialog->raise();
 }
 
 void AddresseeEditorWidget::emitModified()
 {
+  if ( mBlockSignals )
+    return;
+
   mDirty = true;
 
-  if ( !mBlockSignals ) {
-    KABC::Addressee::List list;
-
-    save();
-    list.append( mAddressee );
-
-    // I hate these hacks...
-    if ( !isExtension() )
-      mDirty = true;
-
-    emit modified( list );
-  }
+  emit modified();
 }
 
-void AddresseeEditorWidget::dateChanged( QDate )
+void AddresseeEditorWidget::dateChanged( const QDate& )
 {
   emitModified();
 }
@@ -783,21 +822,30 @@ void AddresseeEditorWidget::pageChanged( QWidget *wdg )
     KAcceleratorManager::manage( wdg );
 }
 
-QString AddresseeEditorWidget::title() const
-{
-  return i18n( "Contact Editor" );
-}
-
-QString AddresseeEditorWidget::identifier() const
-{
-  return "contact_editor";
-}
-
 void AddresseeEditorWidget::setInitialFocus()
 {
   mNameEdit->setFocus();
 }
 
+bool AddresseeEditorWidget::readyToClose()
+{
+  bool ok = true;
+
+  QDate date = mBirthdayPicker->date();
+  if ( !date.isValid() && !mBirthdayPicker->currentText().isEmpty() ) {
+    KMessageBox::error( this, i18n( "You have to enter a valid birthdate." ) );
+    ok = false;
+  }
+
+  date = mAnniversaryPicker->date();
+  if ( !date.isValid() && !mAnniversaryPicker->currentText().isEmpty() ) {
+    KMessageBox::error( this, i18n( "You have to enter a valid anniversary." ) );
+    ok = false;
+  }
+
+  return ok;
+}
+
 void AddresseeEditorWidget::setReadOnly( bool readOnly )
 {
   mReadOnly = readOnly;
@@ -809,7 +857,8 @@ void AddresseeEditorWidget::setReadOnly( bool readOnly )
   mAddressEditWidget->setReadOnly( readOnly );
   mEmailWidget->setReadOnly( readOnly );
   mURLEdit->setReadOnly( readOnly );
-  mIMAddressEdit->setReadOnly( readOnly );
+  mBlogEdit->setReadOnly( readOnly );
+  mIMWidget->setReadOnly( readOnly );
   mCategoryButton->setEnabled( !readOnly );
   mSecrecyWidget->setReadOnly( readOnly );
   mDepartmentEdit->setReadOnly( readOnly );
@@ -817,6 +866,7 @@ void AddresseeEditorWidget::setReadOnly( bool readOnly )
   mProfessionEdit->setReadOnly( readOnly );
   mManagerEdit->setReadOnly( readOnly );
   mAssistantEdit->setReadOnly( readOnly );
+  mTitleEdit->setReadOnly( readOnly );
   mNicknameEdit->setReadOnly( readOnly );
   mSpouseEdit->setReadOnly( readOnly );
   mBirthdayPicker->setEnabled( !readOnly );
