@@ -1,36 +1,44 @@
-/*                                                                      
-    This file is part of KAddressBook.                                  
-    Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>                   
-                                                                        
+/*
+    This file is part of KAddressBook.
+    Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or   
-    (at your option) any later version.                                 
-                                                                        
-    This program is distributed in the hope that it will be useful,     
-    but WITHOUT ANY WARRANTY; without even the implied warranty of      
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the        
-    GNU General Public License for more details.                        
-                                                                        
-    You should have received a copy of the GNU General Public License   
-    along with this program; if not, write to the Free Software         
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           
-                                                                        
-    As a special exception, permission is given to link this program    
-    with any edition of Qt, and distribute the resulting executable,    
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
+    As a special exception, permission is given to link this program
+    with any edition of Qt, and distribute the resulting executable,
     without including the source code for Qt in the source distribution.
-*/                                                                      
+*/
 
 #include <kabc/picture.h>
 #include <kdebug.h>
 #include <kdialog.h>
 #include <kglobalsettings.h>
 #include <kiconloader.h>
+#include <kimageio.h>
 #include <kio/netaccess.h>
 #include <klocale.h>
+#include <kmessagebox.h>
+#include <kurldrag.h>
 #include <kurlrequester.h>
-#include <kimageio.h>
+#include <libkdepim/kpixmapregionselectordialog.h>
 
+#include <librss/loader.h>
+#include <librss/document.h>
+#include <librss/image.h>
+
+#include <qapplication.h>
 #include <qcheckbox.h>
 #include <qdragobject.h>
 #include <qgroupbox.h>
@@ -62,14 +70,41 @@ void ImageLabel::startDrag()
 
 void ImageLabel::dragEnterEvent( QDragEnterEvent *event )
 {
-  event->accept( QImageDrag::canDecode( event ) );
+  bool accepted = false;
+
+  if ( QImageDrag::canDecode( event ) )
+    accepted = true;
+
+  if ( QUriDrag::canDecode( event ) )
+    accepted = true;
+
+  event->accept( accepted );
 }
 
 void ImageLabel::dropEvent( QDropEvent *event )
 {
-  QPixmap pm;
-  if ( QImageDrag::decode( event, pm ) && !mReadOnly ) {
-    setPixmap( pm );
+  if ( mReadOnly )
+    return;
+
+  if ( QImageDrag::canDecode( event ) ) {
+    QPixmap pm;
+
+    if ( QImageDrag::decode( event, pm ) ) {
+      setPixmap( pm );
+      emit changed();
+    }
+  }
+
+  if ( QUriDrag::canDecode( event ) ) {
+    KURL::List urls;
+    if ( KURLDrag::decode( event, urls ) ) {
+      if ( urls.isEmpty() ) { // oops, no data
+        event->accept( false );
+        return;
+      }
+    }
+
+    emit urlDropped( urls.first() );
     emit changed();
   }
 }
@@ -90,20 +125,21 @@ void ImageLabel::mouseMoveEvent( QMouseEvent *event )
 }
 
 
-ImageBaseWidget::ImageBaseWidget( const QString &title, QWidget *parent, const char *name )
-  : QWidget( parent, name ), mReadOnly( false )
+ImageBaseWidget::ImageBaseWidget( const QString &title, QWidget *parent,
+                                  const char *name )
+  : QWidget( parent, name ), mReadOnly( false ), mRssLoader( 0 )
 {
   QHBoxLayout *topLayout = new QHBoxLayout( this, KDialog::marginHint(),
                                             KDialog::spacingHint() );
   QGroupBox *box = new QGroupBox( 0, Qt::Vertical, title, this );
   QGridLayout *boxLayout = new QGridLayout( box->layout(), 3, 3,
                                             KDialog::spacingHint() );
-  boxLayout->setRowStretch( 2, 1 );
+  boxLayout->setRowStretch( 3, 1 );
 
   mImageLabel = new ImageLabel( i18n( "Picture" ), box );
   mImageLabel->setFixedSize( 100, 140 );
   mImageLabel->setFrameStyle( QFrame::Panel | QFrame::Sunken );
-  boxLayout->addMultiCellWidget( mImageLabel, 0, 2, 0, 0, AlignTop );
+  boxLayout->addMultiCellWidget( mImageLabel, 0, 3, 0, 0, AlignTop );
 
   mImageUrl = new KURLRequester( box );
   mImageUrl->setFilter( KImageIO::pattern() );
@@ -112,13 +148,18 @@ ImageBaseWidget::ImageBaseWidget( const QString &title, QWidget *parent, const c
 
   mClearButton = new QPushButton( box );
   mClearButton->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum ) );
-  mClearButton->setPixmap( SmallIcon( "clear_left" ) );
+  mClearButton->setIconSet( SmallIconSet( QApplication::reverseLayout() ? "locationbar_erase" : "clear_left" ) );
   mClearButton->setEnabled( false );
   boxLayout->addWidget( mClearButton, 0, 2 );
 
+  mBlogButton = new QPushButton( i18n("Get From Blog"), box );
+  boxLayout->addMultiCellWidget( mBlogButton, 1, 1, 1, 2 );
+  connect( mBlogButton, SIGNAL( clicked() ), SLOT( getPictureFromBlog() ) );
+  showBlogButton( false );
+
   mUseImageUrl = new QCheckBox( i18n( "Store as URL" ), box );
   mUseImageUrl->setEnabled( false );
-  boxLayout->addMultiCellWidget( mUseImageUrl, 1, 1, 1, 2 );
+  boxLayout->addMultiCellWidget( mUseImageUrl, 2, 2, 1, 2 );
 
   topLayout->addWidget( box );
 
@@ -126,6 +167,8 @@ ImageBaseWidget::ImageBaseWidget( const QString &title, QWidget *parent, const c
 
   connect( mImageLabel, SIGNAL( changed() ),
            SIGNAL( changed() ) );
+  connect( mImageLabel, SIGNAL( urlDropped( const KURL& ) ),
+           SLOT( urlDropped( const KURL& ) ) );
   connect( mImageUrl, SIGNAL( textChanged( const QString& ) ),
            SIGNAL( changed() ) );
   connect( mImageUrl, SIGNAL( urlSelected( const QString& ) ),
@@ -136,6 +179,7 @@ ImageBaseWidget::ImageBaseWidget( const QString &title, QWidget *parent, const c
            SLOT( updateGUI() ) );
   connect( mUseImageUrl, SIGNAL( toggled( bool ) ),
            SIGNAL( changed() ) );
+  // FIXME: Is there really no race condition with all these mImageUrl signals?
   connect( mClearButton, SIGNAL( clicked() ),
            SLOT( clear() ) );
 }
@@ -149,6 +193,21 @@ void ImageBaseWidget::setReadOnly( bool readOnly )
   mReadOnly = readOnly;
   mImageLabel->setReadOnly( mReadOnly );
   mImageUrl->setEnabled( !mReadOnly );
+  if ( !mBlogFeed.isEmpty() ) mBlogButton->setEnabled( !readOnly );
+}
+
+void ImageBaseWidget::showBlogButton( bool show )
+{
+  if ( show )
+    mBlogButton->show();
+  else
+    mBlogButton->hide();
+}
+
+void ImageBaseWidget::setBlogFeed( const QString &feed )
+{
+  mBlogFeed = feed;
+  mBlogButton->setEnabled( !feed.isEmpty() );
 }
 
 void ImageBaseWidget::setImage( const KABC::Picture &photo )
@@ -187,16 +246,28 @@ KABC::Picture ImageBaseWidget::image() const
   if ( mUseImageUrl->isChecked() )
     photo.setUrl( mImageUrl->url() );
   else {
-    if ( mImageLabel->pixmap() )
+    if ( mImageLabel->pixmap() ) {
       photo.setData( mImageLabel->pixmap()->convertToImage() );
+    }
   }
 
   return photo;
 }
 
+void ImageBaseWidget::urlDropped( const KURL &url )
+{
+  mImageUrl->setURL( url.url() );
+  loadImage();
+  mImageUrl->setURL( url.url() );
+
+  emit changed();
+}
+
 void ImageBaseWidget::loadImage()
 {
-  mImageLabel->setPixmap( loadPixmap( KURL( mImageUrl->url() ) ) );
+  QPixmap pixmap = loadPixmap( KURL( mImageUrl->url() ) );
+  if(! pixmap.isNull() )
+    mImageLabel->setPixmap( pixmap );
 }
 
 void ImageBaseWidget::updateGUI()
@@ -221,7 +292,7 @@ void ImageBaseWidget::clear()
 void ImageBaseWidget::imageChanged()
 {
   updateGUI();
-  
+
   emit changed();
 }
 
@@ -240,6 +311,18 @@ QPixmap ImageBaseWidget::loadPixmap( const KURL &url )
     KIO::NetAccess::removeTempFile( tempFile );
   }
 
+  if ( pixmap.isNull() ) {
+    // image does not exist (any more)
+    KMessageBox::sorry( this, i18n( "This contact's image cannot be found." ) );
+    return pixmap;
+  }
+
+  QPixmap selectedPixmap = KPIM::KPixmapRegionSelectorDialog::getSelectedImage( pixmap, 100, 140, this );
+  if ( selectedPixmap.isNull() )
+    return QPixmap();
+  pixmap = selectedPixmap;
+  mImageUrl->clear();
+
   if ( pixmap.height() != 140 || pixmap.width() != 100 ) {
     if ( pixmap.height() > pixmap.width() )
       pixmap = pixmap.convertToImage().scaleHeight( 140 );
@@ -250,14 +333,55 @@ QPixmap ImageBaseWidget::loadPixmap( const KURL &url )
   return pixmap;
 }
 
+void ImageBaseWidget::getPictureFromBlog()
+{
+  if ( mRssLoader ) {
+    return;
+  }
+
+  mRssLoader = RSS::Loader::create();
+  connect( mRssLoader, SIGNAL( loadingComplete( Loader *, Document,
+    Status ) ),
+    SLOT( slotLoadingComplete( Loader *, Document, Status ) ) );
+  mRssLoader->loadFrom( mBlogFeed, new RSS::FileRetriever );
+
+  // TODO: Show progress for fetching image from blog.
+}
+
+void ImageBaseWidget::slotLoadingComplete( RSS::Loader *loader,
+  RSS::Document doc, RSS::Status status )
+{
+  if ( status != RSS::Success ) {
+    KMessageBox::sorry( this,
+      i18n("Unable to retrieve blog feed from '%1': %2").arg( mBlogFeed )
+      .arg( loader->errorCode() ) );
+    return;
+  }
+
+  if ( !doc.image() ) {
+    KMessageBox::sorry( this,
+      i18n("Blog feed at '%1' does not contain an image.").arg( mBlogFeed ) );
+    return;
+  }
+
+  blockSignals( true );
+  mImageUrl->setURL( doc.image()->url().url() );
+  loadImage();
+  blockSignals( false );
+  imageChanged();
+
+  mRssLoader = 0;
+}
+
 ImageWidget::ImageWidget( KABC::AddressBook *ab, QWidget *parent, const char *name )
   : KAB::ContactEditorWidget( ab, parent, name )
 {
-  QHBoxLayout *layout = new QHBoxLayout( this, KDialog::marginHint(), 
+  QHBoxLayout *layout = new QHBoxLayout( this, KDialog::marginHint(),
                                          KDialog::spacingHint() );
 
   mPhotoWidget = new ImageBaseWidget( KABC::Addressee::photoLabel(), this );
   layout->addWidget( mPhotoWidget );
+  mPhotoWidget->showBlogButton( true );
 
   mLogoWidget = new ImageBaseWidget( KABC::Addressee::logoLabel(), this );
   layout->addWidget( mLogoWidget );
@@ -269,6 +393,7 @@ ImageWidget::ImageWidget( KABC::AddressBook *ab, QWidget *parent, const char *na
 void ImageWidget::loadContact( KABC::Addressee *addr )
 {
   mPhotoWidget->setImage( addr->photo() );
+  mPhotoWidget->setBlogFeed( addr->custom( "KADDRESSBOOK", "BlogFeed" ) );
   mLogoWidget->setImage( addr->logo() );
 }
 
