@@ -1,7 +1,7 @@
 /*
  *  kamail.cpp  -  email functions
  *  Program:  kalarm
- *  (C) 2002 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (c) 2002 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
@@ -40,18 +40,23 @@
 #include <kfileitem.h>
 #include <kio/netaccess.h>
 #include <ktempfile.h>
+#include <kemailsettings.h>
 #include <kdebug.h>
 
+#include <libkpimidentities/identitymanager.h>
+#include <libkpimidentities/identity.h>
 #include <libkcal/person.h>
 
 #include <kmime_header_parsing.h>
 
 #include "alarmevent.h"
-#include "preferences.h"
+#include "functions.h"
 #include "kalarmapp.h"
 #include "mainwindow.h"
+#include "preferences.h"
 #include "kamail.h"
 
+
 namespace HeaderParsing
 {
 bool parseAddress( const char* & scursor, const char * const send,
@@ -65,54 +70,102 @@ namespace
 QString getHostName();
 }
 
-const QString KAMail::EMAIL_QUEUED_NOTIFY = QString::fromLatin1("EmailQueuedNotify");
+struct KAMailData
+{
+	KAMailData(const KAEvent& e, const QString& fr, const QString& bc, bool allownotify)
+	                 : event(e), from(fr), bcc(bc), allowNotify(allownotify) { }
+	const KAEvent& event;
+	QString        from;
+	QString        bcc;
+	bool           allowNotify;
+};
+
 
 QString KAMail::i18n_NeedFromEmailAddress()
 { return i18n("A 'From' email address must be configured in order to execute email alarms."); }
 
+QString KAMail::i18n_sent_mail()
+{ return i18n("KMail folder name: this should be translated the same as in kmail", "sent-mail"); }
+
+KPIM::IdentityManager* KAMail::mIdentityManager = 0;
+KPIM::IdentityManager* KAMail::identityManager()
+{
+	if (!mIdentityManager)
+		mIdentityManager = new KPIM::IdentityManager(true);   // create a read-only kmail identity manager
+	return mIdentityManager;
+}
+
 
 /******************************************************************************
 * Send the email message specified in an event.
-* Reply = reason for failure (which may be the empty string)
-*       = null string if success.
+* Reply = true if the message was sent - 'errmsgs' may contain copy error messages.
+*       = false if the message was not sent - 'errmsgs' contains the error messages.
 */
-QString KAMail::send(const KAEvent& event, bool allowNotify)
+bool KAMail::send(const KAEvent& event, QStringList& errmsgs, bool allowNotify)
 {
-	Preferences* preferences = Preferences::instance();
-	QString from = preferences->emailAddress();
+	QString err;
+	QString from;
+	if (event.emailFromKMail().isEmpty())
+		from = Preferences::emailAddress();
+	else
+	{
+		from = mIdentityManager->identityForName(event.emailFromKMail()).fullEmailAddr();
+		if (from.isEmpty())
+		{
+			errmsgs = errors(i18n("Invalid 'From' email address.\nKMail identity '%1' not found.").arg(event.emailFromKMail()));
+			return false;
+		}
+	}
 	if (from.isEmpty())
 	{
-		return preferences->emailUseControlCentre()
-		       ? i18n("No 'From' email address is configured.\nPlease set it in the KDE Control Center or in the %1 Preferences dialog.").arg(kapp->aboutData()->programName())
-		       : i18n("No 'From' email address is configured.\nPlease set it in the %1 Preferences dialog.").arg(kapp->aboutData()->programName());
+		switch (Preferences::emailFrom())
+		{
+			case Preferences::MAIL_FROM_KMAIL:
+				errmsgs = errors(i18n("No 'From' email address is configured (no default KMail identity found)\nPlease set it in KMail or in the KAlarm Preferences dialog."));
+				break;
+			case Preferences::MAIL_FROM_CONTROL_CENTRE:
+				errmsgs = errors(i18n("No 'From' email address is configured.\nPlease set it in the KDE Control Center or in the KAlarm Preferences dialog."));
+				break;
+			case Preferences::MAIL_FROM_ADDR:
+			default:
+				errmsgs = errors(i18n("No 'From' email address is configured.\nPlease set it in the KAlarm Preferences dialog."));
+				break;
+		}
+		return false;
 	}
-	QString bcc = event.emailBcc() ? preferences->emailBccAddress() : QString::null;
+	KAMailData data(event, from,
+	                (event.emailBcc() ? Preferences::emailBccAddress() : QString::null),
+	                allowNotify);
 	kdDebug(5950) << "KAlarmApp::sendEmail(): To: " << event.emailAddresses(", ")
 	              << "\nSubject: " << event.emailSubject() << endl;
 
-	if (preferences->emailClient() == Preferences::SENDMAIL)
+	if (Preferences::emailClient() == Preferences::SENDMAIL)
 	{
+		// Use sendmail to send the message
 		QString textComplete;
 		QString command = KStandardDirs::findExe(QString::fromLatin1("sendmail"),
 		                                         QString::fromLatin1("/sbin:/usr/sbin:/usr/lib"));
 		if (!command.isNull())
 		{
 			command += QString::fromLatin1(" -oi -t ");
-			textComplete = initHeaders(event, from, bcc, false);
+			textComplete = initHeaders(data, false);
 		}
 		else
 		{
 			command = KStandardDirs::findExe(QString::fromLatin1("mail"));
 			if (command.isNull())
-				return i18n("%1 not found").arg(QString::fromLatin1("sendmail")); // give up
+			{
+				errmsgs = errors(i18n("%1 not found").arg(QString::fromLatin1("sendmail"))); // give up
+				return false;
+			}
 
 			command += QString::fromLatin1(" -s ");
 			command += KShellProcess::quote(event.emailSubject());
 
-			if (!bcc.isEmpty())
+			if (!data.bcc.isEmpty())
 			{
 				command += QString::fromLatin1(" -b ");
-				command += KShellProcess::quote(bcc);
+				command += KShellProcess::quote(data.bcc);
 			}
 
 			command += " ";
@@ -121,157 +174,195 @@ QString KAMail::send(const KAEvent& event, bool allowNotify)
 
 		// Add the body and attachments to the message.
 		// (Sendmail requires attachments to have already been included in the message.)
-		QString err = appendBodyAttachments(textComplete, event);
+		err = appendBodyAttachments(textComplete, event);
 		if (!err.isNull())
-			return err;
+		{
+			errmsgs = errors(err);
+			return false;
+		}
 
-//kdDebug(5950)<<"Email:"<<command<<"\n-----\n"<<textComplete<<"\n-----\n";
+		// Execute the send command
 		FILE* fd = popen(command.local8Bit(), "w");
 		if (!fd)
 		{
 			kdError(5950) << "KAMail::send(): Unable to open a pipe to " << command << endl;
-			return QString("");
+			errmsgs = errors();
+			return false;
 		}
 		fwrite(textComplete.local8Bit(), textComplete.length(), 1, fd);
 		pclose(fd);
+
+		if (Preferences::emailCopyToKMail())
+		{
+			// Create a copy of the sent email in KMail's 'Sent-mail' folder
+			err = addToKMailFolder(data, "sent-mail", true);
+			if (!err.isNull())
+				errmsgs = errors(err, false);    // not a fatal error - continue
+		}
+
 		if (allowNotify)
 			notifyQueued(event);
-		return QString::null;
 	}
 	else
 	{
-		return sendKMail(event, from, bcc, allowNotify);
+		// Use KMail to send the message
+		err = sendKMail(data);
+		if (!err.isNull())
+		{
+			errmsgs = errors(err);
+			return false;
+		}
 	}
+	return true;
 }
 
 /******************************************************************************
 * Send the email message via KMail.
+* Reply = reason for failure (which may be the empty string)
+*       = null string if success.
 */
-QString KAMail::sendKMail(const KAEvent& event, const QString& from, const QString& bcc, bool allowNotify)
+QString KAMail::sendKMail(const KAMailData& data)
 {
-	if (kapp->dcopClient()->isApplicationRegistered("kmail"))
+	QString err = KAlarm::runKMail(true);
+	if (!err.isNull())
+		return err;
+
+	// KMail is now running. Determine which DCOP call to use.
+	bool useSend = false;
+	QCString sendFunction = "sendMessage(QString,QString,QString,QString,QString,QString,KURL::List)";
+	QCStringList funcs = kapp->dcopClient()->remoteFunctions("kmail", "MailTransportServiceIface");
+	for (QCStringList::Iterator it=funcs.begin();  it != funcs.end() && !useSend;  ++it)
 	{
-		// KMail is running - use a DCOP call.
-		// First, determine which DCOP call to use.
-		bool useSend = false;
-		QCString funcname = "sendMessage()";
-		QCString function = "sendMessage(QString,QString,QString,QString,QString,QString,KURL::List)";
-		QCStringList funcs = kapp->dcopClient()->remoteFunctions("kmail", "MailTransportServiceIface");
-		for (QCStringList::Iterator it=funcs.begin();  it != funcs.end() && !useSend;  ++it)
+		QCString func = DCOPClient::normalizeFunctionSignature(*it);
+		if (func.left(5) == "bool ")
 		{
-			QCString func = DCOPClient::normalizeFunctionSignature(*it);
-			if (func.left(5) == "bool ")
-			{
-				func = func.mid(5);
-				func.replace(QRegExp(" [0-9A-Za-z_:]+"), "");
-				useSend = (func == function);
-			}
-		}
-		if (!useSend)
-		{
-			funcname = "dcopAddMessage()";
-			function = "dcopAddMessage(QString,QString)";
+			func = func.mid(5);
+			func.replace(QRegExp(" [0-9A-Za-z_:]+"), "");
+			useSend = (func == sendFunction);
 		}
+	}
 
-		QCString    replyType;
-		QByteArray  replyData;
-		QByteArray  data;
-		QDataStream arg(data, IO_WriteOnly);
-		int result = 0;
-		kdDebug(5950) << "KAMail::sendKMail(): using " << funcname << endl;
-		if (useSend)
-		{
-			// This version of KMail has the sendMessage() function,
-			// which transmits the message immediately.
-			arg << from;
-			arg << event.emailAddresses(", ");
-			arg << "";
-			arg << bcc;
-			arg << event.emailSubject();
-			arg << event.message();
-			arg << KURL::List(event.emailAttachments());
-			if (kapp->dcopClient()->call("kmail", "MailTransportServiceIface", function,
-			                             data, replyType, replyData)
-			&&  replyType == "bool")
-				result = 1;
-		}
-		else
-		{
-			// KMail is an older version, so use dcopAddMessage()
-			// to add the message to the outbox for later transmission.
-			QString message = initHeaders(event, from, bcc, true);
-			QString err = appendBodyAttachments(message, event);
-			if (!err.isNull())
-				return err;
-
-			// Write to a temporary file for feeding to KMail
-			KTempFile tmpFile;
-			tmpFile.setAutoDelete(true);     // delete file when it is destructed
-			QTextStream* stream = tmpFile.textStream();
-			if (!stream)
-			{
-				kdError(5950) << "KAMail::sendKMail(): Unable to open a temporary mail file" << endl;
-				return QString("");
-			}
-			*stream << message;
-			tmpFile.close();
-			if (tmpFile.status())
-			{
-				kdError(5950) << "KAMail::sendKMail(): Error " << tmpFile.status() << " writing to temporary mail file" << endl;
-				return QString("");
-			}
-
-			// Notify KMail of the message in the temporary file
-			arg << QString::fromLatin1("outbox") << tmpFile.name();
-			if (kapp->dcopClient()->call("kmail", "KMailIface", "dcopAddMessage(QString,QString)",
-			                             data, replyType, replyData)
-			&&  replyType == "int")
-				result = 1;
-		}
-		if (result)
-		{
-			QDataStream _reply_stream(replyData, IO_ReadOnly);
-			_reply_stream >> result;
-		}
-		if (result <= 0)
-		{
-			kdError(5950) << "KAMail::sendKMail(): kmail " << funcname << " call failed (error code = " << result << ")" << endl;
+	QByteArray  callData;
+	QDataStream arg(callData, IO_WriteOnly);
+	kdDebug(5950) << "KAMail::sendKMail(): using " << (useSend ? "sendMessage()" : "dcopAddMessage()") << endl;
+	if (useSend)
+	{
+		// This version of KMail has the sendMessage() function,
+		// which transmits the message immediately.
+		arg << data.from;
+		arg << data.event.emailAddresses(", ");
+		arg << "";    // CC:
+		arg << data.bcc;
+		arg << data.event.emailSubject();
+		arg << data.event.message();
+		arg << KURL::List(data.event.emailAttachments());
+		if (!callKMail(callData, "MailTransportServiceIface", sendFunction, "bool"))
 			return i18n("Error calling KMail");
-		}
-		if (allowNotify)
-			notifyQueued(event);
 	}
 	else
 	{
-		// KMail isn't running - start it
-		KProcess proc;
-		proc << "kmail"
-		     << "--subject" << event.emailSubject().local8Bit()
-		     << "--body" << event.message().local8Bit();
-		if (!bcc.isEmpty())
-			proc << "--bcc" << bcc.local8Bit();
-		QStringList attachments = event.emailAttachments();
-		if (attachments.count())
+		// KMail is an older version, so use dcopAddMessage()
+		// to add the message to the outbox for later transmission.
+		err = addToKMailFolder(data, "outbox", false);
+		if (!err.isNull())
+			return err;
+	}
+	if (data.allowNotify)
+		notifyQueued(data.event);
+	return QString::null;
+}
+
+/******************************************************************************
+* Add the message to a KMail folder.
+* Reply = reason for failure (which may be the empty string)
+*       = null string if success.
+*/
+QString KAMail::addToKMailFolder(const KAMailData& data, const char* folder, bool checkKmailRunning)
+{
+	QString err;
+	if (checkKmailRunning)
+		err = KAlarm::runKMail(true);
+	if (err.isNull())
+	{
+		QString message = initHeaders(data, true);
+		err = appendBodyAttachments(message, data.event);
+		if (!err.isNull())
+			return err;
+
+		// Write to a temporary file for feeding to KMail
+		KTempFile tmpFile;
+		tmpFile.setAutoDelete(true);     // delete file when it is destructed
+		QTextStream* stream = tmpFile.textStream();
+		if (!stream)
 		{
-			for (QStringList::Iterator at = attachments.begin();  at != attachments.end();  ++at)
-				proc << "--attach" << (*at).local8Bit();
+			kdError(5950) << "KAMail::addToKMailFolder(" << folder << "): Unable to open a temporary mail file" << endl;
+			return QString("");
 		}
-		EmailAddressList addresses = event.emailAddresses();
-		for (EmailAddressList::Iterator ad = addresses.begin();  ad != addresses.end();  ++ad)
-			proc << (*ad).fullName().local8Bit();
-		if (!proc.start(KProcess::DontCare))
+		*stream << message;
+		tmpFile.close();
+		if (tmpFile.status())
 		{
-			kdDebug(5950) << "sendKMail(): kmail start failed" << endl;
-			return i18n("Error starting KMail");
+			kdError(5950) << "KAMail::addToKMailFolder(" << folder << "): Error " << tmpFile.status() << " writing to temporary mail file" << endl;
+			return QString("");
+		}
+
+		// Notify KMail of the message in the temporary file
+		QByteArray  callData;
+		QDataStream arg(callData, IO_WriteOnly);
+		arg << QString::fromLatin1(folder) << tmpFile.name();
+		if (callKMail(callData, "KMailIface", "dcopAddMessage(QString,QString)", "int"))
+			return QString::null;
+		err = i18n("Error calling KMail");
+	}
+	kdError(5950) << "KAMail::addToKMailFolder(" << folder << "): " << err << endl;
+	return err;
+}
+
+/******************************************************************************
+* Call KMail via DCOP. The DCOP function must return an 'int'.
+*/
+bool KAMail::callKMail(const QByteArray& callData, const QCString& iface, const QCString& function, const QCString& funcType)
+{
+	QCString   replyType;
+	QByteArray replyData;
+	if (!kapp->dcopClient()->call("kmail", iface, function, callData, replyType, replyData)
+	||  replyType != funcType)
+	{
+		QCString funcname = function;
+		funcname.replace(QRegExp("(.+$"), "()");
+		kdError(5950) << "KAMail::callKMail(): kmail " << funcname << " call failed\n";;
+		return false;
+	}
+	QDataStream replyStream(replyData, IO_ReadOnly);
+	QCString funcname = function;
+	funcname.replace(QRegExp("(.+$"), "()");
+	if (replyType == "int")
+	{
+		int result;
+		replyStream >> result;
+		if (result <= 0)
+		{
+			kdError(5950) << "KAMail::callKMail(): kmail " << funcname << " call returned error code = " << result << endl;
+			return false;
 		}
 	}
-	return QString::null;
+	else if (replyType == "bool")
+	{
+		bool result;
+		replyStream >> result;
+		if (!result)
+		{
+			kdError(5950) << "KAMail::callKMail(): kmail " << funcname << " call returned error\n";
+			return false;
+		}
+	}
+	return true;
 }
 
 /******************************************************************************
 * Create the headers part of the email.
 */
-QString KAMail::initHeaders(const KAEvent& event, const QString& from, const QString& bcc, bool dateId)
+QString KAMail::initHeaders(const KAMailData& data, bool dateId)
 {
 	QString message;
 	if (dateId)
@@ -282,13 +373,13 @@ QString KAMail::initHeaders(const KAEvent& event, const QString& from, const QSt
 		char buff[64];
 		strftime(buff, sizeof(buff), "Date: %a, %d %b %Y %H:%M:%S %z", localtime(&timenow));
 		message = QString::fromLatin1(buff);
-		message += QString::fromLatin1("\nMessage-Id: <%1.%2.%3>\n").arg(timenow).arg(tod.tv_usec).arg(from);
+		message += QString::fromLatin1("\nMessage-Id: <%1.%2.%3>\n").arg(timenow).arg(tod.tv_usec).arg(data.from);
 	}
-	message += QString::fromLatin1("From: ") + from;
-	message += QString::fromLatin1("\nTo: ") + event.emailAddresses(", ");
-	if (!bcc.isEmpty())
-		message += QString::fromLatin1("\nBcc: ") + bcc;
-	message += QString::fromLatin1("\nSubject: ") + event.emailSubject();
+	message += QString::fromLatin1("From: ") + data.from;
+	message += QString::fromLatin1("\nTo: ") + data.event.emailAddresses(", ");
+	if (!data.bcc.isEmpty())
+		message += QString::fromLatin1("\nBcc: ") + data.bcc;
+	message += QString::fromLatin1("\nSubject: ") + data.event.emailSubject();
 	message += QString::fromLatin1("\nX-Mailer: %1" KALARM_VERSION).arg(kapp->aboutData()->programName());
 	return message;
 }
@@ -338,7 +429,7 @@ QString KAMail::appendBodyAttachments(QString& message, const KAEvent& event)
 			KURL url(attachment);
 			url.cleanPath();
 			KIO::UDSEntry uds;
-			if (!KIO::NetAccess::stat(url, uds)) {
+			if (!KIO::NetAccess::stat(url, uds, MainWindow::mainMainWindow())) {
 				kdError(5950) << "KAMail::appendBodyAttachments(): not found: " << attachment << endl;
 				return i18n("Attachment not found:\n%1").arg(attachment);
 			}
@@ -364,7 +455,7 @@ QString KAMail::appendBodyAttachments(QString& message, const KAEvent& event)
 
 			// Read the file contents
 			QString tmpFile;
-			if (!KIO::NetAccess::download(url, tmpFile)) {
+			if (!KIO::NetAccess::download(url, tmpFile, MainWindow::mainMainWindow())) {
 				kdError(5950) << "KAMail::appendBodyAttachments(): load failure: " << attachment << endl;
 				return attachError.arg(attachment);
 			}
@@ -373,16 +464,13 @@ QString KAMail::appendBodyAttachments(QString& message, const KAEvent& event)
 				kdDebug(5950) << "KAMail::appendBodyAttachments() tmp load error: " << attachment << endl;
 				return attachError.arg(attachment);
 			}
-			Offset size = file.size();
+			QIODevice::Offset size = file.size();
 			char* contents = new char [size + 1];
-#if QT_VERSION < 300
-			typedef int Q_LONG;
-#endif
 			Q_LONG bytes = file.readBlock(contents, size);
 			file.close();
 			contents[size] = 0;
 			bool atterror = false;
-			if (bytes == -1  ||  (Offset)bytes < size) {
+			if (bytes == -1  ||  (QIODevice::Offset)bytes < size) {
 				kdDebug(5950) << "KAMail::appendBodyAttachments() read error: " << attachment << endl;
 				atterror = true;
 			}
@@ -394,9 +482,9 @@ QString KAMail::appendBodyAttachments(QString& message, const KAEvent& event)
 			else
 			{
 				// Convert the attachment to BASE64 encoding
-				Offset base64Size;
+				QIODevice::Offset base64Size;
 				char* base64 = base64Encode(contents, size, base64Size);
-				if (base64Size == (Offset)-1) {
+				if (base64Size == (QIODevice::Offset)-1) {
 					kdDebug(5950) << "KAMail::appendBodyAttachments() base64 buffer overflow: " << attachment << endl;
 					atterror = true;
 				}
@@ -433,16 +521,34 @@ void KAMail::notifyQueued(const KAEvent& event)
 			QString domain = addr.mailboxList.first().addrSpec.domain;
 			if (!domain.isEmpty()  &&  domain != localhost  &&  domain != hostname)
 			{
-				QString text = (Preferences::instance()->emailClient() == Preferences::KMAIL)
+				QString text = (Preferences::emailClient() == Preferences::KMAIL)
 				             ? i18n("An email has been queued to be sent by KMail")
 				             : i18n("An email has been queued to be sent");
-				KMessageBox::information(0, text, QString::null, EMAIL_QUEUED_NOTIFY);
+				KMessageBox::information(0, text, QString::null, Preferences::EMAIL_QUEUED_NOTIFY);
 				return;
 			}
 		}
 	}
 }
 
+/******************************************************************************
+*  Return whether any KMail identities exist.
+*/
+bool KAMail::identitiesExist()
+{
+	identityManager();    // create identity manager if not already done
+	return mIdentityManager->begin() != mIdentityManager->end();
+}
+
+/******************************************************************************
+*  Fetch the user's email address configured in the KDE Control Centre.
+*/
+QString KAMail::controlCentreAddress()
+{
+	KEMailSettings e;
+	return e.getSetting(KEMailSettings::EmailAddress);
+}
+
 /******************************************************************************
 *  Parse a list of email addresses, optionally containing display names,
 *  entered by the user.
@@ -457,44 +563,85 @@ QString KAMail::convertAddresses(const QString& items, EmailAddressList& list)
 	// parse an address-list
 	QValueList<KMime::Types::Address> maybeAddressList;
 	if (!HeaderParsing::parseAddressList(ad, ad + addrs.length(), maybeAddressList))
-		return QString::fromLocal8Bit(ad);
+		return QString::fromLocal8Bit(ad);    // return the address in error
 
 	// extract the mailboxes and complain if there are groups
 	for (QValueList<KMime::Types::Address>::ConstIterator it = maybeAddressList.begin();
 	     it != maybeAddressList.end();  ++it)
 	{
-		if (!(*it).displayName.isEmpty())
-		{
-			kdDebug(5950) << "mailbox groups not allowed! Name: \"" << (*it).displayName << "\"" << endl;
-			return (*it).displayName;
-		}
-		const QValueList<KMime::Types::Mailbox>& mblist = (*it).mailboxList;
-		for (QValueList<KMime::Types::Mailbox>::ConstIterator mb = mblist.begin();
-		     mb != mblist.end();  ++mb)
-		{
-			QString addrPart = (*mb).addrSpec.localPart;
-			if (!(*mb).addrSpec.domain.isEmpty())
-			{
-				addrPart += QChar('@');
-				addrPart += (*mb).addrSpec.domain;
-			}
-			list += KCal::Person((*mb).displayName, addrPart);
-		}
+		QString bad = convertAddress(*it, list);
+		if (!bad.isEmpty())
+			return bad;
 	}
 	return QString::null;
 }
 
+#if 0
+/******************************************************************************
+*  Parse an email address, optionally containing display name, entered by the
+*  user, and append it to the specified list.
+*  Reply = the invalid item if error, else empty string.
+*/
+QString KAMail::convertAddress(const QString& item, EmailAddressList& list)
+{
+	QCString addr = item.local8Bit();
+	const char* ad = static_cast<const char*>(addr);
+	KMime::Types::Address maybeAddress;
+	if (!HeaderParsing::parseAddress(ad, ad + addr.length(), maybeAddress))
+		return item;     // error
+	return convertAddress(maybeAddress, list);
+}
+#endif
+
+/******************************************************************************
+*  Convert a single KMime::Types address to a KCal::Person instance and append
+*  it to the specified list.
+*/
+QString KAMail::convertAddress(KMime::Types::Address addr, EmailAddressList& list)
+{
+	if (!addr.displayName.isEmpty())
+	{
+		kdDebug(5950) << "mailbox groups not allowed! Name: \"" << addr.displayName << "\"" << endl;
+		return addr.displayName;
+	}
+	const QValueList<KMime::Types::Mailbox>& mblist = addr.mailboxList;
+	for (QValueList<KMime::Types::Mailbox>::ConstIterator mb = mblist.begin();
+	     mb != mblist.end();  ++mb)
+	{
+		QString addrPart = (*mb).addrSpec.localPart;
+		if (!(*mb).addrSpec.domain.isEmpty())
+		{
+			addrPart += QChar('@');
+			addrPart += (*mb).addrSpec.domain;
+		}
+		list += KCal::Person((*mb).displayName, addrPart);
+	}
+	return QString::null;
+}
+
+/*
 QString KAMail::convertAddresses(const QString& items, QStringList& list)
 {
 	EmailAddressList addrs;
 	QString item = convertAddresses(items, addrs);
-	if (item.isEmpty())
+	if (!item.isEmpty())
+		return item;
+	for (EmailAddressList::Iterator ad = addrs.begin();  ad != addrs.end();  ++ad)
 	{
-		for (EmailAddressList::Iterator ad = addrs.begin();  ad != addrs.end();  ++ad)
-			list += (*ad).fullName().local8Bit();
+		item = (*ad).fullName().local8Bit();
+		switch (checkAddress(item))
+		{
+			case 1:      // OK
+				list += item;
+				break;
+			case 0:      // null address
+				break;
+			case -1:     // invalid address
+				return item;
+		}
 	}
-	return item;
-}
+	return QString::null;
+}*/
 
 /******************************************************************************
 *  Check the validity of an email address.
@@ -580,13 +727,53 @@ QString KAMail::convertAttachments(const QString& items, QStringList& list)
 	return QString::null;
 }
 
+#if 0
+/******************************************************************************
+*  Convert a comma or semicolon delimited list of attachments into a
+*  KURL::List. The items are checked for validity.
+*  Reply = the invalid item if error, else empty string.
+*/
+QString KAMail::convertAttachments(const QString& items, KURL::List& list)
+{
+	KURL url;
+	list.clear();
+	QCString addrs = items.local8Bit();
+	int length = items.length();
+	for (int next = 0;  next < length;  )
+	{
+		// Find the first delimiter character (, or ;)
+		int i = items.find(',', next);
+		if (i < 0)
+			i = items.length();
+		int sc = items.find(';', next);
+		if (sc < 0)
+			sc = items.length();
+		if (sc < i)
+			i = sc;
+		QString item = items.mid(next, i - next);
+		switch (checkAttachment(item, &url))
+		{
+			case 1:   list += url;  break;
+			case 0:   break;          // empty attachment name
+			case -1:
+			default:  return item;    // error
+		}
+		next = i + 1;
+	}
+	return QString::null;
+}
+#endif
+
 /******************************************************************************
 *  Check for the existence of the attachment file.
 *  If non-null, '*url' receives the KURL of the attachment.
+*  Reply = 1 if attachment exists
+*        = 0 if null name
+*        = -1 if doesn't exist.
 */
 int KAMail::checkAttachment(QString& attachment, KURL* url)
 {
-	attachment.stripWhiteSpace();
+	attachment = attachment.stripWhiteSpace();
 	if (attachment.isEmpty())
 	{
 		if (url)
@@ -598,13 +785,21 @@ int KAMail::checkAttachment(QString& attachment, KURL* url)
 	u.cleanPath();
 	if (url)
 		*url = u;
+	return checkAttachment(u) ? 1 : -1;
+}
+
+/******************************************************************************
+*  Check for the existence of the attachment file.
+*/
+bool KAMail::checkAttachment(const KURL& url)
+{
 	KIO::UDSEntry uds;
-	if (!KIO::NetAccess::stat(u, uds))
-		return -1;       // doesn't exist
-	KFileItem fi(uds, u);
+	if (!KIO::NetAccess::stat(url, uds, MainWindow::mainMainWindow()))
+		return false;       // doesn't exist
+	KFileItem fi(uds, url);
 	if (fi.isDir()  ||  !fi.isReadable())
-		return -1;
-	return 1;
+		return false;
+	return true;
 }
 
 
@@ -614,7 +809,7 @@ int KAMail::checkAttachment(QString& attachment, KURL* url)
 *            -1 if overflow.
 *  Reply = BASE64 buffer, which the caller must delete[] afterwards.
 */
-char* KAMail::base64Encode(const char* in, KAMail::Offset size, KAMail::Offset& outSize)
+char* KAMail::base64Encode(const char* in, QIODevice::Offset size, QIODevice::Offset& outSize)
 {
 	const int MAX_LINELEN = 72;
 	static unsigned char dtable[65] =
@@ -623,10 +818,10 @@ char* KAMail::base64Encode(const char* in, KAMail::Offset size, KAMail::Offset&
 		"0123456789+/";
 
 	char* out = new char [2*size + 5];
-	outSize = (Offset)-1;
-	Offset outIndex = 0;
+	outSize = (QIODevice::Offset)-1;
+	QIODevice::Offset outIndex = 0;
 	int lineLength = 0;
-	for (Offset inIndex = 0;  inIndex < size;  )
+	for (QIODevice::Offset inIndex = 0;  inIndex < size;  )
 	{
 		unsigned char igroup[3];
 		int n;
@@ -683,6 +878,44 @@ char* KAMail::base64Encode(const char* in, KAMail::Offset size, KAMail::Offset&
 	return out;
 }
 
+/******************************************************************************
+* Set the appropriate error messages for a given error string.
+*/
+QStringList KAMail::errors(const QString& err, bool sendfail)
+{
+	QString error1 = sendfail ? i18n("Failed to send email")
+	                          : i18n("Error copying sent email to KMail %1 folder").arg(i18n_sent_mail());
+	if (err.isEmpty())
+		return QStringList(error1);
+	QStringList errs(QString::fromLatin1("%1:").arg(error1));
+	errs += err;
+	return errs;
+}
+
+/******************************************************************************
+*  Get the body of an email, given its serial number.
+*/
+QString KAMail::getMailBody(Q_UINT32 serialNumber)
+{
+	// Get the body of the email from KMail
+	QCString    replyType;
+	QByteArray  replyData;
+	QByteArray  data;
+	QDataStream arg(data, IO_WriteOnly);
+	arg << serialNumber;
+	arg << (int)0;
+	QString body;
+	if (kapp->dcopClient()->call("kmail", "KMailIface", "getDecodedBodyPart(Q_UINT32,int)", data, replyType, replyData)
+	&&  replyType == "QString")
+	{
+		QDataStream reply_stream(replyData, IO_ReadOnly);
+		reply_stream >> body;
+	}
+	else
+		kdDebug(5950) << "KAMail::getMailBody(): kmail getDecodedBodyPart() call failed\n";
+	return body;
+}
+
 namespace
 {
 /******************************************************************************
