@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     ------
     ELSA and MicroLink are trademarks of ELSA AG, Aachen.
@@ -201,7 +201,9 @@ bool Modem::open()
 
   tcflush( fd, TCIOFLUSH );
   if ( tcgetattr( fd, &init_tty ) == -1 ) {
-    emit errorMessage( i18n( "tcgetattr() failed." ) );
+    int errnumber = errno;
+    emit errorMessage( i18n( "Communication setup failed (tcgetattr code: %1)" )
+        .arg(strerror(errnumber)) );
     ::close( fd );
     fd = 0;
     return false;
@@ -264,9 +266,22 @@ void Modem::flush()
   }
 }
 
+#ifdef HAVE_LOCKDEV
+#include <lockdev.h>
+#endif
 
 bool Modem::lockDevice()
 {
+  if ( is_locked )
+    return true;
+
+#ifdef HAVE_LOCKDEV
+  is_locked = !dev_lock( (*prefs).serialDevice().local8Bit() );
+  if (!is_locked)
+      emit errorMessage( i18n( "Unable to lock device '%1'." ).arg(
+                             (*prefs).serialDevice() ));
+  return is_locked;
+#else
   ssize_t count;
   pid_t pid;
   int lfd;
@@ -274,10 +289,6 @@ bool Modem::lockDevice()
   QStringList pathList;
   QString fileName, content;
 
-
-  if ( is_locked )
-    return true;
-
   pathList = QStringList::split( "/", (*prefs).serialDevice() );
   fileName = (*prefs).lockDirectory() + "/LCK.." + pathList.last();
 
@@ -337,17 +348,22 @@ bool Modem::lockDevice()
   is_locked = true;
 
   return true;
+#endif
 }
 
 
 void Modem::unlockDevice()
 {
+#ifdef HAVE_LOCKDEV
+  dev_unlock( (*prefs).serialDevice().local8Bit(), getpid() );
+#else
   if ( is_locked ) {
     QStringList pathList = QStringList::split( "/", (*prefs).serialDevice() );
 
     QFile::remove( (*prefs).lockDirectory() + "/LCK.." + pathList.last() );
     is_locked = false;
   }
+#endif
 }
 
 
