@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 #ifndef KSYNC_KONNECTORVIEW_H
 #define KSYNC_KONNECTORVIEW_H
@@ -28,17 +28,22 @@
 namespace KSync
 {
 
-class KonnectorView : public QWidget
+class KDE_EXPORT KonnectorView : public QWidget
 {
   Q_OBJECT
 
   public:
     KonnectorView( QWidget *parent, const char *name = 0 );
-    
+
+    //@{
+    void setSelectedKonnectors(const QStringList&);
+    QStringList selectedKonnectorsList()const;
+    //@}
+
     Konnector::List selectedKonnectors();
 
     void updateKonnectorList();
-    
+
   private:
     KListView *mKonnectorList;
 };
