@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -37,12 +37,15 @@
 
 // other KMail headers:
 #include "signatureconfigurator.h"
-#include "kmfoldercombobox.h"
+#include "xfaceconfigurator.h"
+#include "folderrequester.h"
+using KMail::FolderRequester;
 #include "kmfoldermgr.h"
 #include "transportmanager.h"
-#include "kmkernel.h"
 #include "dictionarycombobox.h"
 #include "kleo_util.h"
+#include "kmmainwidget.h"
+#include "kmfolder.h"
 
 // other kdepim headers:
 // libkdepim
@@ -52,10 +55,15 @@
 #include <ui/keyrequester.h>
 #include <kleo/cryptobackendfactory.h>
 
+#include <libemailfunctions/email.h>
+
 // other KDE headers:
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kconfig.h>
+#include <kfileitem.h>
+#include <kurl.h>
+#include <kdebug.h>
 
 // Qt headers:
 #include <qtabwidget.h>
@@ -71,6 +79,8 @@
 #include <iterator>
 #include <algorithm>
 
+using namespace KPIM;
+
 namespace KMail {
 
   IdentityDialog::IdentityDialog( QWidget * parent, const char * name )
@@ -326,16 +336,18 @@ namespace KMail {
 
     // "Sent-mail Folder" combo box and label:
     ++row;
-    mFccCombo = new KMFolderComboBox( tab );
-    mFccCombo->showOutboxFolder( false );
+    mFccCombo = new FolderRequester( tab, 
+        kmkernel->getKMMainWidget()->folderTree() );
+    mFccCombo->setShowOutbox( false );
     glay->addWidget( mFccCombo, row, 1 );
     glay->addWidget( new QLabel( mFccCombo, i18n("Sent-mail &folder:"), tab ),
                      row, 0 );
 
     // "Drafts Folder" combo box and label:
     ++row;
-    mDraftsCombo = new KMFolderComboBox( tab );
-    mDraftsCombo->showOutboxFolder( false );
+    mDraftsCombo = new FolderRequester( tab,
+        kmkernel->getKMMainWidget()->folderTree() );
+    mDraftsCombo->setShowOutbox( false );
     glay->addWidget( mDraftsCombo, row, 1 );
     glay->addWidget( new QLabel( mDraftsCombo, i18n("&Drafts folder:"), tab ),
                      row, 0 );
@@ -360,6 +372,10 @@ namespace KMail {
     mSignatureConfigurator->layout()->setMargin( KDialog::marginHint() );
     tabWidget->addTab( mSignatureConfigurator, i18n("&Signature") );
 
+    mXFaceConfigurator = new XFaceConfigurator( tabWidget );
+    mXFaceConfigurator->layout()->setMargin( KDialog::marginHint() );
+    tabWidget->addTab( mXFaceConfigurator, i18n("&Picture") );
+
     KConfigGroup geometry( KMKernel::config(), "Geometry" );
     if ( geometry.hasKey( "Identity Dialog size" ) )
       resize( geometry.readSizeEntry( "Identity Dialog size" ) );
@@ -421,47 +437,96 @@ namespace KMail {
     }
   }
 
-  void IdentityDialog::slotOk() {
+bool IdentityDialog::validateAddresses( const QString & addresses )
+{
+  QString brokenAddress;
+  KPIM::EmailParseResult errorCode = KMMessage::isValidEmailAddressList( KMMessage::expandAliases( addresses ), brokenAddress );
+  if ( !( errorCode == KPIM::AddressOk || errorCode == KPIM::AddressEmpty ) ) {
+    QString errorMsg( "<qt><p><b>" + brokenAddress +
+                      "</b></p><p>" + KPIM::emailParseResultToString( errorCode ) +
+                      "</p></qt>" );
+    KMessageBox::sorry( this, errorMsg, i18n("Invalid Email Address") );
+    return false;
+  } 
+  return true;
+}
+
+void IdentityDialog::slotOk() {
     const QString email = mEmailEdit->text().stripWhiteSpace();
-    if ( email.isEmpty() )
-      return KDialogBase::slotOk();
+ 
+    // Validate email addresses
+    if ( !isValidSimpleEmailAddress( email )) {
+      QString errorMsg( simpleEmailAddressErrorMsg());
+      KMessageBox::sorry( this, errorMsg, i18n("Invalid Email Address") );
+      return;
+    }
+
+    if ( !validateAddresses( mReplyToEdit->text().stripWhiteSpace() ) ) {
+      return;
+    }
+ 
+    if ( !validateAddresses( mBccEdit->text().stripWhiteSpace() ) ) {
+      return;
+    }
+
     const std::vector<GpgME::Key> & pgpSigningKeys = mPGPSigningKeyRequester->keys();
     const std::vector<GpgME::Key> & pgpEncryptionKeys = mPGPEncryptionKeyRequester->keys();
     const std::vector<GpgME::Key> & smimeSigningKeys = mSMIMESigningKeyRequester->keys();
     const std::vector<GpgME::Key> & smimeEncryptionKeys = mSMIMEEncryptionKeyRequester->keys();
     QString msg;
+    bool err = false;
     if ( std::find_if( pgpSigningKeys.begin(), pgpSigningKeys.end(),
-		       DoesntMatchEMailAddress( email ) ) != pgpSigningKeys.end() )
+		       DoesntMatchEMailAddress( email ) ) != pgpSigningKeys.end() ) {
       msg = i18n("One of the configured OpenPGP signing keys does not contain "
 		 "any user ID with the configured email address for this "
 		 "identity (%1).\n"
 		 "This might result in warning messages on the receiving side "
 		 "when trying to verify signatures made with this configuration.");
+      err = true;
+    }
     else if ( std::find_if( pgpEncryptionKeys.begin(), pgpEncryptionKeys.end(),
-			    DoesntMatchEMailAddress( email ) ) != pgpEncryptionKeys.end() )
+			    DoesntMatchEMailAddress( email ) ) != pgpEncryptionKeys.end() ) {
       msg = i18n("One of the configured OpenPGP encryption keys does not contain "
 		 "any user ID with the configured email address for this "
 		 "identity (%1).");
+      err = true;
+    }
     else if ( std::find_if( smimeSigningKeys.begin(), smimeSigningKeys.end(),
-			    DoesntMatchEMailAddress( email ) ) != smimeSigningKeys.end() )
+			    DoesntMatchEMailAddress( email ) ) != smimeSigningKeys.end() ) {
       msg = i18n("One of the configured S/MIME signing certificates does not contain "
 		 "the configured email address for this "
 		 "identity (%1).\n"
 		 "This might result in warning messages on the receiving side "
 		 "when trying to verify signatures made with this configuration.");
+      err = true;
+    }
     else if ( std::find_if( smimeEncryptionKeys.begin(), smimeEncryptionKeys.end(),
-			    DoesntMatchEMailAddress( email ) ) != smimeEncryptionKeys.end() )
+			    DoesntMatchEMailAddress( email ) ) != smimeEncryptionKeys.end() ) {
       msg = i18n("One of the configured S/MIME encryption certificates does not contain "
 		 "the configured email address for this "
 		 "identity (%1).");
-    else
-      return KDialogBase::slotOk();
+      err = true;
+    }
 
-    if ( KMessageBox::warningContinueCancel( this, msg.arg( email ),
-					     i18n("Email Address Not Found in Key/Certificates"),
-					     KStdGuiItem::cont(), "warn_email_not_in_certificate" )
-	 == KMessageBox::Continue )
-      return KDialogBase::slotOk();
+    if ( err )
+      if ( KMessageBox::warningContinueCancel( this, msg.arg( email ),
+                                          i18n("Email Address Not Found in Key/Certificates"),
+                                          KStdGuiItem::cont(), "warn_email_not_in_certificate" )
+	 != KMessageBox::Continue)
+        return;
+
+
+    if ( mSignatureConfigurator->isSignatureEnabled() &&
+         mSignatureConfigurator->signatureType()==Signature::FromFile ) {
+      KURL url( mSignatureConfigurator->fileURL() );
+      KFileItem signatureFile( KFileItem::Unknown, KFileItem::Unknown, url );
+      if ( !signatureFile.isFile() || !signatureFile.isReadable() || !signatureFile.isLocalFile() ) {
+        KMessageBox::error( this, i18n( "The signature file is not valid" ) );
+        return;
+      }
+    }
+
+    return KDialogBase::slotOk();
   }
 
   bool IdentityDialog::checkFolderExists( const QString & folderID,
@@ -522,6 +587,8 @@ namespace KMail {
 
     // "Signature" tab:
     mSignatureConfigurator->setSignature( ident.signature() );
+    mXFaceConfigurator->setXFace( ident.xface() );
+    mXFaceConfigurator->setXFaceEnabled( ident.isXFaceEnabled() );
   }
 
   void IdentityDialog::updateIdentity( KPIM::Identity & ident ) {
@@ -529,22 +596,6 @@ namespace KMail {
     ident.setFullName( mNameEdit->text() );
     ident.setOrganization( mOrganizationEdit->text() );
     QString email = mEmailEdit->text();
-    int atCount = email.contains('@');
-    if ( email.isEmpty() || atCount == 0 )
-      KMessageBox::sorry( this, "<qt>"+
-                          i18n("Your email address is not valid because it "
-                               "does not contain a <emph>@</emph>: "
-                               "you will not create valid messages if you do not "
-                               "change your address.") + "</qt>",
-                          i18n("Invalid Email Address") );
-    else if ( atCount > 1 ) {
-      KMessageBox::sorry( this, "<qt>" +
-                          i18n("Your email address is not valid because it "
-                               "contains more than one <emph>@</emph>: "
-                               "you will not create valid messages if you do not "
-                               "change your address.") + "</qt>",
-                          i18n("Invalid Email Address") );
-    }
     ident.setEmailAddr( email );
     // "Cryptography" tab:
     ident.setPGPSigningKey( mPGPSigningKeyRequester->fingerprint().latin1() );
@@ -558,12 +609,14 @@ namespace KMail {
     ident.setTransport( ( mTransportCheck->isChecked() ) ?
                         mTransportCombo->currentText() : QString::null );
     ident.setDictionary( mDictionaryCombo->currentDictionary() );
-    ident.setFcc( mFccCombo->getFolder() ?
-                  mFccCombo->getFolder()->idString() : QString::null );
-    ident.setDrafts( mDraftsCombo->getFolder() ?
-                     mDraftsCombo->getFolder()->idString() : QString::null );
+    ident.setFcc( mFccCombo->folder() ?
+                  mFccCombo->folder()->idString() : QString::null );
+    ident.setDrafts( mDraftsCombo->folder() ?
+                     mDraftsCombo->folder()->idString() : QString::null );
     // "Signature" tab:
     ident.setSignature( mSignatureConfigurator->signature() );
+    ident.setXFace( mXFaceConfigurator->xface() );
+    ident.setXFaceEnabled( mXFaceConfigurator->isXFaceEnabled() );
   }
 
   void IdentityDialog::slotUpdateTransportCombo( const QStringList & sl ) {
