@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  In addition, as a special exception, the copyright holders give
  *  permission to link the code of this program with any edition of
@@ -39,7 +39,9 @@
 #include "kmmsgpart.h"
 #include "progressmanager.h"
 using KPIM::ProgressManager;
+#include "util.h"
 
+#include <qstylesheet.h>
 #include <kio/scheduler.h>
 #include <kdebug.h>
 #include <klocale.h>
@@ -110,54 +112,64 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
   if ( jt == tPutMessage )
   {
     // transfers the complete message to the server
-    KURL url = account->getUrl();
-    QString flags = KMFolderImap::statusToFlags( msg->status() );
-    url.setPath( folder->imapPath() + ";SECTION=" + flags );
-    ImapAccountBase::jobData jd;
-    jd.parent = 0; jd.offset = 0; jd.done = 0;
-    jd.total = msg->msgSizeServer();
-    jd.msgList.append(msg);
-    QCString cstr( msg->asString() );
-    int a = cstr.find("\nX-UID: ");
-    int b = cstr.find('\n', a);
-    if (a != -1 && b != -1 && cstr.find("\n\n") > a) cstr.remove(a, b-a);
-    mData.resize( cstr.length() + cstr.contains( "\n" ) - cstr.contains( "\r\n" ) );
-    unsigned int i = 0;
-    char prevChar = '\0';
-    // according to RFC 2060 we need CRLF
-    for ( char *ch = cstr.data(); *ch; ch++ )
+    QPtrListIterator<KMMessage> it( msgList );
+    KMMessage* curMsg;
+    while ( ( curMsg = it.current() ) != 0 )
     {
-      if ( *ch == '\n' && (prevChar != '\r') ) {
-        mData.at( i ) = '\r';
+      ++it;
+      if ( mSrcFolder && !curMsg->isMessage() )
+      {
+        int idx = mSrcFolder->find( curMsg );
+        curMsg = mSrcFolder->getMsg( idx );
+      }
+      KURL url = account->getUrl();
+      QString flags = KMFolderImap::statusToFlags( curMsg->status() );
+      url.setPath( folder->imapPath() + ";SECTION=" + flags );
+      ImapAccountBase::jobData jd;
+      jd.parent = 0; jd.offset = 0; jd.done = 0;
+      jd.total = ( curMsg->msgSizeServer() > 0 ) ? 
+        curMsg->msgSizeServer() : curMsg->msgSize();
+      jd.msgList.append( curMsg );
+      QCString cstr( curMsg->asString() );
+      int a = cstr.find("\nX-UID: ");
+      int b = cstr.find('\n', a);
+      if (a != -1 && b != -1 && cstr.find("\n\n") > a) cstr.remove(a, b-a);
+      jd.data.resize( cstr.length() + cstr.contains( "\n" ) - cstr.contains( "\r\n" ) );
+      unsigned int i = 0;
+      char prevChar = '\0';
+      // according to RFC 2060 we need CRLF
+      for ( char *ch = cstr.data(); *ch; ch++ )
+      {
+        if ( *ch == '\n' && (prevChar != '\r') ) {
+          jd.data.at( i ) = '\r';
+          i++;
+        }
+        jd.data.at( i ) = *ch;
+        prevChar = *ch;
         i++;
       }
-      mData.at( i ) = *ch;
-      prevChar = *ch;
-      i++;
+      jd.progressItem = ProgressManager::createProgressItem(
+          mParentProgressItem,
+          "ImapJobUploading"+ProgressManager::getUniqueID(),
+          i18n("Uploading message data"),
+          curMsg->subject(),
+          true,
+          account->useSSL() || account->useTLS() );
+      jd.progressItem->setTotalItems( jd.total );
+      connect ( jd.progressItem, SIGNAL( progressItemCanceled( KPIM::ProgressItem*)),
+          account, SLOT( slotAbortRequested( KPIM::ProgressItem* ) ) );
+      KIO::SimpleJob *job = KIO::put( url, 0, FALSE, FALSE, FALSE );
+      KIO::Scheduler::assignJobToSlave( account->slave(), job );
+      account->insertJob( job, jd );
+      connect( job, SIGNAL(result(KIO::Job *)),
+          SLOT(slotPutMessageResult(KIO::Job *)) );
+      connect( job, SIGNAL(dataReq(KIO::Job *, QByteArray &)),
+          SLOT(slotPutMessageDataReq(KIO::Job *, QByteArray &)) );
+      connect( job, SIGNAL(infoMessage(KIO::Job *, const QString &)),
+          SLOT(slotPutMessageInfoData(KIO::Job *, const QString &)) );
+      connect( job, SIGNAL(processedSize(KIO::Job *, KIO::filesize_t)),
+          SLOT(slotProcessedSize(KIO::Job *, KIO::filesize_t)));
     }
-    jd.data = mData;
-    jd.progressItem = ProgressManager::createProgressItem(
-                          mParentProgressItem,
-                          "ImapJobUploading"+ProgressManager::getUniqueID(),
-                          i18n("Uploading message data"),
-                          i18n("Destination folder: ") + mDestFolder->prettyURL(),
-                          true,
-                          account->useSSL() || account->useTLS() );
-    jd.progressItem->setTotalItems( jd.total );
-    connect ( jd.progressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
-              account, SLOT( slotAbortRequested( ProgressItem* ) ) );
-    KIO::SimpleJob *simpleJob = KIO::put( url, 0, FALSE, FALSE, FALSE );
-    KIO::Scheduler::assignJobToSlave( account->slave(), simpleJob );
-    mJob = simpleJob;
-    account->insertJob( mJob, jd );
-    connect( mJob, SIGNAL(result(KIO::Job *)),
-             SLOT(slotPutMessageResult(KIO::Job *)) );
-    connect( mJob, SIGNAL(dataReq(KIO::Job *, QByteArray &)),
-             SLOT(slotPutMessageDataReq(KIO::Job *, QByteArray &)) );
-    connect( mJob, SIGNAL(infoMessage(KIO::Job *, const QString &)),
-             SLOT(slotPutMessageInfoData(KIO::Job *, const QString &)) );
-    connect( mJob, SIGNAL(processedSize(KIO::Job *, KIO::filesize_t)),
-             SLOT(slotProcessedSize(KIO::Job *, KIO::filesize_t)));
   }
   else if ( jt == tCopyMessage || jt == tMoveMessage )
   {
@@ -179,13 +191,14 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
                           mParentProgressItem,
                           "ImapJobCopyMove"+ProgressManager::getUniqueID(),
                           i18n("Server operation"),
-                          i18n("Source folder: ") + msg_parent->prettyURL()
-                          + " - destination folder: " + mDestFolder->prettyURL(),
+                          i18n("Source folder: %1 - Destination folder: %2")
+                            .arg( msg_parent->prettyURL(), 
+                                  mDestFolder->prettyURL() ),
                           true,
                           account->useSSL() || account->useTLS() );
     jd.progressItem->setTotalItems( jd.total );
-    connect ( jd.progressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
-              account, SLOT( slotAbortRequested( ProgressItem* ) ) );
+    connect ( jd.progressItem, SIGNAL(progressItemCanceled(KPIM::ProgressItem*)),
+              account, SLOT( slotAbortRequested(KPIM::ProgressItem* ) ) );
     KIO::SimpleJob *simpleJob = KIO::special( url, packedArgs, FALSE );
     KIO::Scheduler::assignJobToSlave( account->slave(), simpleJob );
     mJob = simpleJob;
@@ -261,8 +274,8 @@ void ImapJob::slotGetNextMessage()
   KMAcctImap *account = msgParent->account();
   if ( msg->UID() == 0 )
   {
-    emit messageRetrieved( msg );
-    account->mJobList.remove( this );
+    // broken message
+    emit messageRetrieved( 0 );
     deleteLater();
     return;
   }
@@ -293,15 +306,16 @@ void ImapJob::slotGetNextMessage()
 //  kdDebug(5006) << "ImapJob::slotGetNextMessage - retrieve " << url.path() << endl;
   // protect the message, otherwise we'll get crashes afterwards
   msg->setTransferInProgress( true );
+  const QString escapedSubject = QStyleSheet::escape( msg->subject() );
   jd.progressItem = ProgressManager::createProgressItem(
                           mParentProgressItem,
                           "ImapJobDownloading"+ProgressManager::getUniqueID(),
                           i18n("Downloading message data"),
-                          i18n("Message with subject: ") + msg->subject(),
+                          i18n("Message with subject: ") + escapedSubject,
                           true,
                           account->useSSL() || account->useTLS() );
-  connect ( jd.progressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
-            account, SLOT( slotAbortRequested( ProgressItem* ) ) );
+  connect ( jd.progressItem, SIGNAL( progressItemCanceled( KPIM::ProgressItem*)),
+            account, SLOT( slotAbortRequested( KPIM::ProgressItem* ) ) );
   jd.progressItem->setTotalItems( jd.total );
 
   KIO::SimpleJob *simpleJob = KIO::get( url, FALSE, FALSE );
@@ -331,6 +345,7 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
 {
   KMMessage *msg = mMsgList.first();
   if (!msg || !msg->parent() || !job) {
+    emit messageRetrieved( 0 );
     deleteLater();
     return;
   }
@@ -339,6 +354,7 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
     msg->setTransferInProgress( false );
   KMAcctImap *account = parent->account();
   if ( !account ) {
+    emit messageRetrieved( 0 );
     deleteLater();
     return;
   }
@@ -372,16 +388,28 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
 
         // Convert CR/LF to LF.
         size_t dataSize = (*it).data.size();
-        dataSize = FolderStorage::crlf2lf( (*it).data.data(), dataSize ); // always <=
+        dataSize = Util::crlf2lf( (*it).data.data(), dataSize ); // always <=
         (*it).data.resize( dataSize );
 
+        // During the construction of the message from the byteArray it does 
+        // not have a uid. Therefore we have to make sure that no connected
+        // slots are called, since they would operate on uid == 0.
+        msg->parent()->storage()->blockSignals( true );
         msg->fromByteArray( (*it).data );
-        // reconstruct as it may be overwritten above
-        msg->setUID(uid);
-        if ( size > 0 && msg->msgSizeServer() == 0 )
+        // now let others react
+        msg->parent()->storage()->blockSignals( false );
+        if ( size > 0 && msg->msgSizeServer() == 0 ) {
           msg->setMsgSizeServer(size);
+        }
+        // reconstruct the UID as it gets overwritten above
+        msg->setUID(uid);
 
       } else {
+        // Convert CR/LF to LF.
+        size_t dataSize = (*it).data.size();
+        dataSize = Util::crlf2lf( (*it).data.data(), dataSize ); // always <=
+        (*it).data.resize( dataSize );
+
         // Update the body of the retrieved part (the message notifies all observers)
         msg->updateBodyPart( mPartSpecifier, (*it).data );
         msg->setReadyToShow( true );
@@ -413,10 +441,13 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
     {
       /* we got an answer but not data
        * this means that the msg is not on the server anymore so delete it */
+      emit messageRetrieved( 0 );
       parent->ignoreJobsForMessage( msg );
       int idx = parent->find( msg );
       if (idx != -1) parent->removeMsg( idx, true );
-      emit messageRetrieved( 0 );
+      // the removeMsg will unGet the message, which will delete all 
+      // jobs, including this one
+      return;
     }
   } else {
     emit messageUpdated(msg, mPartSpecifier);
@@ -485,10 +516,10 @@ void ImapJob::slotPutMessageDataReq( KIO::Job *job, QByteArray &data )
 //-----------------------------------------------------------------------------
 void ImapJob::slotPutMessageResult( KIO::Job *job )
 {
-  KMMessage *msg = mMsgList.first();
   KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
+  bool deleteMe = false;
   if (job->error())
   {
     if ( (*it).progressItem )
@@ -496,23 +527,29 @@ void ImapJob::slotPutMessageResult( KIO::Job *job )
     account->handlePutError( job, *it, mDestFolder );
     return;
   } else {
-    if ( !(*it).msgList.isEmpty() )
-    {
-      emit messageStored((*it).msgList.last());
-      (*it).msgList.removeLast();
-    } else if (msg)
-    {
-      emit messageStored(msg);
-    }
-    msg = 0;
     if ( (*it).progressItem )
       (*it).progressItem->setStatus( i18n("Uploading message data completed.") );
+    if ( mParentProgressItem )
+    {
+      mParentProgressItem->incCompletedItems();
+      mParentProgressItem->updateProgress();
+    }
+    KMMessage *msg = (*it).msgList.first();
+    emit messageStored( msg );
+    if ( msg == mMsgList.getLast() )
+    {
+      emit messageCopied( mMsgList );
+      if (account->slave()) {
+        account->mJobList.remove( this );
+      }
+      deleteMe = true;
+    }
   }
   if (account->slave()) {
-    account->removeJob(it);
-    account->mJobList.remove(this);
+    account->removeJob( it ); // also clears progressitem
   }
-  deleteLater();
+  if ( deleteMe )
+    deleteLater();
 }
 
 //-----------------------------------------------------------------------------
@@ -534,28 +571,15 @@ void ImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
     QValueList<ulong> newuids = KMFolderImap::splitSets(newUid);
 
     int index = -1;
-    if ( !(*it).msgList.isEmpty() )
+    KMMessage * msg;
+    for ( msg = (*it).msgList.first(); msg; msg = (*it).msgList.next() )
     {
-      KMMessage * msg;
-      for ( msg = (*it).msgList.first(); msg; msg = (*it).msgList.next() )
-      {
-        ulong uid = msg->UID();
-        index = olduids.findIndex(uid);
-        if (index > -1)
-        {
-          // found, get the new uid
-          const ulong * sernum = (ulong *)msg->getMsgSerNum();
-          imapFolder->insertUidSerNumEntry(newuids[index], sernum);
-        }
-      }
-    } else if (mMsgList.first()) {
-      ulong uid = mMsgList.first()->UID();
+      ulong uid = msg->UID();
       index = olduids.findIndex(uid);
       if (index > -1)
       {
         // found, get the new uid
-        const ulong * sernum = (ulong *)mMsgList.first()->getMsgSerNum();
-        imapFolder->insertUidSerNumEntry(newuids[index], sernum);
+        imapFolder->saveMsgMetaData( msg, newuids[index] );
       }
     }
   }
@@ -569,18 +593,12 @@ void ImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
-  if (data.find("UID") != -1)
+  if ( data.find("UID") != -1 )
   {
-    ulong uid = (data.right(data.length()-4)).toInt();
-
+    ulong uid = ( data.right(data.length()-4) ).toInt();
     if ( !(*it).msgList.isEmpty() )
     {
-      const ulong * sernum = (ulong *)(*it).msgList.last()->getMsgSerNum();
-      imapFolder->insertUidSerNumEntry(uid, sernum);
-    } else if (mMsgList.first())
-    {
-      const ulong * sernum = (ulong *)mMsgList.first()->getMsgSerNum();
-      imapFolder->insertUidSerNumEntry(uid, sernum);
+      imapFolder->saveMsgMetaData( (*it).msgList.first(), uid );
     }
   }
 }
@@ -595,7 +613,12 @@ void ImapJob::slotCopyMessageResult( KIO::Job *job )
 
   if (job->error())
   {
-    account->handleJobError( job, i18n("Error while copying messages.") );
+    mErrorCode = job->error();
+    QString errStr = i18n("Error while copying messages.");
+    if ( (*it).progressItem )
+      (*it).progressItem->setStatus( errStr );
+    account->handleJobError( job, errStr  );
+    deleteLater();
     return;
   } else {
     if ( !(*it).msgList.isEmpty() )
