@@ -5,7 +5,14 @@
 #ifndef __KMComposeWin
 #define __KMComposeWin
 
-#include "secondarywindow.h"
+#ifndef KDE_USE_FINAL
+# ifndef REALLY_WANT_KMCOMPOSEWIN_H
+#  error Do not include kmcomposewin.h anymore. Include composer.h instead.
+# endif
+#endif
+
+#include "composer.h"
+#include "messagesender.h"
 
 #include <qlabel.h>
 #include <qlistview.h>
@@ -17,14 +24,14 @@
 #include <qfont.h>
 #include <qptrlist.h>
 #include <qvaluevector.h>
+#include <qsplitter.h>
 
 #include <kio/job.h>
 #include <kglobalsettings.h>
 #include <kdeversion.h>
 #include <keditcl.h>
+#include <ktempdir.h>
 
-#include "kmmsgpart.h"
-#include "kmmsgbase.h"
 #include "mailcomposerIface.h"
 
 #include <libkdepim/addresseelineedit.h>
@@ -32,12 +39,6 @@
 
 #include <kleo/enum.h>
 
-class _StringPair {
- public:
-   QString name;
-   QString value;
-};
-
 class QCloseEvent;
 class QComboBox;
 class QFrame;
@@ -47,19 +48,18 @@ class QPopupMenu;
 class QPushButton;
 class QCString;
 class KCompletion;
-class KEdit;
+class KMEdit;
 class KMComposeWin;
 class KMFolderComboBox;
+class KMFolder;
 class KMMessage;
+class KMMessagePart;
 class KProcess;
 class KDirWatch;
 class KSelectAction;
 class KFontAction;
 class KFontSizeAction;
 class KSelectAction;
-class KSpell;
-class KSpellConfig;
-class KDictSpellingHighlighter;
 class KStatusBar;
 class KAction;
 class KToggleAction;
@@ -68,8 +68,13 @@ class KToolBar;
 class KToggleAction;
 class KSelectColorAction;
 class KURL;
+class KRecentFilesAction;
 class SpellingFilter;
 class MessageComposer;
+class RecipientsEditor;
+class KMLineEdit;
+class KMLineEditSpell;
+class KMAtmListViewItem;
 
 namespace KPIM {
   class IdentityCombo;
@@ -85,179 +90,23 @@ namespace GpgME {
   class Error;
 }
 
-
 //-----------------------------------------------------------------------------
-class KMEdit: public KEdit
+class KMComposeWin : public KMail::Composer, virtual public MailComposerIface
 {
   Q_OBJECT
-public:
-  KMEdit(QWidget *parent=0,KMComposeWin* composer=0,
-         KSpellConfig* spellConfig = 0,
-	 const char *name=0);
-  virtual ~KMEdit();
+  friend class ::KMEdit;
+  friend class ::MessageComposer;
 
-  /**
-   * Start the spell checker.
-   */
-  void spellcheck();
-
-  /**
-   * Text with lines breaks inserted after every row
-   */
-  QString brokenText();
-
-   /**
-   * Toggle automatic spellchecking
-   */
-  int autoSpellChecking( bool );
-
-  /**
-   * For the external editor
-   */
-  void setUseExternalEditor( bool use ) { mUseExtEditor = use; }
-  void setExternalEditorPath( const QString & path ) { mExtEditor = path; }
-
-  /**
-   * Check that the external editor has finished and output a warning
-   * if it hasn't.
-   * @return false if the user chose to cancel whatever operation
-   * called this method.
-   */
-  bool checkExternalEditorFinished();
-
-  void setSpellCheckingActive(bool spellCheckingActive);
-
-  /** Drag and drop methods */
-  void contentsDragEnterEvent(QDragEnterEvent *e);
-  void contentsDragMoveEvent(QDragMoveEvent *e);
-  void contentsDropEvent(QDropEvent *e);
-
-  void initializeAutoSpellChecking( KSpellConfig* autoSpellConfig );
-  void deleteAutoSpellChecking();
-
-signals:
-  void spellcheck_done(int result);
-public slots:
-  void slotSpellcheck2(KSpell*);
-  void slotSpellResult(const QString&);
-  void slotSpellDone();
-  void slotExternalEditorDone(KProcess*);
-  void slotMisspelling(const QString &, const QStringList &, unsigned int);
-  void slotCorrected (const QString &, const QString &, unsigned int);
-  void addSuggestion(const QString& text, const QStringList& lst, unsigned int );
-  virtual void cut();
-  virtual void clear();
-  virtual void del();
-protected:
-  /**
-   * Event filter that does Tab-key handling.
-   */
-  virtual bool eventFilter(QObject*, QEvent*);
-  virtual void keyPressEvent( QKeyEvent* );
-
-  KMComposeWin* mComposer;
-
-private slots:
-  void slotExternalEditorTempFileChanged( const QString & fileName );
-
-private:
-  void killExternalEditor();
-
-private:
-  KSpell *mKSpell;
-  QMap<QString,QStringList> mReplacements;
-  SpellingFilter* mSpellingFilter;
-  KTempFile *mExtEditorTempFile;
-  KDirWatch *mExtEditorTempFileWatcher;
-  KProcess  *mExtEditorProcess;
-  bool      mUseExtEditor;
-  QString   mExtEditor;
-  bool      mWasModifiedBeforeSpellCheck;
-  KDictSpellingHighlighter *mSpellChecker;
-  bool mSpellLineEdit;
-};
-
-
-//-----------------------------------------------------------------------------
-class KMLineEdit : public KPIM::AddresseeLineEdit
-{
-    Q_OBJECT
-public:
-    KMLineEdit(KMComposeWin* composer, bool useCompletion, QWidget *parent = 0,
-               const char *name = 0);
-protected:
-    // Inherited. Always called by the parent when this widget is created.
-    virtual void loadContacts();
-
-    virtual void keyPressEvent(QKeyEvent*);
-
-    virtual QPopupMenu *createPopupMenu();
-
-private slots:
-    void editRecentAddresses();
-
-private:
-    KMComposeWin* mComposer;
-};
-
-
-class KMLineEditSpell : public KMLineEdit
-{
-    Q_OBJECT
-public:
-    KMLineEditSpell(KMComposeWin* composer, bool useCompletion, QWidget *parent = 0,
-               const char *name = 0);
-    void highLightWord( unsigned int length, unsigned int pos );
-    void spellCheckDone( const QString &s );
-    void spellCheckerMisspelling( const QString &text, const QStringList &, unsigned int pos);
-    void spellCheckerCorrected( const QString &old, const QString &corr, unsigned int pos);
-
- signals:
-  void subjectTextSpellChecked();
-};
-
-
-//-----------------------------------------------------------------------------
-class KMAtmListViewItem : public QObject, public QListViewItem
-{
-  Q_OBJECT
-  friend class KMComposeWin;
-  friend class MessageComposer;
-
-public:
-  KMAtmListViewItem(QListView * parent);
-  virtual ~KMAtmListViewItem();
-  virtual void paintCell( QPainter * p, const QColorGroup & cg,
-                          int column, int width, int align );
-
-protected:
-  void enableCryptoCBs(bool on);
-  void setEncrypt(bool on);
-  bool isEncrypt();
-  void setSign(bool on);
-  bool isSign();
-
-private:
-  QListView* mListview;
-  QCheckBox* mCBEncrypt;
-  QCheckBox* mCBSign;
-  bool mCBSignEnabled, mCBEncryptEnabled;
-};
-
-
-class KMHeaders;
-
-//-----------------------------------------------------------------------------
-class KMComposeWin : public KMail::SecondaryWindow, virtual public MailComposerIface
-{
-  Q_OBJECT
-  friend class KMHeaders;         // needed for the digest forward
-  friend class MessageComposer;
-
-public:
+private: // mailserviceimpl, kmkernel, kmcommands, callback, kmmainwidget
   KMComposeWin( KMMessage* msg=0, uint identity=0 );
   ~KMComposeWin();
+public:
+  static Composer * create( KMMessage * msg = 0, uint identity = 0 );
 
+  MailComposerIface * asMailComposerIFace() { return this; }
+  const MailComposerIface * asMailComposerIFace() const { return this; }
+
+public: // mailserviceimpl
   /**
    * From MailComposerIface
    */
@@ -271,8 +120,10 @@ public:
                     const QCString &paramAttr,
                     const QString &paramValue,
                     const QCString &contDisp);
+public: // kmcommand
   void setBody (QString body);
 
+private:
   /**
    * To catch palette changes
    */
@@ -294,6 +145,7 @@ public:
    */
    void verifyWordWrapLengthIsAdequate(const QString&);
 
+public: // kmkernel, kmcommands, callback
   /**
    * Set the message the composer shall work with. This discards
    * previous messages without calling applyChanges() on them before.
@@ -301,12 +153,31 @@ public:
    void setMsg(KMMessage* newMsg, bool mayAutoSign=TRUE,
 	       bool allowDecryption=FALSE, bool isModified=FALSE);
 
+private: // kmedit
   /**
    * Returns message of the composer. To apply the user changes to the
    * message, call applyChanges() first.
    */
-   KMMessage* msg(void) const { return mMsg; }
+   KMMessage* msg() const { return mMsg; }
 
+public: // kmkernel
+  /**
+   * Set the filename which is used for autosaving.
+   */
+  void setAutoSaveFilename( const QString & filename );
+
+private:
+  /**
+   * Returns true if the message was modified by the user.
+   */
+  bool isModified() const;
+
+  /**
+   * Set whether the message should be treated as modified or not.
+   */
+  void setModified( bool modified );
+
+public: // kmkernel, callback
   /**
    * If this flag is set the message of the composer is deleted when
    * the composer is closed and the message was not sent. Default: FALSE
@@ -319,17 +190,19 @@ public:
    */
   void setAutoDeleteWindow( bool f );
 
+public: // kmcommand
   /**
    * If this folder is set, the original message is inserted back after
    * cancelling
    */
    void setFolder(KMFolder* aFolder) { mFolder = aFolder; }
-
+public: // kmkernel, kmcommand, mailserviceimpl
   /**
    * Recode to the specified charset
    */
    void setCharset(const QCString& aCharset, bool forceDefault = FALSE);
 
+public: // kmcommand
   /**
    * Sets the focus to the edit-widget and the cursor below the
    * "On ... you wrote" line when hasMessage is true.
@@ -343,6 +216,7 @@ public:
    */
    void setFocusToSubject();
 
+private:
   /**
    * determines whether inline signing/encryption is selected
    */
@@ -356,27 +230,38 @@ public:
    static QString prettyMimeType( const QString& type );
     QString quotePrefixName() const;
 
-    KMLineEditSpell *sujectLineWidget() const { return mEdtSubject;}
+private: // kmedit:
+  KMLineEditSpell *sujectLineWidget() const { return mEdtSubject;}
   void setSubjectTextWasSpellChecked( bool _spell ) {
     mSubjectTextWasSpellChecked = _spell;
   }
   bool subjectTextWasSpellChecked() const { return mSubjectTextWasSpellChecked; }
 
 
+public: // callback
   /** Disabled signing and encryption completely for this composer window. */
   void setSigningAndEncryptionDisabled( bool v )
   {
     mSigningAndEncryptionExplicitlyDisabled = v;
   }
 
-public slots:
+private slots:
+  void polish();
   /**
    * Actions:
    */
   void slotPrint();
   void slotAttachFile();
+  void slotInsertRecentFile(const KURL&);
+public slots: // kmkernel, callback
   void slotSendNow();
+private slots:
+  void slotSendNowVia( int item );
   void slotSendLater();
+  void slotSendLaterVia( int item );
+
+  void getTransportMenu();
+
   /**
    * Returns true when saving was successful.
    */
@@ -395,6 +280,7 @@ public slots:
   void slotCopy();
   void slotPaste();
   void slotPasteAsQuotation();
+  void slotPasteAsAttachment();
   void slotAddQuotes();
   void slotRemoveQuotes();
 
@@ -402,6 +288,7 @@ public slots:
 
   void slotFolderRemoved(KMFolder*);
 
+public slots: // kmkernel
   /**
      Tell the composer to always send the message, even if the user
      hasn't changed the next. This is useful if a message is
@@ -409,7 +296,7 @@ public slots:
      simply be able to confirm the message and send it.
   */
   void slotSetAlwaysSend( bool bAlwaysSend );
-
+private slots:
   /**
    * toggle fixed width font.
    */
@@ -436,7 +323,7 @@ public slots:
    * Change crypto plugin to be used for signing/encrypting messages,
    * or switch to built-in OpenPGP code.
    */
-  void slotSelectCryptoModule();
+  void slotSelectCryptoModule( bool init = false );
 
   /**
    * XML-GUI stuff
@@ -465,11 +352,13 @@ public slots:
    */
   void slotSignToggled(bool);
 
+public slots: // kmkernel, callback
   /**
    * Switch wordWrap on/off
    */
   void slotWordWrapToggled(bool);
 
+private slots:
   /**
    * Append signature file to the end of the text in the editor.
    */
@@ -504,6 +393,7 @@ public slots:
   /**
    * Attachment operations.
    */
+  void slotAttachOpen();
   void slotAttachView();
   void slotAttachRemove();
   void slotAttachSave();
@@ -527,20 +417,14 @@ public slots:
   void slotSpellcheckDone(int result);
   void slotSpellcheckDoneClearStatus();
 
-  /**
-   * Append current message to ~/dead.letter
-   */
-  void deadLetter(void);
+public slots: // kmkernel
+  void autoSaveMessage();
 
+private slots:
   void updateCursorPosition();
 
   void slotView();
 
-  /**
-   * Move focus to next/prev edit widget
-   */
-  void focusNextPrevEdit(const QWidget* current, bool next);
-
   /**
    * Update composer field to reflect new identity
    */
@@ -566,27 +450,31 @@ public slots:
   void fontChanged( const QFont & );
   void alignmentChanged( int );
 
+public: // kmkernel, attachmentlistview
   void addAttach(const KURL url);
 
+public: // kmcommand
   /**
    * Add an attachment to the list.
    */
-   void addAttach(const KMMessagePart* msgPart);
+  void addAttach(const KMMessagePart* msgPart);
 
-public:
+private:
+  /**
+   * Add an image from the clipboard as attachment
+   */
+  void addImageFromClipboard();
+
+private:
   const KPIM::Identity & identity() const;
+  uint identityUid() const;
   Kleo::CryptoMessageFormat cryptoMessageFormat() const;
   bool encryptToSelf() const;
 
 signals:
-  /**
-   * A message has been queued or saved in the drafts folder
-   */
-  void messageQueuedOrDrafted();
-
   void applyChangesDone( bool );
 
-protected:
+private:
   /**
    * Applies the user changes to the message object of the composer
    * and signs/encrypts the message if activated. Returns FALSE in
@@ -603,6 +491,11 @@ protected:
    */
   void rethinkFields(bool fromslot=false);
 
+  /**
+    Connect signals for moving focus by arrow keys. Returns next edit.
+  */
+  QWidget *connectFocusMoving( QWidget *prev, QWidget *next );
+
   /**
    * Show or hide header lines
    */
@@ -620,7 +513,7 @@ protected:
   /**
    * Initialization methods
    */
-  void setupActions(int aCryptoMessageFormat=-1);
+  void setupActions();
   void setupStatusBar();
   void setupEditor();
 
@@ -651,6 +544,11 @@ protected:
   */
   virtual bool queryExit ();
 
+  /**
+   * Open the attachment with the given index.
+   */
+  void openAttach( int index );
+
   /**
    * View the attachment with the given index.
    */
@@ -665,7 +563,8 @@ protected:
    /**
     * Updates an item in the QListView to represnet a given message part
     */
-   void msgPartToItem(const KMMessagePart* msgPart, KMAtmListViewItem *lvi);
+   void msgPartToItem(const KMMessagePart* msgPart, KMAtmListViewItem *lvi,
+        bool loadDefaults = true );
 
   /**
    * Open addressbook and append selected addresses to the given
@@ -673,6 +572,9 @@ protected:
    */
   void addrBookSelInto();
 
+  void addrBookSelIntoOld();
+  void addrBookSelIntoNew();
+
 private:
   /**
    * Turn encryption on/off. If setByUser is true then a message box is shown
@@ -716,9 +618,45 @@ private:
   /**
    * Send the message. Returns true if the message was sent successfully.
    */
-  void doSend(int sendNow=-1, bool saveInDrafts = false);
+  void doSend( KMail::MessageSender::SendMethod method=KMail::MessageSender::SendDefault, bool saveInDrafts = false);
 
-protected:
+  /**
+   * Returns the autosave interval in milliseconds (as needed for QTimer).
+   */
+  int autoSaveInterval() const;
+
+  /**
+   * Initialize autosaving (timer and filename).
+   */
+  void initAutoSave();
+
+  /**
+   * Enables/disables autosaving depending on the value of the autosave
+   * interval.
+   */
+  void updateAutoSave();
+
+  /**
+   * Stop autosaving and delete the autosaved message.
+   */
+  void cleanupAutoSave();
+
+  /**
+   * Validates a list of email addresses.
+   * @return true if all addresses are valid.
+   * @return false if one or several addresses are invalid.
+   */
+  static bool validateAddresses( QWidget * parent, const QString & addresses );
+
+private slots:
+   /**
+    * Compress an attachemnt with the given index
+    */
+    void compressAttach(int idx);
+    void uncompressAttach(int idx);
+    void editorFocusChanged(bool gained);
+
+private:
   QWidget   *mMainWidget;
   QComboBox *mTransport;
   KMail::DictionaryComboBox *mDictionaryCombo;
@@ -742,41 +680,35 @@ protected:
   KMail::AttachmentListView* mAtmListView;
   int mAtmColEncrypt;
   int mAtmColSign;
+  int mAtmColCompress;
   int mAtmEncryptColWidth;
   int mAtmSignColWidth;
+  int mAtmCompressColWidth;
   QPtrList<QListViewItem> mAtmItemList;
   QPtrList<KMMessagePart> mAtmList;
   QPopupMenu *mAttachMenu;
-  int mViewId, mRemoveId, mSaveAsId, mPropertiesId;
-  bool mAutoSign, mAutoPgpSign, mAutoPgpEncrypt, mAutoDeleteMsg;
-  bool mNeverSignWhenSavingInDrafts, mNeverEncryptWhenSavingInDrafts;
+  int mOpenId, mViewId, mRemoveId, mSaveAsId, mPropertiesId;
+  bool mAutoDeleteMsg;
   bool mSigningAndEncryptionExplicitlyDisabled;
-  bool mAutoRequestMDN;
   bool mLastSignActionState, mLastEncryptActionState;
   bool mLastIdentityHasSigningKey, mLastIdentityHasEncryptionKey;
   KMFolder *mFolder;
   long mShowHeaders;
-  QString mExtEditor;
-  bool mUseHTMLEditor;
-  bool mUseExtEditor;
-  QPtrList<_StringPair> mCustHeaders;
   bool mConfirmSend;
   bool mDisableBreaking; // Move
   int mNumHeaders;
-  int mLineBreak;
-  int mWordWrap;
-  bool mUseFixedFont;
+  bool mUseHTMLEditor;
+  bool mHtmlMarkup;
   QFont mBodyFont, mFixedFont;
-  //  QList<QLineEdit> mEdtList;
-  QPtrList<QWidget> mEdtList;
   QPtrList<KTempFile> mAtmTempList;
   QPalette mPalette;
   uint mId;
   QString mOldSigText;
-  QStringList mTransportHistory;
 
   KAction *mAttachPK, *mAttachMPK,
-          *mAttachRemoveAction, *mAttachSaveAction, *mAttachPropertiesAction;
+          *mAttachRemoveAction, *mAttachSaveAction, *mAttachPropertiesAction,
+          *mPasteQuotation, *mAddQuoteChars, *mRemQuoteChars;
+  KRecentFilesAction *mRecentAction;
 
   KToggleAction *mSignAction, *mEncryptAction, *mRequestMDNAction;
   KToggleAction *mUrgentAction, *mAllFieldsAction, *mFromAction;
@@ -804,7 +736,6 @@ protected:
   bool mAutoCharset;
 
   bool mAlwaysSend;
-  bool mOutlookCompatible;
 
   QStringList mFolderNames;
   QValueList<QGuardedPtr<KMFolder> > mFolderList;
@@ -818,6 +749,10 @@ private:
 
   bool mSubjectTextWasSpellChecked;
 
+  QString addQuotesToText(const QString &inputText);
+  // helper method for rethinkFields
+  int calcColumnWidth(int which, long allShowing, int width);
+
 private slots:
   void slotCompletionModeChanged( KGlobalSettings::Completion );
   void slotConfigChanged();
@@ -826,7 +761,9 @@ private slots:
 
   void slotContinueDoSend( bool );
   void slotContinuePrint( bool );
-  void slotContinueDeadLetter( bool );
+  void slotContinueAutoSave( bool );
+  
+  void slotEncryptChiasmusToggled( bool );
 
   /**
    * Helper method (you could call is a bottom-half :) for
@@ -842,6 +779,7 @@ private slots:
 private:
   QColor mForeColor,mBackColor;
   QFont mSaveFont;
+  QSplitter *mSplitter;
   struct atmLoadData
   {
     KURL url;
@@ -850,12 +788,14 @@ private:
     QCString encoding;
   };
   QMap<KIO::Job *, atmLoadData> mMapAtmLoadData;
-  bool mForceReplyCharset;
 
   // These are for passing on methods over the applyChanges calls
-  int mSendNow;
+  KMail::MessageSender::SendMethod mSendMethod;
   bool mSaveInDrafts;
 
+  KToggleAction *mEncryptChiasmusAction;
+  bool mEncryptWithChiasmus;
+
   // This is the temporary object that constructs the message out of the
   // window
   MessageComposer* mComposer;
@@ -865,6 +805,21 @@ private:
 
   // Temp var for slotInsert(My)PublicKey():
   QString mFingerprint;
+
+  // Temp ptr for saving image from clipboard
+  KTempDir *mTempDir;
+
+  bool mClassicalRecipients;
+
+  RecipientsEditor *mRecipientsEditor;
+  int mLabelWidth;
+
+  QTimer *mAutoSaveTimer;
+  QString mAutoSaveFilename;
+  int mLastAutoSaveErrno; // holds the errno of the last try to autosave
+
+  QPopupMenu *mActNowMenu;
+  QPopupMenu *mActLaterMenu;
 };
 
 #endif
