@@ -14,7 +14,8 @@
 #include "kmfiltermgr.h"
 #include "kmfolderindex.h"
 #include "kmfoldermgr.h"
-#include "kmsender.h"
+#include "messagesender.h"
+#include "kmmainwidget.h"
 #include <libkpimidentities/identity.h>
 #include <libkpimidentities/identitymanager.h>
 #include <libkpimidentities/identitycombo.h>
@@ -22,12 +23,15 @@
 #include <libkdepim/collectingprocess.h>
 using KPIM::CollectingProcess;
 #include "kmfawidgets.h"
-#include "kmfoldercombobox.h"
+#include "folderrequester.h"
+using KMail::FolderRequester;
 #include "kmmsgbase.h"
 #include "messageproperty.h"
 #include "actionscheduler.h"
 using KMail::MessageProperty;
 using KMail::ActionScheduler;
+#include "regexplineedit.h"
+using KMail::RegExpLineEdit;
 #include <kregexp3.h>
 #include <ktempfile.h>
 #include <kdebug.h>
@@ -41,6 +45,7 @@ using KMail::ActionScheduler;
 #include <qtextcodec.h>
 #include <qtimer.h>
 #include <qobject.h>
+#include <qstylesheet.h>
 #include <assert.h>
 
 
@@ -109,7 +114,7 @@ void KMFilterAction::sendMDN( KMMessage * msg, KMime::MDN::DispositionType d,
                               const QValueList<KMime::MDN::DispositionModifier> & m ) {
   if ( !msg ) return;
   KMMessage * mdn = msg->createMDN( KMime::MDN::AutomaticAction, d, false, m );
-  if ( mdn && !kmkernel->msgSender()->send( mdn, FALSE ) ) {
+  if ( mdn && !kmkernel->msgSender()->send( mdn, KMail::MessageSender::SendLater ) ) {
     kdDebug(5006) << "KMFilterAction::sendMDN(): sending failed." << endl;
     //delete mdn;
   }
@@ -127,6 +132,11 @@ KMFilterActionWithNone::KMFilterActionWithNone( const char* aName, const QString
 {
 }
 
+const QString KMFilterActionWithNone::displayString() const
+{
+  return label();
+}
+
 
 //=============================================================================
 //
@@ -149,6 +159,14 @@ const QString KMFilterActionWithUOID::argsAsString() const
   return QString::number( mParameter );
 }
 
+const QString KMFilterActionWithUOID::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
+
+
 //=============================================================================
 //
 // KMFilterActionWithString
@@ -192,6 +210,13 @@ const QString KMFilterActionWithString::argsAsString() const
   return mParameter;
 }
 
+const QString KMFilterActionWithString::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
+
 //=============================================================================
 //
 // class KMFilterActionWithStringList
@@ -252,36 +277,29 @@ KMFilterActionWithFolder::KMFilterActionWithFolder( const char* aName, const QSt
 
 QWidget* KMFilterActionWithFolder::createParamWidget( QWidget* parent ) const
 {
-  KMFolderComboBox *cb = new KMFolderComboBox( parent );
-  cb->showImapFolders( false );
-  setParamWidgetValue( cb );
-  return cb;
+  FolderRequester *req = new FolderRequester( parent,
+      kmkernel->getKMMainWidget()->folderTree() );
+  setParamWidgetValue( req );
+  return req;
 }
 
 void KMFilterActionWithFolder::applyParamWidgetValue( QWidget* paramWidget )
 {
-  mFolder = ((KMFolderComboBox *)paramWidget)->getFolder();
-  if (mFolder)
-  {
-     mFolderName = QString::null;
-  }
-  else
-  {
-     mFolderName = ((KMFolderComboBox *)paramWidget)->currentText();
-  }
+  mFolder = ((FolderRequester *)paramWidget)->folder();
+  mFolderName = ((FolderRequester *)paramWidget)->folderId();
 }
 
 void KMFilterActionWithFolder::setParamWidgetValue( QWidget* paramWidget ) const
 {
   if ( mFolder )
-    ((KMFolderComboBox *)paramWidget)->setFolder( mFolder );
+    ((FolderRequester *)paramWidget)->setFolder( mFolder );
   else
-    ((KMFolderComboBox *)paramWidget)->setFolder( mFolderName );
+    ((FolderRequester *)paramWidget)->setFolder( mFolderName );
 }
 
 void KMFilterActionWithFolder::clearParamWidget( QWidget* paramWidget ) const
 {
-  ((KMFolderComboBox *)paramWidget)->setFolder( kmkernel->draftsFolder() );
+  ((FolderRequester *)paramWidget)->setFolder( kmkernel->draftsFolder() );
 }
 
 void KMFilterActionWithFolder::argsFromString( const QString argsStr )
@@ -289,8 +307,10 @@ void KMFilterActionWithFolder::argsFromString( const QString argsStr )
   mFolder = kmkernel->folderMgr()->findIdString( argsStr );
   if (!mFolder)
      mFolder = kmkernel->dimapFolderMgr()->findIdString( argsStr );
+  if (!mFolder)
+     mFolder = kmkernel->imapFolderMgr()->findIdString( argsStr );
   if (mFolder)
-     mFolderName = QString::null;
+     mFolderName = mFolder->idString();
   else
      mFolderName = argsStr;
 }
@@ -305,14 +325,22 @@ const QString KMFilterActionWithFolder::argsAsString() const
   return result;
 }
 
+const QString KMFilterActionWithFolder::displayString() const
+{
+  QString result;
+  if ( mFolder )
+    result = mFolder->prettyURL();
+  else
+    result = mFolderName;
+  return label() + " \"" + QStyleSheet::escape( result ) + "\"";
+}
+
 bool KMFilterActionWithFolder::folderRemoved( KMFolder* aFolder, KMFolder* aNewFolder )
 {
   if ( aFolder == mFolder ) {
     mFolder = aNewFolder;
     if ( aNewFolder )
-      mFolderName = QString::null;
-    else
-      mFolderName = i18n( "<select a folder>" );
+      mFolderName = mFolder->idString();
     return TRUE;
   } else
     return FALSE;
@@ -529,41 +557,6 @@ KMFilterAction::ReturnCode KMFilterActionWithCommand::genericProcess(KMMessage*
 //
 //=============================================================================
 
-//=============================================================================
-// KMFilterActionBounce - bounce
-// Return mail as undelivered.
-//=============================================================================
-class KMFilterActionBounce : public KMFilterActionWithNone
-{
-public:
-  KMFilterActionBounce();
-  virtual ReturnCode process(KMMessage* msg) const;
-  static KMFilterAction* newAction(void);
-};
-
-KMFilterAction* KMFilterActionBounce::newAction(void)
-{
-  return (new KMFilterActionBounce);
-}
-
-KMFilterActionBounce::KMFilterActionBounce()
-  : KMFilterActionWithNone( "bounce", i18n("Bounce") )
-{
-}
-
-KMFilterAction::ReturnCode KMFilterActionBounce::process(KMMessage* msg) const
-{
-  KMMessage *bounceMsg = msg->createBounce( FALSE );
-  if ( !bounceMsg ) return ErrorButGoOn;
-
-  // Queue message. This is a) so that the user can check
-  // the bounces before sending and b) for speed reasons.
-  kmkernel->msgSender()->send( bounceMsg, FALSE );
-
-  return GoOn;
-}
-
-
 //=============================================================================
 // KMFilterActionSendReceipt - send receipt
 // Return delivery receipt.
@@ -593,7 +586,7 @@ KMFilterAction::ReturnCode KMFilterActionSendReceipt::process(KMMessage* msg) co
 
   // Queue message. This is a) so that the user can check
   // the receipt before sending and b) for speed reasons.
-  kmkernel->msgSender()->send( receipt, FALSE );
+  kmkernel->msgSender()->send( receipt, KMail::MessageSender::SendLater );
 
   return GoOn;
 }
@@ -742,6 +735,7 @@ public:
 
   virtual void argsFromString( const QString argsStr );
   virtual const QString argsAsString() const;
+  virtual const QString displayString() const;
 };
 
 
@@ -823,6 +817,12 @@ const QString KMFilterActionSetStatus::argsAsString() const
   return KMMsgBase::statusToStr(status);
 }
 
+const QString KMFilterActionSetStatus::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
 
 //=============================================================================
 // KMFilterActionFakeDisposition - send fake MDN
@@ -841,6 +841,7 @@ public:
 
   virtual void argsFromString( const QString argsStr );
   virtual const QString argsAsString() const;
+  virtual const QString displayString() const;
 };
 
 
@@ -911,6 +912,12 @@ const QString KMFilterActionFakeDisposition::argsAsString() const
   return QString( QChar( idx < 2 ? 'I' : char(mdns[idx-2]) ) );
 }
 
+const QString KMFilterActionFakeDisposition::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
 
 //=============================================================================
 // KMFilterActionRemoveHeader - remove header
@@ -995,6 +1002,8 @@ public:
   virtual const QString argsAsString() const;
   virtual void argsFromString( const QString argsStr );
 
+  virtual const QString displayString() const;
+
   static KMFilterAction* newAction()
   {
     return (new KMFilterActionAddHeader);
@@ -1088,6 +1097,13 @@ const QString KMFilterActionAddHeader::argsAsString() const
   return result;
 }
 
+const QString KMFilterActionAddHeader::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
+
 void KMFilterActionAddHeader::argsFromString( const QString argsStr )
 {
   QStringList l = QStringList::split( '\t', argsStr, TRUE /*allow empty entries*/ );
@@ -1126,6 +1142,8 @@ public:
   virtual const QString argsAsString() const;
   virtual void argsFromString( const QString argsStr );
 
+  virtual const QString displayString() const;
+
   static KMFilterAction* newAction()
   {
     return (new KMFilterActionRewriteHeader);
@@ -1176,14 +1194,14 @@ QWidget* KMFilterActionRewriteHeader::createParamWidget( QWidget* parent ) const
   l->setFixedWidth( l->sizeHint().width() );
   hbl->addWidget( l, 0 );
 
-  QLineEdit *le = new KLineEdit( w, "search" );
-  hbl->addWidget( le, 1 );
+  RegExpLineEdit *rele = new RegExpLineEdit( w, "search" );
+  hbl->addWidget( rele, 1 );
 
   l = new QLabel( i18n("With:"), w );
   l->setFixedWidth( l->sizeHint().width() );
   hbl->addWidget( l, 0 );
 
-  le = new KLineEdit( w, "replace" );
+  QLineEdit *le = new KLineEdit( w, "replace" );
   hbl->addWidget( le, 1 );
 
   setParamWidgetValue( w );
@@ -1205,11 +1223,11 @@ void KMFilterActionRewriteHeader::setParamWidgetValue( QWidget* paramWidget ) co
     cb->setCurrentItem( idx );
   }
 
-  QLineEdit *le = (QLineEdit*)paramWidget->child("search");
-  Q_ASSERT( le );
-  le->setText( mRegExp.pattern() );
+  RegExpLineEdit *rele = (RegExpLineEdit*)paramWidget->child("search");
+  Q_ASSERT( rele );
+  rele->setText( mRegExp.pattern() );
 
-  le = (QLineEdit*)paramWidget->child("replace");
+  QLineEdit *le = (QLineEdit*)paramWidget->child("replace");
   Q_ASSERT( le );
   le->setText( mReplacementString );
 }
@@ -1220,11 +1238,11 @@ void KMFilterActionRewriteHeader::applyParamWidgetValue( QWidget* paramWidget )
   Q_ASSERT( cb );
   mParameter = cb->currentText();
 
-  QLineEdit *le = (QLineEdit*)paramWidget->child("search");
-  Q_ASSERT( le );
-  mRegExp.setPattern( le->text() );
+  RegExpLineEdit *rele = (RegExpLineEdit*)paramWidget->child("search");
+  Q_ASSERT( rele );
+  mRegExp.setPattern( rele->text() );
 
-  le = (QLineEdit*)paramWidget->child("replace");
+  QLineEdit *le = (QLineEdit*)paramWidget->child("replace");
   Q_ASSERT( le );
   mReplacementString = le->text();
 }
@@ -1235,11 +1253,11 @@ void KMFilterActionRewriteHeader::clearParamWidget( QWidget* paramWidget ) const
   Q_ASSERT( cb );
   cb->setCurrentItem(0);
 
-  QLineEdit *le = (QLineEdit*)paramWidget->child("search");
-  Q_ASSERT( le );
-  le->clear();
+  RegExpLineEdit *rele = (RegExpLineEdit*)paramWidget->child("search");
+  Q_ASSERT( rele );
+  rele->clear();
 
-  le = (QLineEdit*)paramWidget->child("replace");
+  QLineEdit *le = (QLineEdit*)paramWidget->child("replace");
   Q_ASSERT( le );
   le->clear();
 }
@@ -1255,6 +1273,13 @@ const QString KMFilterActionRewriteHeader::argsAsString() const
   return result;
 }
 
+const QString KMFilterActionRewriteHeader::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
+
 void KMFilterActionRewriteHeader::argsFromString( const QString argsStr )
 {
   QStringList l = QStringList::split( '\t', argsStr, TRUE /*allow empty entries*/ );
@@ -1274,7 +1299,7 @@ void KMFilterActionRewriteHeader::argsFromString( const QString argsStr )
 
 
 //=============================================================================
-// KMFilterActionMove - file into folder
+// KMFilterActionMove - move into folder
 // File message into another mail folder
 //=============================================================================
 class KMFilterActionMove: public KMFilterActionWithFolder
@@ -1292,7 +1317,7 @@ KMFilterAction* KMFilterActionMove::newAction(void)
 }
 
 KMFilterActionMove::KMFilterActionMove()
-  : KMFilterActionWithFolder( "transfer", i18n("File into Folder") )
+  : KMFilterActionWithFolder( "transfer", i18n("Move Into Folder") )
 {
 }
 
@@ -1301,7 +1326,18 @@ KMFilterAction::ReturnCode KMFilterActionMove::process(KMMessage* msg) const
   if ( !mFolder )
     return ErrorButGoOn;
 
-  MessageProperty::setFilterFolder( msg, mFolder );
+  ActionScheduler *handler = MessageProperty::filterHandler( msg );
+  if (handler) {
+    MessageProperty::setFilterFolder( msg, mFolder );
+  } else {
+    // The old filtering system does not support online imap targets.
+    // Skip online imap targets when using the old system.
+    KMFolder *check;
+    check = kmkernel->imapFolderMgr()->findIdString( argsAsString() );
+    if (mFolder && (check != mFolder)) {
+      MessageProperty::setFilterFolder( msg, mFolder );
+    }
+  }
   return GoOn;
 }
 
@@ -1310,6 +1346,70 @@ bool KMFilterActionMove::requiresBody(KMMsgBase*) const
     return false; //iff mFolder->folderMgr == msgBase->parent()->folderMgr;
 }
 
+
+//=============================================================================
+// KMFilterActionCopy - copy into folder
+// Copy message into another mail folder
+//=============================================================================
+class KMFilterActionCopy: public KMFilterActionWithFolder
+{
+public:
+  KMFilterActionCopy();
+  virtual ReturnCode process(KMMessage* msg) const;
+  virtual void processAsync(KMMessage* msg) const;
+  virtual bool requiresBody(KMMsgBase*) const;
+  static KMFilterAction* newAction(void);
+};
+
+KMFilterAction* KMFilterActionCopy::newAction(void)
+{
+  return (new KMFilterActionCopy);
+}
+
+KMFilterActionCopy::KMFilterActionCopy()
+  : KMFilterActionWithFolder( "copy", i18n("Copy Into Folder") )
+{
+}
+
+KMFilterAction::ReturnCode KMFilterActionCopy::process(KMMessage* msg) const
+{
+  // TODO opening and closing the folder is a trade off.
+  // Perhaps Copy is a seldomly used action for now,
+  // but I gonna look at improvements ASAP.
+  if ( !mFolder && mFolder->open() != 0 )
+    return ErrorButGoOn;
+
+  // copy the message 1:1
+  KMMessage* msgCopy = new KMMessage;
+  msgCopy->fromDwString(msg->asDwString());
+
+  int index;
+  int rc = mFolder->addMsg(msgCopy, &index);
+  if (rc == 0 && index != -1)
+    mFolder->unGetMsg( index );
+  mFolder->close();
+
+  return GoOn;
+}
+
+void KMFilterActionCopy::processAsync(KMMessage* msg) const
+{
+  // FIXME remove the debug output
+  kdDebug(5006) << "##### KMFilterActionCopy::processAsync(KMMessage* msg)" << endl;
+  ActionScheduler *handler = MessageProperty::filterHandler( msg );
+
+  KMCommand *cmd = new KMCopyCommand( mFolder, msg );
+  QObject::connect( cmd, SIGNAL( completed( KMCommand * ) ),
+                    handler, SLOT( copyMessageFinished( KMCommand * ) ) );
+  cmd->start();
+}
+
+bool KMFilterActionCopy::requiresBody(KMMsgBase*) const
+{
+    return true;
+}
+
+
 //=============================================================================
 // KMFilterActionForward - forward to
 // Forward message to another user
@@ -1410,7 +1510,7 @@ KMFilterAction::ReturnCode KMFilterActionForward::process(KMMessage* aMsg) const
 
   sendMDN( aMsg, KMime::MDN::Dispatched );
 
-  if ( !kmkernel->msgSender()->send( msg, FALSE ) ) {
+  if ( !kmkernel->msgSender()->send( msg, KMail::MessageSender::SendLater ) ) {
     kdDebug(5006) << "KMFilterAction: could not forward message (sending failed)" << endl;
     return ErrorButGoOn; // error: couldn't send
   }
@@ -1446,11 +1546,11 @@ KMFilterAction::ReturnCode KMFilterActionRedirect::process(KMMessage* aMsg) cons
   if ( mParameter.isEmpty() )
     return ErrorButGoOn;
 
-  msg = aMsg->createRedirect2( mParameter );
-  
+  msg = aMsg->createRedirect( mParameter );
+
   sendMDN( aMsg, KMime::MDN::Dispatched );
 
-  if ( !kmkernel->msgSender()->send( msg, FALSE ) ) {
+  if ( !kmkernel->msgSender()->send( msg, KMail::MessageSender::SendLater ) ) {
     kdDebug(5006) << "KMFilterAction: could not redirect message (sending failed)" << endl;
     return ErrorButGoOn; // error: couldn't send
   }
@@ -1655,6 +1755,13 @@ const QString KMFilterActionWithTest::argsAsString() const
   return mParameter;
 }
 
+const QString KMFilterActionWithTest::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
+
 
 KMFilterActionExecSound::KMFilterActionExecSound()
   : KMFilterActionWithTest( "play sound", i18n("Play Sound") )
@@ -1725,6 +1832,13 @@ const QString KMFilterActionWithUrl::argsAsString() const
   return mParameter;
 }
 
+const QString KMFilterActionWithUrl::displayString() const
+{
+  // FIXME after string freeze:
+  // return i18n("").arg( );
+  return label() + " \"" + QStyleSheet::escape( argsAsString() ) + "\"";
+}
+
 
 //=============================================================================
 //
@@ -1734,6 +1848,7 @@ const QString KMFilterActionWithUrl::argsAsString() const
 void KMFilterActionDict::init(void)
 {
   insert( KMFilterActionMove::newAction );
+  insert( KMFilterActionCopy::newAction );
   insert( KMFilterActionIdentity::newAction );
   insert( KMFilterActionSetStatus::newAction );
   insert( KMFilterActionFakeDisposition::newAction );
@@ -1741,7 +1856,6 @@ void KMFilterActionDict::init(void)
   insert( KMFilterActionReplyTo::newAction );
   insert( KMFilterActionForward::newAction );
   insert( KMFilterActionRedirect::newAction );
-  insert( KMFilterActionBounce::newAction );
   insert( KMFilterActionSendReceipt::newAction );
   insert( KMFilterActionExec::newAction );
   insert( KMFilterActionExtFilter::newAction );
@@ -1751,10 +1865,10 @@ void KMFilterActionDict::init(void)
   insert( KMFilterActionExecSound::newAction );
   // Register custom filter actions below this line.
 }
-// The int in the QDict constructor (23) must be a prime
+// The int in the QDict constructor (41) must be a prime
 // and should be greater than the double number of KMFilterAction types
 KMFilterActionDict::KMFilterActionDict()
-  : QDict<KMFilterActionDesc>(23)
+  : QDict<KMFilterActionDesc>(41)
 {
   mList.setAutoDelete(TRUE);
   init();
