@@ -1,7 +1,22 @@
-// -*- mode: C++; c-file-style: "gnu" -*-
-// kmfolder.cpp
-// Author: Stefan Taferner <taferner@alpin.or.at>
-
+/* -*- mode: C++; c-file-style: "gnu" -*-
+ * kmail: KDE mail client
+ * Copyright (c) 1996-1998 Stefan Taferner <taferner@kde.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ */
 #include <config.h>
 
 #include "kmfolder.h"
@@ -17,25 +32,34 @@
 #include "expirejob.h"
 #include "compactionjob.h"
 #include "kmfoldertree.h"
+#include "kmailicalifaceimpl.h"
 
 #include <errno.h>
 
 #include <kdebug.h>
 #include <klocale.h>
+#include <kshortcut.h>
 #include <kmessagebox.h>
 #include <qfile.h>
+#include <qfileinfo.h>
 
 
 KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
-                    KMFolderType aFolderType )
+                             KMFolderType aFolderType, bool withIndex, bool exportedSernums )
   : KMFolderNode( aParent, aFolderName ), mStorage(0),
-    mParent( aParent ), mChild( 0 ),
+    mChild( 0 ),
     mIsSystemFolder( false ),
+    mHasIndex( withIndex ),
+    mExportsSernums( exportedSernums ),
     mExpireMessages( false ), mUnreadExpireAge( 28 ),
     mReadExpireAge( 14 ), mUnreadExpireUnits( expireNever ),
     mReadExpireUnits( expireNever ),
     mExpireAction( ExpireDelete ),
-    mUseCustomIcons( false ), mMailingListEnabled( false )
+    mUseCustomIcons( false ), mMailingListEnabled( false ),
+    mAcctList( 0 ),
+    mIdentity( 0 ), // default identity
+    mPutRepliesInSameFolder( false ),
+    mIgnoreNewMail( false )
 {
   if( aFolderType == KMFolderTypeCachedImap )
     mStorage = new KMFolderCachedImap( this, aFolderName.latin1() );
@@ -48,6 +72,24 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
   else
     mStorage = new KMFolderMbox( this, aFolderName.latin1() );
 
+  assert( mStorage );
+
+  QFileInfo dirinfo;
+  dirinfo.setFile( mStorage->location() );
+  if ( !dirinfo.exists() ) {
+    int rc = mStorage->create();
+    QString msg = i18n("<qt>Error while creating file <b>%1</b>:<br>%2</qt>").arg(aFolderName).arg(strerror(rc));
+    if ( rc ) {
+      KMessageBox::information(0, msg);
+    }
+  }
+
+   // trigger from here, since it needs a fully constructed FolderStorage
+  if ( mExportsSernums )
+    mStorage->registerWithMessageDict();
+  if ( !mHasIndex )
+    mStorage->setAutoCreateIndex( false );
+
   if ( aParent ) {
     connect( mStorage, SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ),
              aParent->manager(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ) );
@@ -57,6 +99,8 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
              parent()->manager(), SIGNAL( msgChanged( KMFolder*, Q_UINT32, int ) ) );
     connect( this, SIGNAL( msgHeaderChanged( KMFolder*,  int ) ),
              parent()->manager(), SIGNAL( msgHeaderChanged( KMFolder*, int ) ) );
+    connect( mStorage, SIGNAL( invalidated( KMFolder* ) ),
+             parent()->manager(), SIGNAL( folderInvalidated( KMFolder* ) ) );
   }
 
   // Resend all mStorage signals
@@ -67,8 +111,8 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
   connect( mStorage, SIGNAL( nameChanged() ), SIGNAL( nameChanged() ) );
   connect( mStorage, SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ),
            SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ) );
-  connect( mStorage, SIGNAL( msgRemoved( int, QString, QString ) ),
-           SIGNAL( msgRemoved( int, QString, QString ) ) );
+  connect( mStorage, SIGNAL( msgRemoved( int, QString ) ),
+           SIGNAL( msgRemoved( int, QString ) ) );
   connect( mStorage, SIGNAL( msgRemoved( KMFolder* ) ),
            SIGNAL( msgRemoved( KMFolder* ) ) );
   connect( mStorage, SIGNAL( msgAdded( int ) ), SIGNAL( msgAdded( int ) ) );
@@ -85,6 +129,9 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
   connect( mStorage, SIGNAL( removed( KMFolder*, bool ) ),
            SIGNAL( removed( KMFolder*, bool ) ) );
 
+  connect( mStorage, SIGNAL( contentsTypeChanged( KMail::FolderContentsType ) ),
+                this, SLOT( slotContentsTypeChanged( KMail::FolderContentsType ) ) );
+
   //FIXME: Centralize all the readConfig calls somehow - Zack
   mStorage->readConfig();
 
@@ -94,6 +141,8 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
 
 KMFolder::~KMFolder()
 {
+  delete mAcctList;
+  if ( mHasIndex ) mStorage->deregisterFromMessageDict();
   delete mStorage;
 }
 
@@ -128,6 +177,12 @@ void KMFolder::readConfig( KConfig* config )
 
   if ( mUseCustomIcons )
     emit iconsChanged();
+
+  QString shortcut( config->readEntry( "Shortcut" ) );
+  if ( !shortcut.isEmpty() ) {
+    KShortcut sc( shortcut );
+    setShortcut( sc );
+  }
 }
 
 void KMFolder::writeConfig( KConfig* config ) const
@@ -154,6 +209,10 @@ void KMFolder::writeConfig( KConfig* config ) const
   config->writeEntry("Id", mId);
   config->writeEntry( "PutRepliesInSameFolder", mPutRepliesInSameFolder );
   config->writeEntry( "IgnoreNewMail", mIgnoreNewMail );
+  if ( !mShortcut.isNull() )
+    config->writeEntry( "Shortcut", mShortcut.toString() );
+  else
+    config->deleteEntry( "Shortcut" );
 }
 
 KMFolderType KMFolder::folderType() const
@@ -323,6 +382,11 @@ int KMFolder::addMsgKeepUID( KMMessage* msg, int* index_return )
   return mStorage->addMsgKeepUID( msg, index_return );
 }
 
+int KMFolder::addMsg( QPtrList<KMMessage>& list, QValueList<int>& index_return )
+{
+  return mStorage->addMsg( list, index_return );
+}
+
 void KMFolder::emitMsgAddedSignals( int idx )
 {
   mStorage->emitMsgAddedSignals( idx );
@@ -432,11 +496,6 @@ void KMFolder::markUnreadAsRead()
   mStorage->markUnreadAsRead();
 }
 
-int KMFolder::create( bool imap )
-{
-  return mStorage->create( imap );
-}
-
 void KMFolder::remove()
 {
   mStorage->remove();
@@ -452,16 +511,6 @@ int KMFolder::rename( const QString& newName, KMFolderDir *aParent )
   return mStorage->rename( newName, aParent );
 }
 
-bool KMFolder::autoCreateIndex() const
-{
-  return mStorage->autoCreateIndex();
-}
-
-void KMFolder::setAutoCreateIndex( bool b )
-{
-  mStorage->setAutoCreateIndex( b );
-}
-
 bool KMFolder::dirty() const
 {
   return mStorage->dirty();
@@ -499,17 +548,20 @@ QString KMFolder::label() const
   if ( !mLabel.isEmpty() )
      return mLabel;
   if ( isSystemFolder() )
-     return i18n( name().latin1() );
+     return i18n( name().utf8() );
   return name();
 }
 
 //-----------------------------------------------------------------------------
 QString KMFolder::prettyURL() const
 {
+  QString parentUrl;
   if ( parent() )
-    return parent()->prettyURL() + "/" + label();
+    parentUrl = parent()->prettyURL();
+  if ( !parentUrl.isEmpty() )
+    return parentUrl + '/' + label();
   else
-    return "/" + label();
+    return label();
 }
 
 //--------------------------------------------------------------------------
@@ -528,16 +580,6 @@ QString KMFolder::mailingListPostAddress() const
   return QString::null;
 }
 
-const char* KMFolder::type() const
-{
-  return mStorage->type();
-}
-
-bool KMFolder::hasAccounts() const
-{
-  return mStorage->hasAccounts();
-}
-
 void KMFolder::setMailingListEnabled( bool enabled )
 {
   mMailingListEnabled = enabled;
@@ -567,7 +609,8 @@ void KMFolder::setWhoField(const QString& aWhoField )
 
 void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
 {
-  mUserWhoField = whoField;
+  if ( mUserWhoField == whoField )
+    return;
   if ( whoField.isEmpty() )
   {
     // default setting
@@ -592,13 +635,17 @@ void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
   } else if ( whoField == "From" || whoField == "To" )
     // set the whoField according to the user-setting
     mWhoField = whoField;
-  else
+  else {
     // this should not happen...
     kdDebug(5006) << "Illegal setting " << whoField << " for userWhoField!"
                   << endl;
+    return; // don't use the value
+  }
+  mUserWhoField = whoField;
 
   if (writeConfig)
     mStorage->writeConfig();
+  emit viewConfigChanged();
 }
 
 void KMFolder::correctUnreadMsgsCount()
@@ -730,36 +777,6 @@ int KMFolder::writeIndex( bool createEmptyIndex )
   return mStorage->writeIndex( createEmptyIndex );
 }
 
-void KMFolder::fillMsgDict( KMMsgDict* dict )
-{
-  mStorage->fillMsgDict( dict );
-}
-
-int KMFolder::writeMsgDict( KMMsgDict* dict)
-{
-  return mStorage->writeMsgDict( dict );
-}
-
-int KMFolder::touchMsgDict()
-{
-  return mStorage->touchMsgDict();
-}
-
-int KMFolder::appendtoMsgDict( int idx )
-{
-  return mStorage->appendtoMsgDict( idx );
-}
-
-void KMFolder::setRDict( KMMsgDictREntry* rentry )
-{
-  mStorage->setRDict( rentry );
-}
-
-KMMsgDictREntry* KMFolder::rDict() const
-{
-  return mStorage->rDict();
-}
-
 void KMFolder::setStatus( int idx, KMMsgStatus status, bool toggle )
 {
   mStorage->setStatus( idx, status, toggle );
@@ -785,11 +802,6 @@ void KMFolder::removeJobs()
   mStorage->removeJobs();
 }
 
-size_t KMFolder::crlf2lf( char* str, const size_t strLen )
-{
-  return FolderStorage::crlf2lf( str, strLen );
-}
-
 int KMFolder::updateIndex()
 {
   return mStorage->updateIndex();
@@ -805,4 +817,23 @@ void KMFolder::reallyAddCopyOfMsg( KMMessage* aMsg )
   mStorage->reallyAddCopyOfMsg( aMsg );
 }
 
+void KMFolder::setShortcut( const KShortcut &sc )
+{
+  if ( mShortcut != sc ) {
+    mShortcut = sc;
+    emit shortcutChanged( this );
+  }
+}
+
+bool KMFolder::isMoveable() const
+{
+  return mStorage->isMoveable();
+}
+
+void KMFolder::slotContentsTypeChanged( KMail::FolderContentsType type )
+{
+  kmkernel->iCalIface().folderContentsTypeChanged( this, type );
+  emit iconsChanged();
+}
+
 #include "kmfolder.moc"
