@@ -19,8 +19,9 @@
 #include "jobscheduler.h"
 using KMail::MaildirJob;
 #include "compactionjob.h"
+#include "kmmsgdict.h"
+#include "util.h"
 
-#include <kio/netaccess.h>
 #include <kapplication.h>
 #include <kdebug.h>
 #include <klocale.h>
@@ -50,13 +51,14 @@ using KMail::MaildirJob;
 KMFolderMaildir::KMFolderMaildir(KMFolder* folder, const char* name)
   : KMFolderIndex(folder, name)
 {
+
 }
 
 
 //-----------------------------------------------------------------------------
 KMFolderMaildir::~KMFolderMaildir()
 {
-  if (mOpenCount>0) close(TRUE);
+  if (mOpenCount>0) close(true);
   if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed( folder() );
 }
 
@@ -134,11 +136,11 @@ int KMFolderMaildir::open()
   }
   else
   {
-    mAutoCreateIndex = FALSE;
+    mAutoCreateIndex = false;
     rc = createIndexFromContents();
   }
 
-  mChanged = FALSE;
+  mChanged = false;
 
   //readConfig();
 
@@ -147,7 +149,40 @@ int KMFolderMaildir::open()
 
 
 //-----------------------------------------------------------------------------
-int KMFolderMaildir::create(bool imap)
+int KMFolderMaildir::createMaildirFolders( const QString & folderPath )
+{
+  // Make sure that neither a new, cur or tmp subfolder exists already.
+  QFileInfo dirinfo;
+  dirinfo.setFile( folderPath + "/new" );
+  if ( dirinfo.exists() ) return EEXIST;
+  dirinfo.setFile( folderPath + "/cur" );
+  if ( dirinfo.exists() ) return EEXIST;
+  dirinfo.setFile( folderPath + "/tmp" );
+  if ( dirinfo.exists() ) return EEXIST;
+
+  // create the maildir directory structure
+  if ( ::mkdir( QFile::encodeName( folderPath ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << endl;
+    return errno;
+  }
+  if ( ::mkdir( QFile::encodeName( folderPath + "/new" ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << "/new" << endl;
+    return errno;
+  }
+  if ( ::mkdir( QFile::encodeName( folderPath + "/cur" ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << "/cur" << endl;
+    return errno;
+  }
+  if ( ::mkdir( QFile::encodeName( folderPath + "/tmp" ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << "/tmp" << endl;
+    return errno;
+  }
+
+  return 0; // no error
+}
+
+//-----------------------------------------------------------------------------
+int KMFolderMaildir::create()
 {
   int rc;
   int old_umask;
@@ -155,42 +190,16 @@ int KMFolderMaildir::create(bool imap)
   assert(!folder()->name().isEmpty());
   assert(mOpenCount == 0);
 
-  // Make sure that neither a new, cur or tmp subfolder exists already.
-  QFileInfo dirinfo;
-  dirinfo.setFile(location() + "/new");
-  if (dirinfo.exists()) return 1;
-  dirinfo.setFile(location() + "/cur");
-  if (dirinfo.exists()) return 1;
-  dirinfo.setFile(location() + "/tmp");
-  if (dirinfo.exists()) return 1;
-
-  // create the maildir directory structure
-  if (::mkdir(QFile::encodeName(location()), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << " maildir" << endl;
-    return errno;
-  }
-  if (::mkdir(QFile::encodeName(location() + "/new"), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << "/new" << endl;
-    return errno;
-  }
-  if (::mkdir(QFile::encodeName(location() + "/cur"), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << "/cur" << endl;
-    return errno;
-  }
-  if (::mkdir(QFile::encodeName(location() + "/tmp"), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << "/new" << endl;
-    return errno;
-  }
+  rc = createMaildirFolders( location() );
+  if ( rc != 0 )
+    return rc;
 
+  // FIXME no path == no index? - till
   if (!folder()->path().isEmpty())
   {
     old_umask = umask(077);
     mIndexStream = fopen(QFile::encodeName(indexLocation()), "w+"); //sven; open RW
-    updateIndexStreamPtr(TRUE);
+    updateIndexStreamPtr(true);
     umask(old_umask);
 
     if (!mIndexStream) return errno;
@@ -198,15 +207,11 @@ int KMFolderMaildir::create(bool imap)
   }
   else
   {
-    mAutoCreateIndex = FALSE;
+    mAutoCreateIndex = false;
   }
 
   mOpenCount++;
-  mChanged = FALSE;
-  if (imap) {
-    readConfig();
-    mUnreadMsgs = -1;
-  }
+  mChanged = false;
 
   rc = writeIndex();
   return rc;
@@ -236,11 +241,11 @@ void KMFolderMaildir::close(bool aForced)
       writeConfig();
   }
 
-  mMsgList.clear(TRUE);
+  mMsgList.clear(true);
 
     if (mIndexStream) {
 	fclose(mIndexStream);
-	updateIndexStreamPtr(TRUE);
+	updateIndexStreamPtr(true);
     }
 
   mOpenCount   = 0;
@@ -301,7 +306,7 @@ int KMFolderMaildir::compact( unsigned int startIndex, int nbMessages, const QSt
 
     // construct a valid filename.  if it's already valid, then
     // nothing happens
-    constructValidFileName(filename, mi->status());
+    filename = constructValidFileName( filename, mi->status() );
 
     // if the name changed, then we need to update the actual filename
     if (filename != mi->fileName())
@@ -355,6 +360,14 @@ KMFolderMaildir::doCreateJob( QPtrList<KMMessage>& msgList, const QString& sets,
 
 //-------------------------------------------------------------
 int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
+{
+  if (!canAddMsgNow(aMsg, index_return)) return 0;
+  return addMsgInternal( aMsg, index_return );
+}
+
+//-------------------------------------------------------------
+int KMFolderMaildir::addMsgInternal( KMMessage* aMsg, int* index_return,
+                                     bool stripUid )
 {
 /*
 QFile fileD0( "testdat_xx-kmfoldermaildir-0" );
@@ -364,11 +377,9 @@ if( fileD0.open( IO_WriteOnly ) ) {
     fileD0.close();  // If data is 0 we just create a zero length file.
 }
 */
-  if (!canAddMsgNow(aMsg, index_return)) return 0;
-
   long len;
   unsigned long size;
-  bool opened = FALSE;
+  bool opened = false;
   KMFolder* msgParent;
   QCString msgText;
   int idx(-1);
@@ -388,9 +399,20 @@ if( fileD0.open( IO_WriteOnly ) ) {
   aMsg->setStatusFields();
   if (aMsg->headerField("Content-Type").isEmpty())  // This might be added by
     aMsg->removeHeaderField("Content-Type");        // the line above
+
+
+  const QString uidHeader = aMsg->headerField( "X-UID" );
+  if ( !uidHeader.isEmpty() && stripUid )
+    aMsg->removeHeaderField( "X-UID" );
+
   msgText = aMsg->asString();
   len = msgText.length();
 
+  // Re-add the uid so that the take can make use of it, in case the
+  // message is currently in an imap folder
+  if ( !uidHeader.isEmpty() && stripUid )
+    aMsg->setHeaderField( "X-UID", uidHeader );
+
   if (len <= 0)
   {
     kdDebug(5006) << "Message added to folder `" << name() << "' contains no data. Ignoring it." << endl;
@@ -398,21 +420,20 @@ if( fileD0.open( IO_WriteOnly ) ) {
   }
 
   // make sure the filename has the correct extension
-  QString filename(aMsg->fileName());
-  constructValidFileName(filename, aMsg->status());
+  QString filename = constructValidFileName( aMsg->fileName(), aMsg->status() );
 
   QString tmp_file(location() + "/tmp/");
   tmp_file += filename;
 
   if (!KPIM::kCStringToFile(msgText, tmp_file, false, false, false))
-    kmkernel->emergencyExit( "" ); // KPIM::kCStringToFile already showed an errormessage
+    kmkernel->emergencyExit( i18n("Message could not be added to the folder, possibly disk space is low.") ); 
 
   QFile file(tmp_file);
   size = msgText.length();
 
   if (!isOpened())
   {
-    opened = TRUE;
+    opened = true;
     rc = open();
     kdDebug(5006) << "KMFolderMaildir::addMsg-open: " << rc << " of folder: " << label() << endl;
     if (rc) return rc;
@@ -431,6 +452,9 @@ if( fileD0.open( IO_WriteOnly ) ) {
   if (msgParent)
     if (idx >= 0) msgParent->take(idx);
 
+  // just to be sure it does not end up in the index
+  if ( stripUid ) aMsg->setUID( 0 );
+
   if (filename != aMsg->fileName())
     aMsg->setFileName(filename);
 
@@ -440,16 +464,31 @@ if( fileD0.open( IO_WriteOnly ) ) {
       mUnreadMsgs = 1;
     else
       ++mUnreadMsgs;
-    emit numUnreadMsgsChanged( folder() );
+    if ( !mQuiet ) {
+      kdDebug( 5006 ) << "FolderStorage::msgStatusChanged" << endl;
+      emit numUnreadMsgsChanged( folder() );
+    }else{
+      if ( !mEmitChangedTimer->isActive() ) {
+//        kdDebug( 5006 )<< "QuietTimer started" << endl;
+        mEmitChangedTimer->start( 3000 );
+      }
+      mChanged = true;
+    }
   }
   ++mTotalMsgs;
 
+  if ( aMsg->attachmentState() == KMMsgAttachmentUnknown && 
+       aMsg->readyToShow() )
+    aMsg->updateAttachmentState();
+  
   // store information about the position in the folder file in the message
   aMsg->setParent(folder());
   aMsg->setMsgSize(size);
-  idx = mMsgList.append(&aMsg->toMsgBase());
+  idx = mMsgList.append( &aMsg->toMsgBase(), mExportsSernums );
   if (aMsg->getMsgSerNum() <= 0)
     aMsg->setMsgSerNum();
+  else
+    replaceMsgSerNum( aMsg->getMsgSerNum(), &aMsg->toMsgBase(), idx );
 
   // write index entry if desired
   if (mAutoCreateIndex)
@@ -471,7 +510,8 @@ if( fileD0.open( IO_WriteOnly ) ) {
     fflush(mIndexStream);
     int error = ferror(mIndexStream);
 
-    error |= appendtoMsgDict(idx);
+    if ( mExportsSernums )
+      error |= appendToFolderIdsFile( idx );
 
     if (error) {
       kdDebug(5006) << "Error: Could not add message to folder (No space left on device?)" << endl;
@@ -520,6 +560,7 @@ KMMessage* KMFolderMaildir::readMsg(int idx)
   KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
   KMMessage *msg = new KMMessage(*mi); // note that mi is deleted by the line below
   mMsgList.set(idx,&msg->toMsgBase()); // done now so that the serial number can be computed
+  msg->setComplete( true );
   msg->fromDwString(getDwString(idx));
   return msg;
 }
@@ -540,7 +581,7 @@ DwString KMFolderMaildir::getDwString(int idx)
       fread(msgText, msgSize, 1, stream);
       fclose( stream );
       msgText[msgSize] = '\0';
-      size_t newMsgSize = crlf2lf( msgText, msgSize );
+      size_t newMsgSize = KMail::Util::crlf2lf( msgText, msgSize );
       DwString str;
       // the DwString takes possession of msgText, so we must not delete it
       str.TakeBuffer( msgText, msgSize + 1, 0, newMsgSize );
@@ -570,7 +611,7 @@ QCString& KMFolderMaildir::getMsgString(int idx, QCString& mDest)
   QFileInfo fi( abs_file );
   mDest.resize(fi.size()+2);
   mDest = KPIM::kFileToString(abs_file, false, false);
-  size_t newMsgSize = crlf2lf( mDest.data(), fi.size() );
+  size_t newMsgSize = KMail::Util::crlf2lf( mDest.data(), fi.size() );
   mDest[newMsgSize] = '\0';
   return mDest;
 }
@@ -699,7 +740,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
       if ( !uidStr.isEmpty() )
          mi->setUID( uidStr.toULong() );
       mi->setDirty(false);
-      mMsgList.append(mi);
+      mMsgList.append( mi, mExportsSernums );
 
       // if this is a New file and is in 'new', we move it to 'cur'
       if (status & KMMsgStatusNew)
@@ -845,8 +886,7 @@ int KMFolderMaildir::createIndexFromContents()
     ++it;
   }
 
-  if (autoCreateIndex())
-  {
+  if ( autoCreateIndex() ) {
     emit statusMsg(i18n("Writing index file"));
     writeIndex();
   }
@@ -861,8 +901,7 @@ int KMFolderMaildir::createIndexFromContents()
 
   needsCompact = true;
 
-  if (folder()->parent())
-    folder()->parent()->manager()->invalidateFolder(kmkernel->msgDict(), folder());
+  invalidateFolder();
   return 0;
 }
 
@@ -901,67 +940,102 @@ KMMessage* KMFolderMaildir::take(int idx)
   // first, we do the high-level stuff.. then delete later
   KMMessage *msg = KMFolderIndex::take(idx);
 
-  if (!msg || !msg->fileName()) return 0;
-
-  if (removeFile(msg->fileName()))
-    return msg;
-  else
+  if (!msg || !msg->fileName()) {
     return 0;
+  }
+
+  if ( removeFile(msg->fileName()) ) {
+    return msg;
+  } else {
+    return 0;
+  }
 }
 
-bool KMFolderMaildir::removeFile(const QString& filename)
+// static
+bool KMFolderMaildir::removeFile( const QString & folderPath,
+                                  const QString & filename )
 {
   // we need to look in both 'new' and 'cur' since it's possible to
-  // delete a message before the folder is compacted.  since the file
+  // delete a message before the folder is compacted. Since the file
   // naming and moving is done in ::compact, we can't assume any
-  // location at this point
-  QCString abs_file(QFile::encodeName(location() + "/cur/"));
-  abs_file += QFile::encodeName(filename);
+  // location at this point.
+  QCString abs_file( QFile::encodeName( folderPath + "/cur/" + filename ) );
+  if ( ::unlink( abs_file ) == 0 )
+    return true;
 
-  if (::unlink( abs_file ) == 0)
+  if ( errno == ENOENT ) { // doesn't exist
+    abs_file = QFile::encodeName( folderPath + "/new/" + filename );
+    if ( ::unlink( abs_file ) == 0 )
       return true;
-
-  if (errno == ENOENT)  {// doesn't exist
-
-    abs_file = QFile::encodeName(location() + "/new/");
-    abs_file += QFile::encodeName(filename);
-
-    if (::unlink( abs_file ) == 0)
-        return true;
-
   }
 
   kdDebug(5006) << "Can't delete " << abs_file << " " << perror << endl;
   return false;
 }
 
+bool KMFolderMaildir::removeFile( const QString & filename )
+{
+  return removeFile( location(), filename );
+}
+
+#include <sys/types.h>
+#include <dirent.h>
+static bool removeDirAndContentsRecursively( const QString & path )
+{
+  bool success = true;
+
+  QDir d;
+  d.setPath( path );
+  d.setFilter( QDir::Files | QDir::Dirs | QDir::Hidden | QDir::NoSymLinks );
+
+  const QFileInfoList *list = d.entryInfoList();
+  QFileInfoListIterator it( *list );
+  QFileInfo *fi;
+
+  while ( (fi = it.current()) != 0 ) {
+    if( fi->isDir() ) {
+      if ( fi->fileName() != "." && fi->fileName() != ".." )
+        success = success && removeDirAndContentsRecursively( fi->absFilePath() );
+    } else {
+      success = success && d.remove( fi->absFilePath() );
+    }
+    ++it;
+  }
+
+  if ( success ) {
+    success = success && d.rmdir( path ); // nuke ourselves, we should be empty now
+  }
+  return success;
+}
+
 //-----------------------------------------------------------------------------
 int KMFolderMaildir::removeContents()
 {
-    if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()+ "/new/"), 0))
-        return 1;
-    if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()+ "/cur/"), 0))
-        return 1;
-    if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()+ "/tmp/"), 0))
-        return 1;
-
-    /* The subdirs are removed now. Check if there is anything else in the dir
-     * and only if not delete the dir itself. The user could have data stored
-     * that would otherwise be deleted. */
-    QDir dir(location());
-    if ( dir.count() == 2 ) { // only . and ..
-        if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()), 0))
-            return 1;
-    }
-    return 0;
+  // NOTE: Don' use KIO::netaccess, it has reentrancy problems and multiple
+  // mailchecks going on trigger them, when removing dirs
+  if ( !removeDirAndContentsRecursively( location() + "/new/" ) ) return 1;
+  if ( !removeDirAndContentsRecursively( location() + "/cur/" ) ) return 1;
+  if ( !removeDirAndContentsRecursively( location() + "/tmp/" ) ) return 1;
+  /* The subdirs are removed now. Check if there is anything else in the dir
+   * and only if not delete the dir itself. The user could have data stored
+   * that would otherwise be deleted. */
+  QDir dir(location());
+  if ( dir.count() == 2 ) { // only . and ..
+    if ( !removeDirAndContentsRecursively( location() ), 0 ) return 1;
+  }
+  return 0;
 }
 
 static QRegExp *suffix_regex = 0;
 static KStaticDeleter<QRegExp> suffix_regex_sd;
 
 //-----------------------------------------------------------------------------
-QString KMFolderMaildir::constructValidFileName(QString& aFileName, KMMsgStatus status)
+// static
+QString KMFolderMaildir::constructValidFileName( const QString & filename,
+                                                 KMMsgStatus status )
 {
+  QString aFileName( filename );
+
   if (aFileName.isEmpty())
   {
     aFileName.sprintf("%ld.%d.", (long)time(0), getpid());
@@ -973,18 +1047,17 @@ QString KMFolderMaildir::constructValidFileName(QString& aFileName, KMMsgStatus
 
   aFileName.truncate(aFileName.findRev(*suffix_regex));
 
-  QString suffix;
+  // only add status suffix if the message is neither new nor unread
   if (! ((status & KMMsgStatusNew) || (status & KMMsgStatusUnread)) )
   {
-    suffix += ":2,";
+    QString suffix( ":2," );
     if (status & KMMsgStatusReplied)
       suffix += "RS";
     else
       suffix += "S";
+    aFileName += suffix;
   }
 
-  aFileName += suffix;
-
   return aFileName;
 }
 
@@ -1007,8 +1080,7 @@ QString KMFolderMaildir::moveInternal(const QString& oldLoc, const QString& newL
   // make sure that our destination filename doesn't already exist
   while (QFile::exists(dest))
   {
-    aFileName = "";
-    constructValidFileName(aFileName, status);
+    aFileName = constructValidFileName( QString(), status );
 
     QFileInfo fi(dest);
     dest = fi.dirPath(true) + "/" + aFileName;
