@@ -5,6 +5,8 @@
 #ifndef kmmessage_h
 #define kmmessage_h
 
+/** @file This file defines Mime Message classes. */
+
 // for large file support
 #include <config.h>
 #include <sys/types.h>
@@ -15,6 +17,8 @@
 
 #include <kmime_mdn.h>
 
+#include<libemailfunctions/email.h>
+
 template <typename T>
 class QValueList;
 
@@ -29,6 +33,7 @@ class DwMessage;
 class KMMessagePart;
 class KMMsgInfo;
 class KMHeaders;
+class KMForwardCommand;
 
 namespace KMime {
   class CharFreq;
@@ -58,9 +63,10 @@ namespace KMail {
                        ReplyNone };
 }
 
+/** This is a Mime Message. */
 class KMMessage: public KMMsgBase, public KMail::ISubject
 {
-  friend class KMForwardCommand;    // needed for MIME Digest forward
+  friend class ::KMForwardCommand;    // needed for MIME Digest forward
 
 public:
   // promote some of KMMsgBase's methods to public:
@@ -80,7 +86,9 @@ public:
   /** Straight forward initialization. */
   KMMessage(KMFolder* parent=0);
 
-  /** Constructor from a DwMessage. */
+  /** Constructor from a DwMessage. KMMessage takes possession of the
+      DwMessage, so don't dare to delete it.
+  */
   KMMessage(DwMessage*);
 
   /** Copy constructor. Does *not* automatically load the message. */
@@ -88,27 +96,16 @@ public:
 
   /** Copy constructor. */
   KMMessage( const KMMessage& other );
-    //KMMessage( const KMMessage& other,
-    //           bool preserveArrivalTime=false  );
-    //  note: By setting preserveArrivalTime true you get
-    //        a message containing the arrival time of the
-    //        old one - this is useful if this new message
-    //        is to replace the old one in the same folder
-    // note2: temporarily uncommented this again (khz)
 
+#if 0 // currently unused
   /** Assignment operator. */
   const KMMessage& operator=( const KMMessage& other ) {
-    //const KMMessage& operator=( const KMMessage& other,
-    //                          bool preserveArrivalTime=false ) {
-    //  note: By setting preserveArrivalTime true you get
-    //        a message containing the arrival time of the
-    //        old one - this is useful if this new message
-    //        is to replace the old one in the same folder
-    // note2: temporarily uncommented this again (khz)
     if( &other == this )
       return *this;
-    assign( other ); return *this;
+    assign( other );
+    return *this;
   }
+#endif
 
   /** Destructor. */
   virtual ~KMMessage();
@@ -174,15 +171,7 @@ public:
     user, mail is not changed and the reply-to field is set to
     the email address of the original sender
    */
-  KMMessage* createRedirect();
-  // new implementation, ATM only used for redirect filter action
-  KMMessage* createRedirect2( const QString &toStr ); 
-
-  /** Create a new message that is a "failed delivery" reply to this
-    message, filling all required header fields with the proper
-    values. The returned message is not stored in any folder. If @p
-    withUI is true, asks the user if he really wants that. */
-  KMMessage* createBounce( bool withUI );
+  KMMessage* createRedirect( const QString &toStr );
 
   /** Create the forwarded body for the message. */
   QCString createForwardBody();
@@ -203,17 +192,17 @@ public:
 
       @param a Use AutomaticAction for filtering and ManualAction for
                user-induced events.
-      @param d See docs for @ref KMime::MDN::DispositionType
-      @param m See docs for @ref KMime::MDN::DispositionModifier
+      @param d See docs for KMime::MDN::DispositionType
+      @param m See docs for KMime::MDN::DispositionModifier
       @param allowGUI Set to true if this method is allowed to ask the
                       user questions
 
       @return The notification message or 0, if none should be sent.
    **/
   KMMessage* createMDN( KMime::MDN::ActionMode a,
-			KMime::MDN::DispositionType d,
-			bool allowGUI=false,
-			QValueList<KMime::MDN::DispositionModifier> m=QValueList<KMime::MDN::DispositionModifier>() );
+          KMime::MDN::DispositionType d,
+          bool allowGUI=false,
+          QValueList<KMime::MDN::DispositionModifier> m=QValueList<KMime::MDN::DispositionModifier>() );
 
   /** Parse the string and create this message from it. */
   void fromDwString(const DwString& str, bool setStatus=FALSE);
@@ -231,9 +220,7 @@ public:
       slow for large message since it involves a string copy. If you
       need the string representation only for a short time
       (i.e. without the chance of calling any function in the
-      underlying mimelib, then you should use the @ref asByteArray,
-      which is more efficient or use the @ref asDwString function.
-      @see asByteArray
+      underlying mimelib, then you should use the asDwString function.
       @see asDwString
   */
   QCString asString() const;
@@ -281,11 +268,15 @@ public:
 
   /** @return the UOID of the identity for this message.
       Searches the "x-kmail-identity" header and if that fails,
-      searches with @ref KPIM::IdentityManager::identityForAddress()
-      and if that fails queries the @ref #parent() folder for a default.
+      searches with KPIM::IdentityManager::identityForAddress()
+      and if that fails queries the KMMsgBase::parent() folder for a default.
    **/
   uint identityUoid() const;
 
+  /** Set the from, to, cc, bcc, encrytion etc headers as specified in the
+   * given identity. */
+  void applyIdentity( uint id );
+
   /** Removes empty fields from the header, e.g. an empty Cc: or Bcc:
     field. */
   void cleanupHeader();
@@ -341,7 +332,6 @@ public:
   QString from() const;
   void setFrom(const QString& aStr);
   QString fromStrip() const;
-  QCString fromEmail() const;
 
   /** @return The addr-spec of either the Sender: (if one is given) or
    * the first addr-spec in From: */
@@ -390,6 +380,19 @@ public:
   */
   QString strippedSubjectMD5() const;
 
+  /**
+    Validate a list of email addresses, and also allow
+    aliases and distribution lists to be expanded
+    before validation.
+    @return Enum to describe the error.
+    @return brokenAddress the address that was faulty.
+    FIXME: this should be in libemailfucntions but that
+           requires moving expandAliases and all that
+           it brings
+  */
+  static KPIM::EmailParseResult isValidEmailAddressList( const QString& aStr,
+                                                         QString& brokenAddress );
+
   /**
     Get a hash of the subject.
     Used for threading.
@@ -411,8 +414,10 @@ public:
   /** Returns the message ID, useful for followups */
   QCString id() const;
 
-  /** Sets the message serial number.  If defaulted to zero, the
-    serial number will be assigned using the dictionary. */
+  /** Sets the message serial number. If defaulted to zero, the
+    serial number will be assigned using the dictionary. Note that
+    unless it is explicitely set the serial number will remain 0
+    as long as the mail is not in a folder. */
   void setMsgSerNum(unsigned long newMsgSerNum = 0);
 
   /** Returns the value of a header field with the given name. If multiple
@@ -428,7 +433,7 @@ public:
       and does not overwrite an existing header field with the same name.
   */
   void setHeaderField( const QCString& name, const QString& value,
-                       HeaderFieldType type = Unstructured, 
+                       HeaderFieldType type = Unstructured,
                        bool prepend = false );
 
   /** Returns a list of the values of all header fields with the given name. */
@@ -586,7 +591,7 @@ public:
       If withBody is false the body of the KMMessagePart will be left
       empty and only the headers of the part will be filled in*/
   static void bodyPart(DwBodyPart* aDwBodyPart, KMMessagePart* aPart,
-		       bool withBody = true );
+          bool withBody = true );
 
   /** Get the body part at position in aIdx.  Indexing starts at 0.
       If there is no body part at that index, aPart will have its
@@ -622,39 +627,6 @@ public:
    */
   static QCString html2source( const QCString & src );
 
-  /** Convert LF line-ends to CRLF
-   */
-  static QCString lf2crlf( const QCString & src );
-
-  /** Returns a normalized address built from the given parts. The normalized
-      address is of one the following forms:
-      - displayName (comment) <addrSpec>
-      - displayName <addrSpec>
-      - comment <addrSpec>
-      - addrSpec
-   */
-  static QString normalizedAddress( const QString & displayName,
-                                    const QString & addrSpec,
-                                    const QString & comment );
-
-  /** Decodes the punycode domain part of the given addr-spec if it's an IDN.
-   */
-  static QString decodeIDN( const QString & addrSpec );
-
-  /** Encodes the domain part of the given addr-spec in punycode if it's an
-      IDN.
-   */
-  static QString encodeIDN( const QString & addrSpec );
-
-  /** Normalizes all email addresses in the given list and decodes all IDNs.
-   */
-  static QString normalizeAddressesAndDecodeIDNs( const QString & str );
-
-  /** Normalizes all email addresses in the given list and encodes all IDNs
-      in punycode.
-   */
-  static QString normalizeAddressesAndEncodeIDNs( const QString & str );
-
   /** Encodes an email address as mailto URL
    */
   static QString encodeMailtoUrl( const QString& str );
@@ -663,21 +635,6 @@ public:
     */
   static QString decodeMailtoUrl( const QString& url );
 
-  enum AddressParseResult { AddressOk, AddressEmpty, NoAddressSpec,
-                            UnbalancedQuote, UnbalancedParens,
-                            UnclosedAngleAddr, UnexpectedComma,
-                            UnexpectedEnd };
-  /** Splits the given address into display name, email address and comment.
-      Returns AddressOk if no error was encountered. Otherwise an appropriate
-      error code is returned. In case of an error the values of displayName,
-      addrSpec and comment are undefined.
-
-   */
-  static AddressParseResult splitAddress( const QCString& address,
-                                          QCString & displayName,
-                                          QCString & addrSpec,
-                                          QCString & comment );
-
   /** This function generates a displayable string from a list of email
       addresses.
       Input : mailbox-list
@@ -691,9 +648,9 @@ public:
   static QString stripEmailAddr(const QString& emailAddr);
 
   /** Quotes the following characters which have a special meaning in HTML:
-   * '<'  '>'  '&'  '"'. Additionally '\n' is converted to "<br />" if
+   * '<'  '>'  '&'  '"'. Additionally '\\n' is converted to "<br />" if
    * @p removeLineBreaks is false. If @p removeLineBreaks is true, then
-   * '\n' is removed. Last but not least '\r' is removed.
+   * '\\n' is removed. Last but not least '\\r' is removed.
    */
   static QString quoteHtmlChars( const QString& str,
                                  bool removeLineBreaks = false );
@@ -703,7 +660,7 @@ public:
    * only the name part and not the given emailAddr.
    */
   static QString emailAddrAsAnchor(const QString& emailAddr,
-					 bool stripped=TRUE);
+          bool stripped=TRUE);
 
   /** Strips an address from an address list. This is for example used
       when replying to all.
@@ -731,6 +688,13 @@ public:
   */
   static QString guessEmailAddressFromLoginName( const QString& userName );
 
+  /**
+   *  Given argument msg add quoting characters and relayout for max width maxLength
+   *  @param msg the string which it to be quoted
+   *  @param maxLineLength reformat text to be this amount of columns at maximum, adding
+   *    linefeeds at word boundaries to make it fit.
+   */
+  static QString smartQuote( const QString &msg, int maxLineLength );
 
   /** Get the default message charset.*/
   static QCString defaultCharset();
@@ -738,8 +702,7 @@ public:
   /** Get a list of preferred message charsets.*/
   static const QStringList &preferredCharsets();
 
-  /** Replaces every occurrence of "${foo}" in @p s with @ref
-      headerField("foo") */
+  /** Replaces every occurrence of "${foo}" in @p s with headerField("foo") */
   QString replaceHeadersInString( const QString & s ) const;
 
   /** Get the message charset.*/
@@ -748,7 +711,7 @@ public:
   /** Set the message charset. */
   void setCharset(const QCString& aStr);
 
-  /** Get a @ref QTextCodec suitable for this message part */
+  /** Get a QTextCodec suitable for this message part */
   const QTextCodec * codec() const;
 
   /** Set the charset the user selected for the message to display */
@@ -832,46 +795,66 @@ public:
   /** Returns the last DwBodyPart that was updated */
   DwBodyPart* lastUpdatedPart() { return mLastUpdated; }
 
+  /** Return true if the complete message is available without referring to the backing store.*/
+  bool isComplete() const { return mComplete; }
+  /** Set if the message is a complete message */
+  void setComplete( bool v ) { mComplete = v; }
+
+  /** Return if the message is ready to be shown */
+  bool readyToShow() const { return mReadyToShow; }
+  /** Set if the message is ready to be shown */
+  void setReadyToShow( bool v ) { mReadyToShow = v; }
+
   void updateAttachmentState(DwBodyPart * part = 0);
 
+  /** Return, if the message should not be deleted */
+  bool transferInProgress() const;
+  /** Set that the message shall not be deleted because it is still required */
+  void setTransferInProgress(bool value, bool force = false);
+  
   /** Returns an mbox message separator line for this message, i.e. a
       string of the form
       "From local@domain.invalid Sat Jun 12 14:00:00 2004\n".
   */
   QCString mboxMessageSeparator();
 
-private:
   /** Returns message body with quoting header and indented by the
     given indentation string. This is suitable for including the message
     in another message of for replies, forwards. The header string is
     a template where the following fields are replaced with the
     corresponding values:
     <pre>
-	%D: date of this message
-	%S: subject of this message
-	%F: sender (from) of this message
-	%%: a single percent sign
+        %D: date of this message
+        %S: subject of this message
+        %F: sender (from) of this message
+        %%: a single percent sign
     </pre>
     No attachments are handled if includeAttach is false.
     The signature is stripped if aStripSignature is true and
     smart quoting is turned on. Signed or encrypted texts
     get converted to plain text when allowDecryption is true. */
   QString asQuotedString( const QString & headerStr,
-			  const QString & indentStr,
-			  const QString & selection=QString::null,
-			  bool aStripSignature=true,
-			  bool allowDecryption=true) const;
+          const QString & indentStr,
+          const QString & selection=QString::null,
+          bool aStripSignature=true,
+          bool allowDecryption=true) const;
 
   /** Return the textual content of the message as plain text,
       converting HTML to plain text if necessary. */
   QString asPlainText( bool stripSignature, bool allowDecryption ) const;
+private:
 
+  /** Initialization shared by the ctors. */
+  void init();
+  /** Assign the values of @param other to this message. Used in the copy c'tor. */
   void assign( const KMMessage& other );
 
   QString mDrafts;
   mutable DwMessage* mMsg;
-  mutable bool       mNeedsAssembly;
-  bool mDecodeHTML;
+  mutable bool       mNeedsAssembly :1;
+  bool mDecodeHTML :1;
+  bool mReadyToShow :1;
+  bool mComplete :1;
   static const KMail::HeaderStrategy * sHeaderStrategy;
   static QString sForwardStr;
   const QTextCodec * mOverrideCodec;
