@@ -14,7 +14,6 @@
 #include "kmversion.h"
 #include "kmmainwidget.h"
 #include "kmreadermainwin.h"
-#include "kmailicalifaceimpl.h"
 #include <libkdepim/kfileio.h>
 #include "kmfolderindex.h"
 #include "kmcommands.h"
@@ -23,9 +22,8 @@
 using KMail::MailSourceViewer;
 #include "partNode.h"
 #include "kmmsgdict.h"
-#include "kmsender.h"
+#include "messagesender.h"
 #include "kcursorsaver.h"
-#include "kmkernel.h"
 #include "kmfolder.h"
 #include "vcardviewer.h"
 using KMail::VCardViewer;
@@ -53,6 +51,7 @@ using KMail::ISubject;
 #include "urlhandlermanager.h"
 using KMail::URLHandlerManager;
 #include "interfaces/observable.h"
+#include "util.h"
 
 #include "broadcaststatus.h"
 
@@ -71,6 +70,10 @@ using KMail::TeeHtmlWriter;
 #include <mimelib/body.h>
 #include <mimelib/utility.h>
 
+#include <kleo/specialjob.h>
+#include <kleo/cryptobackend.h>
+#include <kleo/cryptobackendfactory.h>
+
 // KABC includes
 #include <kabc/addressee.h>
 #include <kabc/vcardconverter.h>
@@ -83,6 +86,7 @@ using KMail::TeeHtmlWriter;
 #include <dom/html_document.h>
 #include <dom/dom_string.h>
 
+
 #include <kapplication.h>
 // for the click on attachment stuff (dnaber):
 #include <kuserprofile.h>
@@ -189,8 +193,6 @@ QByteArray& NewByteArray::qByteArray()
     return *((QByteArray*)this);
 }
 
-
-
 // This function returns the complete data that were in this
 // message parts - *after* all encryption has been removed that
 // could be removed.
@@ -487,7 +489,7 @@ KMReaderWin::KMReaderWin(QWidget *aParent,
     mAttachmentStrategy( 0 ),
     mHeaderStrategy( 0 ),
     mHeaderStyle( 0 ),
-    mOverrideCodec( 0 ),
+    mOldGlobalOverrideEncoding( "---" ), // init with dummy value
     mCSSHelper( 0 ),
     mRootNode( 0 ),
     mMainWindow( mainWindow ),
@@ -504,8 +506,10 @@ KMReaderWin::KMReaderWin(QWidget *aParent,
     mAddBookmarksAction( 0 ),
     mStartIMChatAction( 0 ),
     mSelectAllAction( 0 ),
+    mSelectEncodingAction( 0 ),
     mToggleFixFontAction( 0 ),
-    mHtmlWriter( 0 )
+    mHtmlWriter( 0 ),
+    mSavedRelativePosition( 0 )
 {
   mSplitterSizes << 180 << 100;
   mMimeTreeMode = 1;
@@ -528,12 +532,15 @@ KMReaderWin::KMReaderWin(QWidget *aParent,
   mHtmlOverride = false;
   mHtmlLoadExtOverride = false;
 
+  mLevelQuote = GlobalSettings::self()->collapseQuoteLevelSpin() - 1;
+
   connect( &updateReaderWinTimer, SIGNAL(timeout()),
   	   this, SLOT(updateReaderWin()) );
   connect( &mResizeTimer, SIGNAL(timeout()),
   	   this, SLOT(slotDelayedResize()) );
   connect( &mDelayedMarkTimer, SIGNAL(timeout()),
            this, SLOT(slotTouchMessage()) );
+
 }
 
 void KMReaderWin::createActions( KActionCollection * ac ) {
@@ -620,6 +627,15 @@ void KMReaderWin::createActions( KActionCollection * ac ) {
   raction->setExclusiveGroup( "view_attachments_group" );
   attachmentMenu->insert( raction );
 
+  // Set Encoding submenu
+  mSelectEncodingAction = new KSelectAction( i18n( "&Set Encoding" ), "charset", 0,
+                                 this, SLOT( slotSetEncoding() ),
+                                 ac, "encoding" );
+  QStringList encodings = KMMsgBase::supportedEncodings( false );
+  encodings.prepend( i18n( "Auto" ) );
+  mSelectEncodingAction->setItems( encodings );
+  mSelectEncodingAction->setCurrentItem( 0 );
+
   mMailToComposeAction = new KAction( i18n("New Message To..."), 0, this,
                                       SLOT(slotMailtoCompose()), ac,
                                       "mailto_compose" );
@@ -636,7 +652,7 @@ void KMReaderWin::createActions( KActionCollection * ac ) {
                                      0, this, SLOT(slotMailtoOpenAddrBook()),
                                      ac, "openin_addr_book" );
   mCopyAction = KStdAction::copy( this, SLOT(slotCopySelectedText()), ac, "kmail_copy");
-  mSelectAllAction = new KAction( i18n("Select All"), CTRL+SHIFT+Key_A, this,
+  mSelectAllAction = new KAction( i18n("Select All Text"), CTRL+SHIFT+Key_A, this,
                                   SLOT(selectAll()), ac, "mark_all_text" );
   mCopyURLAction = new KAction( i18n("Copy Link Address"), 0, this,
 				SLOT(slotUrlCopy()), ac, "copy_url" );
@@ -724,6 +740,16 @@ void KMReaderWin::slotAllHeaders() {
                              HeaderStrategy::all() );
 }
 
+void KMReaderWin::slotLevelQuote( int l )
+{
+  kdDebug( 5006 ) << "Old Level: " << mLevelQuote << " New Level: " << l << endl;
+	mLevelQuote = l;
+  QScrollView * scrollview = static_cast<QScrollView *>(mViewer->widget());
+  mSavedRelativePosition = (float)scrollview->contentsY() / scrollview->contentsHeight();
+
+  update(true);
+}
+
 void KMReaderWin::slotCycleHeaderStyles() {
   const HeaderStrategy * strategy = headerStrategy();
   const HeaderStyle * style = headerStyle();
@@ -781,6 +807,7 @@ void KMReaderWin::slotCycleAttachmentStrategy() {
 KMReaderWin::~KMReaderWin()
 {
   delete mHtmlWriter; mHtmlWriter = 0;
+  delete mCSSHelper;
   if (mAutoDelete) delete message();
   delete mRootNode; mRootNode = 0;
   removeTempFiles();
@@ -829,7 +856,7 @@ void KMReaderWin::update( KMail::Interface::Observable * observable )
   QByteArray data = node->msgPart().bodyDecodedBinary();
   size_t size = data.size();
   if ( node->msgPart().type() == DwMime::kTypeText && size) {
-    size = KMFolder::crlf2lf( data.data(), size );
+    size = KMail::Util::crlf2lf( data.data(), size );
   }
   KPIM::kBytesToFile( data.data(), size, mAtmCurrentName, false, false, false );
   ::chmod( QFile::encodeName( mAtmCurrentName ), S_IRUSR );
@@ -859,7 +886,7 @@ bool KMReaderWin::event(QEvent *e)
   if (e->type() == QEvent::ApplicationPaletteChange)
   {
     delete mCSSHelper;
-    mCSSHelper = new CSSHelper( QPaintDeviceMetrics( mViewer->view() ), this );
+    mCSSHelper = new KMail::CSSHelper( 	QPaintDeviceMetrics( mViewer->view() ) );
     if (message())
       message()->readConfig();
     update( true ); // Force update
@@ -876,7 +903,7 @@ void KMReaderWin::readConfig(void)
   /*should be: const*/ KConfigGroup reader( KMKernel::config(), "Reader" );
 
   delete mCSSHelper;
-  mCSSHelper = new CSSHelper( QPaintDeviceMetrics( mViewer->view() ), this );
+  mCSSHelper = new KMail::CSSHelper( QPaintDeviceMetrics( mViewer->view() ) );
 
   mNoMDNsWhenEncrypted = mdnGroup.readBoolEntry( "not-send-when-encrypted", true );
 
@@ -925,6 +952,8 @@ void KMReaderWin::readConfig(void)
 
   adjustLayout();
 
+  readGlobalOverrideCodec();
+
   if (message())
     update();
   KMMessage::readConfig();
@@ -990,6 +1019,8 @@ void KMReaderWin::initHtmlWidget(void)
   mViewer->setURLCursor(KCursor::handCursor());
   // Espen 2000-05-14: Getting rid of thick ugly frames
   mViewer->view()->setLineWidth(0);
+  // register our own event filter for shift-click
+  mViewer->view()->viewport()->installEventFilter( this );
 
   if ( !htmlWriter() )
 #ifdef KMAIL_READER_HTML_DEBUG
@@ -1046,13 +1077,62 @@ void KMReaderWin::setHeaderStyleAndStrategy( const HeaderStyle * style,
   update( true );
 }
 
-void KMReaderWin::setOverrideCodec( const QTextCodec * codec ) {
-  if ( mOverrideCodec == codec )
+//-----------------------------------------------------------------------------
+void KMReaderWin::setOverrideEncoding( const QString & encoding )
+{
+  if ( encoding == mOverrideEncoding )
     return;
-  mOverrideCodec = codec;
+
+  mOverrideEncoding = encoding;
+  if ( mSelectEncodingAction ) {
+    if ( encoding.isEmpty() ) {
+      mSelectEncodingAction->setCurrentItem( 0 );
+    }
+    else {
+      QStringList encodings = mSelectEncodingAction->items();
+      int i = 0;
+      for ( QStringList::const_iterator it = encodings.begin(), end = encodings.end(); it != end; ++it, ++i ) {
+        if ( KGlobal::charsets()->encodingForName( *it ) == encoding ) {
+          mSelectEncodingAction->setCurrentItem( i );
+          break;
+        }
+      }
+    }
+  }
   update( true );
 }
 
+//-----------------------------------------------------------------------------
+const QTextCodec * KMReaderWin::overrideCodec() const
+{
+  kdDebug(5006) << k_funcinfo << " mOverrideEncoding == '" << mOverrideEncoding << "'" << endl;
+  if ( mOverrideEncoding.isEmpty() || mOverrideEncoding == "Auto" ) // Auto
+    return 0;
+  else
+    return KMMsgBase::codecForName( mOverrideEncoding.latin1() );
+}
+
+//-----------------------------------------------------------------------------
+void KMReaderWin::slotSetEncoding()
+{
+  if ( mSelectEncodingAction->currentItem() == 0 ) // Auto
+    mOverrideEncoding = QString();
+  else
+    mOverrideEncoding = KGlobal::charsets()->encodingForName( mSelectEncodingAction->currentText() );
+  update( true );
+}
+
+//-----------------------------------------------------------------------------
+void KMReaderWin::readGlobalOverrideCodec()
+{
+  // if the global character encoding wasn't changed then there's nothing to do
+  if ( GlobalSettings::self()->overrideCharacterEncoding() == mOldGlobalOverrideEncoding )
+    return;
+
+  setOverrideEncoding( GlobalSettings::self()->overrideCharacterEncoding() );
+  mOldGlobalOverrideEncoding = GlobalSettings::self()->overrideCharacterEncoding();
+}
+
 //-----------------------------------------------------------------------------
 void KMReaderWin::setMsg(KMMessage* aMsg, bool force)
 {
@@ -1060,6 +1140,13 @@ void KMReaderWin::setMsg(KMMessage* aMsg, bool force)
       kdDebug(5006) << "(" << aMsg->getMsgSerNum() << ", last " << mLastSerNum << ") " << aMsg->subject() << " "
         << aMsg->fromStrip() << ", readyToShow " << (aMsg->readyToShow()) << endl;
 
+	//Reset the level quote if the msg has changed.
+  if (aMsg && aMsg->getMsgSerNum() != mLastSerNum ){
+    mLevelQuote = GlobalSettings::self()->collapseQuoteLevelSpin()-1;
+  }
+  if ( mPrinting )
+    mLevelQuote = -1;
+
   bool complete = true;
   if ( aMsg &&
        !aMsg->readyToShow() &&
@@ -1082,18 +1169,21 @@ void KMReaderWin::setMsg(KMMessage* aMsg, bool force)
 
   mDelayedMarkTimer.stop();
 
-  mLastSerNum = (aMsg) ? aMsg->getMsgSerNum() : 0;
-  if ( !aMsg ) mWaitingForSerNum = 0; // otherwise it has been set
-
-  // assume if a serial number exists it can be used to find the assoc KMMessage
-  if (mLastSerNum <= 0)
-    mMessage = aMsg;
-  else
-    mMessage = 0;
-  if (message() != aMsg) {
-    mMessage = aMsg;
-    mLastSerNum = 0; // serial number was invalid
-    Q_ASSERT(0);
+  mMessage = 0;
+  if ( !aMsg ) {
+    mWaitingForSerNum = 0; // otherwise it has been set
+    mLastSerNum = 0;
+  } else {
+    mLastSerNum = aMsg->getMsgSerNum();
+    // Check if the serial number can be used to find the assoc KMMessage
+    // If so, keep only the serial number (and not mMessage), to avoid a dangling mMessage
+    // when going to another message in the mainwindow.
+    // Otherwise, keep only mMessage, this is fine for standalone KMReaderMainWins since
+    // we're working on a copy of the KMMessage, which we own.
+    if (message() != aMsg) {
+      mMessage = aMsg;
+      mLastSerNum = 0;
+    }
   }
 
   if (aMsg) {
@@ -1125,9 +1215,9 @@ void KMReaderWin::setMsg(KMMessage* aMsg, bool force)
       updateReaderWinTimer.start( 0, TRUE );
   }
 
-  if ( GlobalSettings::delayedMarkAsRead() ) {
-    if ( GlobalSettings::delayedMarkTime() != 0 )
-      mDelayedMarkTimer.start( GlobalSettings::delayedMarkTime() * 1000, TRUE );
+  if ( aMsg && (aMsg->isUnread() || aMsg->isNew()) && GlobalSettings::self()->delayedMarkAsRead() ) {
+    if ( GlobalSettings::self()->delayedMarkTime() != 0 )
+      mDelayedMarkTimer.start( GlobalSettings::self()->delayedMarkTime() * 1000, TRUE );
     else
       slotTouchMessage();
   }
@@ -1146,6 +1236,7 @@ void KMReaderWin::clearCache()
 
 // enter items for the "Important changes" list here:
 static const char * const kmailChanges[] = {
+  ""
 };
 static const int numKMailChanges =
   sizeof kmailChanges / sizeof *kmailChanges;
@@ -1155,23 +1246,13 @@ static const int numKMailChanges =
 // the translators). Note that the <li>...</li> tags are added
 // automatically below:
 static const char * const kmailNewFeatures[] = {
-  I18N_NOOP("Optional columns in the message list: status, attachment, important, etc."),
-  I18N_NOOP("Load external references for certain folders and on request"),
-  I18N_NOOP("Replace smileys by emoticons"),
-  I18N_NOOP("Show the sender's picture if it is in the address book"),
-  I18N_NOOP("Support for X-Face (b/w pictures in messages)"),
-  I18N_NOOP("Optional spam probability meter"),
-  I18N_NOOP("Configurable default character encoding"),
-  I18N_NOOP("Searching in IMAP messages fully supported"),
-  I18N_NOOP("Move IMAP folders"),
-  I18N_NOOP("KWallet support"),
-  I18N_NOOP("New authentication methods for IMAP, POP3, SMTP: NTLM (Microsoft Windows) and GSSAPI (Kerberos)"),
-  I18N_NOOP("Optional compression of attachments"),
-  I18N_NOOP("Attach images from the clipboard"),
-  I18N_NOOP("New recipient editor and picker in the composer"),
-  I18N_NOOP("Quick folder switching via configurable shortcuts"),
-  I18N_NOOP("Anti-Spam Wizard supports additional tools"),
-  I18N_NOOP("Import mail from Evolution 2.x, Opera and Thunderbird")
+  I18N_NOOP("Full namespace support for IMAP"),
+  I18N_NOOP("Offline mode"),
+  I18N_NOOP("Sieve script management and editing"),
+  I18N_NOOP("Account specific filtering"),
+  I18N_NOOP("Filtering of incoming mail for online IMAP accounts"),
+  I18N_NOOP("Online IMAP folders can be used when filtering into folders"),
+  I18N_NOOP("Automatically delete older mails on POP servers")
 };
 static const int numKMailNewFeatures =
   sizeof kmailNewFeatures / sizeof *kmailNewFeatures;
@@ -1191,7 +1272,7 @@ QString KMReaderWin::newFeaturesMD5()
 }
 
 //-----------------------------------------------------------------------------
-void KMReaderWin::displayAboutPage()
+void KMReaderWin::displaySplashPage( const QString &info )
 {
   mMsgDisplay = false;
   adjustLayout();
@@ -1205,13 +1286,42 @@ void KMReaderWin::displayAboutPage()
     content = content.arg( "" );
 
   mViewer->begin(KURL( location ));
+
+  QString fontSize = QString::number( pointsToPixel( mCSSHelper->bodyFont().pointSize() ) );
+  QString appTitle = i18n("KMail");
+  QString catchPhrase = ""; //not enough space for a catch phrase at default window size i18n("Part of the Kontact Suite");
+  QString quickDescription = i18n("The email client for the K Desktop Environment.");
+  mViewer->write(content.arg(fontSize).arg(appTitle).arg(catchPhrase).arg(quickDescription).arg(info));
+  mViewer->end();
+}
+
+void KMReaderWin::displayBusyPage()
+{
+  QString info =
+    i18n( "<h2 style='margin-top: 0px;'>Retrieving Folder Contents</h2><p>Please wait . . .</p>&nbsp;" );
+
+  displaySplashPage( info );
+}
+
+void KMReaderWin::displayOfflinePage()
+{
+  QString info =
+    i18n( "<h2 style='margin-top: 0px;'>Offline</h2><p>KMail is currently in offline mode. "
+        "Click <a href=\"kmail:goOnline\">here</a> to go online . . .</p>&nbsp;" );
+
+  displaySplashPage( info );
+}
+
+
+//-----------------------------------------------------------------------------
+void KMReaderWin::displayAboutPage()
+{
   QString info =
     i18n("%1: KMail version; %2: help:// URL; %3: homepage URL; "
 	 "%4: prior KMail version; %5: prior KDE version; "
 	 "%6: generated list of new features; "
 	 "%7: First-time user text (only shown on first start); "
-	 "%8: prior KMail version; "
-         "%9: generated list of important changes; "
+         "%8: generated list of important changes; "
 	 "--- end of comment ---",
 	 "<h2 style='margin-top: 0px;'>Welcome to KMail %1</h2><p>KMail is the email client for the K "
 	 "Desktop Environment. It is designed to be fully compatible with "
@@ -1232,7 +1342,7 @@ void KMReaderWin::displayAboutPage()
     .arg(KMAIL_VERSION) // KMail version
     .arg("help:/kmail/index.html") // KMail help:// URL
     .arg("http://kmail.kde.org/") // KMail homepage URL
-    .arg("1.7").arg("3.3"); // prior KMail and KDE version
+    .arg("1.8").arg("3.4"); // prior KMail and KDE version
 
   QString featureItems;
   for ( int i = 0 ; i < numKMailNewFeatures ; i++ )
@@ -1251,11 +1361,11 @@ void KMReaderWin::displayAboutPage()
     info = info.arg( QString::null );
   }
 
-  if ( numKMailChanges > 0 ) {
+  if ( ( numKMailChanges > 1 ) || ( numKMailChanges == 1 && strlen(kmailChanges[0]) > 0 ) ) {
     QString changesText =
       i18n("<p><span style='font-size:125%; font-weight:bold;'>"
            "Important changes</span> (compared to KMail %1):</p>\n")
-      .arg("1.7");
+      .arg("1.8");
     changesText += "<ul>\n";
     for ( int i = 0 ; i < numKMailChanges ; i++ )
       changesText += i18n("<li>%1</li>\n").arg( i18n( kmailChanges[i] ) );
@@ -1265,12 +1375,7 @@ void KMReaderWin::displayAboutPage()
   else
     info = info.arg(""); // remove the %8
 
-  QString fontSize = QString::number( pointsToPixel( mCSSHelper->bodyFont().pointSize() ) );
-  QString appTitle = i18n("KMail");
-  QString catchPhrase = ""; //not enough space for a catch phrase at default window size i18n("Part of the Kontact Suite");
-  QString quickDescription = i18n("The email client for the K Desktop Environment.");
-  mViewer->write(content.arg(fontSize).arg(appTitle).arg(catchPhrase).arg(quickDescription).arg(info));
-  mViewer->end();
+  displaySplashPage( info );
 }
 
 void KMReaderWin::enableMsgDisplay() {
@@ -1292,13 +1397,11 @@ void KMReaderWin::updateReaderWin()
   KMFolder* folder;
   if (message(&folder))
   {
-    if( !kmkernel->iCalIface().isResourceFolder( folder ) ){
-      if ( mShowColorbar )
-        mColorBar->show();
-      else
-        mColorBar->hide();
-      displayMessage();
-    }
+    if ( mShowColorbar )
+      mColorBar->show();
+    else
+      mColorBar->hide();
+    displayMessage();
   }
   else
   {
@@ -1309,6 +1412,13 @@ void KMReaderWin::updateReaderWin()
     htmlWriter()->write( mCSSHelper->htmlHead( isFixedFont() ) + "</body></html>" );
     htmlWriter()->end();
   }
+
+  if (mSavedRelativePosition)
+  {
+    QScrollView * scrollview = static_cast<QScrollView *>(mViewer->widget());
+    scrollview->setContentsPos ( 0, qRound(  scrollview->contentsHeight() * mSavedRelativePosition ) );
+    mSavedRelativePosition = 0;
+  }
 }
 
 //-----------------------------------------------------------------------------
@@ -1574,7 +1684,7 @@ QString KMReaderWin::writeMessagePartToTempFile( KMMessagePart* aMsgPart,
   size_t size = data.size();
   if ( aMsgPart->type() == DwMime::kTypeText && size) {
     // convert CRLF to LF before writing text attachments to disk
-    size = KMFolder::crlf2lf( data.data(), size );
+    size = KMail::Util::crlf2lf( data.data(), size );
   }
   if( !KPIM::kBytesToFile( data.data(), size, fname, false, false, false ) )
     return QString::null;
@@ -1769,10 +1879,13 @@ void KMReaderWin::showAttachmentPopup( int id, const QString & name, const QPoin
   mAtmCurrent = id;
   mAtmCurrentName = name;
   KPopupMenu *menu = new KPopupMenu();
-  menu->insertItem(SmallIcon("fileopen"),i18n("Open"), 1);
+  menu->insertItem(SmallIcon("fileopen"),i18n("to open", "Open"), 1);
   menu->insertItem(i18n("Open With..."), 2);
   menu->insertItem(i18n("to view something", "View"), 3);
   menu->insertItem(SmallIcon("filesaveas"),i18n("Save As..."), 4);
+  if ( name.endsWith( ".xia", false ) &&
+       Kleo::CryptoBackendFactory::instance()->protocol( "Chiasmus" ) )
+    menu->insertItem( i18n( "Decrypt With Chiasmus..." ), 6 );
   menu->insertItem(i18n("Properties"), 5);
   connect(menu, SIGNAL(activated(int)), this, SLOT(slotHandleAttachment(int)));
   menu->exec( p ,0 );
@@ -1808,9 +1921,9 @@ void KMReaderWin::slotHandleAttachment( int choice )
 {
   mAtmUpdate = true;
   partNode* node = mRootNode ? mRootNode->findId( mAtmCurrent ) : 0;
-  KMHandleAttachmentCommand* command = new KMHandleAttachmentCommand( node,
-      message(), mAtmCurrent, mAtmCurrentName,
-      KMHandleAttachmentCommand::AttachmentAction( choice ), 0 );
+  KMHandleAttachmentCommand* command = new KMHandleAttachmentCommand(
+      node, message(), mAtmCurrent, mAtmCurrentName,
+      KMHandleAttachmentCommand::AttachmentAction( choice ), 0, this );
   connect( command, SIGNAL( showAttachment( int, const QString& ) ),
       this, SLOT( slotAtmView( int, const QString& ) ) );
   command->start();
@@ -1819,15 +1932,15 @@ void KMReaderWin::slotHandleAttachment( int choice )
 //-----------------------------------------------------------------------------
 void KMReaderWin::slotFind()
 {
-  //dnaber:
-  KAction *act = mViewer->actionCollection()->action("find");
-  if( act )
-    act->activate();
+  mViewer->findText();
 }
 
 //-----------------------------------------------------------------------------
 void KMReaderWin::slotToggleFixedFont()
 {
+  QScrollView * scrollview = static_cast<QScrollView *>(mViewer->widget());
+  mSavedRelativePosition = (float)scrollview->contentsY() / scrollview->contentsHeight();
+
   mUseFixedFont = !mUseFixedFont;
   update(true);
 }
@@ -1860,7 +1973,7 @@ void KMReaderWin::atmViewMsg(KMMessagePart* aMsgPart)
   msg->setUID(message()->UID());
   msg->setReadyToShow(true);
   KMReaderMainWin *win = new KMReaderMainWin();
-  win->showMsg( overrideCodec(), msg );
+  win->showMsg( overrideEncoding(), msg );
   win->show();
 }
 
@@ -1994,7 +2107,7 @@ void KMReaderWin::slotAtmView( int id, const QString& name )
       setMsgPart( &msgPart, htmlMail(), name, pname );
     } else {
       KMReaderMainWin *win = new KMReaderMainWin(&msgPart, htmlMail(),
-          name, pname, overrideCodec() );
+          name, pname, overrideEncoding() );
       win->show();
     }
   }
@@ -2068,16 +2181,17 @@ void KMReaderWin::openAttachment( int id, const QString & name )
     mAtmUpdate = true;
     KMHandleAttachmentCommand* command = new KMHandleAttachmentCommand( node,
         message(), mAtmCurrent, mAtmCurrentName, KMHandleAttachmentCommand::Save,
-        offer );
+        offer, this );
     connect( command, SIGNAL( showAttachment( int, const QString& ) ),
         this, SLOT( slotAtmView( int, const QString& ) ) );
     command->start();
   }
   else if( choice == KMessageBox::No ) {	// Open
+    KMHandleAttachmentCommand::AttachmentAction action = ( offer ?
+        KMHandleAttachmentCommand::Open : KMHandleAttachmentCommand::OpenWith );
     mAtmUpdate = true;
     KMHandleAttachmentCommand* command = new KMHandleAttachmentCommand( node,
-        message(), mAtmCurrent, mAtmCurrentName, KMHandleAttachmentCommand::Open,
-        offer );
+        message(), mAtmCurrent, mAtmCurrentName, action, offer, this );
     connect( command, SIGNAL( showAttachment( int, const QString& ) ),
         this, SLOT( slotAtmView( int, const QString& ) ) );
     command->start();
@@ -2214,7 +2328,7 @@ KMMessage* KMReaderWin::message( KMFolder** aFolder ) const
   if (mLastSerNum) {
     KMMessage *message = 0;
     int index;
-    kmkernel->msgDict()->getLocation( mLastSerNum, &folder, &index );
+    KMMsgDict::instance()->getLocation( mLastSerNum, &folder, &index );
     if (folder )
       message = folder->getMsg( index );
     if (!message)
@@ -2367,6 +2481,24 @@ QString KMReaderWin::createAtmFileLink() const
   return QString::null;
 }
 
+//-----------------------------------------------------------------------------
+bool KMReaderWin::eventFilter( QObject *, QEvent *e )
+{
+  if ( e->type() == QEvent::MouseButtonPress ) {
+    QMouseEvent* me = static_cast<QMouseEvent*>(e);
+    if ( me->button() == LeftButton && ( me->state() & ShiftButton ) ) {
+      // special processing for shift+click
+      mAtmCurrent = msgPartFromUrl( mUrlClicked );
+      if ( mAtmCurrent < 0 ) return false; // not an attachment
+      mAtmCurrentName = mUrlClicked.path();
+      slotHandleAttachment( KMHandleAttachmentCommand::Save ); // save
+      return true; // eat event
+    }
+  }
+  // standard event processing
+  return false;
+}
+
 #include "kmreaderwin.moc"
 
 
