@@ -2,7 +2,10 @@
 
 #include <config.h>
 
+#define REALLY_WANT_KMSENDER
 #include "kmsender.h"
+#include "kmsender_p.h"
+#undef REALLY_WANT_KMSENDER
 
 #include <kmime_header_parsing.h>
 using namespace KMime::Types;
@@ -111,17 +114,19 @@ bool KMSender::settingsOk() const
   return true;
 }
 
+static void handleRedirections( KMMessage * m ) {
+  const QString from  = m->headerField("X-KMail-Redirect-From");
+  const QString msgId = m->msgId();
+  if( from.isEmpty() || msgId.isEmpty() )
+    m->setMsgId( KMMessage::generateMessageId( m->sender() ) );
+}
 
 //-----------------------------------------------------------------------------
-bool KMSender::send(KMMessage* aMsg, short sendNow)
+bool KMSender::doSend(KMMessage* aMsg, short sendNow)
 {
-  int rc;
-
-  //assert(aMsg != 0);
   if(!aMsg)
-    {
       return false;
-    }
+
   if (!settingsOk()) return FALSE;
 
   if (aMsg->to().isEmpty())
@@ -149,36 +154,28 @@ bool KMSender::send(KMMessage* aMsg, short sendNow)
     aMsg->setTo("Undisclosed.Recipients: ;");
   }
 
-
-  // Handle redirections
-  QString from  = aMsg->headerField("X-KMail-Redirect-From");
-  QString msgId = aMsg->msgId();
-  if( from.isEmpty() || msgId.isEmpty() ) {
-    msgId = KMMessage::generateMessageId( aMsg->sender() );
-    //kdDebug(5006) << "Setting Message-Id to '" << msgId << "'\n";
-    aMsg->setMsgId( msgId );
-  }
+  handleRedirections( aMsg );
 
   if (sendNow==-1) sendNow = mSendImmediate;
 
   kmkernel->outboxFolder()->open();
+  const KMFolderCloser openOutbox( kmkernel->outboxFolder() );
+
   aMsg->setStatus(KMMsgStatusQueued);
 
-  rc = kmkernel->outboxFolder()->addMsg(aMsg);
-  if (rc)
-  {
+  if ( const int err = openOutbox.folder()->addMsg(aMsg) ) {
+    Q_UNUSED( err );
     KMessageBox::information(0,i18n("Cannot add message to outbox folder"));
-    return FALSE;
+    return false;
   }
 
   //Ensure the message is correctly and fully parsed
-  kmkernel->outboxFolder()->unGetMsg( kmkernel->outboxFolder()->count() - 1 );
+  openOutbox.folder()->unGetMsg( openOutbox.folder()->count() - 1 );
 
-  if (sendNow && !mSendInProgress) rc = sendQueued();
-  else rc = TRUE;
-  kmkernel->outboxFolder()->close();
+  if ( !sendNow || mSendInProgress )
+    return true;
 
-  return rc;
+  return sendQueued();
 }
 
 
@@ -194,7 +191,7 @@ void KMSender::outboxMsgAdded(int idx)
 
 
 //-----------------------------------------------------------------------------
-bool KMSender::sendQueued(void)
+bool KMSender::doSendQueued( const QString &customTransport )
 {
   if (!settingsOk()) return FALSE;
 
@@ -226,6 +223,7 @@ bool KMSender::sendQueued(void)
   kmkernel->filterMgr()->ref();
 
   // start sending the messages
+  mCustomTransport = customTransport;
   doSendMsg();
   return TRUE;
 }
@@ -244,15 +242,14 @@ void KMSender::doSendMsg()
   if (!kmkernel)  //To handle message sending in progress when kaplan is exited
     return;	//TODO: handle this case better
 
-  KMFolder *sentFolder = 0, *imapSentFolder = 0;
-  bool someSent = mCurrentMsg;
-  int rc;
+  const bool someSent = mCurrentMsg;
   if (someSent) {
       mSentMessages++;
       mSentBytes += mCurrentMsg->msgSize();
   }
 
   // Post-process sent message (filtering)
+  KMFolder *sentFolder = 0, *imapSentFolder = 0;
   if (mCurrentMsg  && kmkernel->filterMgr())
   {
     mCurrentMsg->setTransferInProgress( FALSE );
@@ -316,8 +313,8 @@ void KMSender::doSendMsg()
       sentFolder = kmkernel->sentFolder();
 
     if ( sentFolder ) {
-      rc = sentFolder->open();
-      if (rc != 0) {
+      if ( const int err = sentFolder->open() ) {
+        Q_UNUSED( err );
         cleanup();
         return;
       }
@@ -327,7 +324,7 @@ void KMSender::doSendMsg()
     // current folder (outbox) and re-added, to make filter actions changing the message
     // work. We don't want that to screw up message counts.
     if ( mCurrentMsg->parent() ) mCurrentMsg->parent()->quiet( true );
-    int processResult = kmkernel->filterMgr()->process(mCurrentMsg,KMFilterMgr::Outbound);
+    const int processResult = kmkernel->filterMgr()->process(mCurrentMsg,KMFilterMgr::Outbound);
     if ( mCurrentMsg->parent() ) mCurrentMsg->parent()->quiet( false );
 
     // 0==processed ok, 1==no filter matched, 2==critical error, abort!
@@ -413,7 +410,7 @@ void KMSender::doSendMsg()
       if ( mSentMessages == mTotalMessages ) {
         setStatusMsg(i18n("%n queued message successfully sent.",
                           "%n queued messages successfully sent.",
-                     mSentMessages));
+                          mSentMessages));
       } else {
         setStatusMsg(i18n("%1 of %2 queued messages successfully sent.")
             .arg(mSentMessages).arg( mTotalMessages ));
@@ -439,23 +436,41 @@ void KMSender::doSendMsg()
     mSendInProgress = TRUE;
   }
 
-  QString msgTransport = mCurrentMsg->headerField("X-KMail-Transport");
-  if (msgTransport.isEmpty())
-  {
-    QStringList sl = KMTransportInfo::availableTransports();
-    if (!sl.isEmpty()) msgTransport = sl[0];
+  QString msgTransport = mCustomTransport;
+  if ( msgTransport.isEmpty() ) {
+    msgTransport = mCurrentMsg->headerField("X-KMail-Transport");
   }
+  if ( msgTransport.isEmpty() ) {
+    const QStringList sl = KMTransportInfo::availableTransports();
+    if (!sl.empty()) msgTransport = sl.front();
+  }
+
   if (!mSendProc || msgTransport != mMethodStr) {
     if (mSendProcStarted && mSendProc) {
-      mSendProc->finish(true);
+      mSendProc->finish();
       mSendProcStarted = FALSE;
     }
 
     mSendProc = createSendProcFromString(msgTransport);
     mMethodStr = msgTransport;
 
-    if( mTransportInfo->encryption == "TLS" || mTransportInfo->encryption == "SSL" )
+    if( mTransportInfo->encryption == "TLS" || mTransportInfo->encryption == "SSL" ) {
       mProgressItem->setUsesCrypto( true );
+    } else if ( !mCustomTransport.isEmpty() ) {
+        int result = KMessageBox::warningContinueCancel( 0,
+        i18n( "You have chosen to send all queued email using an unencrypted transport, do you want to continue? "),
+        i18n( "Security Warning" ),
+        i18n( "Send Unencrypted" ),
+        "useCustomTransportWithoutAsking", false);
+
+      if( result == KMessageBox::Cancel ) {
+        mProgressItem->cancel();
+        mProgressItem->setComplete();
+        slotAbortSend();
+        cleanup();
+        return;
+      }
+    }
 
     if (!mSendProc)
       sendProcStarted(false);
@@ -464,18 +479,8 @@ void KMSender::doSendMsg()
       connect(mSendProc, SIGNAL(started(bool)), SLOT(sendProcStarted(bool)));
 
       // Run the precommand if there is one
-      if (!mTransportInfo->precommand.isEmpty())
-      {
-        setStatusMsg(i18n("Executing precommand %1")
-          .arg(mTransportInfo->precommand));
-        mPrecommand = new KMPrecommand(mTransportInfo->precommand);
-        connect(mPrecommand, SIGNAL(finished(bool)),
-          SLOT(slotPrecommandFinished(bool)));
-        if (!mPrecommand->start())
-        {
-          delete mPrecommand;
-          mPrecommand = 0;
-        }
+      if ( !mTransportInfo->precommand.isEmpty() ) {
+        runPrecommand( mTransportInfo->precommand );
         return;
       }
 
@@ -488,13 +493,24 @@ void KMSender::doSendMsg()
     doSendMsgAux();
 }
 
+bool KMSender::runPrecommand( const QString & cmd ) {
+  setStatusMsg( i18n("Executing precommand %1").arg( cmd ) );
+  mPrecommand = new KMPrecommand( cmd );
+  connect( mPrecommand, SIGNAL(finished(bool)),
+           SLOT(slotPrecommandFinished(bool)) );
+  if ( !mPrecommand->start() ) {
+    delete mPrecommand; mPrecommand = 0;
+    return false;
+  }
+  return true;
+}
 
 //-----------------------------------------------------------------------------
 void KMSender::sendProcStarted(bool success)
 {
   if (!success) {
     if (mSendProc)
-       mSendProc->finish(true);
+       mSendProc->finish();
     else
       setStatusMsg(i18n("Unrecognized transport protocol. Unable to send message."));
     mSendProc = 0;
@@ -506,6 +522,30 @@ void KMSender::sendProcStarted(bool success)
 }
 
 
+static QStringList addrSpecListToStringList( const AddrSpecList & l, bool allowEmpty=false ) {
+  QStringList result;
+  for ( AddrSpecList::const_iterator it = l.begin(), end = l.end() ; it != end ; ++it ) {
+    const QString s = (*it).asString();
+    if ( allowEmpty || !s.isEmpty() )
+      result.push_back( s );
+  }
+  return result;
+}
+
+static void extractSenderToCCAndBcc( KMMessage * aMsg, QString * sender, QStringList * to, QStringList * cc, QStringList * bcc ) {
+  if ( sender ) *sender = aMsg->sender();
+  if( !aMsg->headerField("X-KMail-Recipients").isEmpty() ) {
+    // extended BCC handling to prevent TOs and CCs from seeing
+    // BBC information by looking at source of an OpenPGP encrypted mail
+    if ( to ) *to = addrSpecListToStringList( aMsg->extractAddrSpecs( "X-KMail-Recipients" ) );
+    aMsg->removeHeaderField( "X-KMail-Recipients" );
+  } else {
+    if ( to ) *to = addrSpecListToStringList( aMsg->extractAddrSpecs( "To" ) );
+    if ( cc ) *cc = addrSpecListToStringList( aMsg->extractAddrSpecs( "Cc" ) );
+    if ( bcc ) *bcc = addrSpecListToStringList( aMsg->extractAddrSpecs( "Bcc" ) );
+  }
+}
+
 //-----------------------------------------------------------------------------
 void KMSender::doSendMsgAux()
 {
@@ -513,12 +553,14 @@ void KMSender::doSendMsgAux()
 
   // start sending the current message
 
-  mSendProc->preSendInit();
   setStatusMsg(i18n("%3: subject of message","Sending message %1 of %2: %3")
 	       .arg(mSentMessages+mFailedMessages+1).arg(mTotalMessages)
 	       .arg(mCurrentMsg->subject()));
-  if (!mSendProc->send(mCurrentMsg))
-  {
+  QStringList to, cc, bcc;
+  QString sender;
+  extractSenderToCCAndBcc( mCurrentMsg, &sender, &to, &cc, &bcc );
+  const QCString message = mCurrentMsg->asSendableString();
+  if ( sender.isEmpty() || !mSendProc->send( sender, to, cc, bcc, message ) ) {
     if ( mCurrentMsg )
       mCurrentMsg->setTransferInProgress( false );
     if ( mOutboxFolder )
@@ -537,7 +579,7 @@ void KMSender::doSendMsgAux()
 void KMSender::cleanup(void)
 {
   kdDebug(5006) << k_funcinfo << endl;
-  if (mSendProc && mSendProcStarted) mSendProc->finish(true);
+  if (mSendProc && mSendProcStarted) mSendProc->finish();
   mSendProc = 0;
   mSendProcStarted = FALSE;
   if (mSendInProgress) kapp->deref();
@@ -592,7 +634,7 @@ void KMSender::slotIdle()
   QString msg;
   QString errString;
   if (mSendProc)
-      errString = mSendProc->message();
+      errString = mSendProc->lastErrorMessage();
 
   if (mSendAborted) {
     // sending of message aborted
@@ -659,7 +701,7 @@ void KMSender::slotIdle()
       return;
     }
   }
-  mSendProc->finish(true);
+  mSendProc->finish();
   mSendProc = 0;
   mSendProcStarted = false;
 
@@ -692,7 +734,7 @@ void KMSender::setSendQuotedPrintable(bool aSendQuotedPrintable)
 
 
 //-----------------------------------------------------------------------------
-KMSendProc* KMSender::createSendProcFromString(QString transport)
+KMSendProc* KMSender::createSendProcFromString( const QString & transport )
 {
   mTransportInfo->type = QString::null;
   int nr = KMTransportInfo::findTransport(transport);
@@ -764,13 +806,13 @@ void KMSender::setStatusByLink(const KMMessage *aMsg)
 
     KMFolder *folder = 0;
     int index = -1;
-    kmkernel->msgDict()->getLocation(msn, &folder, &index);
+    KMMsgDict::instance()->getLocation(msn, &folder, &index);
     if (folder && index != -1) {
       folder->open();
       if ( status == KMMsgStatusDeleted ) {
         // Move the message to the trash folder
-        KMDeleteMsgCommand *cmd = 
-          new KMDeleteMsgCommand( folder, folder->getMsg( index ) ); 
+        KMDeleteMsgCommand *cmd =
+          new KMDeleteMsgCommand( folder, folder->getMsg( index ) );
         cmd->start();
       } else {
         folder->setStatus(index, status);
@@ -784,18 +826,21 @@ void KMSender::setStatusByLink(const KMMessage *aMsg)
 
 //=============================================================================
 //=============================================================================
-KMSendProc::KMSendProc(KMSender* aSender): QObject()
+KMSendProc::KMSendProc( KMSender * sender )
+  : QObject( 0 ),
+    mSender( sender ),
+    mLastErrorMessage(),
+    mSendOk( false ),
+    mSending( false )
 {
-  mSender = aSender;
-  preSendInit();
 }
 
 //-----------------------------------------------------------------------------
-void KMSendProc::preSendInit(void)
+void KMSendProc::reset()
 {
   mSending = FALSE;
   mSendOk = FALSE;
-  mMsg = QString::null;
+  mLastErrorMessage = QString::null;
 }
 
 //-----------------------------------------------------------------------------
@@ -803,20 +848,7 @@ void KMSendProc::failed(const QString &aMsg)
 {
   mSending = FALSE;
   mSendOk = FALSE;
-  mMsg = aMsg;
-}
-
-//-----------------------------------------------------------------------------
-void KMSendProc::start(void)
-{
-  emit started(true);
-}
-
-//-----------------------------------------------------------------------------
-bool KMSendProc::finish(bool destructive)
-{
-  if (destructive) deleteLater();
-  return TRUE;
+  mLastErrorMessage = aMsg;
 }
 
 //-----------------------------------------------------------------------------
@@ -825,47 +857,36 @@ void KMSendProc::statusMsg(const QString& aMsg)
   if (mSender) mSender->setStatusMsg(aMsg);
 }
 
-//-----------------------------------------------------------------------------
-bool KMSendProc::addRecipients( const AddrSpecList & al )
-{
-  for ( AddrSpecList::const_iterator it = al.begin() ; it != al.end() ; ++it )
-    if ( !addOneRecipient( (*it).asString() ) )
-      return false;
-  return true;
-}
-
-
 //=============================================================================
 //=============================================================================
-KMSendSendmail::KMSendSendmail(KMSender* aSender):
-  KMSendProc(aSender)
+KMSendSendmail::KMSendSendmail( KMSender * sender )
+  : KMSendProc( sender ),
+    mMsgStr(),
+    mMsgPos( 0 ),
+    mMsgRest( 0 ),
+    mMailerProc( 0 )
 {
-  mMailerProc = 0;
+
 }
 
-//-----------------------------------------------------------------------------
-KMSendSendmail::~KMSendSendmail()
-{
-  delete mMailerProc;
+KMSendSendmail::~KMSendSendmail() {
+  delete mMailerProc; mMailerProc = 0;
 }
 
-//-----------------------------------------------------------------------------
-void KMSendSendmail::start(void)
-{
+bool KMSendSendmail::doStart() {
+
   if (mSender->transportInfo()->host.isEmpty())
   {
-    QString str = i18n("Please specify a mailer program in the settings.");
-    QString msg;
-    msg = i18n("Sending failed:\n%1\n"
-	"The message will stay in the 'outbox' folder and will be resent.\n"
-        "Please remove it from there if you do not want the message to "
-		"be resent.\n"
-	"The following transport protocol was used:\n  %2")
-    .arg(str + "\n")
-    .arg("sendmail://");
+    const QString str = i18n("Please specify a mailer program in the settings.");
+    const QString msg = i18n("Sending failed:\n%1\n"
+                             "The message will stay in the 'outbox' folder and will be resent.\n"
+                             "Please remove it from there if you do not want the message to "
+                             "be resent.\n"
+                             "The following transport protocol was used:\n  %2")
+                        .arg(str + "\n")
+                        .arg("sendmail://");
     KMessageBox::information(0,msg);
-    emit started(false);
-    return;
+    return false;
   }
 
   if (!mMailerProc)
@@ -879,20 +900,14 @@ void KMSendSendmail::start(void)
     connect(mMailerProc,SIGNAL(receivedStderr(KProcess*,char*,int)),
 	    this, SLOT(receivedStderr(KProcess*, char*, int)));
   }
-  emit started(true);
+  return true;
 }
 
-//-----------------------------------------------------------------------------
-bool KMSendSendmail::finish(bool destructive)
-{
+void KMSendSendmail::doFinish() {
   delete mMailerProc;
   mMailerProc = 0;
-  if (destructive)
-    	deleteLater();
-  return TRUE;
 }
 
-//-----------------------------------------------------------------------------
 void KMSendSendmail::abort()
 {
   delete mMailerProc;
@@ -902,46 +917,27 @@ void KMSendSendmail::abort()
   idle();
 }
 
-
-//-----------------------------------------------------------------------------
-bool KMSendSendmail::send(KMMessage* aMsg)
-{
-  QString bccStr;
-
+bool KMSendSendmail::doSend( const QString & sender, const QStringList & to, const QStringList & cc, const QStringList & bcc, const QCString & message ) {
   mMailerProc->clearArguments();
-  *mMailerProc << mSender->transportInfo()->host;
-  *mMailerProc << "-i";
-  *mMailerProc << "-f";
-  *mMailerProc << aMsg->sender().latin1();
+  *mMailerProc << mSender->transportInfo()->host
+               << "-i" << "-f" << sender
+               << to << cc << bcc ;
 
-  if( !aMsg->headerField("X-KMail-Recipients").isEmpty() ) {
-    // extended BCC handling to prevent TOs and CCs from seeing
-    // BBC information by looking at source of an OpenPGP encrypted mail
-    addRecipients(aMsg->extractAddrSpecs("X-KMail-Recipients"));
-    aMsg->removeHeaderField( "X-KMail-Recipients" );
-  } else {
-    addRecipients(aMsg->extractAddrSpecs("To"));
-    addRecipients(aMsg->extractAddrSpecs("Cc"));
-    addRecipients(aMsg->extractAddrSpecs("Bcc"));
-  }
+  mMsgStr = message;
 
-  mMsgStr = aMsg->asSendableString();
-
-  if (!mMailerProc->start(KProcess::NotifyOnExit,KProcess::All))
-  {
-    KMessageBox::information(0,i18n("Failed to execute mailer program %1")
-			     .arg(mSender->transportInfo()->host));
-    return FALSE;
+  if ( !mMailerProc->start( KProcess::NotifyOnExit, KProcess::All ) ) {
+    KMessageBox::information( 0, i18n("Failed to execute mailer program %1")
+                              .arg( mSender->transportInfo()->host ) );
+    return false;
   }
   mMsgPos  = mMsgStr.data();
   mMsgRest = mMsgStr.length();
-  wroteStdin(mMailerProc);
+  wroteStdin( mMailerProc );
 
-  return TRUE;
+  return true;
 }
 
 
-//-----------------------------------------------------------------------------
 void KMSendSendmail::wroteStdin(KProcess *proc)
 {
   char* str;
@@ -968,16 +964,14 @@ void KMSendSendmail::wroteStdin(KProcess *proc)
 }
 
 
-//-----------------------------------------------------------------------------
 void KMSendSendmail::receivedStderr(KProcess *proc, char *buffer, int buflen)
 {
   assert(proc!=0);
   Q_UNUSED( proc );
-  mMsg.replace(mMsg.length(), buflen, buffer);
+  mLastErrorMessage.replace(mLastErrorMessage.length(), buflen, buffer);
 }
 
 
-//-----------------------------------------------------------------------------
 void KMSendSendmail::sendmailExited(KProcess *proc)
 {
   assert(proc!=0);
@@ -988,15 +982,6 @@ void KMSendSendmail::sendmailExited(KProcess *proc)
 }
 
 
-//-----------------------------------------------------------------------------
-bool KMSendSendmail::addOneRecipient(const QString& aRcpt)
-{
-  assert(mMailerProc!=0);
-  if (!aRcpt.isEmpty()) *mMailerProc << aRcpt;
-  return TRUE;
-}
-
-
 
 //-----------------------------------------------------------------------------
 //=============================================================================
@@ -1017,54 +1002,24 @@ KMSendSMTP::~KMSendSMTP()
   if (mJob) mJob->kill();
 }
 
-bool KMSendSMTP::send(KMMessage *aMsg)
-{
-  KMTransportInfo *ti = mSender->transportInfo();
-  assert(aMsg != 0);
+bool KMSendSMTP::doSend( const QString & sender, const QStringList & to, const QStringList & cc, const QStringList & bcc, const QCString & message ) {
+  QString query = "headers=0&from=";
+  query += KURL::encode_string( sender );
 
-  const QString sender = aMsg->sender();
-  if ( sender.isEmpty() )
-    return false;
+  if ( !to.empty() )
+    query += "&to=" + to.join( "&to=" );
+  if ( !cc.empty() )
+    query += "&cc=" + cc.join( "&cc=" );
+  if ( !bcc.empty() )
+    query += "&bcc=" + bcc.join( "&bcc=" );
 
-  // email this is from
-  mQuery = "headers=0&from=";
-  mQuery += KURL::encode_string( sender );
+  KMTransportInfo * ti = mSender->transportInfo();
 
-  // recipients
-  if( !aMsg->headerField("X-KMail-Recipients").isEmpty() ) {
-    // extended BCC handling to prevent TOs and CCs from seeing
-    // BBC information by looking at source of an OpenPGP encrypted mail
-    mQueryField = "&to=";
-    if( !addRecipients( aMsg->extractAddrSpecs("X-KMail-Recipients")) ) {
-      return FALSE;
-    }
-    aMsg->removeHeaderField( "X-KMail-Recipients" );
-  } else {
-    mQueryField = "&to=";
-    if(!addRecipients(aMsg->extractAddrSpecs("To")))
-    {
-      return FALSE;
-    }
-
-    if(!aMsg->cc().isEmpty())
-    {
-      mQueryField = "&cc=";
-      if(!addRecipients(aMsg->extractAddrSpecs("Cc"))) return FALSE;
-    }
-
-    QString bccStr = aMsg->bcc();
-    if(!bccStr.isEmpty())
-    {
-      mQueryField = "&bcc=";
-      if (!addRecipients(aMsg->extractAddrSpecs("Bcc"))) return FALSE;
-    }
-  }
-
-  if (ti->specifyHostname)
-    mQuery += "&hostname=" + KURL::encode_string(ti->localHostname);
+  if ( ti->specifyHostname )
+    query += "&hostname=" + KURL::encode_string( ti->localHostname );
 
   if ( !kmkernel->msgSender()->sendQuotedPrintable() )
-    mQuery += "&body=8bit";
+    query += "&body=8bit";
 
   KURL destination;
 
@@ -1074,7 +1029,7 @@ bool KMSendSMTP::send(KMMessage *aMsg)
 
   if (ti->auth)
   {
-    if( (ti->user.isEmpty() || ti->passwd().isEmpty()) && 
+    if( (ti->user.isEmpty() || ti->passwd().isEmpty()) &&
       ti->authType != "GSSAPI" )
     {
       bool b = FALSE;
@@ -1115,45 +1070,34 @@ bool KMSendSMTP::send(KMMessage *aMsg)
   }
 
   // dotstuffing is now done by the slave (see setting of metadata)
-  mMessage = aMsg->asSendableString();
+  mMessage = message;
   mMessageLength = mMessage.length();
   mMessageOffset = 0;
 
   if ( mMessageLength )
     // allow +5% for subsequent LF->CRLF and dotstuffing (an average
     // over 2G-lines gives an average line length of 42-43):
-    mQuery += "&size=" + QString::number( qRound( mMessageLength * 1.05 ) );
+    query += "&size=" + QString::number( qRound( mMessageLength * 1.05 ) );
 
   destination.setPath("/send");
-  destination.setQuery(mQuery);
-  mQuery = QString::null;
+  destination.setQuery( query );
 
-  if ((mJob = KIO::put(destination, -1, false, false, false)))
-  {
-    mJob->addMetaData( "lf2crlf+dotstuff", "slave" );
-    KIO::Scheduler::assignJobToSlave(mSlave, mJob);
-    connect(mJob, SIGNAL(result(KIO::Job *)), this, SLOT(result(KIO::Job *)));
-    connect(mJob, SIGNAL(dataReq(KIO::Job *, QByteArray &)),
-	    this, SLOT(dataReq(KIO::Job *, QByteArray &)));
-    mSendOk = true;
-    mInProcess = true;
-    return mSendOk;
-  }
-  else
-  {
+  mJob = KIO::put( destination, -1, false, false, false );
+  if ( !mJob ) {
     abort();
     return false;
   }
+  mJob->addMetaData( "lf2crlf+dotstuff", "slave" );
+  KIO::Scheduler::assignJobToSlave(mSlave, mJob);
+  connect(mJob, SIGNAL(result(KIO::Job *)), this, SLOT(result(KIO::Job *)));
+  connect(mJob, SIGNAL(dataReq(KIO::Job *, QByteArray &)),
+          this, SLOT(dataReq(KIO::Job *, QByteArray &)));
+  mSendOk = true;
+  mInProcess = true;
+  return true;
 }
 
-void KMSendSMTP::abort()
-{
-  finish(false);
-  emit idle();
-}
-
-bool KMSendSMTP::finish(bool b)
-{
+void KMSendSMTP::cleanup() {
   if(mJob)
   {
     mJob->kill(TRUE);
@@ -1168,21 +1112,21 @@ bool KMSendSMTP::finish(bool b)
   }
 
   mInProcess = false;
-  return KMSendProc::finish(b);
 }
 
-bool KMSendSMTP::addOneRecipient(const QString& _addr)
-{
-  if(!_addr.isEmpty())
-    mQuery += mQueryField + KURL::encode_string(_addr);
+void KMSendSMTP::abort() {
+  cleanup();
+  emit idle();
+}
 
-  return true;
+void KMSendSMTP::doFinish() {
+  cleanup();
 }
 
 void KMSendSMTP::dataReq(KIO::Job *, QByteArray &array)
 {
   // Send it by 32K chuncks
-  int chunkSize = QMIN( mMessageLength - mMessageOffset, 0x8000 );
+  const int chunkSize = QMIN( mMessageLength - mMessageOffset, 32*1024 );
   if ( chunkSize > 0 ) {
     array.duplicate(mMessage.data() + mMessageOffset, chunkSize);
     mMessageOffset += chunkSize;
@@ -1223,3 +1167,4 @@ void KMSendSMTP::slaveError(KIO::Slave *aSlave, int error, const QString &errorM
 }
 
 #include "kmsender.moc"
+#include "kmsender_p.moc"
