@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -404,12 +404,11 @@ partNode* partNode::findType( int type, int subType, bool deep, bool wide )
            && (    (subType == DwMime::kSubtypeUnknown)
                    || (subType == mSubType) ) )
         return this;
-    else if( mChild && deep )
+    if ( mChild && deep )
         return mChild->findType( type, subType, deep, wide );
-    else if( mNext && wide )
+    if ( mNext && wide )
         return mNext->findType(  type, subType, deep, wide );
-    else
-        return 0;
+    return 0;
 }
 
 partNode* partNode::findNodeForDwPart( DwBodyPart* part )
@@ -432,12 +431,11 @@ partNode* partNode::findTypeNot( int type, int subType, bool deep, bool wide )
            && (    (subType == DwMime::kSubtypeUnknown)
                    || (subType != mSubType) ) )
         return this;
-    else if( mChild && deep )
+    if ( mChild && deep )
         return mChild->findTypeNot( type, subType, deep, wide );
-    else if( mNext && wide )
+    if ( mNext && wide )
         return mNext->findTypeNot(  type, subType, deep, wide );
-    else
-        return 0;
+    return 0;
 }
 
 void partNode::fillMimePartTree( KMMimePartTreeItem* parentItem,
