@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -45,6 +45,7 @@
 #include <qhbox.h>
 #include <qtimer.h>
 #include <qpushbutton.h>
+#include <qstylesheet.h>
 
 Recipient::Recipient( const QString &email, Recipient::Type type )
   : mEmail( email ), mType( type )
@@ -192,6 +193,8 @@ void RecipientLine::slotFocusDown()
 void RecipientLine::slotTypeModified()
 {
   mModified = true;
+
+  emit typeModified( this );
 }
 
 void RecipientLine::analyzeLine( const QString &text )
@@ -280,9 +283,13 @@ void RecipientLine::keyPressEvent( QKeyEvent *ev )
   }
 }
 
-void RecipientLine::setComboWidth( int w )
+int RecipientLine::setComboWidth( int w )
 {
+  w = QMAX( w, mCombo->sizeHint().width() );
   mCombo->setFixedWidth( w );
+  mCombo->updateGeometry();
+  parentWidget()->updateGeometry();
+  return w;
 }
 
 void RecipientLine::fixTabOrder( QWidget *previous )
@@ -302,6 +309,14 @@ void RecipientLine::clear()
   mEdit->clear();
 }
 
+void RecipientLine::setRemoveLineButtonEnabled( bool b )
+{
+  mRemoveButton->setEnabled( b );
+}
+
+
+// ------------ RecipientsView ---------------------
+
 RecipientsView::RecipientsView( QWidget *parent )
   : QScrollView( parent ), mCurDelLine( 0 ), mModified( false )
 {
@@ -309,6 +324,8 @@ RecipientsView::RecipientsView( QWidget *parent )
   setLineWidth( 0 );
 
   addLine();
+  setResizePolicy( QScrollView::Manual );
+  setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
 }
 
 RecipientLine *RecipientsView::activeLine()
@@ -328,8 +345,6 @@ RecipientLine *RecipientsView::emptyLine()
 
 RecipientLine *RecipientsView::addLine()
 {
-  kdDebug() << "RecipientsView::addLine()" << endl;
-
   RecipientLine *line = new RecipientLine( viewport() );
   addChild( line, 0, mLines.count() * mLineHeight );
   line->show();
@@ -343,9 +358,24 @@ RecipientLine *RecipientsView::addLine()
   connect( line, SIGNAL( deleteLine( RecipientLine * ) ),
     SLOT( slotDecideLineDeletion( RecipientLine * ) ) );
   connect( line, SIGNAL( countChanged() ), SLOT( calculateTotal() ) );
+  connect( line, SIGNAL( typeModified( RecipientLine * ) ),
+    SLOT( slotTypeModified( RecipientLine * ) ) );
 
   if ( mLines.last() ) {
-    line->setRecipientType( mLines.last()->recipientType() );
+    if ( mLines.count() == 1 ) {
+      if ( GlobalSettings::self()->secondRecipientTypeDefault() ==
+         GlobalSettings::EnumSecondRecipientTypeDefault::To ) {
+        line->setRecipientType( Recipient::To );
+      } else {
+        if ( mLines.last()->recipientType() == Recipient::Bcc ) {
+          line->setRecipientType( Recipient::To );
+        } else {
+          line->setRecipientType( Recipient::Cc );
+        }
+      }
+    } else {
+      line->setRecipientType( mLines.last()->recipientType() );
+    }
     line->fixTabOrder( mLines.last()->tabOut() );
   }
 
@@ -357,7 +387,7 @@ RecipientLine *RecipientsView::addLine()
     mLines.first()->setRemoveLineButtonEnabled( true );
   }
 
-  line->setComboWidth( mFirstColumnWidth );
+  mFirstColumnWidth = line->setComboWidth( mFirstColumnWidth );
 
   mLineHeight = line->minimumSizeHint().height();
 
@@ -372,15 +402,22 @@ RecipientLine *RecipientsView::addLine()
   return line;
 }
 
-
-void RecipientLine::setRemoveLineButtonEnabled( bool b )
+void RecipientsView::slotTypeModified( RecipientLine *line )
 {
-  mRemoveButton->setEnabled( b );
+  if ( mLines.count() == 2 ||
+       ( mLines.count() == 3 && mLines.at( 2 )->isEmpty() ) ) {
+    if ( mLines.at( 1 ) == line ) {
+      if ( line->recipientType() == Recipient::To ) {
+        GlobalSettings::self()->setSecondRecipientTypeDefault(
+          GlobalSettings::EnumSecondRecipientTypeDefault::To );
+      } else if ( line->recipientType() == Recipient::Cc ) {
+        GlobalSettings::self()->setSecondRecipientTypeDefault(
+          GlobalSettings::EnumSecondRecipientTypeDefault::Cc );
+      }
+    }
+  }
 }
 
-
-// ------------ RecipientsView ---------------------
-
 void RecipientsView::calculateTotal()
 {
   int count = 0;
@@ -440,26 +477,41 @@ void RecipientsView::slotDecideLineDeletion( RecipientLine *line )
 
 void RecipientsView::slotDeleteLine()
 {
+  if ( !mCurDelLine )
+    return;
+
   RecipientLine *line = mCurDelLine;
   int pos = mLines.find( line );
 
   int newPos;
   if ( pos == 0 ) newPos = pos + 1;
   else newPos = pos - 1;
-  mLines.at( newPos )->activate();
+
+  // if there is something left to activate, do so
+  if ( mLines.at( newPos ) )
+    mLines.at( newPos )->activate();
 
   mLines.remove( line );
   removeChild( line );
   delete line;
 
+  bool atLeastOneToLine = false;
+  unsigned int firstCC = 0;
   for( uint i = pos; i < mLines.count(); ++i ) {
     RecipientLine *line = mLines.at( i );
     moveChild( line, childX( line ), childY( line ) - mLineHeight );
+    if ( line->recipientType() == Recipient::To )
+      atLeastOneToLine = true;
+    else if ( ( line->recipientType() == Recipient::Cc ) && ( i == 0 ) )
+      firstCC = i;
   }
   // only one left, can't remove that one
   if ( mLines.count() == 1 )
     mLines.first()->setRemoveLineButtonEnabled( false );
 
+  if ( !atLeastOneToLine )
+    mLines.at( firstCC )->setRecipientType( Recipient::To );
+
   calculateTotal();
 
   resizeView();
@@ -467,7 +519,7 @@ void RecipientsView::slotDeleteLine()
 
 void RecipientsView::resizeView()
 {
-  resizeContents( viewport()->width(), mLines.count() * mLineHeight );
+  resizeContents( width(), mLines.count() * mLineHeight );
 
   if ( mLines.count() < 6 ) {
     setFixedHeight( mLineHeight * mLines.count() );
@@ -482,8 +534,6 @@ void RecipientsView::activateLine( RecipientLine *line )
 
 void RecipientsView::viewportResizeEvent ( QResizeEvent *ev )
 {
-  kdDebug() << "RESIZE " << ev->size() << endl;
-
   for( uint i = 0; i < mLines.count(); ++i ) {
     mLines.at( i )->resize( ev->size().width(), mLineHeight );
   }
@@ -589,21 +639,21 @@ void RecipientsView::setFocusBottom()
   else  kdWarning() << "No last" << endl;
 }
 
-void RecipientsView::setFirstColumnWidth( int w )
+int RecipientsView::setFirstColumnWidth( int w )
 {
   mFirstColumnWidth = w;
 
   QPtrListIterator<RecipientLine> it( mLines );
   RecipientLine *line;
   while( ( line = it.current() ) ) {
-    line->setComboWidth( mFirstColumnWidth );
+    mFirstColumnWidth = line->setComboWidth( mFirstColumnWidth );
     ++it;
   }
 
   resizeView();
+  return mFirstColumnWidth;
 }
 
-
 RecipientsToolTip::RecipientsToolTip( RecipientsView *view, QWidget *parent )
   : QToolTip( parent ), mView( view )
 {
@@ -659,6 +709,7 @@ SideWidget::SideWidget( RecipientsView *view, QWidget *parent )
 {
   QBoxLayout *topLayout = new QVBoxLayout( this );
 
+  topLayout->setSpacing( KDialog::spacingHint() );
   topLayout->addStretch( 1 );
 
   mTotalLabel = new QLabel( this );
@@ -682,8 +733,6 @@ SideWidget::SideWidget( RecipientsView *view, QWidget *parent )
   topLayout->addWidget( mSelectButton );
   connect( mSelectButton, SIGNAL( clicked() ), SLOT( pickRecipient() ) );
   QToolTip::add( mSelectButton, i18n("Select recipients from address book") );
-
-  initRecipientPicker();
 }
 
 SideWidget::~SideWidget()
@@ -692,6 +741,14 @@ SideWidget::~SideWidget()
 
 RecipientsPicker* SideWidget::picker() const
 {
+  if ( !mRecipientPicker ) {
+    // hacks to allow picker() to be const in the presence of lazy loading
+    SideWidget *non_const_this = const_cast<SideWidget*>( this );
+    mRecipientPicker = new RecipientsPicker( non_const_this );
+    connect( mRecipientPicker, SIGNAL( pickedRecipient( const Recipient & ) ),
+             non_const_this, SIGNAL( pickedRecipient( const Recipient & ) ) );
+    mPickerPositioner = new KWindowPositioner( non_const_this, mRecipientPicker );
+  }
   return mRecipientPicker;
 }
 
@@ -700,17 +757,6 @@ void SideWidget::setFocus()
   mSelectButton->setFocus();
 }
 
-void SideWidget::initRecipientPicker()
-{
-  if ( mRecipientPicker ) return;
-
-  mRecipientPicker = new RecipientsPicker( this );
-  connect( mRecipientPicker, SIGNAL( pickedRecipient( const Recipient & ) ),
-    SIGNAL( pickedRecipient( const Recipient & ) ) );
-
-  mPickerPositioner = new KWindowPositioner( this, mRecipientPicker );
-}
-
 void SideWidget::setTotal( int recipients, int lines )
 {
 #if 0
@@ -737,11 +783,12 @@ void SideWidget::pickRecipient()
     "Email address of recipient" );
   if ( !rec.isEmpty() ) emit pickedRecipient( rec );
 #else
-  mRecipientPicker->setDefaultType( mView->activeLine()->recipientType() );
-  mRecipientPicker->setRecipients( mView->recipients() );
-  mRecipientPicker->show();
+  RecipientsPicker *p = picker();
+  p->setDefaultType( mView->activeLine()->recipientType() );
+  p->setRecipients( mView->recipients() );
+  p->show();
   mPickerPositioner->reposition();
-  mRecipientPicker->raise();
+  p->raise();
 #endif
 }
 
@@ -815,10 +862,10 @@ void RecipientsEditor::setRecipientString( const QString &str,
   QStringList r = KPIM::splitEmailAddrList( str );
   QStringList::ConstIterator it;
   for( it = r.begin(); it != r.end(); ++it ) {
-    if ( count++ > GlobalSettings::maximumRecipients() ) {
+    if ( count++ > GlobalSettings::self()->maximumRecipients() ) {
       KMessageBox::sorry( this,
         i18n("Truncating recipients list to %1 of %2 entries.")
-        .arg( GlobalSettings::maximumRecipients() )
+        .arg( GlobalSettings::self()->maximumRecipients() )
         .arg( r.count() ) );
       break;
     }
@@ -828,8 +875,6 @@ void RecipientsEditor::setRecipientString( const QString &str,
 
 QString RecipientsEditor::recipientString( Recipient::Type type )
 {
-  kdDebug() << "recipientString() " << Recipient::typeLabel( type ) << endl;
-
   QString str;
 
   Recipient::List recipients = mRecipientsView->recipients();
@@ -888,9 +933,9 @@ void RecipientsEditor::setFocusBottom()
   mRecipientsView->setFocusBottom();
 }
 
-void RecipientsEditor::setFirstColumnWidth( int w )
+int RecipientsEditor::setFirstColumnWidth( int w )
 {
-  mRecipientsView->setFirstColumnWidth( w );
+  return mRecipientsView->setFirstColumnWidth( w );
 }
 
 void RecipientsEditor::selectRecipients()
