@@ -2,7 +2,7 @@
     aboutdata.h
 
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2005 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -11,23 +11,24 @@
     (at your option) any later version.
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software Foundation,
-    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, US
+    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, US
 */
 
 #include <kaboutdata.h>
+#include <kdepimmacros.h>
 
 #ifndef KNODE_ABOUTDATA_H
 #define KNODE_ABOUTDATA_H
 
 namespace KNode
 {
-  class AboutData : public KAboutData
+  class KDE_EXPORT AboutData : public KAboutData
   {
   public:
     AboutData();
     ~AboutData();
   };
 
-} // namespace KMail
+} // namespace KNode
 
 #endif
