@@ -1,6 +1,4 @@
 /*
-    knarticlemanager.cpp
-
     KNode, the KDE newsreader
     Copyright (c) 1999-2005 the KNode authors.
     See file AUTHORS for details
@@ -11,7 +9,7 @@
     (at your option) any later version.
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software Foundation,
-    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, US
+    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, US
 */
 
 #include <kmessagebox.h>
@@ -22,13 +20,13 @@
 #include <kwin.h>
 #include <ktempfile.h>
 
+#include "articlewidget.h"
 #include "knmainwidget.h"
 #include "knglobals.h"
 #include "knconfigmanager.h"
 #include "utilities.h"
 #include "knarticlemanager.h"
 #include "kngroupmanager.h"
-#include "knarticlewidget.h"
 #include "knsearchdialog.h"
 #include "knfiltermanager.h"
 #include "knfolder.h"
@@ -43,6 +41,8 @@
 #include "knfoldermanager.h"
 #include "headerview.h"
 
+using namespace KNode;
+
 
 KNArticleManager::KNArticleManager() : QObject(0,0)
 {
@@ -66,13 +66,11 @@ KNArticleManager::~KNArticleManager()
 
 void KNArticleManager::deleteTempFiles()
 {
-  KTempFile *file;
-
-  while ((file = t_empFiles.first())) {
-    file->unlink();                 // deletes file
-    t_empFiles.removeFirst();
-    delete file;
+  for ( QValueList<KTempFile*>::Iterator it = mTempFiles.begin(); it != mTempFiles.end(); ++it ) {
+    (*it)->unlink();
+    delete (*it);
   }
+  mTempFiles.clear();
 }
 
 
@@ -84,7 +82,8 @@ void KNArticleManager::saveContentToFile(KMime::Content *c, QWidget *parent)
 
   if (file) {
     QByteArray data=c->decodedContent();
-    file->writeBlock(data.data(), data.size());
+    if (file->writeBlock(data.data(), data.size()) == -1 )
+      KNHelper::displayExternalFileError( parent );
   }
 }
 
@@ -107,7 +106,8 @@ void KNArticleManager::saveArticleToFile(KNArticle *a, QWidget *parent)
 
   if (file) {
     QCString tmp=a->encodedContent(false);
-    file->writeBlock(tmp.data(), tmp.size());
+    if ( file->writeBlock(tmp.data(), tmp.size()) == -1 )
+      KNHelper::displayExternalFileError( parent );
   }
 }
 
@@ -123,11 +123,12 @@ QString KNArticleManager::saveContentToTemp(KMime::Content *c)
     bool found=false;
 
     // lets see if the tempfile-path is still valid...
-    for (tmpFile=t_empFiles.first(); tmpFile; tmpFile=t_empFiles.next())
-      if (tmpFile->name()==path) {
+    for ( QValueList<KTempFile*>::Iterator it = mTempFiles.begin(); it != mTempFiles.end(); ++it ) {
+      if ( (*it)->name() == path ) {
         found = true;
         break;
       }
+    }
 
     if (found)
       return path;
@@ -142,7 +143,7 @@ QString KNArticleManager::saveContentToTemp(KMime::Content *c)
     return QString::null;
   }
 
-  t_empFiles.append(tmpFile);
+  mTempFiles.append(tmpFile);
   QFile *f=tmpFile->file();
   QByteArray data=c->decodedContent();
   f->writeBlock(data.data(), data.size());
@@ -191,11 +192,11 @@ void KNArticleManager::showHdrs(bool clear)
   if(g_roup) {
     KNRemoteArticle *art, *ref, *current;
 
-    current=static_cast<KNRemoteArticle*>(knGlobals.top->articleView()->article());
+    current = static_cast<KNRemoteArticle*>( knGlobals.top->articleViewer()->article() );
 
     if(current && (current->collection() != g_roup)) {
       current=0;
-      knGlobals.top->articleView()->setArticle(0);
+      knGlobals.top->articleViewer()->setArticle( 0 );
     }
 
     if(g_roup->isLocked())
@@ -320,7 +321,7 @@ void KNArticleManager::showHdrs(bool clear)
 
   if(setFirstChild && v_iew->firstChild()) {
     v_iew->setCurrentItem(v_iew->firstChild());
-    knGlobals.top->articleView()->setArticle(0);
+    knGlobals.top->articleViewer()->setArticle( 0 );
   }
 
   knGlobals.setStatusMsg(QString::null);
@@ -468,7 +469,7 @@ bool KNArticleManager::unloadArticle(KNArticle *a, bool force)
   if (!force && a->isNotUnloadable())
     return false;
 
-  if (!force && KNArticleWidget::articleVisible(a))
+  if ( !force && ( ArticleWidget::articleVisible( a ) ) )
     return false;
 
   if (!force && (a->type()==KMime::Base::ATlocal) &&
@@ -479,7 +480,7 @@ bool KNArticleManager::unloadArticle(KNArticle *a, bool force)
     if (!force)
       return false;
 
-  KNArticleWidget::articleRemoved(a);
+  ArticleWidget::articleRemoved( a );
   if (!a->type()==KMime::Base::ATlocal)
     knGlobals.artFactory->deleteComposerForArticle(static_cast<KNLocalArticle*>(a));
   a->KMime::Content::clear();
@@ -494,42 +495,42 @@ void KNArticleManager::copyIntoFolder(KNArticle::List &l, KNFolder *f)
 {
   if(!f) return;
 
-  KNArticle *org;
   KNLocalArticle *loc;
   KNLocalArticle::List l2;
 
-  for(org=l.first(); org; org=l.next()) {
-    if(!org->hasContent())
+  for ( KNArticle::List::Iterator it = l.begin(); it != l.end(); ++it ) {
+    if ( !(*it)->hasContent() )
       continue;
     loc=new KNLocalArticle(0);
     loc->setEditDisabled(true);
-    loc->setContent(org->encodedContent());
+    loc->setContent( (*it)->encodedContent() );
     loc->parse();
     l2.append(loc);
   }
 
-  if(!l2.isEmpty()) {
+  if ( !l2.isEmpty() ) {
 
     f->setNotUnloadable(true);
 
-    if (!f->isLoaded() && !knGlobals.folderManager()->loadHeaders(f)) {
-      l2.setAutoDelete(true);
+    if ( !f->isLoaded() && !knGlobals.folderManager()->loadHeaders( f ) ) {
+      for ( KNLocalArticle::List::Iterator it = l2.begin(); it != l2.end(); ++it )
+        delete (*it);
       l2.clear();
       f->setNotUnloadable(false);
       return;
     }
 
-    if(!f->saveArticles(&l2)) {
-      for(KNLocalArticle *a=l2.first(); a; a=l2.next()) {
-        if(a->isOrphant())
-          delete a; // ok, this is ugly; we simply delete orphant articles
+    if( !f->saveArticles( l2 ) ) {
+      for ( KNLocalArticle::List::Iterator it = l2.begin(); it != l2.end(); ++it ) {
+        if ( (*it)->isOrphant() )
+          delete (*it); // ok, this is ugly; we simply delete orphant articles
         else
-          a->KMime::Content::clear(); // no need to keep them in memory
+          (*it)->KMime::Content::clear(); // no need to keep them in memory
       }
       KNHelper::displayInternalFileError();
     } else {
-      for(KNLocalArticle *a=l2.first(); a; a=l2.next())
-        a->KMime::Content::clear(); // no need to keep them in memory
+      for ( KNLocalArticle::List::Iterator it = l2.begin(); it != l2.end(); ++it )
+        (*it)->KMime::Content::clear(); // no need to keep them in memory
       knGlobals.memoryManager()->updateCacheEntry(f);
     }
 
@@ -550,14 +551,14 @@ void KNArticleManager::moveIntoFolder(KNLocalArticle::List &l, KNFolder *f)
     return;
   }
 
-  if(f->saveArticles(&l)) {
-    for(KNLocalArticle *a=l.first(); a; a=l.next())
-      knGlobals.memoryManager()->updateCacheEntry( a );
+  if ( f->saveArticles( l ) ) {
+    for ( KNLocalArticle::List::Iterator it = l.begin(); it != l.end(); ++it )
+      knGlobals.memoryManager()->updateCacheEntry( (*it) );
     knGlobals.memoryManager()->updateCacheEntry(f);
   } else {
-    for(KNLocalArticle *a=l.first(); a; a=l.next())
-      if(a->isOrphant())
-        delete a; // ok, this is ugly; we simply delete orphant articles
+    for ( KNLocalArticle::List::Iterator it = l.begin(); it != l.end(); ++it )
+      if ( (*it)->isOrphant() )
+        delete (*it); // ok, this is ugly; we simply delete orphant articles
     KNHelper::displayInternalFileError();
   }
 
@@ -569,13 +570,13 @@ bool KNArticleManager::deleteArticles(KNLocalArticle::List &l, bool ask)
 {
   if(ask) {
     QStringList lst;
-    for(KNLocalArticle *a=l.first(); a; a=l.next()) {
-      if(a->isLocked())
+    for ( KNLocalArticle::List::Iterator it = l.begin(); it != l.end(); ++it ) {
+      if ( (*it)->isLocked() )
         continue;
-      if(a->subject()->isEmpty())
+      if ( (*it)->subject()->isEmpty() )
         lst << i18n("no subject");
       else
-        lst << a->subject()->asUnicodeString();
+        lst << (*it)->subject()->asUnicodeString();
     }
     if( KMessageBox::Cancel == KMessageBox::warningContinueCancelList(
       knGlobals.topWidget, i18n("Do you really want to delete these articles?"), lst,
@@ -583,117 +584,90 @@ bool KNArticleManager::deleteArticles(KNLocalArticle::List &l, bool ask)
       return false;
   }
 
-  for(KNLocalArticle *a=l.first(); a; a=l.next())
-    knGlobals.memoryManager()->removeCacheEntry(a);
+  for ( KNLocalArticle::List::Iterator it = l.begin(); it != l.end(); ++it )
+    knGlobals.memoryManager()->removeCacheEntry( (*it) );
 
   KNFolder *f=static_cast<KNFolder*>(l.first()->collection());
-  if(f) {
-    f->removeArticles(&l, true);
+  if ( f ) {
+    f->removeArticles( l, true );
     knGlobals.memoryManager()->updateCacheEntry( f );
   }
   else {
-    for(KNLocalArticle *a=l.first(); a; a=l.next())
-      delete a;
+    for ( KNLocalArticle::List::Iterator it = l.begin(); it != l.end(); ++it )
+      delete (*it);
   }
 
   return true;
 }
 
 
-void KNArticleManager::setAllRead(bool r)
+void KNArticleManager::setAllRead( bool read, int lastcount )
 {
-  if(!g_roup)
+  if ( !g_roup )
     return;
 
-  int new_count = 0, ignore_count=0;
-  KNRemoteArticle *a;
-  for(int i=0; i<g_roup->length(); i++) {
-    a=g_roup->at(i);
-    if(a->getReadFlag()!=r) {
-      a->setRead(r);
-      a->setChanged(true);
-      if(a->isNew() && !a->isIgnored())
-        new_count++;
-      if(a->isIgnored())
-        ignore_count++;
-    }
-  }
+  int groupLength = g_roup->length();
+  int newCount = g_roup->newCount();
+  int readCount = g_roup->readCount();
+  int offset = lastcount;
 
-  g_roup->updateThreadInfo();
-  if(r) {
-    g_roup->setReadCount(g_roup->length());
-    g_roup->setNewCount(0);
-  } else {
-    g_roup->setReadCount(ignore_count);
-    g_roup->setNewCount(new_count);
-  }
-
-  g_roup->updateListItem();
-  showHdrs(true);
-}
-
-
-void KNArticleManager::setAllRead(int lastcount, bool r)
-{
-  if(!g_roup)
-    return;
-
-  int groupLength=g_roup->length();
-  int newCount=g_roup->newCount();
-  int readCount=g_roup->readCount();
-
-  if(lastcount>groupLength)
-    lastcount=groupLength;
+  if ( lastcount > groupLength || lastcount < 0 )
+    offset = groupLength;
 
   KNRemoteArticle *a;
-  for(int i=groupLength-lastcount; i<groupLength; i++) {
-    a=g_roup->at(i);
-    if(a->getReadFlag()!=r) {
-      a->setRead(r);
-      a->setChanged(true);
-      if(!r) {
+  for ( int i = groupLength - offset; i < groupLength; i++ ) {
+    a = g_roup->at( i );
+    if ( a->getReadFlag() != read && !a->isIgnored() ) {
+      a->setRead( read );
+      a->setChanged( true );
+      if ( !read ) {
         readCount--;
-        if (a->isNew() && !a->isIgnored())
+        if ( a->isNew() )
           newCount++;
-      }
-      else {
+      } else {
         readCount++;
-        if (a->isNew() && !a->isIgnored())
+        if ( a->isNew() )
           newCount--;
       }
     }
   }
 
   g_roup->updateThreadInfo();
-  g_roup->setReadCount(readCount);
-  g_roup->setNewCount(newCount);
+  if ( lastcount < 0 && read ) {
+    // HACK: try to hide the effects of the ignore/filter new/unread count bug
+    g_roup->setReadCount( groupLength );
+    g_roup->setNewCount( 0 );
+  } else {
+    g_roup->setReadCount( readCount );
+    g_roup->setNewCount( newCount );
+  }
 
   g_roup->updateListItem();
-  showHdrs(true);
+  showHdrs( true );
 }
 
 
 void KNArticleManager::setRead(KNRemoteArticle::List &l, bool r, bool handleXPosts)
 {
-  if(l.isEmpty())
+  if ( l.isEmpty() )
     return;
 
-  KNRemoteArticle *a=l.first(), *ref=0;
-  KNGroup *g=static_cast<KNGroup*>(a->collection() );
+  KNRemoteArticle *ref = 0;
+  KNGroup *g=static_cast<KNGroup*>( l.first()->collection() );
   int changeCnt=0, idRef=0;
 
-  for( ; a; a=l.next()) {
+  for ( KNRemoteArticle::List::Iterator it = l.begin(); it != l.end(); ++it ) {
     if( r && knGlobals.configManager()->readNewsGeneral()->markCrossposts() &&
-        handleXPosts && a->newsgroups()->isCrossposted() ) {
+        handleXPosts && (*it)->newsgroups()->isCrossposted() ) {
 
-      QStringList groups = a->newsgroups()->getGroups();
+      QStringList groups = (*it)->newsgroups()->getGroups();
       KNGroup *targetGroup=0;
       KNRemoteArticle *xp=0;
       KNRemoteArticle::List al;
-      QCString mid=a->messageID()->as7BitString(false);
+      QCString mid = (*it)->messageID()->as7BitString( false );
 
-      for (QStringList::Iterator it = groups.begin(); it != groups.end(); ++it) {
-        targetGroup = knGlobals.groupManager()->group(*it, g->account());
+      for ( QStringList::Iterator it2 = groups.begin(); it2 != groups.end(); ++it2 ) {
+        targetGroup = knGlobals.groupManager()->group(*it2, g->account());
         if (targetGroup) {
           if (targetGroup->isLoaded() && (xp=targetGroup->byMessageId(mid)) ) {
             al.clear();
@@ -706,25 +680,25 @@ void KNArticleManager::setRead(KNRemoteArticle::List &l, bool r, bool handleXPos
       }
     }
 
-    else if(a->getReadFlag()!=r) {
-      a->setRead(r);
-      a->setChanged(true);
-      a->updateListItem();
+    else if ( (*it)->getReadFlag() != r ) {
+      (*it)->setRead( r );
+      (*it)->setChanged( true );
+      (*it)->updateListItem();
 
-      if (!a->isIgnored()) {
+      if ( !(*it)->isIgnored() ) {
         changeCnt++;
-        idRef=a->idRef();
+        idRef = (*it)->idRef();
 
-        while(idRef!=0) {
+        while ( idRef != 0 ) {
           ref=g->byId(idRef);
           if(r) {
             ref->decUnreadFollowUps();
-            if(a->isNew())
+            if ( (*it)->isNew() )
               ref->decNewFollowUps();
           }
           else {
             ref->incUnreadFollowUps();
-            if(a->isNew())
+            if ( (*it)->isNew() )
               ref->incNewFollowUps();
           }
 
@@ -738,12 +712,12 @@ void KNArticleManager::setRead(KNRemoteArticle::List &l, bool r, bool handleXPos
 
         if(r) {
           g->incReadCount();
-          if(a->isNew())
+          if ( (*it)->isNew() )
             g->decNewCount();
         }
         else {
           g->decReadCount();
-          if(a->isNew())
+          if ( (*it)->isNew() )
             g->incNewCount();
         }
       }
@@ -762,7 +736,6 @@ void KNArticleManager::setAllNotNew()
 {
   if ( !g_roup )
     return;
-  kdDebug(5003) << k_funcinfo << "begin" << endl;
   KNRemoteArticle *a;
   for ( int i = 0; i < g_roup->length(); ++i) {
     a = g_roup->at(i);
@@ -787,19 +760,19 @@ bool KNArticleManager::toggleWatched(KNRemoteArticle::List &l)
   KNGroup *g=static_cast<KNGroup*>(a->collection() );
   int changeCnt=0, idRef=0;
 
-  for(KNRemoteArticle *a=l.first(); a; a=l.next()) {
-    if (a->isIgnored()) {
-      a->setIgnored(false);
+  for ( KNRemoteArticle::List::Iterator it = l.begin(); it != l.end(); ++it ) {
+    if ( (*it)->isIgnored() ) {
+      (*it)->setIgnored(false);
 
-      if (!a->getReadFlag()) {
+      if ( !(*it)->getReadFlag() ) {
         changeCnt++;
-        idRef=a->idRef();
+        idRef = (*it)->idRef();
 
-        while(idRef!=0) {
+        while ( idRef != 0 ) {
           ref=g->byId(idRef);
 
           ref->incUnreadFollowUps();
-          if(a->isNew())
+          if ( (*it)->isNew() )
             ref->incNewFollowUps();
 
           if(ref->listItem() &&
@@ -810,14 +783,14 @@ bool KNArticleManager::toggleWatched(KNRemoteArticle::List &l)
           idRef=ref->idRef();
         }
         g->decReadCount();
-        if(a->isNew())
+        if ( (*it)->isNew() )
           g->incNewCount();
       }
     }
 
-    a->setWatched(watch);
-    a->updateListItem();
-    a->setChanged(true);
+    (*it)->setWatched( watch );
+    (*it)->updateListItem();
+    (*it)->setChanged( true );
   }
 
   if(changeCnt>0) {
@@ -835,31 +808,30 @@ bool KNArticleManager::toggleIgnored(KNRemoteArticle::List &l)
   if(l.isEmpty())
     return true;
 
-  KNRemoteArticle *a=l.first(), *ref=0;
-  bool ignore=(!a->isIgnored());
-  KNGroup *g=static_cast<KNGroup*>(a->collection() );
-  int changeCnt=0, idRef=0;
+  KNRemoteArticle *ref = 0;
+  bool ignore = !l.first()->isIgnored();
+  KNGroup *g = static_cast<KNGroup*>( l.first()->collection() );
+  int changeCnt = 0, idRef = 0;
 
-  for(; a; a=l.next()) {
-    a->setWatched(false);
-    if (a->isIgnored() != ignore) {
-      a->setIgnored(ignore);
+  for ( KNRemoteArticle::List::Iterator it = l.begin(); it != l.end(); ++it ) {
+    (*it)->setWatched(false);
+    if ( (*it)->isIgnored() != ignore ) {
+      (*it)->setIgnored( ignore );
 
-      if (!a->getReadFlag()) {
+      if ( !(*it)->getReadFlag() ) {
         changeCnt++;
-        idRef=a->idRef();
+        idRef = (*it)->idRef();
 
-        while(idRef!=0) {
-          ref=g->byId(idRef);
+        while ( idRef != 0 ) {
+          ref = g->byId( idRef );
 
-          if(ignore) {
+          if ( ignore ) {
             ref->decUnreadFollowUps();
-            if(a->isNew())
+            if ( (*it)->isNew() )
               ref->decNewFollowUps();
-          }
-          else {
+          } else {
             ref->incUnreadFollowUps();
-            if(a->isNew())
+            if ( (*it)->isNew() )
               ref->incNewFollowUps();
           }
 
@@ -871,21 +843,20 @@ bool KNArticleManager::toggleIgnored(KNRemoteArticle::List &l)
           idRef=ref->idRef();
         }
 
-        if(ignore) {
+        if ( ignore ) {
           g->incReadCount();
-          if(a->isNew())
+          if ( (*it)->isNew() )
             g->decNewCount();
-        }
-        else {
+        } else {
           g->decReadCount();
-          if(a->isNew())
+          if ( (*it)->isNew() )
             g->incNewCount();
         }
 
       }
     }
-    a->updateListItem();
-    a->setChanged(true);
+    (*it)->updateListItem();
+    (*it)->setChanged(true);
   }
 
   if(changeCnt>0) {
@@ -900,26 +871,30 @@ bool KNArticleManager::toggleIgnored(KNRemoteArticle::List &l)
 
 void  KNArticleManager::rescoreArticles(KNRemoteArticle::List &l)
 {
-  KNRemoteArticle *a=l.first();
+  if ( l.isEmpty() )
+    return;
 
-  if (a) {
-    KNGroup *g=static_cast<KNGroup*>(a->collection());
-    KScoringManager *sm = knGlobals.scoringManager();
-    sm->initCache(g->groupname());
+  KNGroup *g = static_cast<KNGroup*>( l.first()->collection() );
+  KScoringManager *sm = knGlobals.scoringManager();
+  sm->initCache(g->groupname());
 
-    for(; a; a=l.next()) {
-      int defScore = 0;
-      if (a->isIgnored())
-        defScore = knGlobals.configManager()->scoring()->ignoredThreshold();
-      else if (a->isWatched())
-        defScore = knGlobals.configManager()->scoring()->watchedThreshold();
-      a->setScore(defScore);
+  for ( KNRemoteArticle::List::Iterator it = l.begin(); it != l.end(); ++it ) {
+    int defScore = 0;
+    if ( (*it)->isIgnored())
+      defScore = knGlobals.configManager()->scoring()->ignoredThreshold();
+    else if ( (*it)->isWatched() )
+      defScore = knGlobals.configManager()->scoring()->watchedThreshold();
+    (*it)->setScore(defScore);
 
-      KNScorableArticle sa(a);
-      sm->applyRules(sa);
-      a->updateListItem();
-      a->setChanged(true);
-    }
+    bool read = (*it)->isRead();
+
+    KNScorableArticle sa( (*it) );
+    sm->applyRules(sa);
+    (*it)->updateListItem();
+    (*it)->setChanged( true );
+
+    if ( !read && (*it)->isRead() != read )
+      g_roup->incReadCount();
   }
 }
 
@@ -929,14 +904,14 @@ void KNArticleManager::processJob(KNJobData *j)
   if(j->type()==KNJobData::JTfetchArticle && !j->canceled()) {
     if(j->success()) {
       KNRemoteArticle *a=static_cast<KNRemoteArticle*>(j->data());
-      KNArticleWidget::articleChanged(a);
+      ArticleWidget::articleChanged( a );
       if(!a->isOrphant()) //orphant articles are deleted by the displaying widget
         knGlobals.memoryManager()->updateCacheEntry(a);
       if(a->listItem())
         a->updateListItem();
     }
     else
-      KNArticleWidget::articleLoadError(static_cast<KNRemoteArticle*>(j->data()), j->errorString());
+      ArticleWidget::articleLoadError(static_cast<KNRemoteArticle*>(j->data()), j->errorString());
   }
 
   delete j;
