@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 
    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
@@ -33,6 +33,7 @@
 #define KNOTES_PART_P_H
 
 #include <qlayout.h>
+#include <qlabel.h>
 
 #include <kactioncollection.h>
 #include <klocale.h>
@@ -40,6 +41,7 @@
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <kiconeffect.h>
+#include <klineedit.h>
 #include <ktoolbar.h>
 #include <kpopupmenu.h>
 #include <kdialogbase.h>
@@ -104,7 +106,15 @@ class KNoteEditDlg : public KDialogBase, virtual public KXMLGUIClient
       QWidget *page = plainPage();
       QVBoxLayout *layout = new QVBoxLayout( page );
 
+      QHBoxLayout *hbl = new QHBoxLayout( layout, marginHint() );
+      QLabel *label = new QLabel( page);
+      label->setText( i18n( "Name:" ) );
+      hbl->addWidget( label,0 );
+      mTitleEdit= new KLineEdit( page, "name" );
+      hbl->addWidget( mTitleEdit, 1,Qt::AlignVCenter  );
+
       mNoteEdit = new KNoteEdit( actionCollection(), page );
+      mNoteEdit->setTextFormat( RichText );
       mNoteEdit->setFocus();
 
       KXMLGUIBuilder builder( page );
@@ -127,7 +137,18 @@ class KNoteEditDlg : public KDialogBase, virtual public KXMLGUIClient
       mNoteEdit->setText( text );
     }
 
+    QString title() const
+    {
+      return mTitleEdit->text();
+    }
+
+    void setTitle( const QString& text )
+    {
+      mTitleEdit->setText( text );
+    }
+
   private:
+    KLineEdit *mTitleEdit;
     KNoteEdit  *mNoteEdit;
     KToolBar   *mTool;
     KPopupMenu *mEditMenu;
