@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -211,7 +211,7 @@ void ExportWebDialog::setupEventPage()
 /*
 void ExportWebDialog::setupJournalPage()
 {
-  mJournalPage = addPage(i18n("Journals"));
+  mJournalPage = addPage(i18n("Journal"));
   QVBoxLayout *topLayout = new QVBoxLayout( mJournalPage, 10 );
 
   QHBox *hbox = new QHBox( mJournalPage );
