@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
     Copyright (c) 2001, 2002 Cornelius Schumacher <schumacher@kde.org>
     Copyright (C) 2004 Reinhold Kainhofer <reinhold@kainhofer.com>
+    Copyright (C) 2005 Thomas Zander <zander@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -25,10 +26,9 @@
 #define _FILTEREDITDIALOG_H
 
 #include <kdialogbase.h>
+#include "filteredit_base.h"
 
-class QComboBox;
-class QPushButton;
-class FilterEdit_base;
+class FilterEdit;
 template<class type> class QPtrList;
 
 namespace KPIM { class CategorySelectDialog; }
@@ -38,9 +38,9 @@ using namespace KCal;
 
 /**
   This is the class to add/edit a calendar filter.
-  
+
   @short Creates a dialog box to create/edit a calendar filter
-  @author Cornelius Schumacher
+  @author Cornelius Schumacher, Thomas Zander
 */
 class FilterEditDialog : public KDialogBase
 {
@@ -49,39 +49,52 @@ class FilterEditDialog : public KDialogBase
     FilterEditDialog(QPtrList<CalFilter> *,QWidget *parent=0, const char *name=0);
     virtual ~FilterEditDialog();
 
-  public slots:
-    void updateFilterList();
-    void updateCategoryConfig();
-
   signals:
     void filterChanged();
     void editCategories();
 
-  protected slots:
-    void slotDefault();
-    void slotApply();
-    void slotOk();
-
-    void slotAdd();
-    void slotRemove();
-    
-    void filterSelected();
-    void editCategorySelection();
-    void updateCategorySelection(const QStringList &categories);
-
-  protected:
-    void readFilter(CalFilter *);
-    void writeFilter(CalFilter *);
+  public slots:
+    void updateFilterList();
+    void updateCategoryConfig();
+    void setDialogConsistent(bool consistent);
 
   private:
+    FilterEdit *mFilterEdit;
+
+  protected slots:
+    void slotApply();
+    void slotOk();
+};
+
+class FilterEdit : public FilterEdit_base
+{
+    Q_OBJECT
+  public:
+    FilterEdit(QPtrList<CalFilter> *filters, QWidget *parent);
+    ~FilterEdit();
+
+    void updateFilterList();
+    void saveChanges();
+    void updateCategoryConfig();
+
+  signals:
+    void dataConsistent(bool);
+    void filterChanged();
+    void editCategories();
+
+  private slots:
+    void filterSelected();
+    void bNewPressed();
+    void bDeletePressed();
+    void updateSelectedName(const QString &);
+    void updateCategorySelection(const QStringList &categories);
+    void editCategorySelection();
+
+  private:
+    void filterSelected(CalFilter *f);
+
     QPtrList<CalFilter> *mFilters;
-
-    QComboBox *mSelectionCombo;
-    QPushButton *mRemoveButton;
-    FilterEdit_base *mEditor;
-
-    QStringList mCategories;
-
+    CalFilter *current;
     KPIM::CategorySelectDialog *mCategorySelectDialog;
 };
 
