@@ -1,6 +1,7 @@
 /*
     This file is part of KOrganizer.
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -46,6 +47,7 @@
 #include <kconfig.h>
 #include <kglobal.h>
 #include <kglobalsettings.h>
+#include <kholidays.h>
 
 #include <libkcal/calendar.h>
 #include <libkcal/icaldrag.h>
@@ -61,9 +63,6 @@
 #include "koprefs.h"
 #include "koagenda.h"
 #include "koagendaitem.h"
-#ifndef KORG_NOPRINTER
-#include "calprinter.h"
-#endif
 
 #include "koincidencetooltip.h"
 #include "kogroupware.h"
@@ -79,6 +78,7 @@ TimeLabels::TimeLabels(int rows,QWidget *parent,const char *name,WFlags f) :
   QScrollView(parent,name,f)
 {
   mRows = rows;
+  mMiniWidth = 0;
 
   mCellHeight = KOPrefs::instance()->mHourSize*4;
 
@@ -87,9 +87,31 @@ TimeLabels::TimeLabels(int rows,QWidget *parent,const char *name,WFlags f) :
   setHScrollBarMode(AlwaysOff);
   setVScrollBarMode(AlwaysOff);
 
-  resizeContents(50,mRows * mCellHeight);
+  resizeContents(50, int(mRows * mCellHeight) );
 
   viewport()->setBackgroundMode( PaletteBackground );
+
+  mMousePos = new QFrame(this);
+  mMousePos->setLineWidth(0);
+  mMousePos->setMargin(0);
+  mMousePos->setBackgroundColor(Qt::red);
+  mMousePos->setFixedSize(width(), 1);
+  addChild(mMousePos, 0, 0);
+}
+
+void TimeLabels::mousePosChanged(const QPoint &pos)
+{
+  moveChild(mMousePos, 0, pos.y());
+}
+
+void TimeLabels::showMousePos()
+{
+  mMousePos->show();
+}
+
+void TimeLabels::hideMousePos()
+{
+  mMousePos->hide();
 }
 
 void TimeLabels::setCellHeight(int height)
@@ -103,51 +125,76 @@ void TimeLabels::setCellHeight(int height)
 */
 void TimeLabels::drawContents(QPainter *p,int cx, int cy, int cw, int ch)
 {
-  // bug:  the parameters cx, cy, cw, ch are the areas that need to be
+  // bug:  the parameters cx and cw are the areas that need to be
   //       redrawn, not the area of the widget.  unfortunately, this
   //       code assumes the latter...
 
   // now, for a workaround...
-  // these two assignments fix the weird redraw bug
-  cx = contentsX() + 2;
-  cw = contentsWidth() - 2;
-  int visWidth = visibleWidth();
-  double cellHeight=mCellHeight;
-  if (mAgenda) cellHeight=(4*mAgenda->gridSpacingY());
+  cx = contentsX() + frameWidth()*2;
+  cw = contentsWidth() ;
   // end of workaround
 
-  int cell = ((int)(cy/cellHeight));
-  double y = (cell * cellHeight);
+  int cell = ((int)(cy/mCellHeight));
+  double y = cell * mCellHeight;
   QFontMetrics fm = fontMetrics();
   QString hour;
-  QString suffix;
-  QString fullTime;
+  QString suffix = "am";
+  int timeHeight =  fm.ascent();
+  QFont nFont = font();
+  p->setFont( font() );
 
-  while (y < cy + ch) {
-    p->drawLine(cx,(int)y,cx+cw,(int)y);
+  if (!KGlobal::locale()->use12Clock()) {
+      suffix = "00";
+  } else
+      if (cell > 11) suffix = "pm";
+
+  if ( timeHeight >  mCellHeight ) {
+    timeHeight = int(mCellHeight-1);
+    int pointS = nFont.pointSize();
+    while ( pointS > 4 ) {
+      nFont.setPointSize( pointS );
+      fm = QFontMetrics( nFont );
+      if ( fm.ascent() < mCellHeight )
+        break;
+      -- pointS;
+    }
+    fm = QFontMetrics( nFont );
+    timeHeight = fm.ascent();
+  }
+  //timeHeight -= (timeHeight/4-2);
+  QFont sFont = nFont;
+  sFont.setPointSize( sFont.pointSize()/2 );
+  QFontMetrics fmS(  sFont );
+  int startW = mMiniWidth - frameWidth()-2 ;
+  int tw2 = fmS.width(suffix);
+  int divTimeHeight = (timeHeight-1) /2 - 1;
+  //testline
+  //p->drawLine(0,0,0,contentsHeight());
+  while (y < cy + ch+mCellHeight) {
+    // hour, full line
+    p->drawLine( cx, int(y), cw+2, int(y) );
     hour.setNum(cell);
-    suffix = "am";
-
     // handle 24h and am/pm time formats
     if (KGlobal::locale()->use12Clock()) {
-      if (cell > 11) suffix = "pm";
+      if (cell == 12) suffix = "pm";
       if (cell == 0) hour.setNum(12);
       if (cell > 12) hour.setNum(cell - 12);
-    } else {
-      suffix = ":00";
     }
 
-    // create string in format of "XX:XX" or "XXpm/am"
-    fullTime = hour + suffix;
-
     // center and draw the time label
-    QRect r( cx, (int)y+3, visWidth-4, (int)(y+cellHeight-3) );
-    p->drawText ( r, Qt::AlignHCenter | Qt::AlignTop | Qt::SingleLine, fullTime );
+    int timeWidth = fm.width(hour);
+    int offset = startW - timeWidth - tw2 -1 ;
+    p->setFont( nFont );
+    p->drawText( offset, int(y+timeHeight), hour);
+    p->setFont( sFont );
+    offset = startW - tw2;
+    p->drawText( offset, int(y+timeHeight-divTimeHeight), suffix);
 
     // increment indices
-    y += cellHeight;
+    y += mCellHeight;
     cell++;
   }
+
 }
 
 /**
@@ -155,33 +202,38 @@ void TimeLabels::drawContents(QPainter *p,int cx, int cy, int cw, int ch)
 */
 int TimeLabels::minimumWidth() const
 {
-  QFontMetrics fm = fontMetrics();
-
-  int borderWidth = 4;
-
-  // the maximum width possible
-  int width = fm.width("88:88") + 2*borderWidth;
-
-  return width;
+  return mMiniWidth;
 }
 
 /** updates widget's internal state */
 void TimeLabels::updateConfig()
 {
-  // set the font
-//  config->setGroup("Fonts");
-//  QFont font = config->readFontEntry("TimeBar Font");
   setFont(KOPrefs::instance()->mTimeBarFont);
 
+  QString test = "20";
+  if ( KGlobal::locale()->use12Clock() )
+      test = "12";
+  mMiniWidth = fontMetrics().width( test );
+  if ( KGlobal::locale()->use12Clock() )
+      test = "pm";
+  else {
+      test = "00";
+  }
+  QFont sFont = font();
+  sFont.setPointSize(  sFont.pointSize()/2 );
+  QFontMetrics fmS(   sFont );
+  mMiniWidth += fmS.width(  test ) + frameWidth()*2+4 ;
   // update geometry restrictions based on new settings
-  setFixedWidth(minimumWidth());
+  setFixedWidth(  mMiniWidth );
 
   // update HourSize
   mCellHeight = KOPrefs::instance()->mHourSize*4;
-  if (mCellHeight>mAgenda->gridSpacingY())
-    mCellHeight=(int)(4*mAgenda->gridSpacingY());
-        // FIXME: Why the heck do we set the width to 50???
-  resizeContents(50,mRows * mCellHeight);
+  // If the agenda is zoomed out so that more then 24 would be shown,
+  // the agenda only shows 24 hours, so we need to take the cell height
+  // from the agenda, which is larger than the configured one!
+  if ( mCellHeight < 4*mAgenda->gridSpacingY() )
+       mCellHeight = 4*mAgenda->gridSpacingY();
+  resizeContents( mMiniWidth, int(mRows * mCellHeight+1) );
 }
 
 /** update time label positions */
@@ -195,6 +247,10 @@ void TimeLabels::positionChanged()
 void TimeLabels::setAgenda(KOAgenda* agenda)
 {
   mAgenda = agenda;
+
+  connect(mAgenda, SIGNAL(mousePosSignal(const QPoint &)), this, SLOT(mousePosChanged(const QPoint &)));
+  connect(mAgenda, SIGNAL(enterAgenda()), this, SLOT(showMousePos()));
+  connect(mAgenda, SIGNAL(leaveAgenda()), this, SLOT(hideMousePos()));
 }
 
 
@@ -218,8 +274,8 @@ EventIndicator::EventIndicator(Location loc,QWidget *parent,const char *name)
   mLocation = loc;
   mTopLayout = 0;
 
-  if (mLocation == Top) mPixmap = KOGlobals::self()->smallIcon("1uparrow");
-  else mPixmap = KOGlobals::self()->smallIcon("1downarrow");
+  if (mLocation == Top) mPixmap = KOGlobals::self()->smallIcon("upindicator");
+  else mPixmap = KOGlobals::self()->smallIcon("downindicator");
 
   setMinimumHeight(mPixmap.height());
 }
@@ -267,8 +323,8 @@ void EventIndicator::enableColumn(int column, bool enable)
 ////////////////////////////////////////////////////////////////////////////
 
 
-KOAlternateLabel::KOAlternateLabel(QString shortlabel, QString longlabel,
-    QString extensivelabel, QWidget *parent, const char *name )
+KOAlternateLabel::KOAlternateLabel(const QString &shortlabel, const QString &longlabel,
+    const QString &extensivelabel, QWidget *parent, const char *name )
   : QLabel(parent, name), mTextTypeFixed(false), mShortText(shortlabel),
     mLongText(longlabel), mExtensiveText(extensivelabel)
 {
@@ -357,7 +413,8 @@ void KOAlternateLabel::setText( const QString &text ) {
 ////////////////////////////////////////////////////////////////////////////
 
 KOAgendaView::KOAgendaView(Calendar *cal,QWidget *parent,const char *name) :
-  KOEventView (cal,parent,name), mAllowAgendaUpdate( true ), mUpdateItem( 0 )
+  KOEventView (cal,parent,name), mExpandButton( 0 ), mAllowAgendaUpdate( true ),
+  mUpdateItem( 0 )
 {
   mSelectedDates.append(QDate::currentDate());
 
@@ -367,12 +424,14 @@ KOAgendaView::KOAgendaView(Calendar *cal,QWidget *parent,const char *name) :
 
   bool isRTL = KOGlobals::self()->reverseLayout();
 
-  if ( KOPrefs::instance()->mVerticalScreen ) {
-    mExpandedPixmap = KOGlobals::self()->smallIcon( "1downarrow" );
-    mNotExpandedPixmap = KOGlobals::self()->smallIcon( "1uparrow" );
-  } else {
-    mExpandedPixmap = KOGlobals::self()->smallIcon( isRTL ? "1leftarrow" : "1rightarrow" );
-    mNotExpandedPixmap = KOGlobals::self()->smallIcon( isRTL ? "1rightarrow" : "1leftarrow" );
+  if ( KOPrefs::instance()->compactDialogs() ) {
+    if ( KOPrefs::instance()->mVerticalScreen ) {
+      mExpandedPixmap = KOGlobals::self()->smallIcon( "1downarrow" );
+      mNotExpandedPixmap = KOGlobals::self()->smallIcon( "1uparrow" );
+    } else {
+      mExpandedPixmap = KOGlobals::self()->smallIcon( isRTL ? "1leftarrow" : "1rightarrow" );
+      mNotExpandedPixmap = KOGlobals::self()->smallIcon( isRTL ? "1rightarrow" : "1leftarrow" );
+    }
   }
 
   QBoxLayout *topLayout = new QVBoxLayout(this);
@@ -407,11 +466,16 @@ KOAgendaView::KOAgendaView(Calendar *cal,QWidget *parent,const char *name) :
   // Create all-day agenda widget
   mDummyAllDayLeft = new QVBox( mAllDayFrame );
 
-  mExpandButton = new QPushButton(mDummyAllDayLeft);
-  mExpandButton->setPixmap( mNotExpandedPixmap );
-  mExpandButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed,
-                                QSizePolicy::Fixed ) );
-  connect( mExpandButton, SIGNAL( clicked() ), SIGNAL( toggleExpand() ) );
+  if ( KOPrefs::instance()->compactDialogs() ) {
+    mExpandButton = new QPushButton(mDummyAllDayLeft);
+    mExpandButton->setPixmap( mNotExpandedPixmap );
+    mExpandButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed,
+                                  QSizePolicy::Fixed ) );
+    connect( mExpandButton, SIGNAL( clicked() ), SIGNAL( toggleExpand() ) );
+  } else {
+    QLabel *label = new QLabel( i18n("All Day"), mDummyAllDayLeft );
+    label->setAlignment( Qt::AlignRight | Qt::AlignVCenter | Qt::WordBreak );
+  }
 
   mAllDayAgenda = new KOAgenda(1,mAllDayFrame);
   QWidget *dummyAllDayRight = new QWidget(mAllDayFrame);
@@ -455,23 +519,21 @@ KOAgendaView::KOAgendaView(Calendar *cal,QWidget *parent,const char *name) :
   // these blank widgets make the All Day Event box line up with the agenda
   dummyAllDayRight->setFixedWidth(mAgenda->verticalScrollBar()->width());
   dummyAgendaRight->setFixedWidth(mAgenda->verticalScrollBar()->width());
-  mDummyAllDayLeft->setFixedWidth(mTimeLabels->width());
+
+  updateTimeBarWidth();
 
   // Scrolling
   connect(mAgenda->verticalScrollBar(),SIGNAL(valueChanged(int)),
           mTimeLabels, SLOT(positionChanged()));
+
+  connect( mAgenda,
+    SIGNAL( zoomView( const int, const QPoint & ,const Qt::Orientation ) ),
+    SLOT( zoomView( const int, const QPoint &, const Qt::Orientation ) ) );
+
   connect(mTimeLabels->verticalScrollBar(),SIGNAL(valueChanged(int)),
           SLOT(setContentsPos(int)));
 
   // Create Events, depends on type of agenda
-  connect( mAgenda, SIGNAL(newEventSignal(const QPoint &)),
-                    SLOT(newEvent(const QPoint &)));
-  connect( mAllDayAgenda, SIGNAL(newEventSignal(const QPoint &)),
-                          SLOT(newEventAllDay(const QPoint &)));
-  connect( mAgenda, SIGNAL(newEventSignal(const QPoint &, const QPoint &)),
-                    SLOT(newEvent(const QPoint &, const QPoint &)));
-  connect( mAllDayAgenda, SIGNAL(newEventSignal(const QPoint &, const QPoint &)),
-                          SLOT(newEventAllDay(const QPoint &)));
   connect( mAgenda, SIGNAL(newTimeSpanSignal(const QPoint &, const QPoint &)),
                     SLOT(newTimeSpanSelected(const QPoint &, const QPoint &)));
   connect( mAllDayAgenda, SIGNAL(newTimeSpanSignal(const QPoint &, const QPoint &)),
@@ -518,11 +580,6 @@ void KOAgendaView::connectAgenda( KOAgenda *agenda, QPopupMenu *popup,
   connect( agenda, SIGNAL( deleteIncidenceSignal( Incidence * ) ),
                    SIGNAL( deleteIncidenceSignal( Incidence * ) ) );
 
-  connect( agenda, SIGNAL( incidenceChanged( Incidence *, Incidence * ) ),
-                   SIGNAL( incidenceChanged( Incidence *, Incidence * ) ) );
-  connect( agenda, SIGNAL( incidenceAdded( Incidence * ) ),
-                   SIGNAL( incidenceAdded( Incidence * ) ) );
-
   connect( agenda, SIGNAL( startMultiModify( const QString & ) ),
                    SIGNAL( startMultiModify( const QString & ) ) );
   connect( agenda, SIGNAL( endMultiModify() ),
@@ -549,6 +606,133 @@ void KOAgendaView::connectAgenda( KOAgenda *agenda, QPopupMenu *popup,
 
 }
 
+void KOAgendaView::zoomInVertically( )
+{
+  KOPrefs::instance()->mHourSize++;
+  mAgenda->updateConfig();
+  mAgenda->checkScrollBoundaries();
+
+  mTimeLabels->updateConfig();
+  mTimeLabels->positionChanged();
+  mTimeLabels->repaint();
+
+  updateView();
+}
+
+void KOAgendaView::zoomOutVertically( )
+{
+
+  if ( KOPrefs::instance()->mHourSize > 4 ) {
+
+    KOPrefs::instance()->mHourSize--;
+    mAgenda->updateConfig();
+    mAgenda->checkScrollBoundaries();
+
+    mTimeLabels->updateConfig();
+    mTimeLabels->positionChanged();
+    mTimeLabels->repaint();
+
+    updateView();
+  }
+}
+
+void KOAgendaView::zoomInHorizontally( const QDate &date)
+{
+  QDate begin;
+  QDate newBegin;
+  QDate dateToZoom = date;
+  int ndays,count;
+
+  begin = mSelectedDates.first();
+  ndays = begin.daysTo( mSelectedDates.last() );
+
+  // zoom with Action and are there a selected Incidence?, Yes, I zoom in to it.
+  if ( ! dateToZoom.isValid () )
+    dateToZoom=mAgenda->selectedIncidenceDate();
+
+  if( !dateToZoom.isValid() ) {
+    if ( ndays > 1 ) {
+      newBegin=begin.addDays(1);
+      count = ndays-1;
+      emit zoomViewHorizontally ( newBegin , count );
+    }
+  } else {
+    if ( ndays <= 2 ) {
+      newBegin = dateToZoom;
+      count = 1;
+    } else  {
+      newBegin = dateToZoom.addDays( -ndays/2 +1  );
+      count = ndays -1 ;
+    }
+    emit zoomViewHorizontally ( newBegin , count );
+  }
+}
+
+void KOAgendaView::zoomOutHorizontally( const QDate &date )
+{
+  QDate begin;
+  QDate newBegin;
+  QDate dateToZoom = date;
+  int ndays,count;
+
+  begin = mSelectedDates.first();
+  ndays = begin.daysTo( mSelectedDates.last() );
+
+  // zoom with Action and are there a selected Incidence?, Yes, I zoom out to it.
+  if ( ! dateToZoom.isValid () )
+    dateToZoom=mAgenda->selectedIncidenceDate();
+
+  if ( !dateToZoom.isValid() ) {
+    newBegin = begin.addDays(-1);
+    count = ndays+3 ;
+  } else {
+    newBegin = dateToZoom.addDays( -ndays/2-1 );
+    count = ndays+3;
+  }
+
+  if ( abs( count ) >= 31 )
+    kdDebug(5850) << "change to the mounth view?"<<endl;
+  else
+    //We want to center the date
+    emit zoomViewHorizontally( newBegin, count );
+}
+
+void KOAgendaView::zoomView( const int delta, const QPoint &pos,
+  const Qt::Orientation orient )
+{
+  static QDate zoomDate;
+  static QTimer t( this );
+
+
+  //Zoom to the selected incidence, on the other way
+  // zoom to the date on screen after the first mousewheel move.
+  if ( orient == Qt::Horizontal ) {
+    QDate date=mAgenda->selectedIncidenceDate();
+    if ( date.isValid() )
+      zoomDate=date;
+    else{
+      if ( !t.isActive() ) {
+        zoomDate= mSelectedDates[pos.x()];
+      }
+      t.start ( 1000,true );
+    }
+    if ( delta > 0 )
+      zoomOutHorizontally( zoomDate );
+    else
+      zoomInHorizontally( zoomDate );
+  } else {
+    // Vertical zoom
+    QPoint posConstentsOld = mAgenda->gridToContents(pos);
+    if ( delta > 0 ) {
+      zoomOutVertically();
+    } else {
+      zoomInVertically();
+    }
+    QPoint posConstentsNew = mAgenda->gridToContents(pos);
+    mAgenda->scrollBy( 0, posConstentsNew.y() - posConstentsOld.y() );
+  }
+}
+
 void KOAgendaView::createDayLabels()
 {
 //  kdDebug(5850) << "KOAgendaView::createDayLabels()" << endl;
@@ -589,6 +773,18 @@ void KOAgendaView::createDayLabels()
     }
     dayLayout->addWidget(dayLabel);
 
+    // if a holiday region is selected, show the holiday name
+    if ( KOGlobals::self()->holidays() ) {
+      QString text = KOGlobals::self()->holidays()->shortText( date );
+      if ( !text.isEmpty() ) {
+        // use a KOAlternateLabel so when the text doesn't fit any more a tooltip is used
+        KOAlternateLabel*label = new KOAlternateLabel( text, text, QString::null, mDayLabels );
+        label->setMinimumWidth(1);
+        label->setAlignment(AlignCenter);
+        dayLayout->addWidget(label);
+      }
+    }
+
 #ifndef KORG_NOPLUGINS
     CalendarDecoration::List cds = KOCore::self()->calendarDecorations();
     CalendarDecoration *it;
@@ -665,8 +861,20 @@ bool KOAgendaView::eventDurationHint( QDateTime &startDt, QDateTime &endDt,
                                       bool &allDay )
 {
   if ( selectionStart().isValid() ) {
-    startDt = selectionStart();
-    endDt = selectionEnd();
+    QDateTime start = selectionStart();
+    QDateTime end = selectionEnd();
+
+    if ( start.secsTo( end ) == 15*60 ) {
+      // One cell in the agenda view selected, e.g.
+      // because of a double-click, => Use the default duration
+      QTime defaultDuration( KOPrefs::instance()->mDefaultDuration.time() );
+      int addSecs = ( defaultDuration.hour()*3600 ) +
+                    ( defaultDuration.minute()*60 );
+      end = start.addSecs( addSecs );
+    }
+
+    startDt = start;
+    endDt = end;
     allDay = selectedIsAllDay();
     return true;
   }
@@ -709,13 +917,13 @@ void KOAgendaView::updateConfig()
   mAllDayAgenda->updateConfig();
 
   // widget synchronization
-  //TODO: find a better way, maybe signal/slot
+  // FIXME: find a better way, maybe signal/slot
   mTimeLabels->positionChanged();
 
   // for some reason, this needs to be called explicitly
   mTimeLabels->repaint();
 
-  mDummyAllDayLeft->setFixedWidth(mTimeLabels->width());
+  updateTimeBarWidth();
 
   // ToolTips displaying summary of events
   KOAgendaItem::toolTipGroup()->setEnabled(KOPrefs::instance()
@@ -728,10 +936,21 @@ void KOAgendaView::updateConfig()
   updateView();
 }
 
+void KOAgendaView::updateTimeBarWidth()
+{
+  int width;
+
+  width = mDummyAllDayLeft->fontMetrics().width( i18n("All Day") );
+  width = QMAX( width, mTimeLabels->width() );
+
+  mDummyAllDayLeft->setFixedWidth( width );
+  mTimeLabels->setFixedWidth( width );
+}
+
 
 void KOAgendaView::updateEventDates( KOAgendaItem *item )
 {
-//  kdDebug(5850) << "KOAgendaView::updateEventDates(): " << item->text() << endl;
+  kdDebug(5850) << "KOAgendaView::updateEventDates(): " << item->text() << endl;
 
   QDateTime startDt,endDt;
 
@@ -753,7 +972,6 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
 
   Incidence *incidence = item->incidence();
   if ( !incidence ) return;
-  Incidence *oldIncidence = incidence->clone();
 
   QTime startTime(0,0,0), endTime(0,0,0);
   if ( incidence->doesFloat() ) {
@@ -769,41 +987,40 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
   }
 
 //  kdDebug(5850) << "KOAgendaView::updateEventDates(): now setting dates" << endl;
-  Incidence *i = incidence->clone();
-  if ( i->type() == "Event" ) {
-    startDt = i->dtStart();
+  // FIXME: use a visitor here
+  if ( incidence->type() == "Event" ) {
+    startDt = incidence->dtStart();
     startDt = startDt.addDays( daysOffset );
     startDt.setTime( startTime );
     endDt = startDt.addDays( daysLength );
     endDt.setTime( endTime );
-    Event*ev = static_cast<Event*>(i);
-    if( i->dtStart() == startDt && ev->dtEnd() == endDt ) {
+    Event*ev = static_cast<Event*>(incidence);
+    if( incidence->dtStart() == startDt && ev->dtEnd() == endDt ) {
       // No change
-      delete i;
       return;
     }
-    i->setDtStart( startDt );
+    incidence->setDtStart( startDt );
     ev->setDtEnd( endDt );
-  } else if ( i->type() == "Todo" ) {
-    Todo *td = static_cast<Todo*>(i);
-    startDt = td->dtStart();
-    startDt = startDt.addDays( daysOffset );
-    endDt = td->dtDue();
-    endDt = endDt.addDays( daysOffset );
+  } else if ( incidence->type() == "Todo" ) {
+    Todo *td = static_cast<Todo*>(incidence);
+    startDt = td->hasStartDate() ? td->dtStart() : td->dtDue();
+    startDt = thisDate.addDays( td->dtDue().daysTo( startDt ) );
+    startDt.setTime( startTime );
+    endDt.setDate( thisDate );
     endDt.setTime( endTime );
 
     if( td->dtDue() == endDt ) {
       // No change
-      delete i;
       return;
     }
-    td->setDtDue( endDt );
   }
-  // TODO: Adjusting the recurrence should really go to CalendarView so this
+  // FIXME: Adjusting the recurrence should really go to CalendarView so this
   // functionality will also be available in other views!
+  // TODO_Recurrence: This does not belong here, and I'm not really sure
+  // how it's supposed to work anyway.
   Recurrence *recur = incidence->recurrence();
-  if ( recur && (recur->doesRecur()!=Recurrence::rNone) && (daysOffset!=0) ) {
-    switch ( recur->doesRecur() ) {
+/*  if ( recur->doesRecur() && daysOffset != 0 ) {
+    switch ( recur->recurrenceType() ) {
       case Recurrence::rYearlyPos: {
         int freq = recur->frequency();
         int duration = recur->duration();
@@ -866,7 +1083,7 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
               QDate endDt( recur->endDate() );
               QPtrList<Recurrence::rMonthPos> monthPos( recur->monthPositions() );
               if ( !monthPos.isEmpty() ) {
-          // TODO: How shall I adapt the day x of week Y if we move the date across month borders???
+          // FIXME: How shall I adapt the day x of week Y if we move the date across month borders???
           // for now, just use the date of the moved item and assume the recurrence only occurs on that day.
           // That's fine for korganizer, but might mess up other organizers.
                 QBitArray rDays( 7 );
@@ -903,7 +1120,7 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
                 } else {
                   recur->setMonthly( Recurrence::rMonthlyDay, freq, duration );
                 }
-        // TODO: How shall I adapt the n-th day if we move the date across month borders???
+        // FIXME: How shall I adapt the n-th day if we move the date across month borders???
         // for now, just use the date of the moved item and assume the recurrence only occurs on that day.
         // That's fine for korganizer, but might mess up other organizers.
                 recur->addMonthlyDay( thisDate.day() );
@@ -934,46 +1151,45 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
     if ( recur->duration()==0 ) { // end by date
       recur->setEndDate( recur->endDate().addDays( daysOffset ) );
     }
-    KMessageBox::information( this, i18n("A recurring incidence was moved to a different day. The recurrence settings have been updated with that move. Please check them in the incidence editor."), i18n("Recurrence Moved"), "RecurrenceMoveInAgendaWarning" );
+    KMessageBox::information( this, i18n("A recurring calendar item was moved "
+                              "to a different day. The recurrence settings "
+                              "have been updated with that move. Please check "
+                              "them in the editor."),
+                              i18n("Recurrence Moved"),
+                              "RecurrenceMoveInAgendaWarning" );
+  }*/
+
+  // FIXME: use a visitor here
+  if ( incidence->type() == "Event" ) {
+    incidence->setDtStart( startDt );
+    (static_cast<Event*>( incidence ) )->setDtEnd( endDt );
+  } else if ( incidence->type() == "Todo" ) {
+    Todo *td = static_cast<Todo*>( incidence );
+    if ( td->hasStartDate() )
+      td->setDtStart( startDt );
+    td->setDtDue( endDt );
   }
 
-  i->setRevision( i->revision() + 1 );
-  if( !KOPrefs::instance()->mUseGroupwareCommunication ||
-       KOGroupware::instance()->sendICalMessage( this, KCal::Scheduler::Request,
-  i ) ) {
-    if ( i->type() == "Event" ) {
-      incidence->setDtStart( startDt );
-      (static_cast<Event*>( incidence ) )->setDtEnd( endDt );
-    } else if ( i->type() == "Todo" ) {
-      (static_cast<Todo*>( incidence ) )->setDtDue( endDt );
-    }
-    incidence->setRevision( i->revision() );
-    item->setItemDate( startDt.date() );
+  item->setItemDate( startDt.date() );
 
-    KOIncidenceToolTip::remove( item );
-    KOIncidenceToolTip::add( item, incidence, KOAgendaItem::toolTipGroup() );
+  KOIncidenceToolTip::remove( item );
+  KOIncidenceToolTip::add( item, incidence, KOAgendaItem::toolTipGroup() );
 
-    // don't update the agenda as the item already has the correct coordinates.
-    // an update would delete the current item and recreate it, but we are still
-    // using a pointer to that item! => CRASH
-    enableAgendaUpdate( false );
-    // We need to do this in a timer to make sure we are not deleting the item
-    // we are currently working on, which would lead to crashes
-    // Only the actually moved agenda item is already at the correct position and mustn't be
-    // recreated. All others have to!!!
-    if ( incidence->doesRecur() ) {
-      mUpdateItem = incidence;
-      QTimer::singleShot( 0, this, SLOT( doUpdateItem() ) );
-    }
+  // don't update the agenda as the item already has the correct coordinates.
+  // an update would delete the current item and recreate it, but we are still
+  // using a pointer to that item! => CRASH
+  enableAgendaUpdate( false );
+  // We need to do this in a timer to make sure we are not deleting the item
+  // we are currently working on, which would lead to crashes
+  // Only the actually moved agenda item is already at the correct position and mustn't be
+  // recreated. All others have to!!!
+  if ( incidence->doesRecur() ) {
+    mUpdateItem = incidence;
+    QTimer::singleShot( 0, this, SLOT( doUpdateItem() ) );
+  }
 
-    emit incidenceChanged( oldIncidence, incidence );
     enableAgendaUpdate( true );
-  } else {
-    updateView();
-  }
 
-  delete i;
-  delete oldIncidence;
 //  kdDebug(5850) << "KOAgendaView::updateEventDates() done " << endl;
 }
 
@@ -1009,9 +1225,10 @@ void KOAgendaView::showIncidences( const Incidence::List & )
   kdDebug(5850) << "KOAgendaView::showIncidences( const Incidence::List & ) is not yet implemented" << endl;
 }
 
-void KOAgendaView::insertIncidence( Incidence *incidence, QDate curDate,
+void KOAgendaView::insertIncidence( Incidence *incidence, const QDate &curDate,
                                     int curCol )
 {
+  // FIXME: Use a visitor here, or some other method to get rid of the dynamic_cast's
   Event *event = dynamic_cast<Event *>( incidence );
   Todo  *todo  = dynamic_cast<Todo  *>( incidence );
 
@@ -1026,25 +1243,33 @@ void KOAgendaView::insertIncidence( Incidence *incidence, QDate curDate,
   int endX;
   if ( event ) {
     beginX = curDate.daysTo( incidence->dtStart().date() ) + curCol;
-    endX = curDate.daysTo( event->dtEnd().date() ) + curCol;
+    endX = curDate.daysTo( event->dateEnd() ) + curCol;
   } else if ( todo ) {
+    if ( ! todo->hasDueDate() ) return;  // todo shall not be displayed if it has no date
     beginX = curDate.daysTo( todo->dtDue().date() ) + curCol;
     endX = beginX;
   } else {
     return;
   }
 
-  if ( incidence->doesFloat() ) {
+  if ( todo && todo->isOverdue() ) {
+    mAllDayAgenda->insertAllDayItem( incidence, curDate, curCol, curCol );
+  } else if ( incidence->doesFloat() ) {
+// FIXME: This breaks with recurring multi-day events!
     if ( incidence->recurrence()->doesRecur() ) {
       mAllDayAgenda->insertAllDayItem( incidence, curDate, curCol, curCol );
     } else {
+      // Insert multi-day events only on the first day, otherwise it will
+      // appear multiple times
       if ( ( beginX <= 0 && curCol == 0 ) || beginX == curCol ) {
         mAllDayAgenda->insertAllDayItem( incidence, curDate, beginX, endX );
       }
     }
   } else if ( event && event->isMultiDay() ) {
     int startY = mAgenda->timeToY( event->dtStart().time() );
-    int endY = mAgenda->timeToY( event->dtEnd().time() ) - 1;
+    QTime endtime( event->dtEnd().time() );
+    if ( endtime == QTime( 0, 0, 0 ) ) endtime = QTime( 23, 59, 59 );
+    int endY = mAgenda->timeToY( endtime ) - 1;
     if ( (beginX <= 0 && curCol == 0) || beginX == curCol ) {
       mAgenda->insertMultiItem( event, curDate, beginX, endX, startY, endY );
     }
@@ -1062,7 +1287,9 @@ void KOAgendaView::insertIncidence( Incidence *incidence, QDate curDate,
     int startY = 0, endY = 0;
     if ( event ) {
       startY = mAgenda->timeToY( incidence->dtStart().time() );
-      endY = mAgenda->timeToY( event->dtEnd().time() ) - 1;
+      QTime endtime( event->dtEnd().time() );
+      if ( endtime == QTime( 0, 0, 0 ) ) endtime = QTime( 23, 59, 59 );
+      endY = mAgenda->timeToY( endtime ) - 1;
     }
     if ( todo ) {
       QTime t = todo->dtDue().time();
@@ -1079,7 +1306,8 @@ void KOAgendaView::insertIncidence( Incidence *incidence, QDate curDate,
 void KOAgendaView::changeIncidenceDisplayAdded( Incidence *incidence )
 {
   Todo *todo = dynamic_cast<Todo *>(incidence);
-  if ( !calendar()->filter()->filterIncidence( incidence ) ||
+  CalFilter *filter = calendar()->filter();
+  if ( filter && !filter->filterIncidence( incidence ) ||
      ( todo && !KOPrefs::instance()->showAllDayTodo() ) )
     return;
 
@@ -1092,6 +1320,7 @@ void KOAgendaView::changeIncidenceDisplayAdded( Incidence *incidence )
     QDate curDate;
     for( dit = mSelectedDates.begin(); dit != mSelectedDates.end(); ++dit ) {
       curDate = *dit;
+// FIXME: This breaks with recurring multi-day events!
       if ( incidence->recursOn( curDate ) ) {
         insertIncidence( incidence, curDate );
       }
@@ -1101,24 +1330,19 @@ void KOAgendaView::changeIncidenceDisplayAdded( Incidence *incidence )
 
   QDate endDt;
   if ( incidence->type() == "Event" )
-    endDt = (static_cast<Event *>(incidence))->dtEnd().date();
+    endDt = (static_cast<Event *>(incidence))->dateEnd();
   if ( todo ) {
-    bool overdue = (!todo->isCompleted()) &&
-                   (todo->dtDue() < QDate::currentDate() );
-    endDt = overdue ? QDate::currentDate()
-                    : todo->dtDue().date();
+    endDt = todo->isOverdue() ? QDate::currentDate()
+                              : todo->dtDue().date();
+
     if ( endDt >= f && endDt <= l ) {
       insertIncidence( incidence, endDt );
       return;
     }
   }
 
-  if ( startDt <= l ) {
-    if ( startDt >= f ) {
-      insertIncidence( incidence, startDt );
-    } else if ( endDt >= f ) {
-      insertIncidence( incidence, endDt );
-    }
+  if ( startDt >= f && startDt <= l ) {
+    insertIncidence( incidence, startDt );
   }
 }
 
@@ -1130,7 +1354,8 @@ void KOAgendaView::changeIncidenceDisplay( Incidence *incidence, int mode )
         // recreating everything even causes troubles: dropping to the day matrix
         // recreates the agenda items, but the evaluation is still in an agendaItems' code,
         // which was deleted in the mean time. Thus KOrg crashes...
-      changeIncidenceDisplayAdded( incidence );
+      if ( mAllowAgendaUpdate )
+        changeIncidenceDisplayAdded( incidence );
       break;
     }
     case KOGlobals::INCIDENCEEDITED: {
@@ -1198,7 +1423,9 @@ void KOAgendaView::fillAgenda()
 //    kdDebug(5850) << "KOAgendaView::fillAgenda(): " << currentDate.toString()
 //              << endl;
 
-    dayEvents = calendar()->events(currentDate,true);
+    dayEvents = calendar()->events(currentDate,
+                                   EventSortStartDate,
+                                   SortDirectionAscending);
 
     // Default values, which can never be reached
     mMinY[curCol] = mAgenda->timeToY(QTime(23,59)) + 1;
@@ -1232,7 +1459,7 @@ void KOAgendaView::fillAgenda()
 
         // ToDo items shall be displayed for the day they are due, but only showed today if they are already overdue.
         // Already completed items can be displayed on their original due date
-        bool overdue = (!todo->isCompleted()) && (todo->dtDue() < today);
+        bool overdue = todo->isOverdue();
 
         if ( (( todo->dtDue().date() == currentDate) && !overdue) ||
              (( currentDate == today) && overdue) ||
@@ -1261,6 +1488,7 @@ void KOAgendaView::fillAgenda()
   }
 
   mAgenda->checkScrollBoundaries();
+  updateEventIndicators();
 
 //  mAgenda->viewport()->update();
 //  mAllDayAgenda->viewport()->update();
@@ -1282,109 +1510,60 @@ void KOAgendaView::clearView()
   mAgenda->clear();
 }
 
-void KOAgendaView::printPreview(CalPrinter *calPrinter, const QDate &fd,
-                               const QDate &td)
-{
-#ifndef KORG_NOPRINTER
-  if (fd == td)
-    calPrinter->preview(CalPrinter::Day, fd, td);
-  else
-    calPrinter->preview(CalPrinter::Week, fd, td);
-#endif
-}
-
 CalPrinter::PrintType KOAgendaView::printType()
 {
   if ( currentDateCount() == 1 ) return CalPrinter::Day;
   else return CalPrinter::Week;
 }
 
-void KOAgendaView::newEvent( const QPoint &pos)
-{
-  if (!mSelectedDates.count()) return;
-
-  QDate day = mSelectedDates[pos.x()];
-
-  QTime time = mAgenda->gyToTime(pos.y());
-  QDateTime dt(day,time);
-
-  emit newEventSignal(dt);
-}
-
-void KOAgendaView::newEvent(const QPoint &start, const QPoint &end)
-{
-  if (!mSelectedDates.count()) return;
-
-  QDate dayStart = mSelectedDates[start.x()];
-  QDate dayEnd = mSelectedDates[end.x()];
-
-  QTime timeStart = mAgenda->gyToTime( start.y() );
-  QTime timeEnd = mAgenda->gyToTime( end.y() + 1 );
-
-  QDateTime dtStart(dayStart,timeStart);
-  QDateTime dtEnd(dayEnd,timeEnd);
-
-  emit newEventSignal(dtStart,dtEnd);
-}
-
-void KOAgendaView::newEventAllDay( const QPoint &start )
-{
-  if (!mSelectedDates.count()) return;
-
-  QDate day = mSelectedDates[start.x()];
-
-  emit newEventSignal(day);
-}
-
-void KOAgendaView::updateEventIndicatorTop(int newY)
+void KOAgendaView::updateEventIndicatorTop( int newY )
 {
   uint i;
-  for(i=0;i<mMinY.size();++i) {
-    if (newY >= mMinY[i]) mEventIndicatorTop->enableColumn(i,true);
-    else mEventIndicatorTop->enableColumn(i,false);
+  for( i = 0; i < mMinY.size(); ++i ) {
+    mEventIndicatorTop->enableColumn( i, newY >= mMinY[i] );
   }
-
   mEventIndicatorTop->update();
 }
 
-void KOAgendaView::updateEventIndicatorBottom(int newY)
+void KOAgendaView::updateEventIndicatorBottom( int newY )
 {
   uint i;
-  for(i=0;i<mMaxY.size();++i) {
-    if (newY <= mMaxY[i]) mEventIndicatorBottom->enableColumn(i,true);
-    else mEventIndicatorBottom->enableColumn(i,false);
+  for( i = 0; i < mMaxY.size(); ++i ) {
+    mEventIndicatorBottom->enableColumn( i, newY <= mMaxY[i] );
   }
-
   mEventIndicatorBottom->update();
 }
 
 void KOAgendaView::slotTodoDropped( Todo *todo, const QPoint &gpos, bool allDay )
 {
-  if (gpos.x()<0 || gpos.y()<0) return;
+  if ( gpos.x()<0 || gpos.y()<0 ) return;
   QDate day = mSelectedDates[gpos.x()];
-  QTime time = mAgenda->gyToTime(gpos.y());
-  QDateTime newTime(day, time);
+  QTime time = mAgenda->gyToTime( gpos.y() );
+  QDateTime newTime( day, time );
 
-  if (todo) {
-    Todo *existingTodo = calendar()->todo(todo->uid());
-    if(existingTodo) {
+  if ( todo ) {
+    Todo *existingTodo = calendar()->todo( todo->uid() );
+    if ( existingTodo ) {
       kdDebug(5850) << "Drop existing Todo" << endl;
       Todo *oldTodo = existingTodo->clone();
-      existingTodo->setDtDue( newTime );
-      existingTodo->setFloats( allDay );
-      existingTodo->setHasDueDate( true );
-      existingTodo->setRevision( existingTodo->revision() + 1 );
-      emit incidenceChanged( oldTodo, existingTodo );
+      if ( mChanger && mChanger->beginChange( existingTodo ) ) {
+        existingTodo->setDtDue( newTime );
+        existingTodo->setFloats( allDay );
+        existingTodo->setHasDueDate( true );
+        mChanger->changeIncidence( oldTodo, existingTodo );
+        mChanger->endChange( existingTodo );
+      } else {
+        KMessageBox::sorry( this, i18n("Unable to modify this to-do, "
+                            "because it cannot be locked.") );
+      }
       delete oldTodo;
     } else {
       kdDebug(5850) << "Drop new Todo" << endl;
       todo->setDtDue( newTime );
       todo->setFloats( allDay );
-      existingTodo->setHasDueDate( true );
-      if ( calendar()->addTodo( todo ) ) {
-        emit incidenceAdded(todo);
-      } else {
-        KODialogManager::errorSaveTodo( this );
+      todo->setHasDueDate( true );
+      if ( !mChanger->addIncidence( todo ) ) {
+        KODialogManager::errorSaveIncidence( this, todo );
       }
     }
   }
@@ -1439,12 +1618,12 @@ void KOAgendaView::setHolidayMasks()
   mHolidayMask.resize( mSelectedDates.count() + 1 );
 
   for( uint i = 0; i < mSelectedDates.count(); ++i ) {
-    mHolidayMask[i] = !KOCore::self()->isWorkDay( mSelectedDates[ i ] );
+    mHolidayMask[i] = !KOGlobals::self()->isWorkDay( mSelectedDates[ i ] );
   }
 
   // Store the information about the day before the visible area (needed for
   // overnight working hours) in the last bit of the mask:
-  bool showDay = !KOCore::self()->isWorkDay( mSelectedDates[ 0 ].addDays( -1 ) );
+  bool showDay = !KOGlobals::self()->isWorkDay( mSelectedDates[ 0 ].addDays( -1 ) );
   mHolidayMask[ mSelectedDates.count() ] = showDay;
 
   mAgenda->setHolidayMask( &mHolidayMask );
@@ -1458,6 +1637,8 @@ void KOAgendaView::setContentsPos( int y )
 
 void KOAgendaView::setExpandedButton( bool expanded )
 {
+  if ( !mExpandButton ) return;
+
   if ( expanded ) {
     mExpandButton->setPixmap( mExpandedPixmap );
   } else {
@@ -1527,4 +1708,13 @@ void KOAgendaView::updateEventIndicators()
   mMaxY = mAgenda->maxContentsY();
 
   mAgenda->checkScrollBoundaries();
+  updateEventIndicatorTop( mAgenda->visibleContentsYMin() );
+  updateEventIndicatorBottom( mAgenda->visibleContentsYMax() );
+}
+
+void KOAgendaView::setIncidenceChanger( IncidenceChangerBase *changer )
+{
+  mChanger = changer;
+  mAgenda->setIncidenceChanger( changer );
+  mAllDayAgenda->setIncidenceChanger( changer );
 }
