@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2000,2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -65,11 +66,20 @@ class TimeLabels : public QScrollView
   protected:
     void drawContents( QPainter *p, int cx, int cy, int cw, int ch );
 
+  private slots:
+    /** update the position of the marker showing the mouse position */
+    void mousePosChanged(const QPoint &pos);
+
+    void showMousePos();
+    void hideMousePos();
+
   private:
     int mRows;
-    int mCellHeight;
-
+    double mCellHeight;
+    int mMiniWidth;
     KOAgenda* mAgenda;
+
+    QFrame *mMousePos;  // shows a marker for the current mouse position in y direction
 };
 
 class EventIndicator : public QFrame
@@ -101,8 +111,8 @@ class KOAlternateLabel : public QLabel
 {
     Q_OBJECT
   public:
-    KOAlternateLabel( QString shortlabel, QString longlabel,
-                      QString extensivelabel = QString::null,
+    KOAlternateLabel( const QString &shortlabel, const QString &longlabel,
+                      const QString &extensivelabel = QString::null,
                       QWidget *parent = 0, const char *name = 0 );
     ~KOAlternateLabel();
 
@@ -133,6 +143,8 @@ class KOAgendaView : public KOEventView
     KOAgendaView( Calendar *cal, QWidget *parent = 0, const char *name = 0 );
     virtual ~KOAgendaView();
 
+
+
     /** Returns maximum number of days supported by the koagendaview */
     virtual int maxDatesHint();
 
@@ -151,9 +163,6 @@ class KOAgendaView : public KOEventView
     /** Remove all events from view */
     void clearView();
 
-    virtual void printPreview( CalPrinter *calPrinter,
-                               const QDate &, const QDate & );
-
     CalPrinter::PrintType printType();
 
     /** start-datetime of selection */
@@ -173,18 +182,14 @@ class KOAgendaView : public KOEventView
     virtual void updateView();
     virtual void updateConfig();
     virtual void showDates( const QDate &start, const QDate &end );
-    virtual void showIncidences( const Incidence::List & );
+    virtual void showIncidences( const Incidence::List &incidenceList );
 
-    void insertIncidence( Incidence *incidence, QDate curDate, int curCol = -1 );
+    void insertIncidence( Incidence *incidence, const QDate &curDate, int curCol = -1 );
     void changeIncidenceDisplayAdded( Incidence *incidence );
     void changeIncidenceDisplay( Incidence *incidence, int mode );
 
     void clearSelection();
 
-    void newEvent( const QPoint &pos );
-    void newEvent( const QPoint &start, const QPoint &end );
-    void newEventAllDay( const QPoint &pos );
-
     void startDrag( Incidence * );
 
     void readSettings();
@@ -201,9 +206,19 @@ class KOAgendaView : public KOEventView
     void slotTodoDropped( Todo *, const QPoint &, bool );
 
     void enableAgendaUpdate( bool enable );
+    void setIncidenceChanger( IncidenceChangerBase *changer );
 
+    void zoomInHorizontally( const QDate& date=QDate() );
+    void zoomOutHorizontally( const QDate& date=QDate() );
+
+    void zoomInVertically( );
+    void zoomOutVertically( );
+
+    void zoomView( const int delta, const QPoint &pos,
+      const Qt::Orientation orient=Qt::Horizontal );
   signals:
     void toggleExpand();
+    void zoomViewHorizontally(const QDate &, int count );
 
   protected:
     /** Fill agenda beginning with date startDate */
@@ -229,6 +244,8 @@ class KOAgendaView : public KOEventView
     */
     void updateEventIndicators();
 
+    void updateTimeBarWidth();
+
   protected slots:
     /** Update event belonging to agenda item */
     void updateEventDates( KOAgendaItem *item );
