@@ -5,6 +5,7 @@
     Parts of the source code have been copied from kdpdatebutton.cpp
 
     Copyright (c) 2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -42,9 +43,6 @@
 
 #include <kcalendarsystem.h>
 
-#ifndef KORG_NOPLUGINS
-#include "kocore.h"
-#endif
 #include "koprefs.h"
 #include "koglobals.h"
 #include "kodialogmanager.h"
@@ -101,7 +99,7 @@ const int KODayMatrix::NOSELECTION = -1000;
 const int KODayMatrix::NUMDAYS = 42;
 
 KODayMatrix::KODayMatrix( QWidget *parent, const char *name )
-  : QFrame( parent, name ), mCalendar( 0 )
+  : QFrame( parent, name ), mCalendar( 0 ), mStartDate( 1970, 1, 1 )
 {
   // initialize dynamic arrays
   mDays = new QDate[ NUMDAYS ];
@@ -109,15 +107,10 @@ KODayMatrix::KODayMatrix( QWidget *parent, const char *name )
   mEvents = new int[ NUMDAYS ];
   mToolTip = new DynamicTip( this );
 
-  // set default values used for drawing the matrix
-  mDefaultBackColor = palette().active().base();
-  mDefaultTextColor = palette().active().foreground();
-  mDefaultTextColorShaded = getShadedColor( mDefaultTextColor );
-  mHolidayColorShaded = getShadedColor( KOPrefs::instance()->mHolidayColor );
-  mSelectedDaysColor = QColor( "white" );
   mTodayMarginWidth = 2;
   mSelEnd = mSelStart = NOSELECTION;
   setBackgroundMode( NoBackground );
+  recalculateToday();
 }
 
 void KODayMatrix::setCalendar( Calendar *cal )
@@ -129,7 +122,7 @@ void KODayMatrix::setCalendar( Calendar *cal )
   updateEvents();
 }
 
-QColor KODayMatrix::getShadedColor( QColor color )
+QColor KODayMatrix::getShadedColor( const QColor &color )
 {
   QColor shaded;
   int h = 0;
@@ -217,7 +210,7 @@ void KODayMatrix::recalculateToday()
   updateView( mStartDate );
 }
 
-void KODayMatrix::updateView( QDate actdate )
+void KODayMatrix::updateView( const QDate &actdate )
 {
 //  kdDebug(5850) << "KODayMatrix::updateView() " << actdate.toString() << endl;
 
@@ -251,14 +244,14 @@ void KODayMatrix::updateView( QDate actdate )
     recalculateToday();
   }
 
+  // TODO_Recurrence: If we just change the selection, but not the data, there's
+  // no need to update the whole list of events... This is just a waste of
+  // computational power (and it takes forever!)
   updateEvents();
   for( int i = 0; i < NUMDAYS; i++ ) {
     //if it is a holy day then draw it red. Sundays are consider holidays, too
-#ifndef KORG_NOPLUGINS
-    QString holiStr = KOCore::self()->holiday( mDays[ i ] );
-#else
-    QString holiStr = QString::null;
-#endif
+    QString holiStr = KOGlobals::self()->holiday( mDays[ i ] );
+
     if ( ( KOGlobals::self()->calendarSystem()->dayOfWeek( mDays[ i ] ) ==
            KOGlobals::self()->calendarSystem()->weekDayOfPray() ) ||
          !holiStr.isEmpty() ) {
@@ -281,8 +274,7 @@ void KODayMatrix::updateEvents()
     Event::List::ConstIterator it;
     for( it = eventlist.begin(); it != eventlist.end(); ++it ) {
       Event *event = *it;
-      ushort recurType = event->doesRecur();
-
+      ushort recurType = event->recurrenceType();
       if ( ( recurType == Recurrence::rDaily &&
              !KOPrefs::instance()->mDailyRecur ) ||
            ( recurType == Recurrence::rWeekly &&
@@ -353,8 +345,8 @@ void KODayMatrix::mouseReleaseEvent( QMouseEvent *e )
 
   DateList daylist;
   if ( mSelStart < 0 ) mSelStart = 0;
-  for (int i = mSelStart; i <= mSelEnd; i++) {
-    daylist.append(mDays[i]);
+  for ( int i = mSelStart; i <= mSelEnd; ++i ) {
+    daylist.append( mDays[i] );
   }
   emit selected((const DateList)daylist);
 }
@@ -448,8 +440,8 @@ void KODayMatrix::dropEvent( QDropEvent *e )
     return;
   }
 
-  Todo *existingTodo = 0, *oldTodo = 0;
-  Event *existingEvent = 0, *oldEvent = 0;
+  Todo *existingTodo = 0;
+  Event *existingEvent = 0;
 
   // Find the incidence in the calendar, then we don't need the drag object any more
   if ( event ) existingEvent = mCalendar->event( event->uid() );
@@ -482,74 +474,19 @@ void KODayMatrix::dropEvent( QDropEvent *e )
   }
 
   if ( action == DRAG_COPY  || action == DRAG_MOVE ) {
-  
-    // When copying, clear the UID:
-    if ( action == DRAG_COPY ) {
-      if ( todo ) todo->recreate();
-      if ( event ) event->recreate();
-    } else {
-      if ( existingEvent ) oldEvent = existingEvent->clone();
-      if ( event ) delete event;
-      event = existingEvent;
-      if ( existingTodo ) oldTodo = existingTodo->clone();
-      if ( todo ) delete todo;
-      todo = existingTodo;
-    }
-
     e->accept();
-    if ( event ) {
-      // Adjust date
-      QDateTime start = event->dtStart();
-      QDateTime end = event->dtEnd();
-      int duration = start.daysTo( end );
-      int idx = getDayIndexFrom( e->pos().x(), e->pos().y() );
+    int idx = getDayIndexFrom( e->pos().x(), e->pos().y() );
 
-      start.setDate( mDays[idx] );
-      end.setDate( mDays[idx].addDays( duration ) );
-
-      event->setDtStart( start );
-      event->setDtEnd( end );
-      // When moving, we don't need to insert  the item!
-      if ( action != DRAG_MOVE ) {
-        if ( !mCalendar->addEvent( event ) ) {
-          KODialogManager::errorSaveEvent( this );
-          return;
-        }
-      }
-
-      if ( oldEvent ) {
-        emit incidenceDroppedMove( oldEvent, event );
-      } else {
-        emit incidenceDropped( event );
-      }
+    if ( action == DRAG_COPY ) {
+      if ( event ) emit incidenceDropped( event, mDays[idx] );
+      if ( todo )  emit incidenceDropped( todo, mDays[idx] );
+    } else if ( action == DRAG_MOVE ) {
+      if ( event ) emit incidenceDroppedMove( event, mDays[idx] );
+      if ( todo )  emit incidenceDroppedMove( todo, mDays[idx] );
     }
-    if ( todo ) {
-      // Adjust date
-      QDateTime due = todo->dtDue();
-      int idx = getDayIndexFrom( e->pos().x(), e->pos().y() );
-      due.setDate( mDays[idx] );
-
-      todo->setDtDue( due );
-      todo->setHasDueDate( true );
-
-      // When moving, we don't need to insert  the item!
-      if ( action != DRAG_MOVE ) {
-        if ( !mCalendar->addTodo( todo ) ) {
-          KODialogManager::errorSaveTodo( this );
-        }
-      }
-
-      if ( oldTodo ) {
-        emit incidenceDroppedMove( oldTodo, todo );
-      } else {
-        emit incidenceDropped( todo );
-      }
-    }
-  } else {
-    if ( todo ) delete todo;
-    if ( event ) delete event;
-    e->ignore();
   }
+  delete event;
+  delete todo;
 #endif
 }
 
@@ -570,12 +507,14 @@ void KODayMatrix::paintEvent( QPaintEvent * )
   int selw, selh;
   bool isRTL = KOGlobals::self()->reverseLayout();
 
+  QColorGroup cg = palette().active();
+
   p.begin(  &pm, this );
-  pm.fill( mDefaultBackColor );
+  pm.fill( cg.base() );
 
   // draw topleft frame
-  p.setPen(mDefaultTextColor);
-  p.drawRect(0, 0, sz.width()+1, sz.height()+1);
+  p.setPen( cg.mid() );
+  p.drawRect(0, 0, sz.width()-1, sz.height()-1);
   // don't paint over borders
   p.translate(1,1);
 
@@ -586,40 +525,47 @@ void KODayMatrix::paintEvent( QPaintEvent * )
     col = mSelStart -row*7;
     QColor selcol = KOPrefs::instance()->mHighlightColor;
 
-    if (row == mSelEnd/7) {
-      // Single row selection
-      p.fillRect(isRTL ? (7 - (mSelEnd-mSelStart+1) - col)*dwidth : col*dwidth,
-                  row*dheight, (mSelEnd-mSelStart+1)*dwidth, dheight, selcol);
-    } else {
-      // draw first row to the right
-      p.fillRect(isRTL ? 0 : col*dwidth, row*dheight, (7-col)*dwidth,
-                 dheight, selcol);
-      // draw full block till last line
-      selh = mSelEnd/7-row;
-      if (selh > 1) {
-        p.fillRect(0, (row+1)*dheight, 7*dwidth, (selh-1)*dheight,selcol);
+    if ( row < 6 && row >= 0 ) {
+      if (row == mSelEnd/7) {
+        // Single row selection
+        p.fillRect(isRTL ? (7 - (mSelEnd-mSelStart+1) - col)*dwidth : col*dwidth,
+                    row*dheight, (mSelEnd-mSelStart+1)*dwidth, dheight, selcol);
+      } else {
+        // draw first row to the right
+        p.fillRect(isRTL ? 0 : col*dwidth, row*dheight, (7-col)*dwidth,
+                   dheight, selcol);
+        // draw full block till last line
+        selh = mSelEnd/7-row;
+        if ( selh + row >= 6 ) selh = 6-row;
+        if ( selh > 1 ) {
+          p.fillRect(0, (row+1)*dheight, 7*dwidth, (selh-1)*dheight,selcol);
+        }
+        // draw last block from left to mSelEnd
+        if ( mSelEnd/7 < 6 ) {
+          selw = mSelEnd-7*(mSelEnd/7)+1;
+          p.fillRect(isRTL ? (7-selw)*dwidth : 0, (row+selh)*dheight,
+                     selw*dwidth, dheight, selcol);
+        }
       }
-      // draw last block from left to mSelEnd
-      selw = mSelEnd-7*(mSelEnd/7)+1;
-      p.fillRect(isRTL ? (7-selw)*dwidth : 0, (row+selh)*dheight,
-                 selw*dwidth, dheight, selcol);
     }
   }
 
   // iterate over all days in the matrix and draw the day label in appropriate colors
-  QColor actcol = mDefaultTextColorShaded;
+  QColor textColor = cg.text();
+  QColor textColorShaded = getShadedColor( textColor );
+  QColor actcol = textColorShaded;
   p.setPen(actcol);
   QPen tmppen;
-  for(int i = 0; i < NUMDAYS; i++) {
+  for ( int i = 0; i < NUMDAYS; ++i ) {
     row = i/7;
     col = isRTL ? 6-(i-row*7) : i-row*7;
 
     // if it is the first day of a month switch color from normal to shaded and vice versa
     if ( KOGlobals::self()->calendarSystem()->day( mDays[i] ) == 1) {
-      if (actcol == mDefaultTextColorShaded) {
-        actcol = mDefaultTextColor;
+      if (actcol == textColorShaded) {
+        actcol = textColor;
       } else {
-        actcol = mDefaultTextColorShaded;
+        actcol = textColorShaded;
       }
       p.setPen(actcol);
     }
@@ -629,6 +575,9 @@ void KODayMatrix::paintEvent( QPaintEvent * )
       p.setPen(actcol);
     }
 
+    bool holiday = ! KOGlobals::self()->isWorkDay( mDays[ i ] );
+
+    QColor holidayColorShaded = getShadedColor( KOPrefs::instance()->mHolidayColor );
     // if today then draw rectangle around day
     if (mToday == i) {
       tmppen = p.pen();
@@ -636,11 +585,11 @@ void KODayMatrix::paintEvent( QPaintEvent * )
 
       mTodayPen.setWidth(mTodayMarginWidth);
       //draw red rectangle for holidays
-      if (!mHolidays[i].isNull()) {
-        if (actcol == mDefaultTextColor) {
+      if (holiday) {
+        if (actcol == textColor) {
           mTodayPen.setColor(KOPrefs::instance()->mHolidayColor);
         } else {
-          mTodayPen.setColor(mHolidayColorShaded);
+          mTodayPen.setColor(holidayColorShaded);
         }
       }
       //draw gray rectangle for today if in selection
@@ -661,25 +610,25 @@ void KODayMatrix::paintEvent( QPaintEvent * )
     }
 
     // if it is a holiday then use the default holiday color
-    if (!mHolidays[i].isNull()) {
-      if (actcol == mDefaultTextColor) {
+    if (holiday) {
+      if (actcol == textColor) {
         p.setPen(KOPrefs::instance()->mHolidayColor);
       } else {
-        p.setPen(mHolidayColorShaded);
+        p.setPen(holidayColorShaded);
       }
     }
 
     // draw selected days with special color
     // DO NOT specially highlight holidays in selection !
     if (i >= mSelStart && i <= mSelEnd) {
-      p.setPen(mSelectedDaysColor);
+      p.setPen( QColor( "white" ) );
     }
 
     p.drawText(col*dwidth, row*dheight, dwidth, dheight,
               Qt::AlignHCenter | Qt::AlignVCenter,  mDayLabels[i]);
 
     // reset color to actual color
-    if (!mHolidays[i].isNull()) {
+    if (holiday) {
       p.setPen(actcol);
     }
     // reset bold font to plain font
