@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2000,2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -45,17 +46,15 @@
 
 #include <kcalendarsystem.h>
 #include <libkcal/calfilter.h>
+#include <libkcal/calendar.h>
+#include <libkcal/incidenceformatter.h>
+#include <libkcal/calendarresources.h>
 
-#ifndef KORG_NOPRINTER
-#include "calprinter.h"
-#endif
 #include "koprefs.h"
-#ifndef KORG_NOPLUGINS
-#include "kocore.h"
-#endif
 #include "koglobals.h"
 #include "koincidencetooltip.h"
 #include "koeventpopupmenu.h"
+#include "kohelper.h"
 
 #include "komonthview.h"
 #include "komonthview.moc"
@@ -79,9 +78,8 @@ void KOMonthCellToolTip::maybeTip( const QPoint & pos )
     /* Calculate the rectangle. */
     r=eventlist->itemRect( it );
     /* Show the tip */
-    QString tipText;
-    ToolTipVisitor v;
-    if ( v.act( i->incidence(), &tipText, true ) ) {
+    QString tipText( IncidenceFormatter::toolTipString( i->incidence() ) );
+    if ( !tipText.isEmpty() ) {
       tip( r, tipText );
     }
   }
@@ -192,13 +190,13 @@ void KNoScrollListBox::resizeEvent( QResizeEvent *e )
   QListBox::resizeEvent( e );
 }
 
-MonthViewItem::MonthViewItem( Incidence *incidence, QDate qd, const QString & s)
-  : QListBoxItem()
+MonthViewItem::MonthViewItem( Incidence *incidence, const QDateTime &qd,
+                              const QString & s ) : QListBoxItem()
 {
   setText( s );
 
   mIncidence = incidence;
-  mDate = qd;
+  mDateTime = qd;
 
   mTodoPixmap      = KOGlobals::self()->smallIcon("todo");
   mTodoDonePixmap  = KOGlobals::self()->smallIcon("checkedbox");
@@ -206,6 +204,7 @@ MonthViewItem::MonthViewItem( Incidence *incidence, QDate qd, const QString & s)
   mRecurPixmap     = KOGlobals::self()->smallIcon("recur");
   mReplyPixmap     = KOGlobals::self()->smallIcon("mail_reply");
 
+  mResourceColor = QColor();
   mTodo      = false;
   mTodoDone  = false;
   mRecur     = false;
@@ -223,9 +222,16 @@ void MonthViewItem::paint( QPainter *p )
 
   QColor bgColor = palette().color( QPalette::Normal,
             sel ? QColorGroup::Highlight : QColorGroup::Background );
+  int offset=0;
+  if ( KOPrefs::instance()->monthViewUsesResourceColor() &&
+    mResourceColor.isValid() ) {
+    p->setBackgroundColor( mResourceColor );
+    p->eraseRect( 0, 0, listBox()->maxItemWidth(), height( listBox() ) );
+    offset=2;
+  }
   if ( KOPrefs::instance()->monthViewUsesCategoryColor() ) {
     p->setBackgroundColor( bgColor );
-    p->eraseRect( 0, 0, listBox()->maxItemWidth(), height( listBox() ) );
+    p->eraseRect( offset, offset, listBox()->maxItemWidth()-2*offset, height( listBox() )-2*offset );
   }
   int x = 3;
   if ( mTodo ) {
@@ -320,10 +326,8 @@ MonthViewCell::MonthViewCell( KOMonthView *parent)
   connect( mItemList, SIGNAL( rightButtonPressed( QListBoxItem *,
                                                   const QPoint &) ),
            SLOT( contextMenu( QListBoxItem * ) ) );
-  connect( mItemList, SIGNAL( highlighted( QListBoxItem *) ),
-           SLOT( selection( QListBoxItem * ) ) );
   connect( mItemList, SIGNAL( clicked( QListBoxItem * ) ),
-           SLOT( cellClicked( QListBoxItem * ) ) );
+           SLOT( select() ) );
 }
 
 void MonthViewCell::setDate( const QDate &date )
@@ -332,15 +336,19 @@ void MonthViewCell::setDate( const QDate &date )
 
   mDate = date;
 
+  setFrameWidth();
+
   QString text;
-   if ( KOGlobals::self()->calendarSystem()->day( date ) == 1 ) {
-     text = KOGlobals::self()->calendarSystem()->monthName( date, true ) + " ";
+  if ( KOGlobals::self()->calendarSystem()->day( date ) == 1 ) {
+    text = i18n("'Month day' for month view cells", "%1 %2")
+        .arg( KOGlobals::self()->calendarSystem()->monthName( date, true ) )
+        .arg( KOGlobals::self()->calendarSystem()->day(mDate) );
     QFontMetrics fm( mLabel->font() );
     mLabel->resize( mLabelSize + QSize( fm.width( text ), 0 ) );
   } else {
     mLabel->resize( mLabelSize );
+    text = QString::number( KOGlobals::self()->calendarSystem()->day(mDate) );
   }
-  text += QString::number( KOGlobals::self()->calendarSystem()->day(mDate) );
   mLabel->setText( text );
 
   resizeEvent( 0 );
@@ -351,6 +359,15 @@ QDate MonthViewCell::date() const
   return mDate;
 }
 
+void MonthViewCell::setFrameWidth()
+{
+  // show current day with a thicker frame
+  if ( mDate == QDate::currentDate() )
+    mItemList->setLineWidth( 3 );
+  else
+    mItemList->setLineWidth( 1 );
+}
+
 void MonthViewCell::setPrimary( bool primary )
 {
   mPrimary = primary;
@@ -361,7 +378,7 @@ void MonthViewCell::setPrimary( bool primary )
     mLabel->setBackgroundMode( PaletteBackground );
   }
 
-  mItemList->setBackground( mPrimary, KOCore::self()->isWorkDay( mDate ) );
+  mItemList->setBackground( mPrimary, KOGlobals::self()->isWorkDay( mDate ) );
 }
 
 bool MonthViewCell::isPrimary() const
@@ -380,109 +397,165 @@ void MonthViewCell::setHoliday( bool holiday )
   }
 }
 
-void MonthViewCell::setHoliday( const QString &holiday )
+void MonthViewCell::setHolidayString( const QString &holiday )
 {
   mHolidayString = holiday;
-
-  if ( !holiday.isEmpty() ) {
-    setHoliday( true );
-  }
 }
 
 void MonthViewCell::updateCell()
 {
+  setFrameWidth();
+
   if ( mDate == QDate::currentDate() ) {
     setPalette( mTodayPalette );
+
+    QPalette pal = mItemList->palette();
+    pal.setColor( QColorGroup::Foreground, KOPrefs::instance()->highlightColor() );
+    mItemList->setPalette( pal );
   }
   else {
     if ( mHoliday )
       setPalette( mHolidayPalette );
     else
       setPalette( mStandardPalette );
+
+    QPalette pal = mItemList->palette();
+    pal.setColor( QColorGroup::Foreground, KOPrefs::instance()->agendaBgColor().dark( 150 ) );
+    mItemList->setPalette( pal );
   }
 
   mItemList->clear();
 
   if ( !mHolidayString.isEmpty() ) {
-    MonthViewItem *item = new MonthViewItem( 0, mDate, mHolidayString );
+    MonthViewItem *item = new MonthViewItem( 0, QDateTime( mDate ), mHolidayString );
     item->setPalette( mHolidayPalette );
     mItemList->insertItem( item );
   }
 }
 
+class MonthViewCell::CreateItemVisitor :
+      public IncidenceBase::Visitor
+{
+  public:
+    CreateItemVisitor() : mItem(0) {}
+
+    bool act( IncidenceBase *incidence, QDate date, QPalette stdPal )
+    {
+      mItem = 0;
+      mDate = date;
+      mStandardPalette = stdPal;
+      return incidence->accept( *this );
+    }
+    MonthViewItem *item() const { return mItem; }
+
+  protected:
+    bool visit( Event *event ) {
+      QString text;
+      QDateTime dt( mDate );
+      if ( event->isMultiDay() ) {
+        if (mDate == event->dtStart().date()) {
+          text = "(-- " + event->summary();
+          dt = event->dtStart();
+        } else if (mDate == event->dtEnd().date()) {
+          text = event->summary() + " --)";
+        } else if (!(event->dtStart().date().daysTo(mDate) % 7)) {
+          text = "-- " + event->summary() + " --";
+        } else {
+          text = "----------------";
+          dt = QDateTime( mDate );
+        }
+      } else {
+        if (event->doesFloat())
+          text = event->summary();
+        else {
+          text = KGlobal::locale()->formatTime(event->dtStart().time());
+          dt.setTime( event->dtStart().time() );
+          text += " " + event->summary();
+        }
+      }
+
+      mItem = new MonthViewItem( event, dt, text );
+      if (KOPrefs::instance()->monthViewUsesCategoryColor()) {
+        QStringList categories = event->categories();
+        QString cat = categories.first();
+        if (cat.isEmpty()) {
+          mItem->setPalette(QPalette(KOPrefs::instance()->mEventColor, KOPrefs::instance()->mEventColor));
+        } else {
+          mItem->setPalette(QPalette(*(KOPrefs::instance()->categoryColor(cat)), *(KOPrefs::instance()->categoryColor(cat))));
+        }
+      } else {
+        mItem->setPalette( mStandardPalette );
+      }
+
+      Attendee *me = event->attendeeByMails( KOPrefs::instance()->allEmails() );
+      if ( me != 0 ) {
+        mItem->setReply( me->status() == Attendee::NeedsAction && me->RSVP() );
+      } else
+        mItem->setReply(false);
+      return true;
+    }
+    bool visit( Todo *todo ) {
+      QString text;
+      if ( !KOPrefs::instance()->showAllDayTodo() )
+        return false;
+      QDateTime dt( mDate );
+      if ( todo->hasDueDate() && !todo->doesFloat() ) {
+        text += KGlobal::locale()->formatTime( todo->dtDue().time() );
+        text += " ";
+        dt.setTime( todo->dtDue().time() );
+      }
+      text += todo->summary();
+
+      mItem = new MonthViewItem( todo, dt, text );
+// FIXME: This breaks with recurring multi-day events!
+      if ( todo->doesRecur() ) {
+        mDate < todo->dtDue().date() ?
+        mItem->setTodoDone( true ) : mItem->setTodo( true );
+      }
+      else
+        todo->isCompleted() ? mItem->setTodoDone( true ) : mItem->setTodo( true );
+      mItem->setPalette( mStandardPalette );
+      return true;
+    }
+  protected:
+    MonthViewItem *mItem;
+    QDate mDate;
+    QPalette mStandardPalette;
+};
+
+
 void MonthViewCell::addIncidence( Incidence *incidence )
 {
-  QString text;
-  MonthViewItem *item = 0;
-  if ( incidence->type() == "Event" ) {
-    Event *event = static_cast<Event *>(incidence);
-    if (event->isMultiDay()) {
-      if (mDate == event->dtStart().date()) {
-        text = "(-- " + event->summary();
-      } else if (mDate == event->dtEnd().date()) {
-        text = event->summary() + " --)";
-      } else if (!(event->dtStart().date().daysTo(mDate) % 7)) {
-        text = "-- " + event->summary() + " --";
-      } else {
-        text = "----------------";
-      }
-    } else {
-      if (event->doesFloat())
-        text = event->summary();
-      else {
-        text = KGlobal::locale()->formatTime(event->dtStart().time());
-        text += " " + event->summary();
+  CreateItemVisitor v;
+
+  if ( v.act( incidence, mDate, mStandardPalette ) ) {
+    MonthViewItem *item = v.item();
+    if ( item ) {
+      item->setAlarm( incidence->isAlarmEnabled() );
+      item->setRecur( incidence->recurrenceType() );
+
+      QColor resourceColor = KOHelper::resourceColor( mCalendar, incidence );
+      if ( !resourceColor.isValid() )
+        resourceColor = KOPrefs::instance()->mEventColor;
+      item->setResourceColor( resourceColor );
+
+      // FIXME: Find the correct position (time-wise) to insert the item.
+      //        Currently, the items are displayed in "random" order instead of
+      //        chronologically sorted.
+      uint i = 0;
+      int pos = -1;
+      QDateTime dt( item->incidenceDateTime() );
+
+      while ( i < mItemList->count() && pos<0 ) {
+        QListBoxItem *item = mItemList->item( i );
+        MonthViewItem *mvitem = dynamic_cast<MonthViewItem*>( item );
+        if ( mvitem && mvitem->incidenceDateTime()>dt ) {
+          pos = i;
+        }
+        ++i;
       }
+      mItemList->insertItem( item, pos );
     }
-
-    item = new MonthViewItem( event, mDate, text );
-    if (KOPrefs::instance()->monthViewUsesCategoryColor()) {
-      QStringList categories = event->categories();
-      QString cat = categories.first();
-      if (cat.isEmpty()) {
-        item->setPalette(QPalette(KOPrefs::instance()->mEventColor, KOPrefs::instance()->mEventColor));
-      } else {
-        item->setPalette(QPalette(*(KOPrefs::instance()->categoryColor(cat)), *(KOPrefs::instance()->categoryColor(cat))));
-      }
-    } else {
-      item->setPalette( mStandardPalette );
-    }
-
-    Attendee *me = event->attendeeByMails( KOPrefs::instance()->allEmails() );
-    if ( me != 0 ) {
-      if ( me->status() == Attendee::NeedsAction && me->RSVP())
-        item->setReply(true);
-      else
-        item->setReply(false);
-    } else
-      item->setReply(false);
-  }
-
-  if ( incidence->type() == "Todo" &&
-       KOPrefs::instance()->showAllDayTodo() ) {
-    Todo *todo = static_cast<Todo *>(incidence);
-    if (todo->hasDueDate()) {
-      if (!todo->doesFloat()) {
-        text += KGlobal::locale()->formatTime(todo->dtDue().time());
-        text += " ";
-      }
-    }
-    text += todo->summary();
-
-    item = new MonthViewItem( todo, mDate, text );
-    if ( todo->doesRecur() ) {
-      mDate < todo->dtDue().date() ?
-      item->setTodoDone( true ) : item->setTodo( true );
-    }
-    else
-      todo->isCompleted() ? item->setTodoDone( true ) : item->setTodo( true );
-    item->setPalette( mStandardPalette );
-  }
-
-  if ( item ) {
-    item->setAlarm( incidence->isAlarmEnabled() );
-    item->setRecur( incidence->doesRecur() );
-    mItemList->insertItem( item );
   }
 }
 
@@ -534,7 +607,7 @@ void MonthViewCell::updateConfig()
                           KOPrefs::instance()->highlightColor() );
   updateCell();
 
-  mItemList->setBackground( mPrimary, KOCore::self()->isWorkDay( mDate ) );
+  mItemList->setBackground( mPrimary, KOGlobals::self()->isWorkDay( mDate ) );
 }
 
 void MonthViewCell::enableScrollBars( bool enabled )
@@ -572,12 +645,29 @@ QDate MonthViewCell::selectedIncidenceDate()
 
   if ( !item ) return qd;
 
-  return item->incidenceDate();
+  return item->incidenceDateTime().date();
+}
+
+void MonthViewCell::select()
+{
+  // setSelectedCall will deselect currently selected cells
+  mMonthView->setSelectedCell( this );
+
+  if( KOPrefs::instance()->enableMonthScroll() )
+    enableScrollBars( true );
+
+  // don't mess up the cell when it represents today
+  if( mDate != QDate::currentDate() ) {
+    mItemList->setFrameStyle( QFrame::Sunken | QFrame::Panel );
+    mItemList->setLineWidth( 3 );
+  }
 }
 
 void MonthViewCell::deselect()
 {
   mItemList->clearSelection();
+  mItemList->setFrameStyle( QFrame::Plain | QFrame::Panel );
+  setFrameWidth();
 
   enableScrollBars( false );
 }
@@ -589,6 +679,8 @@ void MonthViewCell::resizeEvent ( QResizeEvent * )
 
 void MonthViewCell::defaultAction( QListBoxItem *item )
 {
+  select();
+
   if ( !item ) {
     emit newEventSignal( date() );
   } else {
@@ -598,14 +690,10 @@ void MonthViewCell::defaultAction( QListBoxItem *item )
   }
 }
 
-void MonthViewCell::cellClicked( QListBoxItem * )
-{
-  if( KOPrefs::instance()->enableMonthScroll() ) enableScrollBars( true );
-}
-
 void MonthViewCell::contextMenu( QListBoxItem *item )
 {
-  mMonthView->setSelectedCell( this );
+  select();
+
   if ( item ) {
     MonthViewItem *eventItem = static_cast<MonthViewItem *>( item );
     Incidence *incidence = eventItem->incidence();
@@ -616,12 +704,6 @@ void MonthViewCell::contextMenu( QListBoxItem *item )
   }
 }
 
-void MonthViewCell::selection( QListBoxItem *item )
-{
-  if ( !item ) return;
-
-  mMonthView->setSelectedCell( this );
-}
 
 KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name )
     : KOEventView( calendar, parent, name ),
@@ -632,11 +714,24 @@ KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name
 
   QGridLayout *dayLayout = new QGridLayout( this );
 
+  QFont bfont = font();
+  bfont.setBold( true );
+
+  QFont mfont = bfont;
+  mfont.setPointSize( 20 );
+
+  // month name on top
+  mLabel = new QLabel( this );
+  mLabel->setFont( mfont );
+  mLabel->setAlignment( AlignCenter );
+  mLabel->setLineWidth( 0 );
+  mLabel->setFrameStyle( QFrame::Plain );
+
+  dayLayout->addMultiCellWidget( mLabel, 0, 0, 0, mDaysPerWeek );
+
   // create the day of the week labels (Sun, Mon, etc) and add them to
   // the layout.
   mDayLabels.resize( mDaysPerWeek );
-  QFont bfont = font();
-  bfont.setBold( true );
   int i;
   for( i = 0; i < mDaysPerWeek; i++ ) {
     QLabel *label = new QLabel( this );
@@ -647,7 +742,7 @@ KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name
 
     mDayLabels.insert( i, label );
 
-    dayLayout->addWidget( label, 0, i );
+    dayLayout->addWidget( label, 1, i );
     dayLayout->addColSpacing( i, 10 );
     dayLayout->setColStretch( i, 1 );
   }
@@ -658,15 +753,16 @@ KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name
   for( row = 0; row < mNumWeeks; ++row ) {
     for( col = 0; col < mDaysPerWeek; ++col ) {
       MonthViewCell *cell = new MonthViewCell( this );
+      cell->setCalendar(calendar);
       mCells.insert( row * mDaysPerWeek + col, cell );
-      dayLayout->addWidget( cell, row + 1, col );
+      dayLayout->addWidget( cell, row + 2, col );
 
       connect( cell, SIGNAL( defaultAction( Incidence * ) ),
                SLOT( defaultAction( Incidence * ) ) );
-      connect( cell, SIGNAL( newEventSignal( QDate ) ),
-               SIGNAL( newEventSignal( QDate ) ) );
+      connect( cell, SIGNAL( newEventSignal( const QDate & ) ),
+               SIGNAL( newEventSignal( const QDate & ) ) );
     }
-    dayLayout->setRowStretch( row + 1, 1 );
+    dayLayout->setRowStretch( row + 2, 1 );
   }
 
   mEventContextMenu = eventPopup();
@@ -726,14 +822,6 @@ bool KOMonthView::eventDurationHint( QDateTime &startDt, QDateTime &endDt, bool
   return false;
 }
 
-void KOMonthView::printPreview( CalPrinter *calPrinter, const QDate &fd,
-                                const QDate &td )
-{
-#ifndef KORG_NOPRINTER
-  calPrinter->preview( CalPrinter::Month, fd, td );
-#endif
-}
-
 void KOMonthView::updateConfig()
 {
   mWeekStartDay = KGlobal::locale()->weekStartDay();
@@ -741,7 +829,7 @@ void KOMonthView::updateConfig()
   QFontMetrics fontmetric( mDayLabels[0]->font() );
   mWidthLongDayLabel = 0;
 
-  for (int i = 0; i < 7; i++) {
+  for ( int i = 0; i < 7; ++i ) {
     int width =
         fontmetric.width( KOGlobals::self()->calendarSystem()->weekDayName( i + 1 ) );
     if ( width > mWidthLongDayLabel ) mWidthLongDayLabel = width;
@@ -771,35 +859,39 @@ void KOMonthView::showDates( const QDate &start, const QDate & )
 {
 //  kdDebug(5850) << "KOMonthView::showDates(): " << start.toString() << endl;
 
-  mStartDate = start;
+  const KCalendarSystem *calSys = KOGlobals::self()->calendarSystem();
 
+  // show first day of month on top for readability issues
+  mStartDate = start.addDays( -start.day() + 1 );
   // correct begin of week
   int weekdayCol=( mStartDate.dayOfWeek() + 7 - mWeekStartDay ) % 7;
   mStartDate = mStartDate.addDays( -weekdayCol );
 
+  mLabel->setText( i18n( "monthname year", "%1 %2" )
+                   .arg( calSys->monthName( start ) )
+                   .arg( calSys->year( start ) ) );
+
   bool primary = false;
   uint i;
   for( i = 0; i < mCells.size(); ++i ) {
     QDate date = mStartDate.addDays( i );
-    if ( KOGlobals::self()->calendarSystem()->day( date ) == 1 ) {
+    if ( calSys->day( date ) == 1 ) {
       primary = !primary;
     }
+
     mCells[i]->setDate( date );
+    if( date == start )
+      mCells[i]->select();
 
     mCells[i]->setPrimary( primary );
 
-    if ( KOGlobals::self()->calendarSystem()->dayOfWeek( date ) ==
-         KOGlobals::self()->calendarSystem()->weekDayOfPray() ) {
-      mCells[i]->setHoliday( true );
-    } else {
-      mCells[i]->setHoliday( false );
-    }
+    bool isHoliday = calSys->dayOfWeek( date ) == calSys->weekDayOfPray()
+                  || !KOGlobals::self()->isWorkDay( date );
+    mCells[i]->setHoliday( isHoliday );
 
-#ifndef KORG_NOPLUGINS
     // add holiday, if present
-    QString hstring( KOCore::self()->holiday( date ) );
-    mCells[i]->setHoliday( hstring );
-#endif
+    QString hstring( KOGlobals::self()->holiday( date ) );
+    mCells[i]->setHolidayString( hstring );
   }
 
   updateView();
@@ -810,12 +902,45 @@ void KOMonthView::showIncidences( const Incidence::List & )
   kdDebug(5850) << "KOMonthView::showIncidences( const Incidence::List & ) is not implemented yet." << endl;
 }
 
+class KOMonthView::GetDateVisitor : public IncidenceBase::Visitor
+{
+  public:
+    GetDateVisitor() {}
+
+    bool act( IncidenceBase *incidence )
+    {
+      return incidence->accept( *this );
+    }
+    QDate date() const { return mDate; }
+
+  protected:
+    bool visit( Event *event ) {
+      mDate = event->dtStart().date();
+      return true;
+    }
+    bool visit( Todo *todo ) {
+      if ( todo->hasDueDate() ) {
+        mDate = todo->dtDue().date();
+        return true;
+      } else
+        return false;
+    }
+    bool visit( Journal *journal ) {
+      mDate = journal->dtStart().date();
+      return true;
+    }
+  protected:
+    QDate mDate;
+};
+
 void KOMonthView::changeIncidenceDisplayAdded( Incidence *incidence )
 {
   MonthViewCell *mvc;
   Event *event = 0;
   Todo *todo = 0;
   QDate date;
+
+  // FIXME: use a visitor here
   if ( incidence->type() == "Event" ) {
     event = static_cast<Event *>( incidence );
     date = event->dtStart().date();
@@ -827,6 +952,7 @@ void KOMonthView::changeIncidenceDisplayAdded( Incidence *incidence )
   }
 
   if ( incidence->doesRecur() ) {
+// FIXME: This breaks with recurring multi-day events!
      for ( uint i = 0; i < mCells.count(); i++ ) {
        if ( incidence->recursOn( mCells[i]->date() ) ) {
          mCells[i]->addIncidence( incidence );
@@ -898,17 +1024,9 @@ void KOMonthView::resizeEvent( QResizeEvent * )
   }
 }
 
-void KOMonthView::showEventContextMenu( Incidence *incidence, QDate qd )
+void KOMonthView::showEventContextMenu( Incidence *incidence, const QDate &qd )
 {
   mEventContextMenu->showIncidencePopup( incidence, qd );
-  /*
-  if( incidence && incidence->type() == "Event" ) {
-    Event *event = static_cast<Event *>(incidence);
-    mContextMenu->showEventPopup(event);
-  } else {
-    kdDebug(5850) << "MonthView::showContextMenu(): cast failed." << endl;
-  }
-  */
 }
 
 void KOMonthView::showGeneralContextMenu()
