@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -29,6 +30,7 @@
 #include <qstring.h>
 #include <qfont.h>
 #include <qcolor.h>
+#include <qmap.h>
 #include <qstringlist.h>
 
 #include <kglobalsettings.h>
@@ -41,6 +43,11 @@
 #include <kstringhandler.h>
 
 #include "koprefs.h"
+#include <libkpimidentities/identitymanager.h>
+#include <libkpimidentities/identity.h>
+#include <libemailfunctions/email.h>
+#include <kabc/stdaddressbook.h>
+#include "kocore.h"
 
 KOPrefs *KOPrefs::mInstance = 0;
 static KStaticDeleter<KOPrefs> insd;
@@ -56,9 +63,12 @@ KOPrefs::KOPrefs() :
   KOPrefsBase()
 {
   mCategoryColors.setAutoDelete(true);
+  mResourceColors.setAutoDelete(true);
 
   mDefaultCategoryColor = QColor(151, 235, 121);
 
+  mDefaultResourceColor = QColor();//Default is a color invalid
+
   mDefaultMonthViewFont = KGlobalSettings::generalFont();
   // make it a bit smaller
   mDefaultMonthViewFont.setPointSize(mDefaultMonthViewFont.pointSize()-2);
@@ -95,24 +105,31 @@ void KOPrefs::usrSetDefaults()
   // settings for example.
 
   KEMailSettings settings;
-  mName = settings.getSetting(KEMailSettings::RealName);
-  mEmail = settings.getSetting(KEMailSettings::RealName);
+  QString tmp = settings.getSetting(KEMailSettings::RealName);
+  if ( !tmp.isEmpty() ) setUserName( tmp );
+  tmp = settings.getSetting(KEMailSettings::EmailAddress);
+  if ( !tmp.isEmpty() ) setUserEmail( tmp );
   fillMailDefaults();
 
   mMonthViewFont = mDefaultMonthViewFont;
 
   setTimeZoneIdDefault();
 
-  mRememberPublishPw = false;
-  mRememberRetrievePw = false;
-
   KPimPrefs::usrSetDefaults();
 }
 
 void KOPrefs::fillMailDefaults()
 {
-  if (mName.isEmpty()) mName = i18n("Anonymous");
-  if (mEmail.isEmpty()) mEmail = i18n("nobody@nowhere");
+  userEmailItem()->swapDefault();
+  QString defEmail = userEmailItem()->value();
+  userEmailItem()->swapDefault();
+
+  if ( userEmail() == defEmail ) {
+    // No korg settings - but maybe there's a kcontrol[/kmail] setting available
+    KEMailSettings settings;
+    if ( !settings.getSetting( KEMailSettings::EmailAddress ).isEmpty() )
+      mEmailControlCenter = true;
+  }
 }
 
 void KOPrefs::setTimeZoneIdDefault()
@@ -158,11 +175,6 @@ void KOPrefs::usrReadConfig()
   mCustomCategories = config()->readListEntry("Custom Categories");
   if (mCustomCategories.isEmpty()) setCategoryDefaults();
 
-  config()->setGroup("Personal Settings");
-  mName = config()->readEntry("user_name");
-  mEmail = config()->readEntry("user_email");
-  fillMailDefaults();
-
   // old category colors, ignore if they have the old default
   // should be removed a few versions after 3.2...
   config()->setGroup("Category Colors");
@@ -182,17 +194,29 @@ void KOPrefs::usrReadConfig()
     setCategoryColor(*it,config()->readColorEntry(*it, &*it2));
   }
 
+  config()->setGroup( "Resources Colors" );
+  QMap<QString, QString> map = config()->entryMap( "Resources Colors" );
+
+  QMapIterator<QString, QString> it3;
+  for( it3 = map.begin(); it3 != map.end(); ++it3 ) {
+    kdDebug(5850)<< "KOPrefs::usrReadConfig: key: " << it3.key() << " value: "
+      << it3.data()<<endl;
+    setResourceColor( it3.key(), config()->readColorEntry( it3.key(),
+      &mDefaultResourceColor ) );
+  }
+
+
   if (mTimeZoneId.isEmpty()) {
     setTimeZoneIdDefault();
   }
 
-  config()->setGroup("Groupware");
-  if( mRememberPublishPw )
-    mPublishPassword = KStringHandler::obscure( config()->readEntry( "Publish Server Password" ) );
+#if 0
+  config()->setGroup("FreeBusy");
   if( mRememberRetrievePw )
     mRetrievePassword = KStringHandler::obscure( config()->readEntry( "Retrieve Server Password" ) );
-
+#endif
   KPimPrefs::usrReadConfig();
+  fillMailDefaults();
 }
 
 
@@ -201,10 +225,6 @@ void KOPrefs::usrWriteConfig()
   config()->setGroup("General");
   config()->writeEntry("Custom Categories",mCustomCategories);
 
-  config()->setGroup("Personal Settings");
-  config()->writeEntry("user_name",mName);
-  config()->writeEntry("user_email",mEmail);
-
   config()->setGroup("Category Colors2");
   QDictIterator<QColor> it(mCategoryColors);
   while (it.current()) {
@@ -212,60 +232,142 @@ void KOPrefs::usrWriteConfig()
     ++it;
   }
 
-  config()->setGroup( "Groupware" );
-  if( mRememberPublishPw )
-    config()->writeEntry( "Publish Server Password", KStringHandler::obscure( mPublishPassword ) );
-  else
-    config()->deleteEntry( "Publish Server Password" );
+  config()->setGroup( "Resources Colors" );
+  QDictIterator<QColor> it2( mResourceColors );
+  while( it2.current() ) {
+    config()->writeEntry( it2.currentKey(), *( it2.current() ) );
+    ++it2;
+  }
+
+  if( !mFreeBusyPublishSavePassword ) {
+    KConfigSkeleton::ItemPassword *i = freeBusyPublishPasswordItem();
+    i->setValue( "" );
+    i->writeConfig( config() );
+  }
+  if( !mFreeBusyRetrieveSavePassword ) {
+    KConfigSkeleton::ItemPassword *i = freeBusyRetrievePasswordItem();
+    i->setValue( "" );
+    i->writeConfig( config() );
+  }
+
+#if 0
   if( mRememberRetrievePw )
     config()->writeEntry( "Retrieve Server Password", KStringHandler::obscure( mRetrievePassword ) );
   else
     config()->deleteEntry( "Retrieve Server Password" );
+#endif
 
   KPimPrefs::usrWriteConfig();
 }
 
-void KOPrefs::setCategoryColor(QString cat,const QColor & color)
+void KOPrefs::setCategoryColor( const QString &cat, const QColor & color)
 {
-  mCategoryColors.replace(cat,new QColor(color));
+  mCategoryColors.replace( cat, new QColor( color ) );
 }
 
-QColor *KOPrefs::categoryColor(QString cat)
+QColor *KOPrefs::categoryColor( const QString &cat )
 {
   QColor *color = 0;
 
-  if (!cat.isEmpty()) color = mCategoryColors[cat];
+  if ( !cat.isEmpty() ) color = mCategoryColors[ cat ];
 
-  if (color) return color;
+  if ( color ) return color;
   else return &mDefaultCategoryColor;
 }
 
-void KOPrefs::setFullName(const QString &name)
+void KOPrefs::setResourceColor ( const QString &cal, const QColor &color )
 {
-  mName = name;
+  kdDebug(5850)<<"KOPrefs::setResourceColor: " << cal << " color: "<<
+    color.name()<<endl;
+  mResourceColors.replace( cal, new QColor( color ) );
 }
 
-void KOPrefs::setEmail(const QString &email)
+QColor* KOPrefs::resourceColor( const QString &cal )
 {
-  mEmail = email;
+  QColor *color=0;
+  if( !cal.isEmpty() ) color = mResourceColors[cal];
+
+  if (color && color->isValid() )
+    return color;
+  else
+    return &mDefaultResourceColor;
 }
 
 QString KOPrefs::fullName()
 {
-  if (mEmailControlCenter) {
+  if ( mEmailControlCenter ) {
     KEMailSettings settings;
-    return settings.getSetting(KEMailSettings::RealName);
+    return settings.getSetting( KEMailSettings::RealName );
   } else {
-    return mName;
+    return userName();
   }
 }
 
 QString KOPrefs::email()
 {
-  if (mEmailControlCenter) {
+  if ( mEmailControlCenter ) {
     KEMailSettings settings;
-    return settings.getSetting(KEMailSettings::EmailAddress);
+    return settings.getSetting( KEMailSettings::EmailAddress );
   } else {
-    return mEmail;
+    return userEmail();
   }
 }
+
+QStringList KOPrefs::allEmails()
+{
+  // Grab emails from the email identities
+  QStringList lst = KOCore::self()->identityManager()->allEmails();
+  // Add emails configured in korganizer
+  lst += mAdditionalMails;
+  // Add emails from the user's kaddressbook entry
+  lst += KABC::StdAddressBook::self( true )->whoAmI().emails();
+  // Add the email entered as the userEmail here
+  lst += email();
+
+  // Warning, this list could contain duplicates.
+  return lst;
+}
+
+QStringList KOPrefs::fullEmails()
+{
+  QStringList fullEmails;
+  // The user name and email from the config dialog:
+  fullEmails << QString("%1 <%2>").arg( fullName() ).arg( email() );
+
+  QStringList::Iterator it;
+  // Grab emails from the email identities
+  KPIM::IdentityManager *idmanager = KOCore::self()->identityManager();
+  QStringList lst = idmanager->identities();
+  KPIM::IdentityManager::ConstIterator it1;
+  for ( it1 = idmanager->begin() ; it1 != idmanager->end() ; ++it1 ) {
+    fullEmails << (*it1).fullEmailAddr();
+  }
+  // Add emails configured in korganizer
+  lst = mAdditionalMails;
+  for ( it = lst.begin(); it != lst.end(); ++it ) {
+    fullEmails << QString("%1 <%2>").arg( fullName() ).arg( *it );
+  }
+  // Add emails from the user's kaddressbook entry
+  KABC::Addressee me = KABC::StdAddressBook::self( true )->whoAmI();
+  lst = me.emails();
+  for ( it = lst.begin(); it != lst.end(); ++it ) {
+    fullEmails << me.fullEmail( *it );
+  }
+
+  // Warning, this list could contain duplicates.
+  return fullEmails;
+}
+
+bool KOPrefs::thatIsMe( const QString& _email )
+{
+  if ( KOCore::self()->identityManager()->thatIsMe( _email ) )
+    return true;
+  // in case email contains a full name, strip it out
+  QString email = KPIM::getEmailAddress( _email );
+  if ( mAdditionalMails.find( email ) != mAdditionalMails.end() )
+    return true;
+  QStringList lst = KABC::StdAddressBook::self( true )->whoAmI().emails();
+  if ( lst.find( email ) != lst.end() )
+    return true;
+  return false;
+}
