@@ -16,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -26,7 +26,6 @@
 #define KORGANIZER_PART_H
 
 #include <kurl.h>
-#include <kparts/browserextension.h>
 #include <kparts/part.h>
 
 #include <korganizer/mainwindow.h>
@@ -34,7 +33,6 @@
 
 class KInstance;
 class KAboutData;
-class KOrganizerBrowserExtension;
 class KProcess;
 
 class CalendarView;
@@ -97,7 +95,6 @@ class KOrganizerPart: public KParts::ReadOnlyPart,
   private:
     CalendarView *mView;
     ActionManager *mActionManager;
-    KOrganizerBrowserExtension *mBrowserExtension;
     KParts::StatusBarExtension *mStatusBarExtension;
     QWidget *mTopLevelWidget;
 
@@ -105,13 +102,4 @@ class KOrganizerPart: public KParts::ReadOnlyPart,
     void textChanged( const QString & );
 };
 
-class KOrganizerBrowserExtension : public KParts::BrowserExtension
-{
-    Q_OBJECT
-    friend class KOrganizerPart;
-  public:
-    KOrganizerBrowserExtension( KOrganizerPart *parent );
-    virtual ~KOrganizerBrowserExtension();
-};
-
 #endif
