@@ -1,8 +1,9 @@
 /*
     This file is part of KOrganizer.
 
-    Copyright (c) 1997, 1998 Preston Brown
+    Copyright (c) 1997, 1998 Preston Brown <pbrown@kde.org>
     Copyright (c) 2000-2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +17,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -29,6 +30,7 @@
 #include <qlayout.h>
 #include <qdatetime.h>
 
+#include <kabc/addressee.h>
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kmessagebox.h>
@@ -40,12 +42,20 @@
 
 #include "koprefs.h"
 #include "koeditorattachments.h"
+#include "kogroupware.h"
 #include "kodialogmanager.h"
- 
+#include "incidencechanger.h"
+
+#include "koeditorgeneraltodo.h"
+#include "koeditordetails.h"
+#include "koeditorrecurrence.h"
+#include "koeditoralarms.h"
+
 #include "kotodoeditor.h"
+#include "kocore.h"
 
 KOTodoEditor::KOTodoEditor( Calendar *calendar, QWidget *parent ) :
-  KOIncidenceEditor( i18n("Edit To-Do"), calendar, parent )
+  KOIncidenceEditor( QString::null, calendar, parent )
 {
   mTodo = 0;
   mRelatedTodo = 0;
@@ -59,8 +69,15 @@ KOTodoEditor::~KOTodoEditor()
 void KOTodoEditor::init()
 {
   setupGeneral();
+//  setupAlarmsTab();
+  setupRecurrence();
   setupAttendeesTab();
   setupAttachmentsTab();
+
+  connect( mGeneral, SIGNAL( dateTimeStrChanged( const QString & ) ),
+           mRecurrence, SLOT( setDateTimeStr( const QString & ) ) );
+  connect( mGeneral, SIGNAL( signalDateTimeChanged( const QDateTime &, const QDateTime & ) ),
+           mRecurrence, SLOT( setDateTimes( const QDateTime &, const QDateTime & ) ) );
 }
 
 void KOTodoEditor::reload()
@@ -115,6 +132,7 @@ void KOTodoEditor::setupGeneral()
     mGeneral->initStatus(topFrame,topLayout);
     QBoxLayout *alarmLineLayout = new QHBoxLayout(topLayout);
     mGeneral->initAlarm(topFrame,alarmLineLayout);
+    alarmLineLayout->addStretch( 1 );
     mGeneral->initDescription(topFrame,topLayout);
     QBoxLayout *detailsLayout = new QHBoxLayout(topLayout);
     mGeneral->initCategories( topFrame, detailsLayout );
@@ -124,20 +142,42 @@ void KOTodoEditor::setupGeneral()
   mGeneral->finishSetup();
 }
 
-void KOTodoEditor::editTodo(Todo *todo)
+void KOTodoEditor::setupRecurrence()
 {
-  init();
+  QFrame *topFrame = addPage( i18n("Rec&urrence") );
 
-  mTodo = todo;
-  readTodo(mTodo);
+  QBoxLayout *topLayout = new QVBoxLayout( topFrame );
+
+  mRecurrence = new KOEditorRecurrence( topFrame );
+  topLayout->addWidget( mRecurrence );
+
+  mRecurrence->setEnabled( false );
+  connect(mGeneral,SIGNAL(dueDateEditToggle( bool ) ),
+          mRecurrence, SLOT( setEnabled( bool ) ) );
 }
 
-void KOTodoEditor::newTodo(QDateTime due,Todo *relatedTodo,bool allDay)
+void KOTodoEditor::editIncidence(Incidence *incidence)
+{
+  Todo *todo=dynamic_cast<Todo*>(incidence);
+  if (todo)
+  {
+    init();
+
+    mTodo = todo;
+    readTodo(mTodo);
+  }
+
+  setCaption( i18n("Edit To-do") );
+}
+
+void KOTodoEditor::newTodo( const QDateTime &due, Todo *relatedTodo, bool allDay)
 {
   init();
 
   mTodo = 0;
   setDefaults(due,relatedTodo,allDay);
+
+  setCaption( i18n("New To-do") );
 }
 
 void KOTodoEditor::newTodo( const QString &text )
@@ -157,14 +197,16 @@ void KOTodoEditor::newTodo( const QString &text )
   } else {
     mGeneral->setSummary( text );
   }
+
+  setCaption( i18n("New To-do") );
 }
 
 void KOTodoEditor::newTodo( const QString &summary,
-                              const QString &description,
-                              const QString &attachment )
+                            const QString &description,
+                            const QString &attachment )
 {
   init();
-  
+
   mTodo = 0;
 
   loadDefaults();
@@ -175,6 +217,23 @@ void KOTodoEditor::newTodo( const QString &summary,
   if ( !attachment.isEmpty() ) {
     mAttachments->addAttachment( attachment );
   }
+
+  setCaption( i18n("New To-do") );
+}
+
+void KOTodoEditor::newTodo( const QString &summary,
+                            const QString &description,
+                            const QString &attachment,
+                            const QStringList &attendees )
+{
+  newTodo( summary, description, attachment );
+
+  QStringList::ConstIterator it;
+  for ( it = attendees.begin(); it != attendees.end(); ++it ) {
+    QString name, email;
+    KABC::Addressee::parseEmailAddress( *it, name, email );
+    mDetails->insertAttendee( new Attendee( name, email ) );
+  }
 }
 
 void KOTodoEditor::loadDefaults()
@@ -187,103 +246,120 @@ bool KOTodoEditor::processInput()
   if ( !validateInput() ) return false;
 
   if ( mTodo ) {
+    bool rc = true;
     Todo *oldTodo = mTodo->clone();
+    Todo *todo = mTodo->clone();
 
-    writeTodo( mTodo );
-
-    mTodo->setRevision( mTodo->revision() + 1 );
-
-    emit todoChanged( oldTodo, mTodo );
+    kdDebug(5850) << "KOTodoEditor::processInput() write event." << endl;
+    writeTodo( todo );
+    kdDebug(5850) << "KOTodoEditor::processInput() event written." << endl;
 
+    if( *mTodo == *todo )
+      // Don't do anything
+      kdDebug(5850) << "Todo not changed\n";
+    else {
+      kdDebug(5850) << "Todo changed\n";
+      //IncidenceChanger::assignIncidence( mTodo, todo );
+      writeTodo( mTodo );
+      mChanger->changeIncidence( oldTodo, mTodo );
+    }
+    delete todo;
     delete oldTodo;
+    return rc;
+
   } else {
     mTodo = new Todo;
-    mTodo->setOrganizer( KOPrefs::instance()->email() );
+    mTodo->setOrganizer( Person( KOPrefs::instance()->fullName(),
+                         KOPrefs::instance()->email() ) );
 
     writeTodo( mTodo );
 
-    if ( !mCalendar->addTodo( mTodo ) ) {
-      KODialogManager::errorSaveTodo( this );
+    if ( !mChanger->addIncidence( mTodo ) ) {
       delete mTodo;
       mTodo = 0;
       return false;
     }
-
-    emit todoAdded( mTodo );
   }
 
   return true;
-}
 
-void KOTodoEditor::processCancel()
-{
-  if ( mTodo ) {
-    emit editCanceled( mTodo );
-  }
 }
 
 void KOTodoEditor::deleteTodo()
 {
-  if (mTodo) {
-    if (KOPrefs::instance()->mConfirm) {
-      switch (msgItemDelete()) {
-        case KMessageBox::Continue: // OK
-          emit todoToBeDeleted(mTodo);
-          emit dialogClose(mTodo);
-          mCalendar->deleteTodo(mTodo);
-          emit todoDeleted();
-          reject();
-          break;
-      }
-    }
-    else {
-      emit todoToBeDeleted(mTodo);
-      emit dialogClose(mTodo);
-      mCalendar->deleteTodo(mTodo);
-      emit todoDeleted();
-      reject();
-    }
-  } else {
-    reject();
-  }
+  if (mTodo)
+    emit deleteIncidenceSignal( mTodo );
+  emit dialogClose(mTodo);
+  reject();
 }
 
-void KOTodoEditor::setDefaults( QDateTime due, Todo *relatedEvent, bool allDay )
+void KOTodoEditor::setDefaults( const QDateTime &due, Todo *relatedEvent, bool allDay )
 {
   mRelatedTodo = relatedEvent;
 
-  mGeneral->setDefaults( due, allDay );
+  // inherit some properties from parent todo
+  if ( mRelatedTodo ) {
+    mGeneral->setCategories( mRelatedTodo->categoriesStr() );
+    mCategoryDialog->setSelected( mRelatedTodo->categories() );
+    if ( mRelatedTodo->hasDueDate() )
+      mGeneral->setDefaults( mRelatedTodo->dtDue(), allDay );
+    else
+      mGeneral->setDefaults( due, allDay );
+  }
+  else
+    mGeneral->setDefaults( due, allDay );
+
   mDetails->setDefaults();
+  if ( mTodo )
+    mRecurrence->setDefaults( mTodo->dtStart(), due, false );
+  else
+    mRecurrence->setDefaults( QDateTime::currentDateTime(), due, false );
   mAttachments->setDefaults();
 }
 
 void KOTodoEditor::readTodo( Todo *todo )
 {
+  kdDebug(5850)<<"read todo"<<endl;
   mGeneral->readTodo( todo );
   mDetails->readEvent( todo );
+//  mAlarms->readIncidence( todo );
+  mRecurrence->readIncidence( todo );
   mAttachments->readIncidence( todo );
 
   // categories
   mCategoryDialog->setSelected( todo->categories() );
-
-  // TODO: We should handle read-only events here.
+  createEmbeddedURLPages( todo );
+  readDesignerFields( todo );
 }
 
 void KOTodoEditor::writeTodo( Todo *todo )
 {
+  Incidence *oldIncidence = todo->clone();
+
+  mRecurrence->writeIncidence( todo );
   mGeneral->writeTodo( todo );
   mDetails->writeEvent( todo );
   mAttachments->writeIncidence( todo );
 
+  if ( *(oldIncidence->recurrence()) != *(todo->recurrence() ) ) {
+    todo->setDtDue( todo->dtDue(), true );
+    if ( todo->hasStartDate() )
+      todo->setDtStart( todo->dtStart() );
+  }
+  writeDesignerFields( todo );
+
   // set related event, i.e. parent to-do in this case.
   if ( mRelatedTodo ) {
     todo->setRelatedTo( mRelatedTodo );
   }
+
+  cancelRemovedAttendees( todo );
 }
 
 bool KOTodoEditor::validateInput()
 {
   if ( !mGeneral->validateInput() ) return false;
+  if ( !mRecurrence->validateInput() ) return false;
   if ( !mDetails->validateInput() ) return false;
   return true;
 }
@@ -292,44 +368,37 @@ int KOTodoEditor::msgItemDelete()
 {
   return KMessageBox::warningContinueCancel(this,
       i18n("This item will be permanently deleted."),
-      i18n("KOrganizer Confirmation"),i18n("Delete"));
+      i18n("KOrganizer Confirmation"), KStdGuiItem::del() );
 }
 
-void KOTodoEditor::modified (int modification)
+void KOTodoEditor::modified (int /*modification*/)
 {
-  if (modification == KOGlobals::CATEGORY_MODIFIED ||
-      KOGlobals::UNKNOWN_MODIFIED == modification )
-    mCategoryDialog->setSelected (mTodo->categories ());
-  mGeneral->modified (mTodo, modification);
-
+  // Play dump, just reload the todo. This dialog has become so complicated that
+  // there is no point in trying to be smart here...
+  reload();
 }
 
-void KOTodoEditor::slotLoadTemplate()
+void KOTodoEditor::loadTemplate( /*const*/ CalendarLocal& cal )
 {
-  CalendarLocal cal( KOPrefs::instance()->mTimeZoneId );
-  Todo *todo = new Todo;
-  QString templateName = loadTemplate( &cal, todo->type(),
-                                       KOPrefs::instance()->mTodoTemplates );
-  delete todo;
-  if ( templateName.isEmpty() ) {
-    return;
-  }
-
   Todo::List todos = cal.todos();
   if ( todos.count() == 0 ) {
     KMessageBox::error( this,
-        i18n("Template '%1' does not contain a valid todo.")
-        .arg( templateName ) );
+        i18n("Template does not contain a valid to-do.") );
   } else {
     readTodo( todos.first() );
   }
 }
 
-void KOTodoEditor::saveTemplate( const QString &templateName )
+void KOTodoEditor::slotSaveTemplate( const QString &templateName )
 {
   Todo *todo = new Todo;
   writeTodo( todo );
   saveAsTemplate( todo, templateName );
 }
 
+QStringList& KOTodoEditor::templates() const
+{
+  return KOPrefs::instance()->mTodoTemplates;
+}
+
 #include "kotodoeditor.moc"
