@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef MK_IMAPS_PROTO_H
@@ -31,9 +31,9 @@ public:
 
 	virtual KIO_Protocol * clone() const { return new Imaps_Protocol; }
 	
-	virtual QString protocol() const { return "newimaps"; }
+	virtual QString protocol() const { return "imaps"; }
 	virtual QString configName() const { return "imaps"; }
-
+	
 	virtual unsigned short defaultPort() const { return 993; }
 };
 
