@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "kio_count.h"
@@ -52,7 +52,6 @@ KIO_Count::~KIO_Count()
 	// Delete copies of urls.
 	delete _kurl;
 	delete _metadata;
-	delete _protocol;
 }
 
 void KIO_Count::count( KKioDrop *drop )
@@ -62,7 +61,6 @@ void KIO_Count::count( KKioDrop *drop )
 
 	delete _kurl;
 	delete _metadata;
-	delete _protocol;
 	_kio = drop;
 
 	/*
@@ -73,7 +71,7 @@ void KIO_Count::count( KKioDrop *drop )
 	 */
 	_kurl = new KURL( *_kio->_kurl );
 	_metadata = new KIO::MetaData( *_kio->_metadata );
-	_protocol = _kio->_protocol->clone();
+	_protocol = _kio->_protocol;
 
 	KURL kurl = *_kurl;
 	KIO::MetaData metadata = *_metadata;
@@ -89,6 +87,7 @@ void KIO_Count::count( KKioDrop *drop )
 		if( ! ( _slave = KIO::Scheduler::getConnectedSlave( kurl, metadata ) ) ) //Forcing reload
 		{
 			kdWarning() << i18n( "Not able to open a kio slave for %1." ).arg( _protocol->configName() ) << endl;
+			_kio->emitShowPassivePopup( i18n( "Not able to open a kio slave for %1." ).arg( _protocol->configName() ) );
 			_valid = false;
 			_kio->emitValidChanged();
 			_slave = 0;
@@ -122,6 +121,7 @@ void KIO_Count::count( KKioDrop *drop )
 		kurl.setPort( _protocol->defaultPort() );
 
 	//Making job to fetch file-list
+	
 	_job = KIO::listDir( kurl, false );
 	_job->addMetaData( metadata );
 
@@ -156,7 +156,6 @@ void KIO_Count::stopActiveCount()
 	//Deletings settings
 	delete _kurl; _kurl = 0;
 	delete _metadata; _metadata = 0;
-	delete _protocol; _protocol = 0;
 
 	delete _new_mailurls; _new_mailurls = 0;
 }
@@ -193,7 +192,6 @@ void KIO_Count::disconnectSlave()
 
 	//Disconnect slave
 	KIO::Scheduler::disconnectSlave( _slave );
-	delete _protocol;
 	_slave = 0;
 	_protocol = 0;
 }
@@ -205,11 +203,12 @@ void KIO_Count::result( KIO::Job* job )
 	if( job != _job )
 		kdError() << i18n( "Got unknown job; something must be wrong..." ) << endl;
 
-	//look of an error occured. If there is, print the error.
+	//look of an error occurred. If there is, print the error.
 	//This could be very useful by resolving bugs.
 	if( job->error() )
 	{
 		kdError() << i18n( "The next KIO-error occurred by counting: %1" ).arg( job->errorString() ) << endl;
+		_kio->emitShowPassivePopup( i18n( "The next KIO-error occurred by counting: %1" ).arg( job->errorString() ) );
 		_valid = false;
 		_kio->emitValidChanged();
 	}
