@@ -13,11 +13,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef SORTEDMAILSUBJECT_H
-#define SORTEDMAILSUBJECT_H
+#ifndef MK_SORTEDMAILSUBJECT_H
+#define MK_SORTEDMAILSUBJECT_H
 
 #include "mailsubject.h"
 #include <qptrlist.h>
@@ -37,4 +37,4 @@ public:
 	virtual int compareItems ( QPtrCollection::Item item1, QPtrCollection::Item item2 );
 };
 
-#endif
+#endif //MK_SORTEDMAILSUBJECT_H
