@@ -20,8 +20,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -30,7 +30,7 @@
 
 
 static const char *kpilot_id =
-	"$Id: kpilot.cc,v 1.165 2005/02/24 14:01:30 adridg Exp $";
+	"$Id: kpilot.cc 450702 2005-08-18 21:37:28Z adridg $";
 
 
 #include "options.h"
@@ -278,7 +278,6 @@ void KPilotInstaller::setupWidget()
 	setAutoSaveSettings();
 }
 
-
 void KPilotInstaller::initComponents()
 {
 	FUNCTIONSETUP;
@@ -289,43 +288,41 @@ void KPilotInstaller::initComponents()
 	DEBUGKPILOT << fname << ": Creating component pages." << endl;
 #endif
 
+	QPixmap pixmap;
 	QString pixfile;
 	QWidget *w;
 
-#define VIEWICON(a) KGlobal::dirs()->findResource("data",(a))
+#define ADDICONPAGE(a,b) \
+	pixmap = KGlobal::iconLoader()->loadIcon(b, KIcon::Desktop, 64); \
+	w = getManagingWidget()->addVBoxPage(a,QString::null, pixmap) ;
 
-#define ADDICONPAGE(a,b) pixfile = VIEWICON(b); \
-	w = getManagingWidget()->addVBoxPage(a,QString::null, \
-		(pixfile.isEmpty() ? QPixmap() : QPixmap(pixfile))) ;
-
-	ADDICONPAGE(i18n("HotSync"),CSL1("kpilot/icons/kpilot-bhotsync.png"));
+	ADDICONPAGE(i18n("HotSync"),CSL1("kpilotbhotsync"));
 	fLogWidget = new LogWidget(w);
 	addComponentPage(fLogWidget, i18n("HotSync"));
 	fLogWidget->setShowTime(true);
 
-	ADDICONPAGE(i18n("To-do Viewer"),CSL1("kpilot/icons/kpilot-todo.png"));
+	ADDICONPAGE(i18n("To-do Viewer"),CSL1("kpilottodo"));
 	addComponentPage(new TodoWidget(w,defaultDBPath),
 		i18n("To-do Viewer"));
 
-	ADDICONPAGE(i18n("Address Viewer"),CSL1("kpilot/icons/kpilot-address.png"));
+	ADDICONPAGE(i18n("Address Viewer"),CSL1("kpilotaddress"));
 	addComponentPage(new AddressWidget(w,defaultDBPath),
 		i18n("Address Viewer"));
 
-	ADDICONPAGE(i18n("Memo Viewer"),CSL1("kpilot/icons/kpilot-knotes.png"));
+	ADDICONPAGE(i18n("Memo Viewer"),CSL1("kpilotknotes"));
 	addComponentPage(new MemoWidget(w, defaultDBPath),
 		i18n("Memo Viewer"));
 
-	ADDICONPAGE(i18n("File Installer"),CSL1("kpilot/icons/kpilot-fileinstaller.png"));
+	ADDICONPAGE(i18n("File Installer"),CSL1("kpilotfileinstaller"));
 	fFileInstallWidget = new FileInstallWidget(
 		w,defaultDBPath);
 	addComponentPage(fFileInstallWidget, i18n("File Installer"));
 
-	ADDICONPAGE(i18n("Generic DB Viewer"),CSL1("kpilot/icons/kpilot-db.png"));
+	ADDICONPAGE(i18n("Generic DB Viewer"),CSL1("kpilotdb"));
 	addComponentPage(new GenericDBWidget(w,defaultDBPath),
 		i18n("Generic DB Viewer"));
 
 #undef ADDICONPAGE
-#undef VIEWICON
 
 	QTimer::singleShot(500,this,SLOT(initializeComponents()));
 }
@@ -403,7 +400,7 @@ void KPilotInstaller::slotSelectComponent(PilotComponent * c)
 void KPilotInstaller::slotBackupRequested()
 {
 	FUNCTIONSETUP;
-	setupSync(SyncAction::eBackup,
+	setupSync(SyncAction::SyncMode::eBackup,
 		i18n("Next sync will be a backup. ") +
 		i18n("Please press the HotSync button."));
 }
@@ -411,7 +408,7 @@ void KPilotInstaller::slotBackupRequested()
 void KPilotInstaller::slotRestoreRequested()
 {
 	FUNCTIONSETUP;
-	setupSync(SyncAction::eRestore,
+	setupSync(SyncAction::SyncMode::eRestore,
 		i18n("Next sync will restore the Pilot from backup. ") +
 		i18n("Please press the HotSync button."));
 }
@@ -419,7 +416,7 @@ void KPilotInstaller::slotRestoreRequested()
 void KPilotInstaller::slotHotSyncRequested()
 {
 	FUNCTIONSETUP;
-	setupSync(SyncAction::eHotSync,
+	setupSync(SyncAction::SyncMode::eHotSync,
 		i18n("Next sync will be a regular HotSync. ") +
 		i18n("Please press the HotSync button."));
 }
@@ -427,7 +424,7 @@ void KPilotInstaller::slotHotSyncRequested()
 void KPilotInstaller::slotFastSyncRequested()
 {
 	FUNCTIONSETUP;
-	setupSync(SyncAction::eFastSync,
+	setupSync(SyncAction::SyncMode::eFastSync,
 		i18n("Next sync will be a Fast Sync. ") +
 		i18n("Please press the HotSync button."));
 }
@@ -435,7 +432,7 @@ void KPilotInstaller::slotFastSyncRequested()
 void KPilotInstaller::slotFullSyncRequested()
 {
 	FUNCTIONSETUP;
-	setupSync(SyncAction::eFullSync,
+	setupSync(SyncAction::SyncMode::eFullSync,
 		i18n("Next sync will be a Full Sync. ") +
 		i18n("Please press the HotSync button."));
 }
@@ -443,7 +440,7 @@ void KPilotInstaller::slotFullSyncRequested()
 void KPilotInstaller::slotHHtoPCRequested()
 {
 	FUNCTIONSETUP;
-	setupSync(SyncAction::eCopyHHToPC,
+	setupSync(SyncAction::SyncMode::eCopyHHToPC,
 		i18n("Next sync will copy Handheld data to PC. ") +
 		i18n("Please press the HotSync button."));
 }
@@ -451,18 +448,11 @@ void KPilotInstaller::slotHHtoPCRequested()
 void KPilotInstaller::slotPCtoHHRequested()
 {
 	FUNCTIONSETUP;
-	setupSync(SyncAction::eCopyPCToHH,
+	setupSync(SyncAction::SyncMode::eCopyPCToHH,
 		i18n("Next sync will copy PC data to Handheld. ") +
 		i18n("Please press the HotSync button."));
 }
 
-void KPilotInstaller::slotTestSyncRequested()
-{
-	FUNCTIONSETUP;
-	setupSync(SyncAction::eTest,
-		QString::fromLatin1("Listing Pilot databases."));
-}
-
 /* virtual DCOP */ ASYNC KPilotInstaller::daemonStatus(int i)
 {
 	FUNCTIONSETUP;
@@ -655,7 +645,7 @@ void KPilotInstaller::initMenu()
 	syncPopup->insert(a);
 
 
-#ifdef DEBUG
+#if 0
 	a = new KAction(i18n("&List Only"),CSL1("listsync"),0,
 		this,SLOT(slotTestSyncRequested()),
 		actionCollection(), "file_list");
