@@ -1,7 +1,8 @@
-/* pilotDaemon.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
 ** Copyright (C) 2001-2004 by Adriaan de Groot
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** This is the KPilot Daemon, which does the actual communication with
 ** the Pilot and with the conduits.
@@ -20,15 +21,15 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 static const char *pilotdaemon_id =
-	"$Id: pilotDaemon.cc,v 1.137.2.3 2004/11/24 21:53:41 wstephens Exp $";
+	"$Id: pilotDaemon.cc 437980 2005-07-23 19:53:57Z kainhofe $";
 
 #include "options.h"
 
@@ -78,6 +79,7 @@ static KAboutData *aboutData = 0L;
 
 PilotDaemonTray::PilotDaemonTray(PilotDaemon * p) :
 	KSystemTray(0, "pilotDaemon"),
+	fSyncTypeMenu(0L),
 	daemon(p),
 	kap(0L),
 	fBlinkTimer(0L)
@@ -162,21 +164,34 @@ void PilotDaemonTray::setupWidget()
 		daemon, SLOT(slotRunConfig()));
 	menu->insertSeparator();
 
-	KPopupMenu *synctype = new KPopupMenu(menu,"sync_type_menu");
-#define MI(a,b) synctype->insertItem( \
-		KGlobal::iconLoader()->loadIconSet(b,KIcon::Small,0,true), \
-		SyncAction::syncModeName(SyncAction::a), \
-		(int)(SyncAction::a));
-	MI(eHotSync,"hotsync");
-	MI(eFastSync,"fastsync");
-	MI(eBackup,"backup");
+	fSyncTypeMenu = new KPopupMenu(menu,"sync_type_menu");
+	QString once = i18n("Appended to names of sync types to indicate the sync will happen just one time"," (once)");
+#define MI(a) fSyncTypeMenu->insertItem( \
+		SyncAction::SyncMode::name(SyncAction::SyncMode::a) + once, \
+		(int)(SyncAction::SyncMode::a));
+	fSyncTypeMenu->insertItem(i18n("Default (%1)")
+		.arg(SyncAction::SyncMode::name((SyncAction::SyncMode::Mode)KPilotSettings::syncType())),
+		0);
+	fSyncTypeMenu->insertSeparator();
+
+        // Keep this synchronized with kpilotui.rc and kpilot.cc if at all possible.
+	MI(eHotSync);
+	MI(eFastSync);
+	MI(eFullSync);
+	MI(eBackup);
+	MI(eRestore);
+	MI(eCopyHHToPC);
+	MI(eCopyPCToHH);
+
+	fSyncTypeMenu->setCheckable(true);
+	fSyncTypeMenu->setItemChecked(0,true);
 #undef MI
-	connect(synctype,SIGNAL(activated(int)),daemon,SLOT(requestSync(int)));
-	menu->insertItem(i18n("Next &Sync"),synctype);
+	connect(fSyncTypeMenu,SIGNAL(activated(int)),daemon,SLOT(requestSync(int)));
+	menu->insertItem(i18n("Next &Sync"),fSyncTypeMenu);
 
 	KHelpMenu *help = new KHelpMenu(menu,aboutData);
 	menu->insertItem(
-		KGlobal::iconLoader()->loadIconSet("help",KIcon::Small,0,true),
+		KGlobal::iconLoader()->loadIconSet(CSL1("help"),KIcon::Small,0,true),
 		i18n("&Help"),help->menu(),false /* no whatsthis */);
 
 
@@ -267,26 +282,21 @@ void PilotDaemonTray::endHotSync()
 	}
 }
 
-static inline SyncAction::SyncMode getSyncType()
-{
-	unsigned int m = KPilotSettings::syncType();
-	if (m>SyncAction::eRestore) m=SyncAction::eTest;
-	return (SyncAction::SyncMode) m;
-}
 
 PilotDaemon::PilotDaemon() :
 	DCOPObject("KPilotDaemonIface"),
 	fDaemonStatus(INIT),
 	fPostSyncAction(None),
 	fPilotLink(0L),
-	fNextSyncType(SyncAction::eTest),
+	fNextSyncType(SyncAction::SyncMode::eHotSync,true),
 	fSyncStack(0L),
 	fTray(0L),
 	fInstaller(0L),
 	fLogFile(0L),
 	fLogStub(new LoggerDCOP_stub("kpilot", "LogIface")),
 	fLogFileStub(new LoggerDCOP_stub("kpilotDaemon", "LogIface")),
-	fKPilotStub(new KPilotDCOP_stub("kpilot", "KPilotIface"))
+	fKPilotStub(new KPilotDCOP_stub("kpilot", "KPilotIface")),
+	fTempDevice(QString::null)
 {
 	FUNCTIONSETUP;
 
@@ -305,7 +315,7 @@ PilotDaemon::PilotDaemon() :
 	connect(fInstaller, SIGNAL(filesChanged()),
 		this, SLOT(slotFilesChanged()));
 
-	fNextSyncType = getSyncType();
+	fNextSyncType.setMode( KPilotSettings::syncType() );
 
 #ifdef DEBUG
 	DEBUGDAEMON << fname
@@ -403,6 +413,16 @@ void PilotDaemon::showTray()
 	updateTrayStatus();
 }
 
+/* DCOP ASYNC */ void PilotDaemon::setTempDevice(QString d)
+{
+	if ( !d.isEmpty() ){
+		fTempDevice = d;
+		if (fPilotLink)
+			fPilotLink->setTempDevice( fTempDevice );
+		reloadSettings();
+	}
+}
+
 /* DCOP ASYNC */ void PilotDaemon::reloadSettings()
 {
 	FUNCTIONSETUP;
@@ -439,10 +459,12 @@ void PilotDaemon::showTray()
 		<< endl;
 	DEBUGDAEMON << fname
 		<< ": Got conduit list "
-		<< (KPilotSettings::installedConduits().join(","))
+		<< (KPilotSettings::installedConduits().join(CSL1(",")))
 		<< endl;
 #endif
 
+	requestSync(0);
+
 
 	if (fPilotLink)
 	{
@@ -453,7 +475,21 @@ void PilotDaemon::showTray()
 			<< endl;
 #endif
 
-		fPilotLink->reset( KPilotSettings::pilotDevice());
+		fPilotLink->reset( KPilotSettings::pilotDevice() );
+#ifdef DEBUG
+		DEBUGDAEMON << fname
+			<< ": Using workarounds "
+			<< KPilotSettings::workarounds()
+			<< endl;
+#endif
+		if ( KPilotSettings::workarounds() == KPilotSettings::eWorkaroundUSB )
+		{
+#ifdef DEBUG
+			DEBUGDAEMON << fname
+				<< ": Using Zire31 USB workaround." << endl;
+#endif
+			fPilotLink->setWorkarounds(true);
+		}
 	}
 
 	if (KPilotSettings::dockDaemon())
@@ -507,7 +543,7 @@ void PilotDaemon::showTray()
 	s.append(shortStatusString());
 
 	s.append(CSL1("; NextSync="));
-	s.append(syncTypeString(fNextSyncType));
+	s.append(fNextSyncType.name());
 
 	s.append(CSL1(" ("));
 	if (fPilotLink)
@@ -560,7 +596,7 @@ bool PilotDaemon::setupPilotLink()
 	FUNCTIONSETUP;
 
 	KPILOT_DELETE(fPilotLink);
-	fPilotLink = new KPilotDeviceLink();
+	fPilotLink = new KPilotDeviceLink( 0, 0, fTempDevice );
 	if (!fPilotLink)
 	{
 		kdWarning() << k_funcinfo
@@ -610,12 +646,12 @@ bool PilotDaemon::setupPilotLink()
 
 /* DCOP ASYNC */ void PilotDaemon::requestRegularSyncNext()
 {
-	requestSync(SyncAction::eHotSync);
+	requestSync(SyncAction::SyncMode::eHotSync);
 }
 
 /* DCOP ASYNC */ void PilotDaemon::requestFastSyncNext()
 {
-	requestSync(SyncAction::eFastSync);
+	requestSync(SyncAction::SyncMode::eFastSync);
 }
 
 
@@ -623,23 +659,31 @@ bool PilotDaemon::setupPilotLink()
 {
 	FUNCTIONSETUP;
 
-
-	if ( (mode>=0) && (mode<=SyncAction::eRestore))
+	if ( 0==mode )
 	{
-		fNextSyncType = (SyncAction::SyncMode) mode;
-#ifdef DEBUG
-		DEBUGDAEMON << fname
-			<< ": Next sync is: "
-			<< syncTypeString(fNextSyncType)
-			<< endl ;
-#endif
+		mode = KPilotSettings::syncType();
 	}
-	else
+
+	if ( !fNextSyncType.setMode(mode) )
 	{
 		kdWarning() << k_funcinfo << ": Ignored fake sync type " << mode << endl;
+		return;
 	}
 
 	updateTrayStatus();
+
+	if (fTray && (fTray->fSyncTypeMenu))
+	{
+		for (int i=((int)SyncAction::SyncMode::eFastSync);
+			i<=((int)SyncAction::SyncMode::eRestore) /* Restore */ ;
+			++i)
+		{
+			fTray->fSyncTypeMenu->setItemChecked(i,mode==i);
+		}
+	}
+
+	getLogger().logMessage(i18n("Next HotSync will be: %1. ").arg(fNextSyncType.name()) +
+		i18n("Please press the HotSync button."));
 }
 
 /* DCOP ASYNC */ void PilotDaemon::requestSyncType(QString s)
@@ -647,30 +691,38 @@ bool PilotDaemon::setupPilotLink()
 	FUNCTIONSETUP;
 
 	// This checks unique prefixes of the names of the various sync types.
-	if (s.startsWith(CSL1("H"))) requestSync(SyncAction::eHotSync);
-	else if (s.startsWith(CSL1("Fa"))) requestSync(SyncAction::eFastSync);
-	else if (s.startsWith(CSL1("Fu"))) requestSync(SyncAction::eFullSync);
-	else if (s.startsWith(CSL1("B"))) requestSync(SyncAction::eBackup);
-	else if (s.startsWith(CSL1("R"))) requestSync(SyncAction::eRestore);
-	else if (s.startsWith(CSL1("T"))) requestSync(SyncAction::eTest);
-	else if (s.startsWith(CSL1("CopyHHToPC"))) requestSync(SyncAction::eCopyHHToPC);
-	else if (s.startsWith(CSL1("CopyPCToHH"))) requestSync(SyncAction::eCopyPCToHH);
+	if (s.startsWith(CSL1("H"))) requestSync(SyncAction::SyncMode::eHotSync);
+	else if (s.startsWith(CSL1("Fa"))) requestSync(SyncAction::SyncMode::eFastSync);
+	else if (s.startsWith(CSL1("Fu"))) requestSync(SyncAction::SyncMode::eFullSync);
+	else if (s.startsWith(CSL1("B"))) requestSync(SyncAction::SyncMode::eBackup);
+	else if (s.startsWith(CSL1("R"))) requestSync(SyncAction::SyncMode::eRestore);
+	else if (s.startsWith(CSL1("T"))) { fNextSyncType.setOptions(true,false); }
+	else if (s.startsWith(CSL1("CopyHHToPC"))) requestSync(SyncAction::SyncMode::eCopyHHToPC);
+	else if (s.startsWith(CSL1("CopyPCToHH"))) requestSync(SyncAction::SyncMode::eCopyPCToHH);
+	else if (s.startsWith(CSL1("D"))) requestSync(0);
 	else
 	{
-		kdWarning() << ": Unknown sync type " << ( s.isEmpty() ? "<none>" : s )
+		kdWarning() << ": Unknown sync type " << ( s.isEmpty() ? CSL1("<none>") : s )
 			<< endl;
 	}
 }
 
+/* DCOP ASYNC */ void PilotDaemon::requestSyncOptions(bool test, bool local)
+{
+	if ( !fNextSyncType.setOptions(test,local) )
+	{
+		kdWarning() << k_funcinfo << ": Nonsensical request for "
+			<< (test ? "test" : "notest")
+			<< ' '
+			<< (local ? "local" : "nolocal")
+			<< " in mode "
+			<< fNextSyncType.name() << endl;
+	}
+}
+
 /* DCOP */ int PilotDaemon::nextSyncType() const
 {
-	return fNextSyncType;
-}
-
-QString PilotDaemon::syncTypeString(SyncAction::SyncMode i) const
-{
-	FUNCTIONSETUP;
-	return SyncAction::syncModeName(i);
+	return fNextSyncType.mode();
 }
 
 /**
@@ -686,16 +738,19 @@ static QDict<QString> *conduitNameMap = 0L;
 
 static void fillConduitNameMap()
 {
-	if (!conduitNameMap)
+	if ( !conduitNameMap )
 	{
 		conduitNameMap = new QDict<QString>;
-		// Fill with internal settings.
-		conduitNameMap->insert(CSL1("internal_fileinstall"),new QString(i18n("File Installer")));
-
 		conduitNameMap->setAutoDelete(true);
 	}
+	conduitNameMap->clear();
 
 	QStringList l = KPilotSettings::installedConduits();
+	// Fill with internal settings.
+	if ( l.find( CSL1("internal_fileinstall") ) != l.end() ) {
+		conduitNameMap->insert( CSL1("internal_fileinstall"),
+		                        new QString(i18n("File Installer")) );
+	}
 
 	QStringList::ConstIterator end = l.end();
 	for (QStringList::ConstIterator i = l.begin(); i != end; ++i)
@@ -712,7 +767,7 @@ static void fillConduitNameMap()
 			{
 				readableName = o->name();
 			}
-			conduitNameMap->insert(*i,new QString(readableName));
+			conduitNameMap->insert( *i, new QString(readableName) );
 		}
 	}
 }
@@ -902,13 +957,13 @@ static void queueConduits(ActionQueue *fSyncStack,
 	if (conduits.count() > 0)
 	{
 		fSyncStack->queueConduits( conduits,e);
-		QString s = i18n("Conduit flags: ");
-		s.append(ConduitProxy::flagsForMode(e).join(" "));
+		// QString s = i18n("Conduit flags: ");
+		// s.append(ConduitProxy::flagsForMode(e).join(CSL1(" ")));
 		// logMessage(s);
 	}
 }
 
-/* slot */ void PilotDaemon::startHotSync(KPilotDeviceLink*pilotLink)
+/* slot */ void PilotDaemon::startHotSync(KPilotDeviceLink *pilotLink)
 {
 	FUNCTIONSETUP;
 
@@ -920,14 +975,11 @@ static void queueConduits(ActionQueue *fSyncStack,
 #ifdef DEBUG
 	DEBUGDAEMON << fname
 		<< ": Starting Sync with type "
-		<< syncTypeString(fNextSyncType)
-		<< " (" << fNextSyncType << ")" << endl;
+		<< fNextSyncType.name() << endl;
 	DEBUGDAEMON << fname << ": Status is " << shortStatusString() << endl;
 	(void) PilotDatabase::count();
 #endif
 
-
-
 	fDaemonStatus = HOTSYNC_START ;
 	if (fTray)
 	{
@@ -950,7 +1002,7 @@ static void queueConduits(ActionQueue *fSyncStack,
 	// Except when the user has requested a Restore, in which case she knows she doesn't
 	// want to sync with a blank palm and then back up the result over her stored backup files,
 	// do a Full Sync when changing the PC or using a different Palm Desktop app.
-	if (fNextSyncType != SyncAction::eRestore)
+	if (fNextSyncType.mode() != SyncAction::SyncMode::eRestore)
 	{
 		// Use gethostid to determine , since JPilot uses 1+(2000000000.0*random()/(RAND_MAX+1.0))
 		// as PC_ID, so using JPilot and KPilot is the same as using two different PCs
@@ -958,7 +1010,7 @@ static void queueConduits(ActionQueue *fSyncStack,
 		pcchanged = usr->getLastSyncPC() !=(unsigned long) gethostid();
 		if (pcchanged && KPilotSettings::fullSyncOnPCChange() )
 		{
-			fNextSyncType = SyncAction::eFullSync;
+			fNextSyncType = SyncAction::SyncMode::eFullSync;
 		}
 	}
 
@@ -967,60 +1019,50 @@ static void queueConduits(ActionQueue *fSyncStack,
 
 	conduits = KPilotSettings::installedConduits() ;
 
-	switch (fNextSyncType)
+	if (fNextSyncType.isTest())
 	{
-	case SyncAction::eTest:
 		fSyncStack->addAction(new TestLink(pilotLink));
-		// No conduits, nothing.
-		break;
-	case SyncAction::eBackup:
-		if (KPilotSettings::runConduitsWithBackup() && (conduits.count() > 0))
-		{
-			fSyncStack->queueConduits(conduits,SyncAction::eBackup);
-		}
-		fSyncStack->addAction(new BackupAction(pilotLink,true));
-		break;
-	case SyncAction::eRestore:
-		fSyncStack->addAction(new RestoreAction(pilotLink));
-		queueInstaller(fSyncStack,fInstaller,conduits);
-		break;
-	case SyncAction::eFullSync:
-	case SyncAction::eFastSync:
-	case SyncAction::eHotSync:
-		// first install the files, and only then do the conduits
-		// (conduits might want to sync a database that will be installed
-		queueInstaller(fSyncStack,fInstaller,conduits);
-		queueEditors(fSyncStack,pilotLink);
-		queueConduits(fSyncStack,conduits,fNextSyncType);
-		// After running the conduits, install new databases
-		queueInstaller(fSyncStack,fInstaller,conduits);
-		// And sync the remaining databases if needed.
-		if ( (fNextSyncType == SyncAction::eHotSync) ||
-			(fNextSyncType == SyncAction::eFullSync))
-		{
-			fSyncStack->addAction(new BackupAction(pilotLink, (fNextSyncType == SyncAction::eFullSync)));
-		}
-		break;
-	case SyncAction::eCopyPCToHH:
-		queueConduits(fSyncStack,conduits,SyncAction::eCopyPCToHH);
-		break;
-	case SyncAction::eCopyHHToPC:
-		queueConduits(fSyncStack,conduits,SyncAction::eCopyHHToPC);
-		break;
-#if 0
-	default:
-#ifdef DEBUG
-		DEBUGDAEMON << fname
-			<< ": Can't handle sync type "
-			<< syncTypeString(fNextSyncType) << endl;
-#endif
-		fSyncStack->addAction(new SorryAction(pilotLink),
-			i18n("KPilot cannot perform a sync of type <i>%1</i>.")
-				.arg(SyncAction::syncModeName(fNextSyncType)));
-		break;
-#endif
 	}
-
+	else
+	{
+		switch (fNextSyncType.mode())
+		{
+		case SyncAction::SyncMode::eBackup:
+			if (KPilotSettings::runConduitsWithBackup() && (conduits.count() > 0))
+			{
+				queueConduits(fSyncStack,conduits,fNextSyncType);
+			}
+			fSyncStack->addAction(new BackupAction(pilotLink,true));
+			break;
+		case SyncAction::SyncMode::eRestore:
+			fSyncStack->addAction(new RestoreAction(pilotLink));
+			queueInstaller(fSyncStack,fInstaller,conduits);
+			break;
+		case SyncAction::SyncMode::eFullSync:
+		case SyncAction::SyncMode::eFastSync:
+		case SyncAction::SyncMode::eHotSync:
+			// first install the files, and only then do the conduits
+			// (conduits might want to sync a database that will be installed
+			queueInstaller(fSyncStack,fInstaller,conduits);
+			queueEditors(fSyncStack,pilotLink);
+			queueConduits(fSyncStack,conduits,fNextSyncType);
+			// After running the conduits, install new databases
+			queueInstaller(fSyncStack,fInstaller,conduits);
+			// And sync the remaining databases if needed.
+			if ( (fNextSyncType == SyncAction::SyncMode::eHotSync) ||
+				(fNextSyncType == SyncAction::SyncMode::eFullSync))
+			{
+				fSyncStack->addAction(new BackupAction(pilotLink, (fNextSyncType == SyncAction::SyncMode::eFullSync)));
+			}
+			break;
+		case SyncAction::SyncMode::eCopyPCToHH:
+			queueConduits(fSyncStack,conduits,SyncAction::SyncMode::eCopyPCToHH);
+			break;
+		case SyncAction::SyncMode::eCopyHHToPC:
+			queueConduits(fSyncStack,conduits,SyncAction::SyncMode::eCopyHHToPC);
+			break;
+		}
+	}
 
 // Jump here to finalize the connections to the sync action
 // queue and start the actual sync.
@@ -1107,7 +1149,7 @@ launch:
 	}
 
 	fPostSyncAction = None;
-	requestRegularSyncNext();
+	requestSync(0);
 
 	(void) PilotDatabase::count();
 
@@ -1179,7 +1221,7 @@ void PilotDaemon::slotRunConfig()
 	{
 		// KPilot not running
 		KProcess *p = new KProcess;
-		*p << "kpilot" << "-c";
+		*p << "kpilot" << "-s";
 
 		p->start();
 	}
@@ -1193,20 +1235,32 @@ void PilotDaemon::updateTrayStatus(const QString &s)
 	tipText.append( s );
 	tipText.append( CSL1(" ") );
 	tipText.append( i18n("Next sync is %1.")
-		.arg( syncTypeString(fNextSyncType) ) );
+		.arg( fNextSyncType.name() ) );
 	tipText.append( CSL1("</qt>") );
 
 	QToolTip::remove(fTray);
 	QToolTip::add(fTray,tipText);
 	emitDCOPSignal( "kpilotDaemonStatusChanged()", QByteArray() );
+	// emit the same dcop signal but including the information needed by Kontact to update its kpilot summary widget
+	QByteArray data;
+	QDataStream arg(data, IO_WriteOnly);
+	arg << lastSyncDate();
+	arg << shortStatusString();
+	arg << configuredConduitList();
+	arg << logFileName();
+	arg << userName();
+	arg << pilotDevice();
+	arg << killDaemonOnExit();
+	emitDCOPSignal( "kpilotDaemonStatusDetails(QDateTime,QString,QStringList,QString,QString,QString,bool)", data );
 }
 
 static KCmdLineOptions daemonoptions[] = {
 #ifdef DEBUG
 	{"debug <level>", I18N_NOOP("Set debugging level"), "0"},
 #endif
+	{ "device <device>", I18N_NOOP("Device to try first"), ""},
 	{"fail-silently", /* TODO_I18N */ ("Exit instead of complaining "
-		"about bad configuration files."), 0},
+		"about bad configuration files"), 0},
 	KCmdLineLastOption
 } ;
 
@@ -1246,9 +1300,21 @@ int main(int argc, char **argv)
 #ifdef DEBUG
 	KPilotConfig::getDebugLevel(p);
 #endif
-
 	if (!KUniqueApplication::start())
 	{
+		if (p->isSet("device")){
+			// tell the running kpilotDaemon to use
+			// this device now
+			DCOPClient d;
+			QString dev(p->getOption("device"));
+			QByteArray data;
+			QDataStream arg(data, IO_WriteOnly);
+			arg << dev;
+			if (d.attach()){
+				d.send("kpilotDaemon", "KPilotDaemonIface", "setTempDevice(QString)", data );
+				d.detach();
+			}
+		}
 		return 0;
 	}
 	KUniqueApplication a(true, true);
@@ -1281,6 +1347,9 @@ int main(int argc, char **argv)
 
 	PilotDaemon *gPilotDaemon = new PilotDaemon();
 
+	if (p->isSet("device"))
+		gPilotDaemon->setTempDevice(p->getOption("device"));
+
 	if (gPilotDaemon->status() == PilotDaemon::ERROR)
 	{
 		delete gPilotDaemon;
