@@ -14,14 +14,15 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef RESOURCEXMLRPCCONFIG_H
 #define RESOURCEXMLRPCCONFIG_H
 
 #include <kresources/configwidget.h>
+#include <kdepimmacros.h>
 
 class KIntSpinBox;
 class KLineEdit;
@@ -29,7 +30,7 @@ class KURLRequester;
 
 namespace KCal {
 
-class ResourceXMLRPCConfig : public KRES::ConfigWidget
+class KDE_EXPORT ResourceXMLRPCConfig : public KRES::ConfigWidget
 { 
   Q_OBJECT
 
