@@ -1,6 +1,8 @@
 /*
     This file is part of libkcal.
+
     Copyright (c) 2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -14,10 +16,16 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
+/**
+   @file calendarresources.cpp
+   Provides a calendar composed of several calendar resources.
 
+   @author Cornelius Schumacher
+   @author Reinhold Kainhofer
+*/
 #include <stdlib.h>
 
 #include <qdatetime.h>
@@ -47,20 +55,27 @@
 
 using namespace KCal;
 
-ResourceCalendar *CalendarResources::StandardDestinationPolicy::destination( Incidence * )
+ResourceCalendar
+*CalendarResources::StandardDestinationPolicy::destination( Incidence * )
 {
   return resourceManager()->standardResource();
 }
 
-ResourceCalendar *CalendarResources::AskDestinationPolicy::destination( Incidence * )
+ResourceCalendar
+*CalendarResources::AskDestinationPolicy::destination( Incidence * )
 {
   QPtrList<KRES::Resource> list;
 
   CalendarResourceManager::ActiveIterator it;
-  for( it = resourceManager()->activeBegin();
-       it != resourceManager()->activeEnd(); ++it ) {
-    if ( !(*it)->readOnly() )
-      list.append( *it );
+  for ( it = resourceManager()->activeBegin();
+        it != resourceManager()->activeEnd(); ++it ) {
+    if ( !(*it)->readOnly() ) {
+      //Insert the first the Standard resource to get be the default selected.
+      if ( resourceManager()->standardResource() == *it )
+        list.insert( 0, *it );
+      else
+        list.append( *it );
+    }
   }
 
   KRES::Resource *r;
@@ -68,23 +83,18 @@ ResourceCalendar *CalendarResources::AskDestinationPolicy::destination( Incidenc
   return static_cast<ResourceCalendar *>( r );
 }
 
-CalendarResources::CalendarResources()
-  : Calendar()
+CalendarResources::CalendarResources( const QString &timeZoneId,
+                                      const QString &family )
+  : Calendar( timeZoneId )
 {
-  init();
+  init( family );
 }
 
-CalendarResources::CalendarResources(const QString &timeZoneId)
-  : Calendar(timeZoneId)
+void CalendarResources::init( const QString &family )
 {
-  init();
-}
+  kdDebug(5800) << "CalendarResources::init( " << family << " )" << endl;
 
-void CalendarResources::init()
-{
-  kdDebug(5800) << "CalendarResources::init" << endl;
-
-  mManager = new CalendarResourceManager( "calendar" );
+  mManager = new CalendarResourceManager( family );
   mManager->addObserver( this );
 
   mStandardPolicy = new StandardDestinationPolicy( mManager );
@@ -94,10 +104,7 @@ void CalendarResources::init()
 
 CalendarResources::~CalendarResources()
 {
-//  kdDebug(5800) << "CalendarResources::destructor" << endl;
-
   close();
-
   delete mManager;
 }
 
@@ -126,15 +133,40 @@ void CalendarResources::load()
     (*i1)->setTimeZoneId( timeZoneId() );
   }
 
+  QValueList<ResourceCalendar *> failed;
+
   // Open all active resources
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    (*it)->load();
+    if ( !(*it)->load() ) {
+      failed.append( *it );
+    }
+    Incidence::List incidences = (*it)->rawIncidences();
+    Incidence::List::Iterator incit;
+    for ( incit = incidences.begin(); incit != incidences.end(); ++incit ) {
+      (*incit)->registerObserver( this );
+      notifyIncidenceAdded( *incit );
+    }
+  }
+
+  QValueList<ResourceCalendar *>::ConstIterator it2;
+  for ( it2 = failed.begin(); it2 != failed.end(); ++it2 ) {
+    (*it2)->setActive( false );
+    emit signalResourceModified( *it2 );
   }
 
   mOpen = true;
 }
 
+bool CalendarResources::reload( const QString &tz )
+{
+  save();
+  close();
+  setTimeZoneId( tz );
+  load();
+  return true;
+}
+
 void CalendarResources::setStandardDestinationPolicy()
 {
   mDestinationPolicy = mStandardPolicy;
@@ -164,7 +196,7 @@ void CalendarResources::save()
 {
   kdDebug(5800) << "CalendarResources::save()" << endl;
 
-  if ( mOpen ) {
+  if ( mOpen && isModified() ) {
     CalendarResourceManager::ActiveIterator it;
     for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
       (*it)->save();
@@ -186,17 +218,59 @@ bool CalendarResources::isSaving()
   return false;
 }
 
+bool CalendarResources::addIncidence( Incidence *incidence,
+                                      ResourceCalendar *resource )
+{
+  // FIXME: Use proper locking via begin/endChange!
+  bool validRes = false;
+  CalendarResourceManager::ActiveIterator it;
+  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
+    if ( (*it) == resource )
+      validRes = true;
+  }
+  ResourceCalendar *oldResource = 0;
+  if ( mResourceMap.contains( incidence ) ) {
+    oldResource = mResourceMap[incidence];
+  }
+  mResourceMap[incidence] = resource;
+  if ( validRes && beginChange( incidence ) &&
+       resource->addIncidence( incidence ) ) {
+//    mResourceMap[incidence] = resource;
+    incidence->registerObserver( this );
+    notifyIncidenceAdded( incidence );
+    setModified( true );
+    endChange( incidence );
+    return true;
+  } else {
+    if ( oldResource )
+      mResourceMap[incidence] = oldResource;
+    else
+      mResourceMap.remove( incidence );
+  }
+
+  return false;
+}
+
 bool CalendarResources::addIncidence( Incidence *incidence )
 {
-  kdDebug(5800) << "CalendarResources::addIncidence" << endl;
+  kdDebug(5800) << "CalendarResources::addIncidence" << this << endl;
 
   ResourceCalendar *resource = mDestinationPolicy->destination( incidence );
 
   if ( resource ) {
-    if ( resource->addIncidence( incidence ) ) {
+    mResourceMap[ incidence ] = resource;
+
+    if ( beginChange( incidence ) && resource->addIncidence( incidence ) ) {
+      incidence->registerObserver( this );
+      notifyIncidenceAdded( incidence );
+
+
       mResourceMap[ incidence ] = resource;
       setModified( true );
+      endChange( incidence );
       return true;
+    } else {
+      mResourceMap.remove( incidence );
     }
   } else
     kdDebug(5800) << "CalendarResources::addIncidence(): no resource" << endl;
@@ -206,52 +280,42 @@ bool CalendarResources::addIncidence( Incidence *incidence )
 
 bool CalendarResources::addEvent( Event *event )
 {
+  kdDebug(5800) << "CalendarResources::addEvent" << endl;
   return addIncidence( event );
 }
 
-bool CalendarResources::addEvent( Event *anEvent, ResourceCalendar *resource )
+bool CalendarResources::addEvent( Event *Event, ResourceCalendar *resource )
 {
-  bool validRes = false;
-  CalendarResourceManager::ActiveIterator it;
-  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    if ( (*it) == resource ) validRes = true;
-  }
-  if ( validRes && resource->addEvent( anEvent ) ) {
-    mResourceMap[anEvent] = resource;
-    setModified( true );
-    return true;
-  }
-
-  return false;
+  return addIncidence( Event, resource );
 }
 
-void CalendarResources::deleteEvent( Event *event )
+bool CalendarResources::deleteEvent( Event *event )
 {
   kdDebug(5800) << "CalendarResources::deleteEvent" << endl;
 
+  bool status;
   if ( mResourceMap.find( event ) != mResourceMap.end() ) {
-    mResourceMap[event]->deleteEvent( event );
-    mResourceMap.remove( event );
+    status = mResourceMap[event]->deleteEvent( event );
+    if ( status )
+      mResourceMap.remove( event );
   } else {
+    status = false;
     CalendarResourceManager::ActiveIterator it;
     for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-      (*it)->deleteEvent( event );
+      status |= (*it)->deleteEvent( event );
     }
   }
 
-  setModified( true );
+  setModified( status );
+  return status;
 }
 
-
 Event *CalendarResources::event( const QString &uid )
 {
-//  kdDebug(5800) << "CalendarResources::event(): " << uid << endl;
-
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
     Event* event = (*it)->event( uid );
-    if ( event )
-    {
+    if ( event ) {
       mResourceMap[event] = *it;
       return event;
     }
@@ -264,65 +328,50 @@ Event *CalendarResources::event( const QString &uid )
 bool CalendarResources::addTodo( Todo *todo )
 {
   kdDebug(5800) << "CalendarResources::addTodo" << endl;
-
   return addIncidence( todo );
 }
 
-bool CalendarResources::addTodo(Todo *todo, ResourceCalendar *resource)
+bool CalendarResources::addTodo( Todo *todo, ResourceCalendar *resource )
 {
-  bool validRes = false;
-  CalendarResourceManager::ActiveIterator it;
-  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    if ( (*it) == resource ) validRes = true;
-  }
-  if ( validRes && resource->addTodo( todo ) ) {
-    mResourceMap[todo] = resource;
-    setModified( true );
-    return true;
-  }
-
-  return false;
+  return addIncidence( todo, resource );
 }
 
-void CalendarResources::deleteTodo( Todo *todo )
+bool CalendarResources::deleteTodo( Todo *todo )
 {
   kdDebug(5800) << "CalendarResources::deleteTodo" << endl;
 
-  Q_ASSERT(todo);
-
-  if ( mResourceMap.find(todo) != mResourceMap.end() ) {
-    mResourceMap[todo]->deleteTodo( todo );
-    mResourceMap.remove( todo );
+  bool status;
+  if ( mResourceMap.find( todo ) != mResourceMap.end() ) {
+    status = mResourceMap[todo]->deleteTodo( todo );
+    if ( status )
+      mResourceMap.remove( todo );
   } else {
     CalendarResourceManager::ActiveIterator it;
+    status = false;
     for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-      (*it)->deleteTodo( todo );
+      status |= (*it)->deleteTodo( todo );
     }
   }
 
-  setModified( true );
+  setModified( status );
+  return status;
 }
 
-Todo::List CalendarResources::rawTodos()
+Todo::List CalendarResources::rawTodos( TodoSortField sortField,
+                                        SortDirection sortDirection )
 {
-//  kdDebug(5800) << "CalendarResources::rawTodos()" << endl;
-
   Todo::List result;
 
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-//    kdDebug(5800) << "Getting raw todos from '" << (*it)->resourceName()
-//                  << "'" << endl;
-    Todo::List todos = (*it)->rawTodos();
+    Todo::List todos = (*it)->rawTodos( TodoSortUnsorted );
     Todo::List::ConstIterator it2;
     for ( it2 = todos.begin(); it2 != todos.end(); ++it2 ) {
-//      kdDebug(5800) << "Adding todo to result" << endl;
       result.append( *it2 );
       mResourceMap[ *it2 ] = *it;
     }
   }
-
-  return result;
+  return sortTodos( &result, sortField, sortDirection );
 }
 
 Todo *CalendarResources::todo( const QString &uid )
@@ -338,14 +387,12 @@ Todo *CalendarResources::todo( const QString &uid )
     }
   }
 
-  // not found
+  // Not found
   return 0;
 }
 
 Todo::List CalendarResources::rawTodosForDate( const QDate &date )
 {
-//  kdDebug(5800) << "CalendarResources::rawTodosforDate(date)" << endl;
-
   Todo::List result;
 
   CalendarResourceManager::ActiveIterator it;
@@ -361,73 +408,52 @@ Todo::List CalendarResources::rawTodosForDate( const QDate &date )
   return result;
 }
 
-
 Alarm::List CalendarResources::alarmsTo( const QDateTime &to )
 {
   kdDebug(5800) << "CalendarResources::alarmsTo" << endl;
 
   Alarm::List result;
-  CalendarResourceManager::ActiveIterator it;
-  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    Alarm::List list = (*it)->alarmsTo( to );
-    Alarm::List::Iterator it;
-    for ( it = list.begin(); it != list.end(); ++it )
-      result.append( *it );
+  CalendarResourceManager::ActiveIterator resit;
+  for ( resit = mManager->activeBegin(); resit != mManager->activeEnd(); ++resit ) {
+    Alarm::List list = (*resit)->alarmsTo( to );
+    Alarm::List::Iterator alarmit;
+    for ( alarmit = list.begin(); alarmit != list.end(); ++alarmit )
+      result.append( *alarmit );
   }
   return result;
 }
 
-Alarm::List CalendarResources::alarms( const QDateTime &from, const QDateTime &to )
+Alarm::List CalendarResources::alarms( const QDateTime &from,
+                                       const QDateTime &to )
 {
-//  kdDebug(5800) << "CalendarResources::alarms(" << from.toString() << " - "
-//                << to.toString() << ")" << endl;
-
   Alarm::List result;
-  CalendarResourceManager::ActiveIterator it;
-  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    Alarm::List list = (*it)->alarms( from, to );
-    Alarm::List::Iterator it;
-    for ( it = list.begin(); it != list.end(); ++it )
-      result.append( *it );
+  CalendarResourceManager::ActiveIterator resit;
+  for ( resit = mManager->activeBegin(); resit != mManager->activeEnd(); ++resit ) {
+    Alarm::List list = (*resit)->alarms( from, to );
+    Alarm::List::Iterator alarmit;
+    for ( alarmit = list.begin(); alarmit != list.end(); ++alarmit )
+      result.append( *alarmit );
   }
   return result;
 }
 
 /****************************** PROTECTED METHODS ****************************/
 
-
-// taking a QDate, this function will look for an eventlist in the dict
-// with that date attached -
-Event::List CalendarResources::rawEventsForDate( const QDate &qd, bool sorted )
+Event::List CalendarResources::rawEventsForDate( const QDate &date,
+                                                 EventSortField sortField,
+                                                 SortDirection sortDirection )
 {
-//  kdDebug(5800) << "CalendarResources::rawEventsForDate()" << endl;
-
   Event::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-//    kdDebug(5800) << "Getting events from '" << (*it)->resourceName() << "'"
-//                  << endl;
-    Event::List list = (*it)->rawEventsForDate( qd, sorted );
-
+    Event::List list = (*it)->rawEventsForDate( date );
     Event::List::ConstIterator it2;
-    if ( sorted ) {
-      Event::List::Iterator insertionPoint = result.begin();
-      for ( it2 = list.begin(); it2 != list.end(); ++it2 ) {
-        while ( insertionPoint != result.end() &&
-                (*insertionPoint)->dtStart().time() <= (*it2)->dtStart().time() )
-          insertionPoint++;
-        result.insert( insertionPoint, *it2 );
-        mResourceMap[ *it2 ] = *it;
-      }
-    } else {
-      for ( it2 = list.begin(); it2 != list.end(); ++it2 ) {
-        result.append( *it2 );
-        mResourceMap[ *it2 ] = *it;
-      }
+    for ( it2 = list.begin(); it2 != list.end(); ++it2 ) {
+      result.append( *it2 );
+      mResourceMap[ *it2 ] = *it;
     }
   }
-
-  return result;
+  return sortEvents( &result, sortField, sortDirection );
 }
 
 Event::List CalendarResources::rawEvents( const QDate &start, const QDate &end,
@@ -448,11 +474,11 @@ Event::List CalendarResources::rawEvents( const QDate &start, const QDate &end,
   return result;
 }
 
-Event::List CalendarResources::rawEventsForDate(const QDateTime &qdt)
+Event::List CalendarResources::rawEventsForDate( const QDateTime &qdt )
 {
   kdDebug(5800) << "CalendarResources::rawEventsForDate(qdt)" << endl;
 
-  // TODO: Remove the code duplication by the resourcemap iteration block.
+  // @TODO: Remove the code duplication by the resourcemap iteration block.
   Event::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
@@ -466,93 +492,60 @@ Event::List CalendarResources::rawEventsForDate(const QDateTime &qdt)
   return result;
 }
 
-Event::List CalendarResources::rawEvents()
+Event::List CalendarResources::rawEvents( EventSortField sortField,
+                                          SortDirection sortDirection )
 {
   kdDebug(5800) << "CalendarResources::rawEvents()" << endl;
 
   Event::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    Event::List list = (*it)->rawEvents();
+    Event::List list = (*it)->rawEvents( EventSortUnsorted );
     Event::List::ConstIterator it2;
     for ( it2 = list.begin(); it2 != list.end(); ++it2 ) {
       result.append( *it2 );
       mResourceMap[ *it2 ] = *it;
     }
   }
-  return result;
+  return sortEvents( &result, sortField, sortDirection );
 }
 
 
 bool CalendarResources::addJournal( Journal *journal )
 {
-  kdDebug(5800) << "Adding Journal on " << journal->dtStart().toString() << endl;
-
+  kdDebug(5800) << "CalendarResources::addJournal" << endl;
   return addIncidence( journal );
 }
 
-void CalendarResources::deleteJournal( Journal *journal )
+bool CalendarResources::deleteJournal( Journal *journal )
 {
   kdDebug(5800) << "CalendarResources::deleteJournal" << endl;
 
-  if ( mResourceMap.find(journal)!=mResourceMap.end() ) {
-    mResourceMap[journal]->deleteJournal( journal );
-    mResourceMap.remove( journal );
+  bool status;
+  if ( mResourceMap.find( journal ) != mResourceMap.end() ) {
+    status = mResourceMap[journal]->deleteJournal( journal );
+    if ( status )
+      mResourceMap.remove( journal );
   } else {
     CalendarResourceManager::ActiveIterator it;
+    status = false;
     for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-      (*it)->deleteJournal( journal );
+      status |= (*it)->deleteJournal( journal );
     }
   }
 
-  setModified( true );
+  setModified( status );
+  return status;
 }
 
-bool CalendarResources::addJournal(Journal *journal, ResourceCalendar *resource)
+bool CalendarResources::addJournal( Journal *journal,
+                                    ResourceCalendar *resource
+  )
 {
-  bool validRes = false;
-  CalendarResourceManager::ActiveIterator it;
-  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    if ( (*it) == resource ) validRes = true;
-  }
-  if ( validRes && resource->addJournal( journal ) ) {
-    mResourceMap[journal] = resource;
-    setModified( true );
-    return true;
-  }
-
-  return false;
+  return addIncidence( journal, resource );
 }
 
-Journal *CalendarResources::journal(const QDate &date)
-{
-  kdDebug(5800) << "CalendarResources::journal() " << date.toString() << endl;
-  kdDebug(5800) << "FIXME: what to do with the multiple journals from multiple calendar resources?" << endl;
-
-  // If we're on a private resource, return that journal.
-  // Else, first see if the standard resource has a journal for this date. If it has, return that journal.
-  // If not, check all resources for a journal for this date.
-
-  if ( mManager->standardResource() ) {
-    Journal* journal = mManager->standardResource()->journal( date );
-    if ( journal ) {
-      mResourceMap[journal] = mManager->standardResource();
-      return journal;
-    }
-  }
-  CalendarResourceManager::ActiveIterator it;
-  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    Journal* journal = (*it)->journal( date );
-    if ( journal ) {
-      mResourceMap[journal] = *it;
-      return journal;
-    }
-  }
-
-  return 0;
-}
-
-Journal *CalendarResources::journal(const QString &uid)
+Journal *CalendarResources::journal( const QString &uid )
 {
   kdDebug(5800) << "CalendarResources::journal(uid)" << endl;
 
@@ -565,20 +558,38 @@ Journal *CalendarResources::journal(const QString &uid)
     }
   }
 
-  // not found
+  // Not found
   return 0;
 }
 
-Journal::List CalendarResources::journals()
+Journal::List CalendarResources::rawJournals( JournalSortField sortField,
+                                              SortDirection sortDirection )
 {
-  kdDebug(5800) << "CalendarResources::journals()" << endl;
+  kdDebug(5800) << "CalendarResources::rawJournals()" << endl;
 
   Journal::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    Journal::List list = (*it)->journals();
+    Journal::List journals = (*it)->rawJournals( JournalSortUnsorted );
     Journal::List::ConstIterator it2;
-    for ( it2 = list.begin(); it2 != list.end(); ++it2 ) {
+    for ( it2 = journals.begin(); it2 != journals.end(); ++it2 ) {
+      result.append( *it2 );
+      mResourceMap[ *it2 ] = *it;
+    }
+  }
+  return sortJournals( &result, sortField, sortDirection );
+}
+
+Journal::List CalendarResources::rawJournalsForDate( const QDate &date )
+{
+
+  Journal::List result;
+
+  CalendarResourceManager::ActiveIterator it;
+  for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
+    Journal::List journals = (*it)->rawJournalsForDate( date );
+    Journal::List::ConstIterator it2;
+    for ( it2 = journals.begin(); it2 != journals.end(); ++it2 ) {
       result.append( *it2 );
       mResourceMap[ *it2 ] = *it;
     }
@@ -586,12 +597,6 @@ Journal::List CalendarResources::journals()
   return result;
 }
 
-
-void CalendarResources::incidenceUpdated( IncidenceBase * )
-{
-  kdDebug(5800) << "CalendarResources::incidenceUpdated( IncidenceBase * ): Not yet implemented\n";
-}
-
 void CalendarResources::connectResource( ResourceCalendar *resource )
 {
   connect( resource, SIGNAL( resourceChanged( ResourceCalendar * ) ),
@@ -607,10 +612,10 @@ void CalendarResources::connectResource( ResourceCalendar *resource )
            SLOT( slotSaveError( ResourceCalendar *, const QString & ) ) );
 }
 
-ResourceCalendar *CalendarResources::resource(Incidence *inc)
+ResourceCalendar *CalendarResources::resource( Incidence *incidence )
 {
-  if ( mResourceMap.find( inc ) != mResourceMap.end() ) {
-    return mResourceMap[ inc ];
+  if ( mResourceMap.find( incidence ) != mResourceMap.end() ) {
+    return mResourceMap[ incidence ];
   }
   return 0;
 }
@@ -619,7 +624,8 @@ void CalendarResources::resourceAdded( ResourceCalendar *resource )
 {
   kdDebug(5800) << "Resource added: " << resource->resourceName() << endl;
 
-  if ( !resource->isActive() ) return;
+  if ( !resource->isActive() )
+    return;
 
   if ( resource->open() ) {
     resource->load();
@@ -644,35 +650,46 @@ void CalendarResources::resourceDeleted( ResourceCalendar *resource )
   emit signalResourceDeleted( resource );
 }
 
-void CalendarResources::doSetTimeZoneId( const QString &tzid )
+void CalendarResources::doSetTimeZoneId( const QString &timeZoneId )
 {
   // set the timezone for all resources. Otherwise we'll have those terrible
   // tz troubles ;-((
   CalendarResourceManager::Iterator i1;
   for ( i1 = mManager->begin(); i1 != mManager->end(); ++i1 ) {
-    (*i1)->setTimeZoneId( tzid );
+    (*i1)->setTimeZoneId( timeZoneId );
   }
 }
 
-CalendarResources::Ticket *CalendarResources::requestSaveTicket( ResourceCalendar *resource )
+void CalendarResources::setTimeZoneIdViewOnly( const QString &timeZoneId )
+{
+  reload( timeZoneId );
+}
+
+CalendarResources::Ticket
+*CalendarResources::requestSaveTicket( ResourceCalendar *resource )
 {
   kdDebug(5800) << "CalendarResources::requestSaveTicket()" << endl;
 
   KABC::Lock *lock = resource->lock();
-  if ( !lock ) return 0;
-  if ( lock->lock() ) return new Ticket( resource );
-  else return 0;
+  if ( !lock )
+    return 0;
+  if ( lock->lock() )
+    return new Ticket( resource );
+  else
+    return 0;
 }
 
-bool CalendarResources::save( Ticket *ticket )
+bool CalendarResources::save( Ticket *ticket, Incidence *incidence )
 {
   kdDebug(5800) << "CalendarResources::save( Ticket *)" << endl;
 
-  if ( !ticket || !ticket->resource() ) return false;
+  if ( !ticket || !ticket->resource() )
+    return false;
 
   kdDebug(5800) << "tick " << ticket->resource()->resourceName() << endl;
 
-  if ( ticket->resource()->save() ) {
+    // @TODO: Check if the resource was changed at all. If not, don't save.
+  if ( ticket->resource()->save( incidence ) ) {
     releaseSaveTicket( ticket );
     return true;
   }
@@ -721,12 +738,13 @@ bool CalendarResources::endChange( Incidence *incidence )
   kdDebug(5800) << "CalendarResource::endChange()" << endl;
 
   ResourceCalendar *r = resource( incidence );
-  if ( !r ) return false;
+  if ( !r )
+    return false;
 
   int count = decrementChangeCount( r );
 
   if ( count == 0 ) {
-    bool ok = save( mTickets[ r ] );
+    bool ok = save( mTickets[ r ], incidence );
     if ( ok ) {
       mTickets.remove( r );
     } else {
