@@ -2,6 +2,7 @@
     This file is part of libkcal.
 
     Copyright (c) 2001-2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -15,8 +16,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 #ifndef KCAL_ICALFORMATIMPL_H
 #define KCAL_ICALFORMATIMPL_H
@@ -27,6 +28,7 @@
 #include "scheduler.h"
 #include "freebusy.h"
 
+
 extern "C" {
   #include <ical.h>
   #include <icalss.h>
@@ -50,58 +52,61 @@ class ICalFormatImpl
 
     bool populate( Calendar *, icalcomponent *fs);
 
-    icalcomponent *writeIncidence(Incidence *incidence);
+    icalcomponent *writeIncidence(IncidenceBase *incidence, Scheduler::Method method = Scheduler::Request );
     icalcomponent *writeTodo(Todo *todo);
     icalcomponent *writeEvent(Event *event);
     icalcomponent *writeFreeBusy(FreeBusy *freebusy,
-                                 Scheduler::Method method);
+                                 Scheduler::Method method = Scheduler::Publish );
     icalcomponent *writeJournal(Journal *journal);
     void writeIncidence(icalcomponent *parent,Incidence *incidence);
     icalproperty *writeAttendee(Attendee *attendee);
     icalproperty *writeOrganizer( const Person &organizer );
     icalproperty *writeAttachment(Attachment *attach);
     icalproperty *writeRecurrenceRule(Recurrence *);
-    icalproperty *writeAlarm(Alarm *alarm);
+    icalrecurrencetype writeRecurrenceRule( RecurrenceRule *recur );
+    icalcomponent *writeAlarm(Alarm *alarm);
 
     QString extractErrorProperty(icalcomponent *);
     Todo *readTodo(icalcomponent *vtodo);
-    Event *readEvent(icalcomponent *vevent);
+    Event *readEvent(icalcomponent *vevent, icalcomponent *vtimezone);
     FreeBusy *readFreeBusy(icalcomponent *vfreebusy);
     Journal *readJournal(icalcomponent *vjournal);
     Attendee *readAttendee(icalproperty *attendee);
     Person readOrganizer( icalproperty *organizer );
     Attachment *readAttachment(icalproperty *attach);
-    void readIncidence(icalcomponent *parent,Incidence *incidence);
-    void readRecurrenceRule(icalproperty *rrule,Incidence *event);
-    void readRecurrence( const struct icalrecurrencetype &r, Recurrence* recur );
+    void readIncidence(icalcomponent *parent, icaltimezone *timezone, Incidence *incidence);
+    void readRecurrenceRule(icalproperty *rrule,Incidence *event );
+    void readExceptionRule( icalproperty *rrule, Incidence *incidence );
+    void readRecurrence( const struct icalrecurrencetype &r, RecurrenceRule* recur );
     void readAlarm(icalcomponent *alarm,Incidence *incidence);
     /** Return the PRODID string loaded from calendar file */
     const QString &loadedProductId()  { return mLoadedProductId; }
 
-    icaltimetype writeICalDate(const QDate &);
-    QDate readICalDate(icaltimetype);
+    static icaltimetype writeICalDate(const QDate &);
+    static QDate readICalDate(icaltimetype);
     icaltimetype writeICalDateTime(const QDateTime &);
-    QDateTime readICalDateTime(icaltimetype);
-    icaldurationtype writeICalDuration(int seconds);
-    int readICalDuration(icaldurationtype);
+    QDateTime readICalDateTime( icaltimetype&, icaltimezone* tz = 0 );
+    static icaldurationtype writeICalDuration(int seconds);
+    static int readICalDuration(icaldurationtype);
+    static icaldatetimeperiodtype writeICalDatePeriod( const QDate &date );
+    icaldatetimeperiodtype writeICalDateTimePeriod( const QDateTime &date );
+
     icalcomponent *createCalendarComponent(Calendar * = 0);
     icalcomponent *createScheduleComponent(IncidenceBase *,Scheduler::Method);
 
+  protected:
+    void dumpIcalRecurrence(icalrecurrencetype);
   private:
     void writeIncidenceBase(icalcomponent *parent,IncidenceBase *);
-    void readIncidenceBase(icalcomponent *parent,IncidenceBase *);
+    void readIncidenceBase(icalcomponent *parent, IncidenceBase *);
     void writeCustomProperties(icalcomponent *parent,CustomProperties *);
     void readCustomProperties(icalcomponent *parent,CustomProperties *);
-    void dumpIcalRecurrence(icalrecurrencetype);
     void readTimezone(icalcomponent *vtimezone);
-    void readTzidParameter( icalcomponent *parent, icaltimetype &t );
 
     ICalFormat *mParent;
     Calendar *mCalendar;
-    QDict<class Timezone> mTimezones;
 
     QString mLoadedProductId;         // PRODID string loaded from calendar file
-    int mCalendarVersion;             // determines backward compatibility mode on read
 
     Event::List mEventsRelate;           // events with relations
     Todo::List mTodosRelate;             // todos with relations
@@ -113,6 +118,7 @@ class ICalFormatImpl
 
     Compat *mCompat;
 
+    class ToComponentVisitor;
     class Private;
     Private *d;
 };
