@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <kglobal.h>
@@ -84,10 +84,12 @@ void Todo::setDtDue(const QDateTime &dtDue, bool first )
     mDtRecurrence = dtDue;
   } else {
     mDtDue = dtDue;
-    recurrence()->setRecurStart( dtDue );
+    // TODO: This doesn't seem right...
+    recurrence()->setStartDateTime( dtDue );
+    recurrence()->setFloats( doesFloat() );
   }
 
-  if ( doesRecur() && dtDue < recurrence()->recurStart() )
+  if ( doesRecur() && dtDue < recurrence()->startDateTime() )
     setDtStart( dtDue );
 
   //kdDebug(5800) << "setDtDue says date is " << mDtDue.toString() << endl;
@@ -146,7 +148,7 @@ void Todo::setHasStartDate(bool f)
 
   if ( doesRecur() && !f ) {
     if ( !comments().grep("NoStartDate").count() )
-      addComment("NoStartDate");
+      addComment("NoStartDate"); //TODO: --> custom flag?
   } else {
     QString s("NoStartDate");
     removeComment(s);
@@ -165,8 +167,11 @@ QDateTime Todo::dtStart( bool first ) const
 
 void Todo::setDtStart( const QDateTime &dtStart )
 {
-  if ( doesRecur() )
-    recurrence()->setRecurStart( mDtDue );
+  // TODO: This doesn't seem right (rfc 2445/6 says, recurrence is calculated from the dtstart...)
+  if ( doesRecur() ) {
+    recurrence()->setStartDateTime( mDtDue );
+    recurrence()->setFloats( doesFloat() );
+  }
   IncidenceBase::setDtStart( dtStart );
 }
 
@@ -193,14 +198,22 @@ bool Todo::isCompleted() const
 
 void Todo::setCompleted(bool completed)
 {
-  if (completed) mPercentComplete = 100;
-  else mPercentComplete = 0;
+  if (completed)
+    mPercentComplete = 100;
+  else {
+    mPercentComplete = 0;
+    mHasCompletedDate = false;
+    mCompleted = QDateTime();
+  }
   updated();
 }
 
 QDateTime Todo::completed() const
 {
-  return mCompleted;
+  if ( hasCompletedDate() )
+    return mCompleted;
+  else
+    return QDateTime();
 }
 
 QString Todo::completedStr() const
@@ -210,9 +223,11 @@ QString Todo::completedStr() const
 
 void Todo::setCompleted(const QDateTime &completed)
 {
-  mHasCompletedDate = true;
-  mPercentComplete = 100;
-  mCompleted = completed;
+  if( !recurTodo() ) {
+    mHasCompletedDate = true;
+    mPercentComplete = 100;
+    mCompleted = completed;
+  }
   updated();
 }
 
@@ -229,6 +244,7 @@ int Todo::percentComplete() const
 void Todo::setPercentComplete(int v)
 {
   mPercentComplete = v;
+  if ( v != 100 ) mHasCompletedDate = false;
   updated();
 }
 
@@ -239,8 +255,44 @@ void Todo::setDtRecurrence( const QDateTime &dt )
 
 QDateTime Todo::dtRecurrence() const
 {
-  if ( !mDtRecurrence.isValid() )
-    return mDtDue;
+  return mDtRecurrence.isValid() ? mDtRecurrence : mDtDue;
+}
 
-  return mDtRecurrence;
+bool Todo::recursOn( const QDate &date ) const
+{
+  QDate today = QDate::currentDate();
+  return ( Incidence::recursOn(date) &&
+           !( date < today && mDtRecurrence.date() < today &&
+              mDtRecurrence > recurrence()->startDateTime() ) );
+}
+
+bool Todo::recurTodo()
+{
+  if ( doesRecur() ) {
+    Recurrence *r = recurrence();
+    QDateTime endDateTime = r->endDateTime();
+    QDateTime nextDate = r->getNextDateTime( dtDue() );
+
+    if ( ( r->duration() == -1 || ( nextDate.isValid() && endDateTime.isValid()
+           && nextDate <= endDateTime ) ) ) {
+      setDtDue( nextDate );
+      while ( !recursAt( dtDue() ) || dtDue() <= QDateTime::currentDateTime() ) {
+        setDtDue( r->getNextDateTime( dtDue() ) );
+      }
+
+      setCompleted( false );
+      setRevision( revision() + 1 );
+
+      return true;
+    }
+  }
+
+  return false;
+}
+
+bool Todo::isOverdue() const
+{
+  bool inPast = doesFloat() ? mDtDue.date() < QDate::currentDate()
+                            : mDtDue < QDateTime::currentDateTime();
+  return ( inPast && !isCompleted() );
 }
