@@ -19,15 +19,27 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "addresseelineedit.h"
 
+#include "resourceabc.h"
+#include "completionordereditor.h"
+#include "ldapclient.h"
+
+#include <config.h>
+
+#ifdef KDEPIM_NEW_DISTRLISTS
+#include "distributionlist.h"
+#else
 #include <kabc/distributionlist.h>
+#endif
+
 #include <kabc/stdaddressbook.h>
 #include <kabc/resource.h>
+#include <libemailfunctions/email.h>
 
 #include <kcompletionbox.h>
 #include <kcursor.h>
@@ -38,9 +50,6 @@
 #include <kurldrag.h>
 #include <klocale.h>
 
-#include "completionordereditor.h"
-#include "ldapclient.h"
-
 #include <qpopupmenu.h>
 #include <qapplication.h>
 #include <qobject.h>
@@ -49,12 +58,12 @@
 #include <qevent.h>
 #include <qdragobject.h>
 #include <qclipboard.h>
-#include "resourceabc.h"
 
 using namespace KPIM;
 
 KCompletion * AddresseeLineEdit::s_completion = 0L;
 KPIM::CompletionItemsMap* AddresseeLineEdit::s_completionItemMap = 0L;
+QStringList* AddresseeLineEdit::s_completionSources = 0L;
 bool AddresseeLineEdit::s_addressesDirty = false;
 QTimer* AddresseeLineEdit::s_LDAPTimer = 0L;
 KPIM::LdapSearch* AddresseeLineEdit::s_LDAPSearch = 0L;
@@ -66,6 +75,7 @@ static KStaticDeleter<KPIM::CompletionItemsMap> completionItemsDeleter;
 static KStaticDeleter<QTimer> ldapTimerDeleter;
 static KStaticDeleter<KPIM::LdapSearch> ldapSearchDeleter;
 static KStaticDeleter<QString> ldapTextDeleter;
+static KStaticDeleter<QStringList> completionSourcesDeleter;
 
 // needs to be unique, but the actual name doesn't matter much
 static QCString newLineEditDCOPObjectName()
@@ -79,6 +89,8 @@ static QCString newLineEditDCOPObjectName()
     return name;
 }
 
+static const QString s_completionItemIndentString = "     ";
+
 AddresseeLineEdit::AddresseeLineEdit( QWidget* parent, bool useCompletion,
                                       const char *name )
   : ClickLineEdit( parent, QString::null, name ), DCOPObject( newLineEditDCOPObjectName() )
@@ -99,10 +111,11 @@ void AddresseeLineEdit::init()
 {
   if ( !s_completion ) {
     completionDeleter.setObject( s_completion, new KCompletion() );
-    s_completion->setOrder( KCompletion::Weighted );
+    s_completion->setOrder( completionOrder() );
     s_completion->setIgnoreCase( true );
 
     completionItemsDeleter.setObject( s_completionItemMap, new KPIM::CompletionItemsMap() );
+    completionSourcesDeleter.setObject( s_completionSources, new QStringList() );
   }
 
 //  connect( s_completion, SIGNAL( match( const QString& ) ),
@@ -113,6 +126,13 @@ void AddresseeLineEdit::init()
       ldapTimerDeleter.setObject( s_LDAPTimer, new QTimer );
       ldapSearchDeleter.setObject( s_LDAPSearch, new KPIM::LdapSearch );
       ldapTextDeleter.setObject( s_LDAPText, new QString );
+
+      /* Add completion sources for all ldap server, 0 to n. Added first so 
+       * that they map to the ldapclient::clientNumber() */
+      QValueList< LdapClient* > clients =  s_LDAPSearch->clients();
+      for ( QValueList<LdapClient*>::iterator it = clients.begin(); it != clients.end(); ++it ) {
+        addCompletionSource( "LDAP server: " + (*it)->server().host() );
+      }
     }
     if ( !m_completionInitialized ) {
       setCompletionObject( s_completion, false );
@@ -220,13 +240,15 @@ void AddresseeLineEdit::insert( const QString &t )
 
   QString contents = text();
   int start_sel = 0;
-  int pos = cursorPosition();
-
-  if ( hasSelectedText() ) {
+  int end_sel = 0;
+  int pos = cursorPosition( );
+  if ( getSelection( &start_sel, &end_sel ) ) {
     // Cut away the selection.
-    start_sel = selectionStart();
+    if ( pos > end_sel )
+      pos -= (end_sel - start_sel);
+    else if ( pos > start_sel )
       pos = start_sel;
-    contents = contents.left( start_sel ) + contents.mid( start_sel + selectedText().length() );
+    contents = contents.left( start_sel ) + contents.right( end_sel + 1 );
   }
 
   int eot = contents.length();
@@ -247,6 +269,11 @@ void AddresseeLineEdit::insert( const QString &t )
   setCursorPosition( pos + newText.length() );
 }
 
+void AddresseeLineEdit::setText( const QString & text )
+{
+  ClickLineEdit::setText( text.stripWhiteSpace() );
+}
+
 void AddresseeLineEdit::paste()
 {
   if ( m_useCompletion )
@@ -326,17 +353,19 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
   if ( !m_useCompletion )
     return;
 
-  if ( s_addressesDirty )
+  if ( s_addressesDirty ) {
     loadContacts(); // read from local address book
+    s_completion->setOrder( completionOrder() );
+  }
 
   // cursor at end of string - or Ctrl+T pressed for substring completion?
   if ( ctrlT ) {
-    const QStringList completions = s_completion->substringCompletion( m_searchString );
+    const QStringList completions = getAdjustedCompletionItems( false );
 
-    if ( completions.count() == 1 ) {
-      setText( m_searchString + completions.first() );
-      setEdited( true );
-    }
+    if ( completions.count() > 1 )
+      ; //m_previousAddresses = prevAddr;
+    else if ( completions.count() == 1 )
+      setText( m_previousAddresses + completions.first().stripWhiteSpace() );
 
     setCompletedItems( completions, true ); // this makes sure the completion popup is closed if no matching items were found
 
@@ -355,38 +384,16 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
 
     case KGlobalSettings::CompletionPopup:
     {
-      //m_previousAddresses = prevAddr;
-      QStringList items = s_completion->allMatches( m_searchString );
-      items += s_completion->allMatches( "\"" + m_searchString );
-      uint beforeDollarCompletionCount = items.count();
+      const QStringList items = getAdjustedCompletionItems( true );
+      bool autoSuggest = !items.isEmpty() && (mode != KGlobalSettings::CompletionPopupAuto);
+      setCompletedItems( items, autoSuggest );
 
-      if ( m_searchString.find( ' ' ) == -1 ) // one word, possibly given name
-        items += s_completion->allMatches( "$$" + m_searchString );
-
-      if ( items.isEmpty() ) {
-        setCompletedItems( items, false );
-      } else {
-        if ( items.count() > beforeDollarCompletionCount ) {
-          // remove the '$$whatever$' part
-          for ( QStringList::Iterator it = items.begin(); it != items.end(); ++it ) {
-            int pos = (*it).find( '$', 2 );
-            if ( pos < 0 ) // ???
-              continue;
-            (*it) = (*it).mid( pos + 1 );
-          }
-        }
-
-        bool autoSuggest = (mode != KGlobalSettings::CompletionPopupAuto);
-        setCompletedItems( items, autoSuggest );
-
-        if ( !autoSuggest ) {
-          int index = items.first().find( m_searchString );
-          QString newText = m_previousAddresses + items.first().mid( index );
-          setUserSelection( false );
-          setCompletedText( newText, true );
-        }
+      if ( !autoSuggest ) {
+        int index = items.first().find( m_searchString );
+        QString newText = m_previousAddresses + items.first().mid( index ).stripWhiteSpace();
+        setUserSelection( false );
+        setCompletedText( newText, true );
       }
-
       break;
     }
 
@@ -422,8 +429,7 @@ void AddresseeLineEdit::doCompletion( bool ctrlT )
 
 void AddresseeLineEdit::slotPopupCompletion( const QString& completion )
 {
-  setText( m_previousAddresses + completion );
-  setEdited( true );
+  setText( m_previousAddresses + completion.stripWhiteSpace() );
   cursorAtEnd();
 //  slotMatched( m_previousAddresses + completion );
 }
@@ -431,6 +437,7 @@ void AddresseeLineEdit::slotPopupCompletion( const QString& completion )
 void AddresseeLineEdit::loadContacts()
 {
   s_completion->clear();
+  s_completionItemMap->clear();
   s_addressesDirty = false;
   //m_contactMap.clear();
 
@@ -452,26 +459,36 @@ void AddresseeLineEdit::loadContacts()
       for ( it = resource->begin(); it != resource->end(); ++it ) {
         QString uid = (*it).uid();
         QMap<QString, QString>::const_iterator wit = uidToResourceMap.find( uid );
+        const QString subresourceLabel = resabc->subresourceLabel( *wit );
+        int idx = s_completionSources->findIndex( subresourceLabel );
+        if ( idx == -1 ) {
+          s_completionSources->append( subresourceLabel );
+          idx = s_completionSources->size() -1;
+        }
         int weight = ( wit != uidToResourceMap.end() ) ? resabc->subresourceCompletionWeight( *wit ) : 80;
         //kdDebug(5300) << (*it).fullEmail() << " subres=" << *wit << " weight=" << weight << endl;
-        addContact( *it, weight );
+        addContact( *it, weight, idx );
       }
     } else { // KABC non-imap resource
       int weight = config.readNumEntry( resource->identifier(), 60 );
+      s_completionSources->append( resource->resourceName() );
       KABC::Resource::Iterator it;
       for ( it = resource->begin(); it != resource->end(); ++it )
-        addContact( *it, weight );
+        addContact( *it, weight, s_completionSources->size()-1 );
     }
   }
 
+#ifndef KDEPIM_NEW_DISTRLISTS // new distr lists are normal contact, already done above
   int weight = config.readNumEntry( "DistributionLists", 60 );
   KABC::DistributionListManager manager( addressBook );
   manager.load();
   const QStringList distLists = manager.listNames();
   QStringList::const_iterator listIt;
+  int idx = addCompletionSource( i18n( "Distribution Lists" ) );
   for ( listIt = distLists.begin(); listIt != distLists.end(); ++listIt ) {
-    s_completion->addItem( (*listIt).simplifyWhiteSpace(), weight );
+    addCompletionItem( (*listIt).simplifyWhiteSpace(), weight, idx );
   }
+#endif
 
   QApplication::restoreOverrideCursor();
 
@@ -481,12 +498,32 @@ void AddresseeLineEdit::loadContacts()
   }
 }
 
-void AddresseeLineEdit::addContact( const KABC::Addressee& addr, int weight )
+void AddresseeLineEdit::addContact( const KABC::Addressee& addr, int weight, int source )
 {
+#ifdef KDEPIM_NEW_DISTRLISTS
+  if ( KPIM::DistributionList::isDistributionList( addr ) ) {
+    //kdDebug(5300) << "AddresseeLineEdit::addContact() distribution list \"" << addr.formattedName() << "\" weight=" << weight << endl;
+    addCompletionItem( addr.formattedName(), weight, source );
+    return;
+  }
+#endif
   //m_contactMap.insert( addr.realName(), addr );
   const QStringList emails = addr.emails();
   QStringList::ConstIterator it;
   for ( it = emails.begin(); it != emails.end(); ++it ) {
+
+    if ( addr.givenName().isEmpty() && addr.familyName().isEmpty() ) {
+      addCompletionItem( addr.fullEmail( (*it) ), weight, source ); // use whatever is there
+    } else {
+      const QString byFirstName= KPIM::quoteNameIfNecessary( addr.givenName() + " " + addr.familyName() ) + " <" + (*it) + ">";
+      const QString byLastName= "\"" + addr.familyName() + ", " + addr.givenName() + "\" "  + "<" + (*it) + ">";
+      const QString byEmail= (*it) + " (" + KPIM::quoteNameIfNecessary( addr.realName() ) + ")";
+      addCompletionItem( byFirstName, weight, source );
+      addCompletionItem( byLastName, weight, source );
+      addCompletionItem( byEmail, weight, source );
+    }
+
+#if 0
     int len = (*it).length();
     if ( len == 0 ) continue;
     if( '\0' == (*it)[len-1] )
@@ -494,7 +531,7 @@ void AddresseeLineEdit::addContact( const KABC::Addressee& addr, int weight )
     const QString tmp = (*it).left( len );
     const QString fullEmail = addr.fullEmail( tmp );
     //kdDebug(5300) << "AddresseeLineEdit::addContact() \"" << fullEmail << "\" weight=" << weight << endl;
-    addCompletionItem( fullEmail.simplifyWhiteSpace(), weight );
+    addCompletionItem( fullEmail.simplifyWhiteSpace(), weight, source );
     // Try to guess the last name: if found, we add an extra
     // entry to the list to make sure completion works even
     // if the user starts by typing in the last name.
@@ -506,59 +543,57 @@ void AddresseeLineEdit::addContact( const KABC::Addressee& addr, int weight )
 
     // While we're here also add "email (full name)" for completion on the email
     if ( !name.isEmpty() )
-      addCompletionItem( addr.preferredEmail() + " (" + name + ")", weight );
-
-    if ( name.find( ',' ) != -1 ) return; // probably already of the form "Lastname, Firstname"
+      addCompletionItem( addr.preferredEmail() + " (" + name + ")", weight, source );
 
     bool bDone = false;
-    int i = 1;
-    do{
-      i = name.findRev(' ');
-      if( 1 < i ){
-        QString sLastName( name.mid(i+1) );
-        if( ! sLastName.isEmpty() &&
+    int i = -1;
+    while( ( i = name.findRev(' ') ) > 1 && !bDone ) {
+      QString sLastName( name.mid( i+1 ) );
+      if( ! sLastName.isEmpty() &&
             2 <= sLastName.length() &&   // last names must be at least 2 chars long
-            ! sLastName.endsWith(".") ){ // last names must not end with a dot (like "Jr." or "Sr.")
-          name.truncate( i );
-          if( !name.isEmpty() ){
-            sLastName.prepend( "\"" );
-            sLastName.append( ", " + name + "\" <" );
-          }
-          QString sExtraEntry( sLastName );
-          sExtraEntry.append( tmp.isEmpty() ? addr.preferredEmail() : tmp );
-          sExtraEntry.append( ">" );
-          //kdDebug(5300) << "AddresseeLineEdit::addContact() added extra \"" << sExtraEntry.simplifyWhiteSpace() << "\" weight=" << weight << endl;
-          addCompletionItem( sExtraEntry.simplifyWhiteSpace(), weight );
-          bDone = true;
+          ! sLastName.endsWith(".") ) { // last names must not end with a dot (like "Jr." or "Sr.")
+        name.truncate( i );
+        if( !name.isEmpty() ){
+          sLastName.prepend( "\"" );
+          sLastName.append( ", " + name + "\" <" );
         }
+        QString sExtraEntry( sLastName );
+        sExtraEntry.append( tmp.isEmpty() ? addr.preferredEmail() : tmp );
+        sExtraEntry.append( ">" );
+        //kdDebug(5300) << "AddresseeLineEdit::addContact() added extra \"" << sExtraEntry.simplifyWhiteSpace() << "\" weight=" << weight << endl;
+        addCompletionItem( sExtraEntry.simplifyWhiteSpace(), weight, source );
+        bDone = true;
       }
-      if( !bDone ){
+      if( !bDone ) {
         name.truncate( i );
         if( name.endsWith("\"") )
           name.truncate( name.length()-1 );
       }
-    }while( 1 < i && !bDone );
+    }
+#endif
   }
 }
 
-void AddresseeLineEdit::addCompletionItem( const QString& string, int weight )
+void AddresseeLineEdit::addCompletionItem( const QString& string, int weight, int completionItemSource )
 {
   // Check if there is an exact match for item already, and use the max weight if so.
   // Since there's no way to get the information from KCompletion, we have to keep our own QMap
   CompletionItemsMap::iterator it = s_completionItemMap->find( string );
   if ( it != s_completionItemMap->end() ) {
-    weight = QMAX( *it, weight );
-    *it = weight;
+    weight = QMAX( ( *it ).first, weight );
+    ( *it ).first = weight;
   } else {
-    s_completionItemMap->insert( string, weight );
+    s_completionItemMap->insert( string, qMakePair( weight, completionItemSource ) );
   }
-
   s_completion->addItem( string, weight );
 }
 
 void AddresseeLineEdit::slotStartLDAPLookup()
 {
-  if ( !s_LDAPSearch->isAvailable() || s_LDAPLineEdit != this )
+  if ( !s_LDAPSearch->isAvailable() ) {
+    return;
+  }
+  if (  s_LDAPLineEdit != this )
     return;
 
   startLoadingLDAPEntries();
@@ -568,7 +603,6 @@ void AddresseeLineEdit::stopLDAPLookup()
 {
   s_LDAPSearch->cancelSearch();
   s_LDAPLineEdit = NULL;
-  s_completionItemMap->clear();
 }
 
 void AddresseeLineEdit::startLoadingLDAPEntries()
@@ -598,7 +632,8 @@ void AddresseeLineEdit::slotLDAPSearchData( const KPIM::LdapResultList& adrs )
     KABC::Addressee addr;
     addr.setNameFromString( (*it).name );
     addr.setEmails( (*it).email );
-    addContact( addr, (*it).completionWeight );
+    
+    addContact( addr, (*it).completionWeight, (*it ).clientNumber  );
   }
 
   if ( hasFocus() || completionBox()->hasFocus() )
@@ -633,6 +668,11 @@ void AddresseeLineEdit::setCompletedItems( const QStringList& items, bool autoSu
             completionBox->setCancelledText( m_searchString );
           completionBox->setItems( items );
           completionBox->popup();
+          // we have to install the event filter after popup(), since that 
+          // calls show(), and that's where KCompletionBox installs its filter.
+          // We want to be first, though, so do it now.
+          if ( s_completion->order() == KCompletion::Weighted )
+            qApp->installEventFilter( this );
         }
 
         if ( autoSuggest )
@@ -645,8 +685,9 @@ void AddresseeLineEdit::setCompletedItems( const QStringList& items, bool autoSu
     }
     else
     {
-        if ( completionBox && completionBox->isVisible() )
+        if ( completionBox && completionBox->isVisible() ) {
             completionBox->hide();
+        }
     }
 }
 
@@ -657,7 +698,7 @@ QPopupMenu* AddresseeLineEdit::createPopupMenu()
     return 0;
 
   if ( m_useCompletion )
-    menu->insertItem( i18n( "Edit Completion Order..." ),
+    menu->insertItem( i18n( "Configure Completion Order..." ),
                       this, SLOT( slotEditCompletionOrder() ) );
   return menu;
 }
@@ -703,7 +744,169 @@ void KPIM::AddresseeLineEdit::slotCompletion()
   {
     m_previousAddresses = QString::null;
   }
+  if ( completionBox() )
+    completionBox()->setCancelledText( m_searchString );
   doCompletion( false );
 }
 
+// not cached, to make sure we get an up-to-date value when it changes
+KCompletion::CompOrder KPIM::AddresseeLineEdit::completionOrder()
+{
+  KConfig config( "kpimcompletionorder" );
+  config.setGroup( "General" );
+  const QString order = config.readEntry( "CompletionOrder", "Weighted" );
+
+  if ( order == "Weighted" )
+    return KCompletion::Weighted;
+  else
+    return KCompletion::Sorted;
+}
+
+int KPIM::AddresseeLineEdit::addCompletionSource( const QString &source )
+{
+  s_completionSources->append( source );
+  return s_completionSources->size()-1;
+}
+
+bool KPIM::AddresseeLineEdit::eventFilter(QObject *obj, QEvent *e)
+{
+  if ( obj == completionBox() ) {
+    if ( e->type() == QEvent::MouseButtonPress
+      || e->type() == QEvent::MouseMove
+      || e->type() == QEvent::MouseButtonRelease ) {
+      QMouseEvent* me = static_cast<QMouseEvent*>( e );
+      // find list box item at the event position
+      QListBoxItem *item = completionBox()->itemAt( me->pos() );
+      if ( !item ) {
+        // In the case of a mouse move outside of the box we don't want
+        // the parent to fuzzy select a header by mistake.
+        bool eat = e->type() == QEvent::MouseMove;
+        return eat;
+      }
+      // avoid selection of headers on button press, or move or release while
+      // a button is pressed
+      if ( e->type() == QEvent::MouseButtonPress 
+          || me->state() & LeftButton || me->state() & MidButton 
+          || me->state() & RightButton ) {
+        if ( !item->text().startsWith( s_completionItemIndentString ) ) {
+          return true; // eat the event, we don't want anything to happen
+        } else {
+          // if we are not on one of the group heading, make sure the item
+          // below or above is selected, not the heading, inadvertedly, due
+          // to fuzzy auto-selection from QListBox
+          completionBox()->setCurrentItem( item );
+          completionBox()->setSelected( completionBox()->index( item ), true );
+          if ( e->type() == QEvent::MouseMove )
+            return true; // avoid fuzzy selection behavior
+        }
+      }
+    }
+  }
+  if ( ( obj == this ) &&
+     ( e->type() == QEvent::AccelOverride ) ) {
+    QKeyEvent *ke = static_cast<QKeyEvent*>( e );
+    if ( ke->key() == Key_Up || ke->key() == Key_Down ) {
+      ke->accept();
+      return true;
+    }
+  }
+  if ( ( obj == this ) &&
+      ( e->type() == QEvent::KeyPress ) &&
+      completionBox()->isVisible() ) {
+    QKeyEvent *ke = static_cast<QKeyEvent*>( e );
+    unsigned int currentIndex = completionBox()->currentItem();
+    if ( ke->key() == Key_Up ) {
+      //kdDebug() << "EVENTFILTER: Key_Up currentIndex=" << currentIndex << endl;
+      // figure out if the item we would be moving to is one we want
+      // to ignore. If so, go one further
+      QListBoxItem *itemAbove = completionBox()->item( currentIndex - 1 );
+      if ( itemAbove && !itemAbove->text().startsWith( s_completionItemIndentString ) ) {
+        // there is a header above is, check if there is even further up
+        // and if so go one up, so it'll be selected
+        if ( currentIndex > 1 && completionBox()->item( currentIndex - 2 ) ) {
+          //kdDebug() << "EVENTFILTER: Key_Up -> skipping " << currentIndex - 1 << endl;
+          completionBox()->setCurrentItem( itemAbove->prev() );
+          completionBox()->setSelected( currentIndex - 2, true );
+        } else if ( currentIndex == 1 ) {
+            // nothing to skip to, let's stay where we are, but make sure the
+            // first header becomes visible, if we are the first real entry
+            completionBox()->ensureVisible( 0, 0 );
+        }
+        return true;
+      }
+    } else if ( ke->key() == Key_Down ) {
+      // same strategy for downwards
+      //kdDebug() << "EVENTFILTER: Key_Down. currentIndex=" << currentIndex << endl;
+      QListBoxItem *itemBelow = completionBox()->item( currentIndex + 1 );
+      if ( itemBelow && !itemBelow->text().startsWith( s_completionItemIndentString ) ) {
+        if ( completionBox()->item( currentIndex + 2 ) ) {
+          //kdDebug() << "EVENTFILTER: Key_Down -> skipping " << currentIndex+1 << endl;
+          completionBox()->setCurrentItem( itemBelow->next() );
+          completionBox()->setSelected( currentIndex + 2, true );
+        } else {
+          // nothing to skip to, let's stay where we are
+        }
+        return true;
+      }
+      // special case of the initial selection, which is unfortunately a header.
+      // Setting it to selected tricks KCompletionBox into not treating is special
+      // and selecting making it current, instead of the one below.
+      QListBoxItem *item = completionBox()->item( currentIndex );
+      if ( item && !item->text().startsWith( s_completionItemIndentString )  ) {
+        completionBox()->setSelected( currentIndex, true );
+      }
+    }
+  }
+  return ClickLineEdit::eventFilter( obj, e );
+}
+
+const QStringList KPIM::AddresseeLineEdit::getAdjustedCompletionItems( bool fullSearch )
+{
+  QStringList items = fullSearch ?
+    s_completion->allMatches( m_searchString )
+    : s_completion->substringCompletion( m_searchString );
+  if ( fullSearch )
+    items += s_completion->allMatches( "\"" + m_searchString );
+  unsigned int beforeDollarCompletionCount = items.count();
+
+  if ( fullSearch && m_searchString.find( ' ' ) == -1 ) // one word, possibly given name
+    items += s_completion->allMatches( "$$" + m_searchString );
+
+  // kdDebug(5300) << "     AddresseeLineEdit::doCompletion() found: " << items.join(" AND ") << endl;
+
+  int lastSourceIndex = -1;
+  unsigned int i = 0;
+  QMap<int, QStringList> sections;
+  for ( QStringList::Iterator it = items.begin(); it != items.end(); ++it, ++i ) {
+    CompletionItemsMap::const_iterator cit = s_completionItemMap->find(*it);
+    if ( cit == s_completionItemMap->end() )continue;
+    int idx = (*cit).second;
+    if ( s_completion->order() == KCompletion::Weighted ) {
+      if ( lastSourceIndex == -1 || lastSourceIndex != idx ) {
+        const QString sourceLabel(  (*s_completionSources)[idx] );
+        if ( sections.find(idx) == sections.end() ) {
+          items.insert( it, sourceLabel );
+        }
+        lastSourceIndex = idx;
+      }
+      (*it) = (*it).prepend( s_completionItemIndentString );
+      sections[idx].append( *it );
+    }
+    if ( i > beforeDollarCompletionCount ) { 
+      // remove the '$$whatever$' part
+      int pos = (*it).find( '$', 2 );
+      if ( pos < 0 ) // ???
+        continue;
+      (*it) = (*it).mid( pos + 1 );
+    }
+  }
+  QStringList sortedItems;
+  for ( QMap<int, QStringList>::Iterator it( sections.begin() ), end( sections.end() ); it != end; ++it ) {
+    sortedItems.append( (*s_completionSources)[it.key()] );
+    for ( QStringList::Iterator sit( (*it).begin() ), send( (*it).end() ); sit != send; ++sit ) {
+      sortedItems.append( *sit );
+    }
+  }
+  return sortedItems;
+}
 #include "addresseelineedit.moc"
