@@ -2,6 +2,8 @@
     This file is part of libkdepim.
 
     Copyright (c) 2001-2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
+    Copyright (C) 2005 Allen Winter <winter@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -15,8 +17,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 #ifndef KPREFSDIALOG_H
 #define KPREFSDIALOG_H
@@ -24,10 +26,13 @@
 #include <qptrlist.h>
 #include <qlineedit.h>
 #include <qvaluelist.h>
+#include <qdatetimeedit.h>
 
 #include <kdialogbase.h>
 #include <kcmodule.h>
 #include <kconfigskeleton.h>
+#include <kfile.h>
+#include <kdepimmacros.h>
 
 
 class KColorButton;
@@ -35,7 +40,10 @@ class QCheckBox;
 class QLabel;
 class QSpinBox;
 class QButtonGroup;
+class QTimeEdit;
 class KTimeEdit;
+class KDateEdit;
+class KURLRequester;
 
 /**
   @short Base class for GUI control elements used by @ref KPrefsDialog.
@@ -46,7 +54,7 @@ class KTimeEdit;
   KPrefsDialog. The control element consists of a set of widgets for handling
   a certain type of configuration information.
 */
-class KPrefsWid : public QObject
+class KDE_EXPORT KPrefsWid : public QObject
 {
     Q_OBJECT
   public:
@@ -79,7 +87,7 @@ class KPrefsWid : public QObject
   This class provides a control element for configuring bool values. It is meant
   to be used by KPrefsDialog. The user is responsible for the layout management.
 */
-class KPrefsWidBool : public KPrefsWid
+class KDE_EXPORT KPrefsWidBool : public KPrefsWid
 {
   public:
     /**
@@ -113,7 +121,7 @@ class KPrefsWidBool : public KPrefsWid
   meant to be used by KPrefsDialog. The user is responsible for the layout
   management.
 */
-class KPrefsWidInt : public KPrefsWid
+class KDE_EXPORT KPrefsWidInt : public KPrefsWid
 {
   public:
     /**
@@ -154,7 +162,7 @@ class KPrefsWidInt : public KPrefsWid
   meant to be used by KPrefsDialog. The user is responsible for the layout
   management.
 */
-class KPrefsWidTime : public KPrefsWid
+class KDE_EXPORT KPrefsWidTime : public KPrefsWid
 {
   public:
     /**
@@ -184,6 +192,81 @@ class KPrefsWidTime : public KPrefsWid
     KTimeEdit *mTimeEdit;
 };
 
+/**
+  @short Widgets for duration settings in @ref KPrefsDialog.
+
+  This class provides a control element for configuring duration values. It is
+  meant to be used by KPrefsDialog. The user is responsible for the layout
+  management.
+*/
+class KDE_EXPORT KPrefsWidDuration : public KPrefsWid
+{
+  public:
+    /**
+      Create a duration value control element consisting of a label and a
+      spinbox.
+
+      @param item    The KConfigSkeletonItem representing the preferences entry.
+      @param parent  Parent widget.
+    */
+    KPrefsWidDuration( KConfigSkeleton::ItemDateTime *item, QWidget *parent );
+
+    /**
+      Return QLabel used by this widget.
+    */
+    QLabel *label();
+    /**
+      Return QSpinBox used by this widget.
+    */
+    QTimeEdit *timeEdit();
+
+    void readConfig();
+    void writeConfig();
+
+  private:
+    KConfigSkeleton::ItemDateTime *mItem;
+
+    QLabel *mLabel;
+    QTimeEdit *mTimeEdit;
+};
+
+/**
+  @short Widgets for time settings in @ref KPrefsDialog.
+
+  This class provides a control element for configuring date values. It is
+  meant to be used by KPrefsDialog. The user is responsible for the layout
+  management.
+*/
+class KDE_EXPORT KPrefsWidDate : public KPrefsWid
+{
+  public:
+    /**
+      Create a time value control element consisting of a label and a date box.
+
+      @param item    The KConfigSkeletonItem representing the preferences entry.
+      @param parent  Parent widget.
+    */
+    KPrefsWidDate( KConfigSkeleton::ItemDateTime *item, QWidget *parent );
+
+    /**
+      Return QLabel used by this widget.
+    */
+    QLabel *label();
+    /**
+      Return QSpinBox used by this widget.
+    */
+    KDateEdit *dateEdit();
+
+    void readConfig();
+    void writeConfig();
+
+  private:
+    KConfigSkeleton::ItemDateTime *mItem;
+
+    QLabel *mLabel;
+    KDateEdit *mDateEdit;
+};
+
 /**
   @short Widgets for color settings in @ref KPrefsDialog.
 
@@ -191,7 +274,7 @@ class KPrefsWidTime : public KPrefsWid
   meant to be used by KPrefsDialog. The user is responsible for the layout
   management.
 */
-class KPrefsWidColor : public KPrefsWid
+class KDE_EXPORT KPrefsWidColor : public KPrefsWid
 {
     Q_OBJECT
   public:
@@ -233,7 +316,7 @@ class KPrefsWidColor : public KPrefsWid
   This class provides a control element for configuring font values. It is meant
   to be used by KPrefsDialog. The user is responsible for the layout management.
 */
-class KPrefsWidFont : public KPrefsWid
+class KDE_EXPORT KPrefsWidFont : public KPrefsWid
 {
     Q_OBJECT
   public:
@@ -243,6 +326,7 @@ class KPrefsWidFont : public KPrefsWid
 
       @param item    The KConfigSkeletonItem representing the preferences entry.
       @param parent  Parent widget.
+      @param sampleText Sample text for previewing the selected font.
     */
     KPrefsWidFont( KConfigSkeleton::ItemFont *item,
                    QWidget *parent, const QString &sampleText );
@@ -284,12 +368,12 @@ class KPrefsWidFont : public KPrefsWid
 
   This class provides a control element for configuring selections. It is meant
   to be used by KPrefsDialog. The user is responsible for the layout management.
-  
+
   The setting is interpreted as an int value, corresponding to the position of
   the radio button. The position of the button is defined by the sequence of
   @ref addRadio() calls, starting with 0.
 */
-class KPrefsWidRadios : public KPrefsWid
+class KDE_EXPORT KPrefsWidRadios : public KPrefsWid
 {
   public:
     /**
@@ -319,7 +403,7 @@ class KPrefsWidRadios : public KPrefsWid
     void readConfig();
     void writeConfig();
 
-    QValueList<QWidget *> widgets() const;    
+    QValueList<QWidget *> widgets() const;
 
   private:
     KConfigSkeleton::ItemEnum *mItem;
@@ -335,7 +419,7 @@ class KPrefsWidRadios : public KPrefsWid
   meant to be used by KPrefsDialog. The user is responsible for the layout
   management.
 */
-class KPrefsWidString : public KPrefsWid
+class KDE_EXPORT KPrefsWidString : public KPrefsWid
 {
   public:
     /**
@@ -344,6 +428,7 @@ class KPrefsWidString : public KPrefsWid
 
       @param item    The KConfigSkeletonItem representing the preferences entry.
       @param parent  Parent widget.
+      @param echomode  Describes how a line edit should display its contents.
     */
     KPrefsWidString( KConfigSkeleton::ItemString *item, QWidget *parent,
                      QLineEdit::EchoMode echomode=QLineEdit::Normal );
@@ -374,6 +459,54 @@ class KPrefsWidString : public KPrefsWid
 };
 
 
+/**
+  @short Widgets for string settings in @ref KPrefsDialog.
+
+  This class provides a control element for configuring string values. It is
+  meant to be used by KPrefsDialog. The user is responsible for the layout
+  management.
+*/
+class KDE_EXPORT KPrefsWidPath : public KPrefsWid
+{
+  public:
+    /**
+      Create a string value control element consisting of a test label and a
+      line edit.
+
+      @param item    The KConfigSkeletonItem representing the preferences entry.
+      @param parent  Parent widget.
+      @param filter URLRequester filter
+      @param mode  Describes how a line edit should display its contents.
+    */
+    KPrefsWidPath( KConfigSkeleton::ItemPath *item, QWidget *parent,
+                   const QString &filter = QString::null, uint mode = KFile::File );
+    /**
+      Destructor.
+    */
+    virtual ~KPrefsWidPath();
+
+    /**
+      Return QLabel used by this widget.
+    */
+    QLabel *label();
+    /**
+      Return QLineEdit used by this widget.
+    */
+    KURLRequester *urlRequester();
+
+    void readConfig();
+    void writeConfig();
+
+    QValueList<QWidget *> widgets() const;
+
+  private:
+    KConfigSkeleton::ItemPath *mItem;
+
+    QLabel *mLabel;
+    KURLRequester *mURLRequester;
+};
+
+
 /**
   @short Class for managing KPrefsWid objects.
 
@@ -382,7 +515,7 @@ class KPrefsWidString : public KPrefsWid
   transparent way. The user has to add the widgets by the corresponding addWid
   functions and KPrefsWidManager handles the rest automatically.
 */
-class KPrefsWidManager
+class KDE_EXPORT KPrefsWidManager
 {
   public:
     /**
@@ -421,6 +554,15 @@ class KPrefsWidManager
     KPrefsWidInt *addWidInt( KConfigSkeleton::ItemInt *item,
                              QWidget *parent );
 
+    /**
+      Register a @ref KPrefsWidDate object.
+
+      @param item    The KConfigSkeletonItem representing the preferences entry.
+      @param parent  Parent widget.
+    */
+    KPrefsWidDate *addWidDate( KConfigSkeleton::ItemDateTime *item,
+                               QWidget *parent );
+
     /**
       Register a @ref KPrefsWidTime object.
 
@@ -430,6 +572,15 @@ class KPrefsWidManager
     KPrefsWidTime *addWidTime( KConfigSkeleton::ItemDateTime *item,
                                QWidget *parent );
 
+    /**
+      Register a @ref KPrefsWidDuration object.
+
+      @param item    The KConfigSkeletonItem representing the preferences entry.
+      @param parent  Parent widget.
+    */
+    KPrefsWidDuration *addWidDuration( KConfigSkeleton::ItemDateTime *item,
+                                       QWidget *parent );
+
     /**
       Register a @ref KPrefsWidColor object.
 
@@ -438,7 +589,7 @@ class KPrefsWidManager
     */
     KPrefsWidColor *addWidColor( KConfigSkeleton::ItemColor *item,
                                  QWidget *parent );
-        
+
     /**
       Register a @ref KPrefsWidRadios object. The choices represented by the
       given item object are automatically added as radio buttons.
@@ -448,7 +599,7 @@ class KPrefsWidManager
     */
     KPrefsWidRadios *addWidRadios( KConfigSkeleton::ItemEnum *item,
                                    QWidget *parent );
-    
+
     /**
       Register a @ref KPrefsWidString object.
 
@@ -458,6 +609,18 @@ class KPrefsWidManager
     KPrefsWidString *addWidString( KConfigSkeleton::ItemString *item,
                                    QWidget *parent );
 
+    /**
+      Register a path @ref KPrefsWidPath object.
+
+      @param item    The KConfigSkeletonItem representing the preferences entry.
+      @param parent  Parent widget.
+      @param filter URLRequester filter
+      @param mode URLRequester mode
+    */
+    KPrefsWidPath *addWidPath ( KConfigSkeleton::ItemPath *item, QWidget *parent,
+                                const QString &filter = QString::null,
+                                uint mode = KFile::File );
+
     /**
       Register a password @ref KPrefsWidString object, with echomode set to QLineEdit::Password.
 
@@ -506,7 +669,7 @@ class KPrefsWidManager
   default values is handled automatically. Custom widgets have to be handled in
   the functions @ref usrReadConfig() and @ref usrWriteConfig().
 */
-class KPrefsDialog : public KDialogBase, public KPrefsWidManager
+class KDE_EXPORT KPrefsDialog : public KDialogBase, public KPrefsWidManager
 {
     Q_OBJECT
   public:
@@ -559,7 +722,7 @@ class KPrefsDialog : public KDialogBase, public KPrefsWidManager
 };
 
 
-class KPrefsModule : public KCModule, public KPrefsWidManager
+class KDE_EXPORT KPrefsModule : public KCModule, public KPrefsWidManager
 {
     Q_OBJECT
   public:
