@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 
@@ -29,6 +29,8 @@
 #include <qtextstream.h>
 #include <qurl.h>
 
+#include <kabc/ldapurl.h>
+#include <kabc/ldif.h>
 #include <kapplication.h>
 #include <kconfig.h>
 #include <kdebug.h>
@@ -36,21 +38,14 @@
 #include <kmdcodec.h>
 #include <kprotocolinfo.h>
 #include <kstandarddirs.h>
+#include <kstaticdeleter.h>
 
 #include "ldapclient.h"
-#include "ldif.h"
-#include "ldapurl.h"
 
 using namespace KPIM;
 
-class LdapClient::LdapClientPrivate{
-public:
-  QString bindDN;
-  QString pwdBindDN;
-  LDIF ldif;
-  int clientNumber;
-  int completionWeight;
-};
+KConfig *KPIM::LdapSearch::s_config = 0L;
+static KStaticDeleter<KConfig> configDeleter;
 
 QString LdapObject::toString() const
 {
@@ -58,7 +53,7 @@ QString LdapObject::toString() const
   for ( LdapAttrMap::ConstIterator it = attrs.begin(); it != attrs.end(); ++it ) {
     QString attr = it.key();
     for ( LdapAttrValue::ConstIterator it2 = (*it).begin(); it2 != (*it).end(); ++it2 ) {
-      result += QString::fromUtf8( LDIF::assembleLine( attr, *it2, 76 ) ) + "\n";
+      result += QString::fromUtf8( KABC::LDIF::assembleLine( attr, *it2, 76 ) ) + "\n";
     }
   }
 
@@ -84,40 +79,15 @@ void LdapObject::assign( const LdapObject& that )
 LdapClient::LdapClient( int clientNumber, QObject* parent, const char* name )
   : QObject( parent, name ), mJob( 0 ), mActive( false ), mReportObjectClass( false )
 {
-  d = new LdapClientPrivate;
-  d->clientNumber = clientNumber;
-  d->completionWeight = 50 - clientNumber;
+//  d = new LdapClientPrivate;
+  mClientNumber = clientNumber;
+  mCompletionWeight = 50 - mClientNumber;
 }
 
 LdapClient::~LdapClient()
 {
   cancelQuery();
-  delete d; d = 0;
-}
-
-void LdapClient::setHost( const QString& host )
-{
-  mHost = host;
-}
-
-void LdapClient::setPort( const QString& port )
-{
-  mPort = port;
-}
-
-void LdapClient::setBase( const QString& base )
-{
-  mBase = base;
-}
-
-void LdapClient::setBindDN( const QString& bindDN )
-{
-  d->bindDN = bindDN;
-}
-
-void LdapClient::setPwdBindDN( const QString& pwdBindDN )
-{
-  d->pwdBindDN = pwdBindDN;
+//  delete d; d = 0;
 }
 
 void LdapClient::setAttrs( const QStringList& attrs )
@@ -135,16 +105,31 @@ void LdapClient::setAttrs( const QStringList& attrs )
 void LdapClient::startQuery( const QString& filter )
 {
   cancelQuery();
-  LDAPUrl url;
+  KABC::LDAPUrl url;
 
-  url.setProtocol( "ldap" );
-  url.setUser( d->bindDN );
-  url.setPass( d->pwdBindDN );
-  url.setHost( mHost );
-  url.setPort( mPort.toUInt() );
-  url.setDn( mBase );
+  url.setProtocol( ( mServer.security() == LdapServer::SSL ) ? "ldaps" : "ldap" );
+  if ( mServer.auth() != LdapServer::Anonymous ) {
+    url.setUser( mServer.user() );
+    url.setPass( mServer.pwdBindDN() );
+  }
+  url.setHost( mServer.host() );
+  url.setPort( mServer.port() );
+  url.setExtension( "x-ver", QString::number( mServer.version() ) );
+  url.setDn( mServer.baseDN() );
+  url.setDn( mServer.baseDN() );
+  if ( mServer.security() == LdapServer::TLS ) url.setExtension( "x-tls","" );
+  if ( mServer.auth() == LdapServer::SASL ) {
+    url.setExtension( "x-sasl","" );
+    if ( !mServer.bindDN().isEmpty() ) url.setExtension( "x-bindname", mServer.bindDN() );
+    if ( !mServer.mech().isEmpty() ) url.setExtension( "x-mech", mServer.mech() );
+  }
+  if ( mServer.timeLimit() != 0 ) url.setExtension( "x-timelimit", 
+    QString::number( mServer.timeLimit() ) );
+  if ( mServer.sizeLimit() != 0 ) url.setExtension( "x-sizelimit", 
+    QString::number( mServer.sizeLimit() ) );
+  
   url.setAttributes( mAttrs );
-  url.setScope( mScope == "one" ? LDAPUrl::One : LDAPUrl::Sub );
+  url.setScope( mScope == "one" ? KABC::LDAPUrl::One : KABC::LDAPUrl::Sub );
   url.setFilter( "("+filter+")" );
 
   kdDebug(5300) << "LdapClient: Doing query: " << url.prettyURL() << endl;
@@ -172,10 +157,6 @@ void LdapClient::cancelQuery()
 
 void LdapClient::slotData( KIO::Job*, const QByteArray& data )
 {
-#ifndef NDEBUG // don't create the QString
-//  QString str( data );
-//  kdDebug(5700) << "LdapClient: Got \"" << str << "\"\n";
-#endif
   parseLDIF( data );
 }
 
@@ -195,7 +176,7 @@ void LdapClient::slotDone()
 #endif
   int err = mJob->error();
   if ( err && err != KIO::ERR_USER_CANCELED ) {
-    emit error( KIO::buildErrorString( err, QString("%1:%2").arg( mHost ).arg( mPort ) ) );
+    emit error( mJob->errorString() );
   }
   emit done();
 }
@@ -203,10 +184,7 @@ void LdapClient::slotDone()
 void LdapClient::startParseLDIF()
 {
   mCurrentObject.clear();
-  mLastAttrName  = 0;
-  mLastAttrValue = 0;
-  mIsBase64 = false;
-  d->ldif.startParsing();
+  mLdif.startParsing();
 }
 
 void LdapClient::endParseLDIF()
@@ -215,7 +193,7 @@ void LdapClient::endParseLDIF()
 
 void LdapClient::finishCurrentObject()
 {
-  mCurrentObject.dn = d->ldif.dn();
+  mCurrentObject.dn = mLdif.dn();
   const QString sClass( mCurrentObject.objectClass.lower() );
   if( sClass == "groupofnames" || sClass == "kolabgroupofnames" ){
     LdapAttrMap::ConstIterator it = mCurrentObject.attrs.find("mail");
@@ -247,65 +225,117 @@ void LdapClient::finishCurrentObject()
 
 void LdapClient::parseLDIF( const QByteArray& data )
 {
-  //kdDebug(5300) << "LdapClient::parseLDIF( " << QCString(data) << " )" << endl;
+  //kdDebug(5300) << "LdapClient::parseLDIF( " << QCString(data.data(), data.size()+1) << " )" << endl;
   if ( data.size() ) {
-    d->ldif.setLDIF( data );
+    mLdif.setLDIF( data );
   } else {
-    d->ldif.endLDIF();
+    mLdif.endLDIF();
   }
 
-  LDIF::ParseVal ret;
+  KABC::LDIF::ParseVal ret;
   QString name;
-  QByteArray value;
   do {
-    ret = d->ldif.nextItem();
+    ret = mLdif.nextItem();
     switch ( ret ) {
-      case LDIF::Item: 
+      case KABC::LDIF::Item: 
         {
-          name = d->ldif.attr();
-          value = d->ldif.val();
-          bool bFoundOC = name.lower() == "objectclass";
-          if( bFoundOC )
+          name = mLdif.attr();
+          // Must make a copy! QByteArray is explicitely shared
+          QByteArray value = mLdif.val().copy();
+          bool bIsObjectClass = name.lower() == "objectclass";
+          if( bIsObjectClass )
             mCurrentObject.objectClass = QString::fromUtf8( value, value.size() );
-          if( mReportObjectClass || !bFoundOC )
+          if( mReportObjectClass || !bIsObjectClass )
             mCurrentObject.attrs[ name ].append( value );
-          //kdDebug(5300) << "LdapClient::parseLDIF()" << name << " / " << value << endl;
+          //kdDebug(5300) << "LdapClient::parseLDIF(): name=" << name << " value=" << QCString(value.data(), value.size()+1) << endl;
         }
         break;
-     case LDIF::EndEntry:
+     case KABC::LDIF::EndEntry:
         finishCurrentObject();
         break;
       default:
         break;
     }
-  } while ( ret != LDIF::MoreData );
-}
-
-QString LdapClient::bindDN() const
-{
-  return d->bindDN;
-}
-
-QString LdapClient::pwdBindDN() const
-{
-  return d->pwdBindDN;
+  } while ( ret != KABC::LDIF::MoreData );
 }
 
 int LdapClient::clientNumber() const
 {
-  return d->clientNumber;
+  return mClientNumber;
 }
 
 int LdapClient::completionWeight() const
 {
-  return d->completionWeight;
+  return mCompletionWeight;
 }
 
-void KPIM::LdapClient::setCompletionWeight( int weight )
+void LdapClient::setCompletionWeight( int weight )
 {
-  d->completionWeight = weight;
+  mCompletionWeight = weight;
 }
 
+void LdapSearch::readConfig( LdapServer &server, KConfig *config, int j, bool active )
+{
+  QString prefix;
+  if ( active ) prefix = "Selected";
+  QString host =  config->readEntry( prefix + QString( "Host%1" ).arg( j ), "" ).stripWhiteSpace();
+  if ( !host.isEmpty() )
+    server.setHost( host );
+
+  int port = config->readNumEntry( prefix + QString( "Port%1" ).arg( j ), 389 );
+  server.setPort( port );
+
+  QString base = config->readEntry( prefix + QString( "Base%1" ).arg( j ), "" ).stripWhiteSpace();
+  if ( !base.isEmpty() )
+    server.setBaseDN( base );
+
+  QString user = config->readEntry( prefix + QString( "User%1" ).arg( j ) ).stripWhiteSpace();
+  if ( !user.isEmpty() )
+    server.setUser( user );
+
+  QString bindDN = config->readEntry( prefix + QString( "Bind%1" ).arg( j ) ).stripWhiteSpace();
+  if ( !bindDN.isEmpty() )
+    server.setBindDN( bindDN );
+
+  QString pwdBindDN = config->readEntry( prefix + QString( "PwdBind%1" ).arg( j ) );
+  if ( !pwdBindDN.isEmpty() )
+    server.setPwdBindDN( pwdBindDN );
+
+  server.setTimeLimit( config->readNumEntry( prefix + QString( "TimeLimit%1" ).arg( j ) ) );
+  server.setSizeLimit( config->readNumEntry( prefix + QString( "SizeLimit%1" ).arg( j ) ) );
+  server.setVersion( config->readNumEntry( prefix + QString( "Version%1" ).arg( j ), 3 ) );
+  server.setSecurity( config->readNumEntry( prefix + QString( "Security%1" ).arg( j ) ) );
+  server.setAuth( config->readNumEntry( prefix + QString( "Auth%1" ).arg( j ) ) );
+  server.setMech( config->readEntry( prefix + QString( "Mech%1" ).arg( j ) ) );
+}
+
+void LdapSearch::writeConfig( const LdapServer &server, KConfig *config, int j, bool active )
+{
+  QString prefix;
+  if ( active ) prefix = "Selected";
+  config->writeEntry( prefix + QString( "Host%1" ).arg( j ), server.host() );
+  config->writeEntry( prefix + QString( "Port%1" ).arg( j ), server.port() );
+  config->writeEntry( prefix + QString( "Base%1" ).arg( j ), server.baseDN() );
+  config->writeEntry( prefix + QString( "User%1" ).arg( j ), server.user() );
+  config->writeEntry( prefix + QString( "Bind%1" ).arg( j ), server.bindDN() );
+  config->writeEntry( prefix + QString( "PwdBind%1" ).arg( j ), server.pwdBindDN() );
+  config->writeEntry( prefix + QString( "TimeLimit%1" ).arg( j ), server.timeLimit() );
+  config->writeEntry( prefix + QString( "SizeLimit%1" ).arg( j ), server.sizeLimit() );
+  config->writeEntry( prefix + QString( "Version%1" ).arg( j ), server.version() );
+  config->writeEntry( prefix + QString( "Security%1" ).arg( j ), server.security() );
+  config->writeEntry( prefix + QString( "Auth%1" ).arg( j ), server.auth() );
+  config->writeEntry( prefix + QString( "Mech%1" ).arg( j ), server.mech() );
+}
+
+KConfig* LdapSearch::config()
+{
+  if ( !s_config )
+    configDeleter.setObject( s_config, new KConfig( "kabldaprc", false, false ) ); // Open read-write, no kdeglobals
+
+  return s_config;
+}
+
+
 LdapSearch::LdapSearch()
     : mActiveClients( 0 ), mNoLDAPLookup( false )
 {
@@ -321,7 +351,6 @@ LdapSearch::LdapSearch()
 
 void LdapSearch::readConfig()
 {
-  //kdDebug(5300) << "LdapClient::readConfig()" << endl;
   cancelSearch();
   QValueList< LdapClient* >::Iterator it;
   for ( it = mClients.begin(); it != mClients.end(); ++it )
@@ -329,38 +358,20 @@ void LdapSearch::readConfig()
   mClients.clear();
 
   // stolen from KAddressBook
-  KConfig config( "kabldaprc", true );
-  config.setGroup( "LDAP" );
-  int numHosts = config.readUnsignedNumEntry( "NumSelectedHosts");
+  KConfig *config = KPIM::LdapSearch::config();
+  config->setGroup( "LDAP" );
+  int numHosts = config->readUnsignedNumEntry( "NumSelectedHosts");
   if ( !numHosts ) {
     mNoLDAPLookup = true;
   } else {
     for ( int j = 0; j < numHosts; j++ ) {
       LdapClient* ldapClient = new LdapClient( j, this );
+      LdapServer server;
+      readConfig( server, config, j, true );
+      if ( !server.host().isEmpty() ) mNoLDAPLookup = false;
+      ldapClient->setServer( server );
 
-      QString host =  config.readEntry( QString( "SelectedHost%1" ).arg( j ), "" ).stripWhiteSpace();
-      if ( !host.isEmpty() ){
-        ldapClient->setHost( host );
-        mNoLDAPLookup = false;
-      }
-
-      QString port = QString::number( config.readUnsignedNumEntry( QString( "SelectedPort%1" ).arg( j ) ) );
-      if ( !port.isEmpty() )
-        ldapClient->setPort( port );
-
-      QString base = config.readEntry( QString( "SelectedBase%1" ).arg( j ), "" ).stripWhiteSpace();
-      if ( !base.isEmpty() )
-        ldapClient->setBase( base );
-
-      QString bindDN = config.readEntry( QString( "SelectedBind%1" ).arg( j ) ).stripWhiteSpace();
-      if ( !bindDN.isEmpty() )
-        ldapClient->setBindDN( bindDN );
-
-      QString pwdBindDN = config.readEntry( QString( "SelectedPwdBind%1" ).arg( j ) ).stripWhiteSpace();
-      if ( !pwdBindDN.isEmpty() )
-        ldapClient->setPwdBindDN( pwdBindDN );
-
-      int completionWeight = config.readNumEntry( QString( "SelectedCompletionWeight%1" ).arg( j ), -1 );
+      int completionWeight = config->readNumEntry( QString( "SelectedCompletionWeight%1" ).arg( j ), -1 );
       if ( completionWeight != -1 )
         ldapClient->setCompletionWeight( completionWeight );
 
@@ -387,7 +398,6 @@ void LdapSearch::readConfig()
 
 void LdapSearch::slotFileChanged( const QString& file )
 {
-  //kdDebug(5300) << "LdapClient::slotFileChanged( " << file << " )" << endl;
   if ( file == mConfigFile )
     readConfig();
 }
@@ -411,7 +421,13 @@ void LdapSearch::startSearch( const QString& txt )
   } else
     mSearchText = txt;
 
-  QString filter = QString( "|(cn=%1*)(mail=%2*)(givenName=%3*)(sn=%4*)" )
+  /* The reasoning behind this filter is:
+   * If it's a person, or a distlist, show it, even if it doesn't have an email address.
+   * If it's not a person, or a distlist, only show it if it has an email attribute.
+   * This allows both resource accounts with an email address which are not a person and
+   * person entries without an email address to show up, while still not showing things
+   * like structural entries in the ldap tree. */
+  QString filter = QString( "&(|(objectclass=person)(objectclass=groupOfNames)(mail=*))(|(cn=%1*)(mail=%2*)(givenName=%3*)(sn=%4*))" )
       .arg( mSearchText ).arg( mSearchText ).arg( mSearchText ).arg( mSearchText );
 
   QValueList< LdapClient* >::Iterator it;
@@ -527,7 +543,7 @@ void LdapSearch::makeSearchData( QStringList& ret, LdapResultList& resList )
       else if( it2.key() == "sn" )
         sn = tmp;
       else if( it2.key() == "objectClass" &&
-               ( tmp == "groupOfNames" || tmp == "kolabGroupOfNames" ) ) {
+               (tmp == "groupOfNames" || tmp == "kolabGroupOfNames") ) {
         isDistributionList = true;
       }
     }
@@ -579,4 +595,5 @@ bool LdapSearch::isAvailable() const
   return !mNoLDAPLookup;
 }
 
+
 #include "ldapclient.moc"
